/****************************************************************************
** Meta object code from reading C++ file 'AudioPluginDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/AudioPluginDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AudioPluginDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__AudioPluginDialog_t {
    QByteArrayData data[36];
    char stringdata0[494];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__AudioPluginDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__AudioPluginDialog_t qt_meta_stringdata_Rosegarden__AudioPluginDialog = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Rosegarden::AudioPluginDialog"
QT_MOC_LITERAL(1, 30, 14), // "pluginSelected"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 12), // "InstrumentId"
QT_MOC_LITERAL(4, 59, 11), // "pluginIndex"
QT_MOC_LITERAL(5, 71, 6), // "plugin"
QT_MOC_LITERAL(6, 78, 17), // "pluginPortChanged"
QT_MOC_LITERAL(7, 96, 9), // "portIndex"
QT_MOC_LITERAL(8, 106, 20), // "pluginProgramChanged"
QT_MOC_LITERAL(9, 127, 25), // "changePluginConfiguration"
QT_MOC_LITERAL(10, 153, 6), // "global"
QT_MOC_LITERAL(11, 160, 3), // "key"
QT_MOC_LITERAL(12, 164, 5), // "value"
QT_MOC_LITERAL(13, 170, 13), // "showPluginGUI"
QT_MOC_LITERAL(14, 184, 13), // "stopPluginGUI"
QT_MOC_LITERAL(15, 198, 8), // "bypassed"
QT_MOC_LITERAL(16, 207, 2), // "bp"
QT_MOC_LITERAL(17, 210, 9), // "destroyed"
QT_MOC_LITERAL(18, 220, 5), // "index"
QT_MOC_LITERAL(19, 226, 21), // "slotSearchTextChanged"
QT_MOC_LITERAL(20, 248, 4), // "text"
QT_MOC_LITERAL(21, 253, 16), // "slotArchSelected"
QT_MOC_LITERAL(22, 270, 20), // "slotCategorySelected"
QT_MOC_LITERAL(23, 291, 18), // "slotPluginSelected"
QT_MOC_LITERAL(24, 310, 21), // "slotPluginPortChanged"
QT_MOC_LITERAL(25, 332, 24), // "slotPluginProgramChanged"
QT_MOC_LITERAL(26, 357, 17), // "slotBypassChanged"
QT_MOC_LITERAL(27, 375, 8), // "slotCopy"
QT_MOC_LITERAL(28, 384, 9), // "slotPaste"
QT_MOC_LITERAL(29, 394, 11), // "slotDefault"
QT_MOC_LITERAL(30, 406, 11), // "slotShowGUI"
QT_MOC_LITERAL(31, 418, 17), // "slotHelpRequested"
QT_MOC_LITERAL(32, 436, 14), // "slotParameters"
QT_MOC_LITERAL(33, 451, 11), // "slotPresets"
QT_MOC_LITERAL(34, 463, 19), // "slotEditConnections"
QT_MOC_LITERAL(35, 483, 10) // "slotEditor"

    },
    "Rosegarden::AudioPluginDialog\0"
    "pluginSelected\0\0InstrumentId\0pluginIndex\0"
    "plugin\0pluginPortChanged\0portIndex\0"
    "pluginProgramChanged\0changePluginConfiguration\0"
    "global\0key\0value\0showPluginGUI\0"
    "stopPluginGUI\0bypassed\0bp\0destroyed\0"
    "index\0slotSearchTextChanged\0text\0"
    "slotArchSelected\0slotCategorySelected\0"
    "slotPluginSelected\0slotPluginPortChanged\0"
    "slotPluginProgramChanged\0slotBypassChanged\0"
    "slotCopy\0slotPaste\0slotDefault\0"
    "slotShowGUI\0slotHelpRequested\0"
    "slotParameters\0slotPresets\0"
    "slotEditConnections\0slotEditor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__AudioPluginDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  134,    2, 0x06 /* Public */,
       6,    3,  141,    2, 0x06 /* Public */,
       8,    2,  148,    2, 0x06 /* Public */,
       9,    5,  153,    2, 0x06 /* Public */,
      13,    2,  164,    2, 0x06 /* Public */,
      14,    2,  169,    2, 0x06 /* Public */,
      15,    3,  174,    2, 0x06 /* Public */,
      17,    2,  181,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    1,  186,    2, 0x0a /* Public */,
      21,    1,  189,    2, 0x0a /* Public */,
      22,    1,  192,    2, 0x0a /* Public */,
      23,    1,  195,    2, 0x0a /* Public */,
      24,    1,  198,    2, 0x0a /* Public */,
      25,    1,  201,    2, 0x0a /* Public */,
      26,    1,  204,    2, 0x0a /* Public */,
      27,    0,  207,    2, 0x0a /* Public */,
      28,    0,  208,    2, 0x0a /* Public */,
      29,    0,  209,    2, 0x0a /* Public */,
      30,    0,  210,    2, 0x0a /* Public */,
      31,    0,  211,    2, 0x0a /* Public */,
      32,    0,  212,    2, 0x0a /* Public */,
      33,    0,  213,    2, 0x0a /* Public */,
      34,    0,  214,    2, 0x0a /* Public */,
      35,    0,  215,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    4,    5,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    4,    7,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Bool, QMetaType::QString, QMetaType::QString,    2,    4,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Bool,    2,    4,   16,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,   18,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Float,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::AudioPluginDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioPluginDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pluginSelected((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->pluginPortChanged((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 2: _t->pluginProgramChanged((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->changePluginConfiguration((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        case 4: _t->showPluginGUI((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->stopPluginGUI((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->bypassed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 7: _t->destroyed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->slotSearchTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->slotArchSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->slotCategorySelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->slotPluginSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slotPluginPortChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 13: _t->slotPluginProgramChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->slotBypassChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->slotCopy(); break;
        case 16: _t->slotPaste(); break;
        case 17: _t->slotDefault(); break;
        case 18: _t->slotShowGUI(); break;
        case 19: _t->slotHelpRequested(); break;
        case 20: _t->slotParameters(); break;
        case 21: _t->slotPresets(); break;
        case 22: _t->slotEditConnections(); break;
        case 23: _t->slotEditor(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::pluginSelected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::pluginPortChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::pluginProgramChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int , bool , QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::changePluginConfiguration)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::showPluginGUI)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::stopPluginGUI)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::bypassed)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AudioPluginDialog::*)(InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioPluginDialog::destroyed)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::AudioPluginDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__AudioPluginDialog.data,
    qt_meta_data_Rosegarden__AudioPluginDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::AudioPluginDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::AudioPluginDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__AudioPluginDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::AudioPluginDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::AudioPluginDialog::pluginSelected(InstrumentId _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::AudioPluginDialog::pluginPortChanged(InstrumentId _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::AudioPluginDialog::pluginProgramChanged(InstrumentId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::AudioPluginDialog::changePluginConfiguration(InstrumentId _t1, int _t2, bool _t3, QString _t4, QString _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::AudioPluginDialog::showPluginGUI(InstrumentId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::AudioPluginDialog::stopPluginGUI(InstrumentId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Rosegarden::AudioPluginDialog::bypassed(InstrumentId _t1, int _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::AudioPluginDialog::destroyed(InstrumentId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
