/****************************************************************************
** Meta object code from reading C++ file 'CompositionModelImpl.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/compositionview/CompositionModelImpl.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CompositionModelImpl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__CompositionModelImpl_t {
    QByteArrayData data[15];
    char stringdata0[266];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__CompositionModelImpl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__CompositionModelImpl_t qt_meta_stringdata_Rosegarden__CompositionModelImpl = {
    {
QT_MOC_LITERAL(0, 0, 32), // "Rosegarden::CompositionModelImpl"
QT_MOC_LITERAL(1, 33, 10), // "needUpdate"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 19), // "needArtifactsUpdate"
QT_MOC_LITERAL(4, 65, 14), // "needSizeUpdate"
QT_MOC_LITERAL(5, 80, 16), // "selectionChanged"
QT_MOC_LITERAL(6, 97, 16), // "SegmentSelection"
QT_MOC_LITERAL(7, 114, 22), // "slotAudioFileFinalized"
QT_MOC_LITERAL(8, 137, 8), // "Segment*"
QT_MOC_LITERAL(9, 146, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(10, 165, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(11, 185, 20), // "slotDocumentModified"
QT_MOC_LITERAL(12, 206, 22), // "slotAudioPeaksComplete"
QT_MOC_LITERAL(13, 229, 20), // "AudioPeaksGenerator*"
QT_MOC_LITERAL(14, 250, 15) // "slotUpdateTimer"

    },
    "Rosegarden::CompositionModelImpl\0"
    "needUpdate\0\0needArtifactsUpdate\0"
    "needSizeUpdate\0selectionChanged\0"
    "SegmentSelection\0slotAudioFileFinalized\0"
    "Segment*\0slotDocumentLoaded\0"
    "RosegardenDocument*\0slotDocumentModified\0"
    "slotAudioPeaksComplete\0AudioPeaksGenerator*\0"
    "slotUpdateTimer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__CompositionModelImpl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       1,    1,   65,    2, 0x06 /* Public */,
       3,    0,   68,    2, 0x06 /* Public */,
       4,    0,   69,    2, 0x06 /* Public */,
       5,    1,   70,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   73,    2, 0x0a /* Public */,
       9,    1,   76,    2, 0x08 /* Private */,
      11,    1,   79,    2, 0x08 /* Private */,
      12,    1,   82,    2, 0x08 /* Private */,
      14,    0,   85,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRect,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::CompositionModelImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CompositionModelImpl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->needUpdate(); break;
        case 1: _t->needUpdate((*reinterpret_cast< const QRect(*)>(_a[1]))); break;
        case 2: _t->needArtifactsUpdate(); break;
        case 3: _t->needSizeUpdate(); break;
        case 4: _t->selectionChanged((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 5: _t->slotAudioFileFinalized((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 6: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 7: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotAudioPeaksComplete((*reinterpret_cast< AudioPeaksGenerator*(*)>(_a[1]))); break;
        case 9: _t->slotUpdateTimer(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CompositionModelImpl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionModelImpl::needUpdate)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CompositionModelImpl::*)(const QRect & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionModelImpl::needUpdate)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CompositionModelImpl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionModelImpl::needArtifactsUpdate)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CompositionModelImpl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionModelImpl::needSizeUpdate)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CompositionModelImpl::*)(const SegmentSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionModelImpl::selectionChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::CompositionModelImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__CompositionModelImpl.data,
    qt_meta_data_Rosegarden__CompositionModelImpl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::CompositionModelImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::CompositionModelImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__CompositionModelImpl.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    if (!strcmp(_clname, "SegmentObserver"))
        return static_cast< SegmentObserver*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::CompositionModelImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::CompositionModelImpl::needUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::CompositionModelImpl::needUpdate(const QRect & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::CompositionModelImpl::needArtifactsUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::CompositionModelImpl::needSizeUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::CompositionModelImpl::selectionChanged(const SegmentSelection & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
