/****************************************************************************
** Meta object code from reading C++ file 'TranzportClient.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/application/TranzportClient.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TranzportClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TranzportClient_t {
    QByteArrayData data[27];
    char stringdata0[296];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TranzportClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TranzportClient_t qt_meta_stringdata_Rosegarden__TranzportClient = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::TranzportClient"
QT_MOC_LITERAL(1, 28, 4), // "play"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 4), // "stop"
QT_MOC_LITERAL(4, 39, 6), // "record"
QT_MOC_LITERAL(5, 46, 6), // "rewind"
QT_MOC_LITERAL(6, 53, 17), // "rewindToBeginning"
QT_MOC_LITERAL(7, 71, 11), // "fastForward"
QT_MOC_LITERAL(8, 83, 16), // "fastForwardToEnd"
QT_MOC_LITERAL(9, 100, 12), // "toggleRecord"
QT_MOC_LITERAL(10, 113, 9), // "trackDown"
QT_MOC_LITERAL(11, 123, 7), // "trackUp"
QT_MOC_LITERAL(12, 131, 9), // "trackMute"
QT_MOC_LITERAL(13, 141, 11), // "trackRecord"
QT_MOC_LITERAL(14, 153, 4), // "solo"
QT_MOC_LITERAL(15, 158, 4), // "undo"
QT_MOC_LITERAL(16, 163, 4), // "redo"
QT_MOC_LITERAL(17, 168, 11), // "setPosition"
QT_MOC_LITERAL(18, 180, 5), // "timeT"
QT_MOC_LITERAL(19, 186, 8), // "readData"
QT_MOC_LITERAL(20, 195, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(21, 214, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(22, 234, 3), // "doc"
QT_MOC_LITERAL(23, 238, 17), // "writeCommandQueue"
QT_MOC_LITERAL(24, 256, 22), // "pointerPositionChanged"
QT_MOC_LITERAL(25, 279, 4), // "time"
QT_MOC_LITERAL(26, 284, 11) // "loopChanged"

    },
    "Rosegarden::TranzportClient\0play\0\0"
    "stop\0record\0rewind\0rewindToBeginning\0"
    "fastForward\0fastForwardToEnd\0toggleRecord\0"
    "trackDown\0trackUp\0trackMute\0trackRecord\0"
    "solo\0undo\0redo\0setPosition\0timeT\0"
    "readData\0slotDocumentLoaded\0"
    "RosegardenDocument*\0doc\0writeCommandQueue\0"
    "pointerPositionChanged\0time\0loopChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TranzportClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x06 /* Public */,
       3,    0,  120,    2, 0x06 /* Public */,
       4,    0,  121,    2, 0x06 /* Public */,
       5,    0,  122,    2, 0x06 /* Public */,
       6,    0,  123,    2, 0x06 /* Public */,
       7,    0,  124,    2, 0x06 /* Public */,
       8,    0,  125,    2, 0x06 /* Public */,
       9,    0,  126,    2, 0x06 /* Public */,
      10,    0,  127,    2, 0x06 /* Public */,
      11,    0,  128,    2, 0x06 /* Public */,
      12,    0,  129,    2, 0x06 /* Public */,
      13,    0,  130,    2, 0x06 /* Public */,
      14,    1,  131,    2, 0x06 /* Public */,
      15,    0,  134,    2, 0x06 /* Public */,
      16,    0,  135,    2, 0x06 /* Public */,
      17,    1,  136,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    0,  139,    2, 0x0a /* Public */,
      20,    1,  140,    2, 0x0a /* Public */,
      23,    0,  143,    2, 0x0a /* Public */,
      24,    1,  144,    2, 0x0a /* Public */,
      26,    0,  147,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   25,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TranzportClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TranzportClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->play(); break;
        case 1: _t->stop(); break;
        case 2: _t->record(); break;
        case 3: _t->rewind(); break;
        case 4: _t->rewindToBeginning(); break;
        case 5: _t->fastForward(); break;
        case 6: _t->fastForwardToEnd(); break;
        case 7: _t->toggleRecord(); break;
        case 8: _t->trackDown(); break;
        case 9: _t->trackUp(); break;
        case 10: _t->trackMute(); break;
        case 11: _t->trackRecord(); break;
        case 12: _t->solo((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->undo(); break;
        case 14: _t->redo(); break;
        case 15: _t->setPosition((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 16: _t->readData(); break;
        case 17: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 18: _t->writeCommandQueue(); break;
        case 19: _t->pointerPositionChanged((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 20: _t->loopChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::play)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::stop)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::record)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::rewind)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::rewindToBeginning)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::fastForward)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::fastForwardToEnd)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::toggleRecord)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::trackDown)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::trackUp)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::trackMute)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::trackRecord)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::solo)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::undo)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::redo)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (TranzportClient::*)(timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TranzportClient::setPosition)) {
                *result = 15;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TranzportClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TranzportClient.data,
    qt_meta_data_Rosegarden__TranzportClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TranzportClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TranzportClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TranzportClient.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::TranzportClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TranzportClient::play()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::TranzportClient::stop()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::TranzportClient::record()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::TranzportClient::rewind()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::TranzportClient::rewindToBeginning()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::TranzportClient::fastForward()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::TranzportClient::fastForwardToEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::TranzportClient::toggleRecord()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Rosegarden::TranzportClient::trackDown()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Rosegarden::TranzportClient::trackUp()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Rosegarden::TranzportClient::trackMute()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Rosegarden::TranzportClient::trackRecord()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Rosegarden::TranzportClient::solo(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Rosegarden::TranzportClient::undo()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Rosegarden::TranzportClient::redo()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Rosegarden::TranzportClient::setPosition(timeT _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
