/****************************************************************************
** Meta object code from reading C++ file 'MatrixScene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/matrix/MatrixScene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MatrixScene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MatrixScene_t {
    QByteArrayData data[19];
    char stringdata0[261];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MatrixScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MatrixScene_t qt_meta_stringdata_Rosegarden__MatrixScene = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Rosegarden::MatrixScene"
QT_MOC_LITERAL(1, 24, 12), // "mousePressed"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 23), // "const MatrixMouseEvent*"
QT_MOC_LITERAL(4, 62, 1), // "e"
QT_MOC_LITERAL(5, 64, 10), // "mouseMoved"
QT_MOC_LITERAL(6, 75, 13), // "mouseReleased"
QT_MOC_LITERAL(7, 89, 18), // "mouseDoubleClicked"
QT_MOC_LITERAL(8, 108, 12), // "eventRemoved"
QT_MOC_LITERAL(9, 121, 6), // "Event*"
QT_MOC_LITERAL(10, 128, 25), // "currentViewSegmentChanged"
QT_MOC_LITERAL(11, 154, 12), // "ViewSegment*"
QT_MOC_LITERAL(12, 167, 18), // "selectionChangedES"
QT_MOC_LITERAL(13, 186, 15), // "EventSelection*"
QT_MOC_LITERAL(14, 202, 1), // "s"
QT_MOC_LITERAL(15, 204, 14), // "segmentDeleted"
QT_MOC_LITERAL(16, 219, 8), // "Segment*"
QT_MOC_LITERAL(17, 228, 12), // "sceneDeleted"
QT_MOC_LITERAL(18, 241, 19) // "slotCommandExecuted"

    },
    "Rosegarden::MatrixScene\0mousePressed\0"
    "\0const MatrixMouseEvent*\0e\0mouseMoved\0"
    "mouseReleased\0mouseDoubleClicked\0"
    "eventRemoved\0Event*\0currentViewSegmentChanged\0"
    "ViewSegment*\0selectionChangedES\0"
    "EventSelection*\0s\0segmentDeleted\0"
    "Segment*\0sceneDeleted\0slotCommandExecuted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MatrixScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       5,    1,   67,    2, 0x06 /* Public */,
       6,    1,   70,    2, 0x06 /* Public */,
       7,    1,   73,    2, 0x06 /* Public */,
       8,    1,   76,    2, 0x06 /* Public */,
      10,    1,   79,    2, 0x06 /* Public */,
      12,    1,   82,    2, 0x06 /* Public */,
      15,    1,   85,    2, 0x06 /* Public */,
      17,    0,   88,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,   89,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 9,    4,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Rosegarden::MatrixScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MatrixScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->mousePressed((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 1: _t->mouseMoved((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 2: _t->mouseReleased((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 3: _t->mouseDoubleClicked((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 4: _t->eventRemoved((*reinterpret_cast< Event*(*)>(_a[1]))); break;
        case 5: _t->currentViewSegmentChanged((*reinterpret_cast< ViewSegment*(*)>(_a[1]))); break;
        case 6: _t->selectionChangedES((*reinterpret_cast< EventSelection*(*)>(_a[1]))); break;
        case 7: _t->segmentDeleted((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 8: _t->sceneDeleted(); break;
        case 9: _t->slotCommandExecuted(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MatrixScene::*)(const MatrixMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::mousePressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(const MatrixMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::mouseMoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(const MatrixMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::mouseReleased)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(const MatrixMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::mouseDoubleClicked)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(Event * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::eventRemoved)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(ViewSegment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::currentViewSegmentChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(EventSelection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::selectionChangedES)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::segmentDeleted)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (MatrixScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixScene::sceneDeleted)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MatrixScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MatrixScene.data,
    qt_meta_data_Rosegarden__MatrixScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MatrixScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MatrixScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MatrixScene.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int Rosegarden::MatrixScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::MatrixScene::mousePressed(const MatrixMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::MatrixScene::mouseMoved(const MatrixMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::MatrixScene::mouseReleased(const MatrixMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::MatrixScene::mouseDoubleClicked(const MatrixMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::MatrixScene::eventRemoved(Event * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::MatrixScene::currentViewSegmentChanged(ViewSegment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Rosegarden::MatrixScene::selectionChangedES(EventSelection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::MatrixScene::segmentDeleted(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::MatrixScene::sceneDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
