/****************************************************************************
** Meta object code from reading C++ file 'NotationSelector.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/notation/NotationSelector.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NotationSelector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__NotationSelector_t {
    QByteArrayData data[19];
    char stringdata0[322];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__NotationSelector_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__NotationSelector_t qt_meta_stringdata_Rosegarden__NotationSelector = {
    {
QT_MOC_LITERAL(0, 0, 28), // "Rosegarden::NotationSelector"
QT_MOC_LITERAL(1, 29, 11), // "editElement"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 14), // "NotationStaff*"
QT_MOC_LITERAL(4, 57, 16), // "NotationElement*"
QT_MOC_LITERAL(5, 74, 18), // "slotInsertSelected"
QT_MOC_LITERAL(6, 93, 17), // "slotEraseSelected"
QT_MOC_LITERAL(7, 111, 21), // "slotCollapseRestsHard"
QT_MOC_LITERAL(8, 133, 15), // "slotRespellFlat"
QT_MOC_LITERAL(9, 149, 16), // "slotRespellSharp"
QT_MOC_LITERAL(10, 166, 18), // "slotRespellNatural"
QT_MOC_LITERAL(11, 185, 17), // "slotCollapseNotes"
QT_MOC_LITERAL(12, 203, 13), // "slotInterpret"
QT_MOC_LITERAL(13, 217, 14), // "slotStaffAbove"
QT_MOC_LITERAL(14, 232, 14), // "slotStaffBelow"
QT_MOC_LITERAL(15, 247, 17), // "slotMakeInvisible"
QT_MOC_LITERAL(16, 265, 15), // "slotMakeVisible"
QT_MOC_LITERAL(17, 281, 16), // "slotClickTimeout"
QT_MOC_LITERAL(18, 298, 23) // "slotMoveInsertionCursor"

    },
    "Rosegarden::NotationSelector\0editElement\0"
    "\0NotationStaff*\0NotationElement*\0"
    "slotInsertSelected\0slotEraseSelected\0"
    "slotCollapseRestsHard\0slotRespellFlat\0"
    "slotRespellSharp\0slotRespellNatural\0"
    "slotCollapseNotes\0slotInterpret\0"
    "slotStaffAbove\0slotStaffBelow\0"
    "slotMakeInvisible\0slotMakeVisible\0"
    "slotClickTimeout\0slotMoveInsertionCursor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__NotationSelector[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   94,    2, 0x0a /* Public */,
       6,    0,   95,    2, 0x0a /* Public */,
       7,    0,   96,    2, 0x0a /* Public */,
       8,    0,   97,    2, 0x0a /* Public */,
       9,    0,   98,    2, 0x0a /* Public */,
      10,    0,   99,    2, 0x0a /* Public */,
      11,    0,  100,    2, 0x0a /* Public */,
      12,    0,  101,    2, 0x0a /* Public */,
      13,    0,  102,    2, 0x0a /* Public */,
      14,    0,  103,    2, 0x0a /* Public */,
      15,    0,  104,    2, 0x0a /* Public */,
      16,    0,  105,    2, 0x0a /* Public */,
      17,    0,  106,    2, 0x0a /* Public */,
      18,    0,  107,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 4,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::NotationSelector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotationSelector *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editElement((*reinterpret_cast< NotationStaff*(*)>(_a[1])),(*reinterpret_cast< NotationElement*(*)>(_a[2]))); break;
        case 1: _t->slotInsertSelected(); break;
        case 2: _t->slotEraseSelected(); break;
        case 3: _t->slotCollapseRestsHard(); break;
        case 4: _t->slotRespellFlat(); break;
        case 5: _t->slotRespellSharp(); break;
        case 6: _t->slotRespellNatural(); break;
        case 7: _t->slotCollapseNotes(); break;
        case 8: _t->slotInterpret(); break;
        case 9: _t->slotStaffAbove(); break;
        case 10: _t->slotStaffBelow(); break;
        case 11: _t->slotMakeInvisible(); break;
        case 12: _t->slotMakeVisible(); break;
        case 13: _t->slotClickTimeout(); break;
        case 14: _t->slotMoveInsertionCursor(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NotationSelector::*)(NotationStaff * , NotationElement * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationSelector::editElement)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::NotationSelector::staticMetaObject = { {
    QMetaObject::SuperData::link<NotationTool::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__NotationSelector.data,
    qt_meta_data_Rosegarden__NotationSelector,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::NotationSelector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::NotationSelector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__NotationSelector.stringdata0))
        return static_cast<void*>(this);
    return NotationTool::qt_metacast(_clname);
}

int Rosegarden::NotationSelector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NotationTool::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::NotationSelector::editElement(NotationStaff * _t1, NotationElement * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
