/****************************************************************************
** Meta object code from reading C++ file 'AudioMixerWindow2.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/studio/AudioMixerWindow2.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AudioMixerWindow2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__AudioMixerWindow2_t {
    QByteArrayData data[21];
    char stringdata0[358];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__AudioMixerWindow2_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__AudioMixerWindow2_t qt_meta_stringdata_Rosegarden__AudioMixerWindow2 = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Rosegarden::AudioMixerWindow2"
QT_MOC_LITERAL(1, 30, 20), // "slotDocumentModified"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 8), // "modified"
QT_MOC_LITERAL(4, 61, 9), // "slotClose"
QT_MOC_LITERAL(5, 71, 24), // "slotNumberOfStereoInputs"
QT_MOC_LITERAL(6, 96, 22), // "slotNumberOfSubmasters"
QT_MOC_LITERAL(7, 119, 14), // "slotPanningLaw"
QT_MOC_LITERAL(8, 134, 19), // "slotShowAudioFaders"
QT_MOC_LITERAL(9, 154, 19), // "slotShowSynthFaders"
QT_MOC_LITERAL(10, 174, 23), // "slotShowAudioSubmasters"
QT_MOC_LITERAL(11, 198, 24), // "slotShowUnassignedFaders"
QT_MOC_LITERAL(12, 223, 8), // "slotHelp"
QT_MOC_LITERAL(13, 232, 19), // "slotAboutRosegarden"
QT_MOC_LITERAL(14, 252, 22), // "slotExternalController"
QT_MOC_LITERAL(15, 275, 18), // "const MappedEvent*"
QT_MOC_LITERAL(16, 294, 5), // "event"
QT_MOC_LITERAL(17, 300, 17), // "slotControlChange"
QT_MOC_LITERAL(18, 318, 11), // "Instrument*"
QT_MOC_LITERAL(19, 330, 10), // "instrument"
QT_MOC_LITERAL(20, 341, 16) // "controllerNumber"

    },
    "Rosegarden::AudioMixerWindow2\0"
    "slotDocumentModified\0\0modified\0slotClose\0"
    "slotNumberOfStereoInputs\0"
    "slotNumberOfSubmasters\0slotPanningLaw\0"
    "slotShowAudioFaders\0slotShowSynthFaders\0"
    "slotShowAudioSubmasters\0"
    "slotShowUnassignedFaders\0slotHelp\0"
    "slotAboutRosegarden\0slotExternalController\0"
    "const MappedEvent*\0event\0slotControlChange\0"
    "Instrument*\0instrument\0controllerNumber"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__AudioMixerWindow2[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x08 /* Private */,
       4,    0,   82,    2, 0x08 /* Private */,
       5,    0,   83,    2, 0x08 /* Private */,
       6,    0,   84,    2, 0x08 /* Private */,
       7,    0,   85,    2, 0x08 /* Private */,
       8,    0,   86,    2, 0x08 /* Private */,
       9,    0,   87,    2, 0x08 /* Private */,
      10,    0,   88,    2, 0x08 /* Private */,
      11,    0,   89,    2, 0x08 /* Private */,
      12,    0,   90,    2, 0x08 /* Private */,
      13,    0,   91,    2, 0x08 /* Private */,
      14,    1,   92,    2, 0x08 /* Private */,
      17,    2,   95,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 18, QMetaType::Int,   19,   20,

       0        // eod
};

void Rosegarden::AudioMixerWindow2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioMixerWindow2 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->slotClose(); break;
        case 2: _t->slotNumberOfStereoInputs(); break;
        case 3: _t->slotNumberOfSubmasters(); break;
        case 4: _t->slotPanningLaw(); break;
        case 5: _t->slotShowAudioFaders(); break;
        case 6: _t->slotShowSynthFaders(); break;
        case 7: _t->slotShowAudioSubmasters(); break;
        case 8: _t->slotShowUnassignedFaders(); break;
        case 9: _t->slotHelp(); break;
        case 10: _t->slotAboutRosegarden(); break;
        case 11: _t->slotExternalController((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 12: _t->slotControlChange((*reinterpret_cast< Instrument*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Instrument* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::AudioMixerWindow2::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__AudioMixerWindow2.data,
    qt_meta_data_Rosegarden__AudioMixerWindow2,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::AudioMixerWindow2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::AudioMixerWindow2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__AudioMixerWindow2.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::AudioMixerWindow2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
