/****************************************************************************
** Meta object code from reading C++ file 'MIDIInstrumentParameterPanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/parameters/MIDIInstrumentParameterPanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MIDIInstrumentParameterPanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel_t {
    QByteArrayData data[25];
    char stringdata0[404];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel_t qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel = {
    {
QT_MOC_LITERAL(0, 0, 40), // "Rosegarden::MIDIInstrumentPar..."
QT_MOC_LITERAL(1, 41, 25), // "slotExternalProgramChange"
QT_MOC_LITERAL(2, 67, 0), // ""
QT_MOC_LITERAL(3, 68, 7), // "bankMSB"
QT_MOC_LITERAL(4, 76, 7), // "bankLSB"
QT_MOC_LITERAL(5, 84, 13), // "programChange"
QT_MOC_LITERAL(6, 98, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(7, 117, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(8, 137, 20), // "slotDocumentModified"
QT_MOC_LITERAL(9, 158, 17), // "slotControlChange"
QT_MOC_LITERAL(10, 176, 11), // "Instrument*"
QT_MOC_LITERAL(11, 188, 10), // "instrument"
QT_MOC_LITERAL(12, 199, 2), // "cc"
QT_MOC_LITERAL(13, 202, 21), // "slotPercussionClicked"
QT_MOC_LITERAL(14, 224, 7), // "checked"
QT_MOC_LITERAL(15, 232, 15), // "slotBankClicked"
QT_MOC_LITERAL(16, 248, 14), // "slotSelectBank"
QT_MOC_LITERAL(17, 263, 5), // "index"
QT_MOC_LITERAL(18, 269, 18), // "slotProgramClicked"
QT_MOC_LITERAL(19, 288, 17), // "slotSelectProgram"
QT_MOC_LITERAL(20, 306, 20), // "slotVariationClicked"
QT_MOC_LITERAL(21, 327, 19), // "slotSelectVariation"
QT_MOC_LITERAL(22, 347, 17), // "slotSelectChannel"
QT_MOC_LITERAL(23, 365, 21), // "slotControllerChanged"
QT_MOC_LITERAL(24, 387, 16) // "controllerNumber"

    },
    "Rosegarden::MIDIInstrumentParameterPanel\0"
    "slotExternalProgramChange\0\0bankMSB\0"
    "bankLSB\0programChange\0slotDocumentLoaded\0"
    "RosegardenDocument*\0slotDocumentModified\0"
    "slotControlChange\0Instrument*\0instrument\0"
    "cc\0slotPercussionClicked\0checked\0"
    "slotBankClicked\0slotSelectBank\0index\0"
    "slotProgramClicked\0slotSelectProgram\0"
    "slotVariationClicked\0slotSelectVariation\0"
    "slotSelectChannel\0slotControllerChanged\0"
    "controllerNumber"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MIDIInstrumentParameterPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   79,    2, 0x0a /* Public */,
       6,    1,   86,    2, 0x08 /* Private */,
       8,    1,   89,    2, 0x08 /* Private */,
       9,    2,   92,    2, 0x08 /* Private */,
      13,    1,   97,    2, 0x08 /* Private */,
      15,    1,  100,    2, 0x08 /* Private */,
      16,    1,  103,    2, 0x08 /* Private */,
      18,    1,  106,    2, 0x08 /* Private */,
      19,    1,  109,    2, 0x08 /* Private */,
      20,    1,  112,    2, 0x08 /* Private */,
      21,    1,  115,    2, 0x08 /* Private */,
      22,    1,  118,    2, 0x08 /* Private */,
      23,    1,  121,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    4,    5,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,   11,   12,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   24,

       0        // eod
};

void Rosegarden::MIDIInstrumentParameterPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MIDIInstrumentParameterPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotExternalProgramChange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 2: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotControlChange((*reinterpret_cast< Instrument*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->slotPercussionClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotBankClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->slotSelectBank((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotProgramClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotSelectProgram((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotVariationClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotSelectVariation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->slotSelectChannel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slotControllerChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Instrument* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MIDIInstrumentParameterPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<InstrumentParameterPanel::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel.data,
    qt_meta_data_Rosegarden__MIDIInstrumentParameterPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MIDIInstrumentParameterPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MIDIInstrumentParameterPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MIDIInstrumentParameterPanel.stringdata0))
        return static_cast<void*>(this);
    return InstrumentParameterPanel::qt_metacast(_clname);
}

int Rosegarden::MIDIInstrumentParameterPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = InstrumentParameterPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
