/****************************************************************************
** Meta object code from reading C++ file 'SegmentTool.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/compositionview/SegmentTool.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SegmentTool.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__SegmentTool_t {
    QByteArrayData data[23];
    char stringdata0[424];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__SegmentTool_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__SegmentTool_t qt_meta_stringdata_Rosegarden__SegmentTool = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Rosegarden::SegmentTool"
QT_MOC_LITERAL(1, 24, 8), // "slotEdit"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 16), // "slotEditInMatrix"
QT_MOC_LITERAL(4, 51, 26), // "slotEditInPercussionMatrix"
QT_MOC_LITERAL(5, 78, 18), // "slotEditAsNotation"
QT_MOC_LITERAL(6, 97, 19), // "slotEditInEventList"
QT_MOC_LITERAL(7, 117, 22), // "slotEditInPitchTracker"
QT_MOC_LITERAL(8, 140, 11), // "slotEditCut"
QT_MOC_LITERAL(9, 152, 12), // "slotEditCopy"
QT_MOC_LITERAL(10, 165, 13), // "slotEditPaste"
QT_MOC_LITERAL(11, 179, 26), // "slotDeleteSelectedSegments"
QT_MOC_LITERAL(12, 206, 16), // "slotJoinSegments"
QT_MOC_LITERAL(13, 223, 21), // "slotQuantizeSelection"
QT_MOC_LITERAL(14, 245, 27), // "slotRepeatQuantizeSelection"
QT_MOC_LITERAL(15, 273, 19), // "slotRelabelSegments"
QT_MOC_LITERAL(16, 293, 21), // "slotTransposeSegments"
QT_MOC_LITERAL(17, 315, 19), // "slotPointerSelected"
QT_MOC_LITERAL(18, 335, 16), // "slotDrawSelected"
QT_MOC_LITERAL(19, 352, 17), // "slotEraseSelected"
QT_MOC_LITERAL(20, 370, 16), // "slotMoveSelected"
QT_MOC_LITERAL(21, 387, 18), // "slotResizeSelected"
QT_MOC_LITERAL(22, 406, 17) // "slotSplitSelected"

    },
    "Rosegarden::SegmentTool\0slotEdit\0\0"
    "slotEditInMatrix\0slotEditInPercussionMatrix\0"
    "slotEditAsNotation\0slotEditInEventList\0"
    "slotEditInPitchTracker\0slotEditCut\0"
    "slotEditCopy\0slotEditPaste\0"
    "slotDeleteSelectedSegments\0slotJoinSegments\0"
    "slotQuantizeSelection\0slotRepeatQuantizeSelection\0"
    "slotRelabelSegments\0slotTransposeSegments\0"
    "slotPointerSelected\0slotDrawSelected\0"
    "slotEraseSelected\0slotMoveSelected\0"
    "slotResizeSelected\0slotSplitSelected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__SegmentTool[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x08 /* Private */,
       3,    0,  120,    2, 0x08 /* Private */,
       4,    0,  121,    2, 0x08 /* Private */,
       5,    0,  122,    2, 0x08 /* Private */,
       6,    0,  123,    2, 0x08 /* Private */,
       7,    0,  124,    2, 0x08 /* Private */,
       8,    0,  125,    2, 0x08 /* Private */,
       9,    0,  126,    2, 0x08 /* Private */,
      10,    0,  127,    2, 0x08 /* Private */,
      11,    0,  128,    2, 0x08 /* Private */,
      12,    0,  129,    2, 0x08 /* Private */,
      13,    0,  130,    2, 0x08 /* Private */,
      14,    0,  131,    2, 0x08 /* Private */,
      15,    0,  132,    2, 0x08 /* Private */,
      16,    0,  133,    2, 0x08 /* Private */,
      17,    0,  134,    2, 0x08 /* Private */,
      18,    0,  135,    2, 0x08 /* Private */,
      19,    0,  136,    2, 0x08 /* Private */,
      20,    0,  137,    2, 0x08 /* Private */,
      21,    0,  138,    2, 0x08 /* Private */,
      22,    0,  139,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::SegmentTool::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SegmentTool *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotEdit(); break;
        case 1: _t->slotEditInMatrix(); break;
        case 2: _t->slotEditInPercussionMatrix(); break;
        case 3: _t->slotEditAsNotation(); break;
        case 4: _t->slotEditInEventList(); break;
        case 5: _t->slotEditInPitchTracker(); break;
        case 6: _t->slotEditCut(); break;
        case 7: _t->slotEditCopy(); break;
        case 8: _t->slotEditPaste(); break;
        case 9: _t->slotDeleteSelectedSegments(); break;
        case 10: _t->slotJoinSegments(); break;
        case 11: _t->slotQuantizeSelection(); break;
        case 12: _t->slotRepeatQuantizeSelection(); break;
        case 13: _t->slotRelabelSegments(); break;
        case 14: _t->slotTransposeSegments(); break;
        case 15: _t->slotPointerSelected(); break;
        case 16: _t->slotDrawSelected(); break;
        case 17: _t->slotEraseSelected(); break;
        case 18: _t->slotMoveSelected(); break;
        case 19: _t->slotResizeSelected(); break;
        case 20: _t->slotSplitSelected(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::SegmentTool::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseTool::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__SegmentTool.data,
    qt_meta_data_Rosegarden__SegmentTool,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::SegmentTool::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::SegmentTool::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__SegmentTool.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return BaseTool::qt_metacast(_clname);
}

int Rosegarden::SegmentTool::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseTool::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
