/****************************************************************************
** Meta object code from reading C++ file 'TrackButtons.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/TrackButtons.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TrackButtons.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TrackButtons_t {
    QByteArrayData data[26];
    char stringdata0[395];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TrackButtons_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TrackButtons_t qt_meta_stringdata_Rosegarden__TrackButtons = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::TrackButtons"
QT_MOC_LITERAL(1, 25, 13), // "trackSelected"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 7), // "trackId"
QT_MOC_LITERAL(4, 48, 16), // "slotToggleRecord"
QT_MOC_LITERAL(5, 65, 8), // "position"
QT_MOC_LITERAL(6, 74, 16), // "slotUpdateTracks"
QT_MOC_LITERAL(7, 91, 15), // "slotRenameTrack"
QT_MOC_LITERAL(8, 107, 9), // "longLabel"
QT_MOC_LITERAL(9, 117, 10), // "shortLabel"
QT_MOC_LITERAL(10, 128, 7), // "TrackId"
QT_MOC_LITERAL(11, 136, 17), // "slotSetTrackMeter"
QT_MOC_LITERAL(12, 154, 5), // "value"
QT_MOC_LITERAL(13, 160, 25), // "slotSetMetersByInstrument"
QT_MOC_LITERAL(14, 186, 12), // "InstrumentId"
QT_MOC_LITERAL(15, 199, 2), // "id"
QT_MOC_LITERAL(16, 202, 18), // "slotInstrumentMenu"
QT_MOC_LITERAL(17, 221, 22), // "slotInstrumentSelected"
QT_MOC_LITERAL(18, 244, 15), // "instrumentIndex"
QT_MOC_LITERAL(19, 260, 8), // "QAction*"
QT_MOC_LITERAL(20, 269, 25), // "slotTPBInstrumentSelected"
QT_MOC_LITERAL(21, 295, 30), // "slotSynchroniseWithComposition"
QT_MOC_LITERAL(22, 326, 14), // "slotToggleMute"
QT_MOC_LITERAL(23, 341, 14), // "slotToggleSolo"
QT_MOC_LITERAL(24, 356, 20), // "slotDocumentModified"
QT_MOC_LITERAL(25, 377, 17) // "slotTrackSelected"

    },
    "Rosegarden::TrackButtons\0trackSelected\0"
    "\0trackId\0slotToggleRecord\0position\0"
    "slotUpdateTracks\0slotRenameTrack\0"
    "longLabel\0shortLabel\0TrackId\0"
    "slotSetTrackMeter\0value\0"
    "slotSetMetersByInstrument\0InstrumentId\0"
    "id\0slotInstrumentMenu\0slotInstrumentSelected\0"
    "instrumentIndex\0QAction*\0"
    "slotTPBInstrumentSelected\0"
    "slotSynchroniseWithComposition\0"
    "slotToggleMute\0slotToggleSolo\0"
    "slotDocumentModified\0slotTrackSelected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TrackButtons[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   92,    2, 0x0a /* Public */,
       6,    0,   95,    2, 0x0a /* Public */,
       7,    3,   96,    2, 0x0a /* Public */,
      11,    2,  103,    2, 0x0a /* Public */,
      13,    2,  108,    2, 0x0a /* Public */,
      16,    1,  113,    2, 0x0a /* Public */,
      17,    1,  116,    2, 0x0a /* Public */,
      17,    1,  119,    2, 0x0a /* Public */,
      20,    2,  122,    2, 0x0a /* Public */,
      21,    0,  127,    2, 0x0a /* Public */,
      22,    1,  128,    2, 0x09 /* Protected */,
      23,    1,  131,    2, 0x09 /* Protected */,
      24,    1,  134,    2, 0x08 /* Private */,
      25,    1,  137,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, 0x80000000 | 10,    8,    9,    3,
    QMetaType::Void, QMetaType::Float, QMetaType::Int,   12,    5,
    QMetaType::Void, QMetaType::Float, 0x80000000 | 14,   12,   15,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,    3,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void Rosegarden::TrackButtons::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TrackButtons *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->trackSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->slotToggleRecord((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slotUpdateTracks(); break;
        case 3: _t->slotRenameTrack((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< TrackId(*)>(_a[3]))); break;
        case 4: _t->slotSetTrackMeter((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->slotSetMetersByInstrument((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< InstrumentId(*)>(_a[2]))); break;
        case 6: _t->slotInstrumentMenu((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotInstrumentSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotInstrumentSelected((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 9: _t->slotTPBInstrumentSelected((*reinterpret_cast< TrackId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->slotSynchroniseWithComposition(); break;
        case 11: _t->slotToggleMute((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slotToggleSolo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->slotTrackSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TrackButtons::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackButtons::trackSelected)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TrackButtons::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TrackButtons.data,
    qt_meta_data_Rosegarden__TrackButtons,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TrackButtons::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TrackButtons::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TrackButtons.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return QFrame::qt_metacast(_clname);
}

int Rosegarden::TrackButtons::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TrackButtons::trackSelected(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
