/* -*- c-basic-offset: 4 indent-tabs-mode: nil -*- vi:set ts=8 sts=4 sw=4: */

/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.
    Copyright 2000-2026 the Rosegarden development team.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef RG_MUSICXMLLOADER_H
#define RG_MUSICXMLLOADER_H


class QString;


namespace Rosegarden
{


class RosegardenDocument;


/// Load a MusicXML file into a RosegardenDocument.
class MusicXMLLoader
{
public:

    /**
      * Load and parse the Music XML file \a fileName, and write it into the
      * given RosegardenDocument (clearing the existing segment data first).
      *
      * Returns true for success.
      */
    bool load(const QString& fileName, RosegardenDocument *doc);

    /// Error message if load() returns false.
    QString errorMessage() const;

private:
    QString m_message;
};


}


#endif
