/****************************************************************************
** Meta object code from reading C++ file 'EventFilterDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/EventFilterDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EventFilterDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__EventFilterDialog_t {
    QByteArrayData data[16];
    char stringdata0[260];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__EventFilterDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__EventFilterDialog_t qt_meta_stringdata_Rosegarden__EventFilterDialog = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Rosegarden::EventFilterDialog"
QT_MOC_LITERAL(1, 30, 13), // "slotToggleAll"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 14), // "slotToggleNone"
QT_MOC_LITERAL(4, 60, 6), // "accept"
QT_MOC_LITERAL(5, 67, 20), // "slotPitchFromChanged"
QT_MOC_LITERAL(6, 88, 5), // "pitch"
QT_MOC_LITERAL(7, 94, 18), // "slotPitchToChanged"
QT_MOC_LITERAL(8, 113, 23), // "slotVelocityFromChanged"
QT_MOC_LITERAL(9, 137, 8), // "velocity"
QT_MOC_LITERAL(10, 146, 21), // "slotVelocityToChanged"
QT_MOC_LITERAL(11, 168, 23), // "slotDurationFromChanged"
QT_MOC_LITERAL(12, 192, 5), // "index"
QT_MOC_LITERAL(13, 198, 21), // "slotDurationToChanged"
QT_MOC_LITERAL(14, 220, 20), // "slotPitchFromChooser"
QT_MOC_LITERAL(15, 241, 18) // "slotPitchToChooser"

    },
    "Rosegarden::EventFilterDialog\0"
    "slotToggleAll\0\0slotToggleNone\0accept\0"
    "slotPitchFromChanged\0pitch\0"
    "slotPitchToChanged\0slotVelocityFromChanged\0"
    "velocity\0slotVelocityToChanged\0"
    "slotDurationFromChanged\0index\0"
    "slotDurationToChanged\0slotPitchFromChooser\0"
    "slotPitchToChooser"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__EventFilterDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x09 /* Protected */,
       3,    0,   70,    2, 0x09 /* Protected */,
       4,    0,   71,    2, 0x09 /* Protected */,
       5,    1,   72,    2, 0x09 /* Protected */,
       7,    1,   75,    2, 0x09 /* Protected */,
       8,    1,   78,    2, 0x09 /* Protected */,
      10,    1,   81,    2, 0x09 /* Protected */,
      11,    1,   84,    2, 0x09 /* Protected */,
      13,    1,   87,    2, 0x09 /* Protected */,
      14,    0,   90,    2, 0x09 /* Protected */,
      15,    0,   91,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::EventFilterDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventFilterDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotToggleAll(); break;
        case 1: _t->slotToggleNone(); break;
        case 2: _t->accept(); break;
        case 3: _t->slotPitchFromChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotPitchToChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->slotVelocityFromChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotVelocityToChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotDurationFromChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotDurationToChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotPitchFromChooser(); break;
        case 10: _t->slotPitchToChooser(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::EventFilterDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__EventFilterDialog.data,
    qt_meta_data_Rosegarden__EventFilterDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::EventFilterDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::EventFilterDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__EventFilterDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::EventFilterDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
