/****************************************************************************
** Meta object code from reading C++ file 'MarkerEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/MarkerEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MarkerEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MarkerEditor_t {
    QByteArrayData data[20];
    char stringdata0[234];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MarkerEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MarkerEditor_t qt_meta_stringdata_Rosegarden__MarkerEditor = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::MarkerEditor"
QT_MOC_LITERAL(1, 25, 7), // "closing"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "jumpToMarker"
QT_MOC_LITERAL(4, 47, 5), // "timeT"
QT_MOC_LITERAL(5, 53, 10), // "slotUpdate"
QT_MOC_LITERAL(6, 64, 7), // "slotAdd"
QT_MOC_LITERAL(7, 72, 10), // "slotDelete"
QT_MOC_LITERAL(8, 83, 13), // "slotDeleteAll"
QT_MOC_LITERAL(9, 97, 9), // "slotClose"
QT_MOC_LITERAL(10, 107, 8), // "slotEdit"
QT_MOC_LITERAL(11, 116, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(12, 133, 15), // "slotItemClicked"
QT_MOC_LITERAL(13, 149, 4), // "item"
QT_MOC_LITERAL(14, 154, 6), // "column"
QT_MOC_LITERAL(15, 161, 15), // "slotMusicalTime"
QT_MOC_LITERAL(16, 177, 12), // "slotRealTime"
QT_MOC_LITERAL(17, 190, 11), // "slotRawTime"
QT_MOC_LITERAL(18, 202, 17), // "slotHelpRequested"
QT_MOC_LITERAL(19, 220, 13) // "slotHelpAbout"

    },
    "Rosegarden::MarkerEditor\0closing\0\0"
    "jumpToMarker\0timeT\0slotUpdate\0slotAdd\0"
    "slotDelete\0slotDeleteAll\0slotClose\0"
    "slotEdit\0QTreeWidgetItem*\0slotItemClicked\0"
    "item\0column\0slotMusicalTime\0slotRealTime\0"
    "slotRawTime\0slotHelpRequested\0"
    "slotHelpAbout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MarkerEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x06 /* Public */,
       3,    1,   85,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   88,    2, 0x08 /* Private */,
       6,    0,   89,    2, 0x08 /* Private */,
       7,    0,   90,    2, 0x08 /* Private */,
       8,    0,   91,    2, 0x08 /* Private */,
       9,    0,   92,    2, 0x08 /* Private */,
      10,    2,   93,    2, 0x08 /* Private */,
      12,    2,   98,    2, 0x08 /* Private */,
      15,    0,  103,    2, 0x08 /* Private */,
      16,    0,  104,    2, 0x08 /* Private */,
      17,    0,  105,    2, 0x08 /* Private */,
      18,    0,  106,    2, 0x08 /* Private */,
      19,    0,  107,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11, QMetaType::Int,    2,    2,
    QMetaType::Void, 0x80000000 | 11, QMetaType::Int,   13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::MarkerEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MarkerEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closing(); break;
        case 1: _t->jumpToMarker((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 2: _t->slotUpdate(); break;
        case 3: _t->slotAdd(); break;
        case 4: _t->slotDelete(); break;
        case 5: _t->slotDeleteAll(); break;
        case 6: _t->slotClose(); break;
        case 7: _t->slotEdit((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->slotItemClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->slotMusicalTime(); break;
        case 10: _t->slotRealTime(); break;
        case 11: _t->slotRawTime(); break;
        case 12: _t->slotHelpRequested(); break;
        case 13: _t->slotHelpAbout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MarkerEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MarkerEditor::closing)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MarkerEditor::*)(timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MarkerEditor::jumpToMarker)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MarkerEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MarkerEditor.data,
    qt_meta_data_Rosegarden__MarkerEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MarkerEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MarkerEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MarkerEditor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::MarkerEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::MarkerEditor::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::MarkerEditor::jumpToMarker(timeT _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
