/****************************************************************************
** Meta object code from reading C++ file 'MidiMixerWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/studio/MidiMixerWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MidiMixerWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MidiMixerWindow_t {
    QByteArrayData data[11];
    char stringdata0[171];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MidiMixerWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MidiMixerWindow_t qt_meta_stringdata_Rosegarden__MidiMixerWindow = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::MidiMixerWindow"
QT_MOC_LITERAL(1, 28, 20), // "slotDocumentModified"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 8), // "modified"
QT_MOC_LITERAL(4, 59, 21), // "slotCurrentTabChanged"
QT_MOC_LITERAL(5, 81, 22), // "slotExternalController"
QT_MOC_LITERAL(6, 104, 18), // "const MappedEvent*"
QT_MOC_LITERAL(7, 123, 5), // "event"
QT_MOC_LITERAL(8, 129, 17), // "slotHelpRequested"
QT_MOC_LITERAL(9, 147, 13), // "slotHelpAbout"
QT_MOC_LITERAL(10, 161, 9) // "slotClose"

    },
    "Rosegarden::MidiMixerWindow\0"
    "slotDocumentModified\0\0modified\0"
    "slotCurrentTabChanged\0slotExternalController\0"
    "const MappedEvent*\0event\0slotHelpRequested\0"
    "slotHelpAbout\0slotClose"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MidiMixerWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x08 /* Private */,
       4,    1,   47,    2, 0x08 /* Private */,
       5,    1,   50,    2, 0x08 /* Private */,
       8,    0,   53,    2, 0x08 /* Private */,
       9,    0,   54,    2, 0x08 /* Private */,
      10,    0,   55,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::MidiMixerWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MidiMixerWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->slotCurrentTabChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slotExternalController((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 3: _t->slotHelpRequested(); break;
        case 4: _t->slotHelpAbout(); break;
        case 5: _t->slotClose(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MidiMixerWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<MixerWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MidiMixerWindow.data,
    qt_meta_data_Rosegarden__MidiMixerWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MidiMixerWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MidiMixerWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MidiMixerWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return MixerWindow::qt_metacast(_clname);
}

int Rosegarden::MidiMixerWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MixerWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
