/****************************************************************************
** Meta object code from reading C++ file 'EditViewBase.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/general/EditViewBase.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EditViewBase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__EditViewBase_t {
    QByteArrayData data[21];
    char stringdata0[392];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__EditViewBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__EditViewBase_t qt_meta_stringdata_Rosegarden__EditViewBase = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::EditViewBase"
QT_MOC_LITERAL(1, 25, 8), // "saveFile"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 14), // "openInNotation"
QT_MOC_LITERAL(4, 50, 21), // "std::vector<Segment*>"
QT_MOC_LITERAL(5, 72, 12), // "openInMatrix"
QT_MOC_LITERAL(6, 85, 22), // "openInPercussionMatrix"
QT_MOC_LITERAL(7, 108, 15), // "openInEventList"
QT_MOC_LITERAL(8, 124, 18), // "openInPitchTracker"
QT_MOC_LITERAL(9, 143, 19), // "slotToggleStatusBar"
QT_MOC_LITERAL(10, 163, 30), // "slotUpdateClipboardActionState"
QT_MOC_LITERAL(11, 194, 14), // "slotToggleSolo"
QT_MOC_LITERAL(12, 209, 13), // "slotConfigure"
QT_MOC_LITERAL(13, 223, 23), // "slotSetSegmentStartTime"
QT_MOC_LITERAL(14, 247, 22), // "slotSetSegmentDuration"
QT_MOC_LITERAL(15, 270, 16), // "slotOpenInMatrix"
QT_MOC_LITERAL(16, 287, 26), // "slotOpenInPercussionMatrix"
QT_MOC_LITERAL(17, 314, 18), // "slotOpenInNotation"
QT_MOC_LITERAL(18, 333, 19), // "slotOpenInEventList"
QT_MOC_LITERAL(19, 353, 22), // "slotOpenInPitchTracker"
QT_MOC_LITERAL(20, 376, 15) // "slotCloseWindow"

    },
    "Rosegarden::EditViewBase\0saveFile\0\0"
    "openInNotation\0std::vector<Segment*>\0"
    "openInMatrix\0openInPercussionMatrix\0"
    "openInEventList\0openInPitchTracker\0"
    "slotToggleStatusBar\0slotUpdateClipboardActionState\0"
    "slotToggleSolo\0slotConfigure\0"
    "slotSetSegmentStartTime\0slotSetSegmentDuration\0"
    "slotOpenInMatrix\0slotOpenInPercussionMatrix\0"
    "slotOpenInNotation\0slotOpenInEventList\0"
    "slotOpenInPitchTracker\0slotCloseWindow"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__EditViewBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  104,    2, 0x06 /* Public */,
       3,    1,  105,    2, 0x06 /* Public */,
       5,    1,  108,    2, 0x06 /* Public */,
       6,    1,  111,    2, 0x06 /* Public */,
       7,    1,  114,    2, 0x06 /* Public */,
       8,    1,  117,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  120,    2, 0x09 /* Protected */,
      10,    0,  121,    2, 0x09 /* Protected */,
      11,    0,  122,    2, 0x09 /* Protected */,
      12,    0,  123,    2, 0x09 /* Protected */,
      13,    0,  124,    2, 0x08 /* Private */,
      14,    0,  125,    2, 0x08 /* Private */,
      15,    0,  126,    2, 0x08 /* Private */,
      16,    0,  127,    2, 0x08 /* Private */,
      17,    0,  128,    2, 0x08 /* Private */,
      18,    0,  129,    2, 0x08 /* Private */,
      19,    0,  130,    2, 0x08 /* Private */,
      20,    0,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::EditViewBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditViewBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saveFile(); break;
        case 1: _t->openInNotation((*reinterpret_cast< std::vector<Segment*>(*)>(_a[1]))); break;
        case 2: _t->openInMatrix((*reinterpret_cast< std::vector<Segment*>(*)>(_a[1]))); break;
        case 3: _t->openInPercussionMatrix((*reinterpret_cast< std::vector<Segment*>(*)>(_a[1]))); break;
        case 4: _t->openInEventList((*reinterpret_cast< std::vector<Segment*>(*)>(_a[1]))); break;
        case 5: _t->openInPitchTracker((*reinterpret_cast< std::vector<Segment*>(*)>(_a[1]))); break;
        case 6: _t->slotToggleStatusBar(); break;
        case 7: _t->slotUpdateClipboardActionState(); break;
        case 8: _t->slotToggleSolo(); break;
        case 9: _t->slotConfigure(); break;
        case 10: _t->slotSetSegmentStartTime(); break;
        case 11: _t->slotSetSegmentDuration(); break;
        case 12: _t->slotOpenInMatrix(); break;
        case 13: _t->slotOpenInPercussionMatrix(); break;
        case 14: _t->slotOpenInNotation(); break;
        case 15: _t->slotOpenInEventList(); break;
        case 16: _t->slotOpenInPitchTracker(); break;
        case 17: _t->slotCloseWindow(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditViewBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::saveFile)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (EditViewBase::*)(std::vector<Segment*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::openInNotation)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (EditViewBase::*)(std::vector<Segment*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::openInMatrix)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (EditViewBase::*)(std::vector<Segment*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::openInPercussionMatrix)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (EditViewBase::*)(std::vector<Segment*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::openInEventList)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (EditViewBase::*)(std::vector<Segment*> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditViewBase::openInPitchTracker)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::EditViewBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__EditViewBase.data,
    qt_meta_data_Rosegarden__EditViewBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::EditViewBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::EditViewBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__EditViewBase.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::EditViewBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::EditViewBase::saveFile()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::EditViewBase::openInNotation(std::vector<Segment*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::EditViewBase::openInMatrix(std::vector<Segment*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::EditViewBase::openInPercussionMatrix(std::vector<Segment*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::EditViewBase::openInEventList(std::vector<Segment*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::EditViewBase::openInPitchTracker(std::vector<Segment*> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
