/****************************************************************************
** Meta object code from reading C++ file 'TrackEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/TrackEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TrackEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TrackEditor_t {
    QByteArrayData data[19];
    char stringdata0[258];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TrackEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TrackEditor_t qt_meta_stringdata_Rosegarden__TrackEditor = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Rosegarden::TrackEditor"
QT_MOC_LITERAL(1, 24, 11), // "stateChange"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 4), // "name"
QT_MOC_LITERAL(4, 42, 5), // "state"
QT_MOC_LITERAL(5, 48, 15), // "droppedDocument"
QT_MOC_LITERAL(6, 64, 3), // "uri"
QT_MOC_LITERAL(7, 68, 12), // "droppedAudio"
QT_MOC_LITERAL(8, 81, 9), // "audioDesc"
QT_MOC_LITERAL(9, 91, 15), // "droppedNewAudio"
QT_MOC_LITERAL(10, 107, 22), // "slotSetPointerPosition"
QT_MOC_LITERAL(11, 130, 5), // "timeT"
QT_MOC_LITERAL(12, 136, 11), // "pointerTime"
QT_MOC_LITERAL(13, 148, 28), // "slotPointerDraggedToPosition"
QT_MOC_LITERAL(14, 177, 8), // "position"
QT_MOC_LITERAL(15, 186, 30), // "slotVerticalScrollTrackButtons"
QT_MOC_LITERAL(16, 217, 1), // "y"
QT_MOC_LITERAL(17, 219, 19), // "slotCommandExecuted"
QT_MOC_LITERAL(18, 239, 18) // "slotViewportResize"

    },
    "Rosegarden::TrackEditor\0stateChange\0"
    "\0name\0state\0droppedDocument\0uri\0"
    "droppedAudio\0audioDesc\0droppedNewAudio\0"
    "slotSetPointerPosition\0timeT\0pointerTime\0"
    "slotPointerDraggedToPosition\0position\0"
    "slotVerticalScrollTrackButtons\0y\0"
    "slotCommandExecuted\0slotViewportResize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TrackEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   59,    2, 0x06 /* Public */,
       5,    1,   64,    2, 0x06 /* Public */,
       7,    1,   67,    2, 0x06 /* Public */,
       9,    1,   70,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   73,    2, 0x08 /* Private */,
      13,    1,   76,    2, 0x08 /* Private */,
      15,    1,   79,    2, 0x08 /* Private */,
      17,    0,   82,    2, 0x08 /* Private */,
      18,    0,   83,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 11,   14,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TrackEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TrackEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChange((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->droppedDocument((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->droppedAudio((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->droppedNewAudio((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->slotSetPointerPosition((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 5: _t->slotPointerDraggedToPosition((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 6: _t->slotVerticalScrollTrackButtons((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotCommandExecuted(); break;
        case 8: _t->slotViewportResize(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TrackEditor::*)(QString , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackEditor::stateChange)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TrackEditor::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackEditor::droppedDocument)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TrackEditor::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackEditor::droppedAudio)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TrackEditor::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackEditor::droppedNewAudio)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TrackEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TrackEditor.data,
    qt_meta_data_Rosegarden__TrackEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TrackEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TrackEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TrackEditor.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Rosegarden::TrackEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TrackEditor::stateChange(QString _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::TrackEditor::droppedDocument(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::TrackEditor::droppedAudio(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::TrackEditor::droppedNewAudio(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
