/********************************************************************************
** Form generated from reading UI file 'RosegardenTransportUi.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ROSEGARDENTRANSPORTUI_H
#define UI_ROSEGARDENTRANSPORTUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include "gui/widgets/Label.h"

QT_BEGIN_NAMESPACE

class Ui_RosegardenTransport
{
public:
    QFrame *RecordingFrame;
    QPushButton *PanelCloseButton;
    QPushButton *PanicButton;
    QPushButton *MetronomeButton;
    QFrame *MidiEventFrame;
    QLabel *InLabel;
    QLabel *OutLabel;
    QLabel *InDisplay;
    QLabel *OutDisplay;
    QPushButton *RecordButton;
    QPushButton *LoopButton;
    QPushButton *SoloButton;
    QPushButton *SetStartLPButton;
    QPushButton *SetStopLPButton;
    QFrame *MainFrame;
    QFrame *LCDBoxFrame;
    Rosegarden::Label *UnitHoursPixmap;
    Rosegarden::Label *HourColonPixmap;
    Rosegarden::Label *TenMinutesPixmap;
    Rosegarden::Label *UnitMinutesPixmap;
    Rosegarden::Label *TenSecondsPixmap;
    Rosegarden::Label *UnitSecondsPixmap;
    Rosegarden::Label *HundredthsPixmap;
    Rosegarden::Label *NegativePixmap;
    Rosegarden::Label *TenHoursPixmap;
    QLabel *TimeSigLabel;
    QLabel *DivisionLabel;
    QLabel *DivisionDisplay;
    QLabel *TempoLabel;
    Rosegarden::Label *TimeSigDisplay;
    QLabel *ToEndLabel;
    QLabel *TimeDisplayLabel;
    Rosegarden::Label *TenThousandthsPixmap;
    Rosegarden::Label *MinuteColonPixmap;
    Rosegarden::Label *SecondColonPixmap;
    Rosegarden::Label *HundredthColonPixmap;
    Rosegarden::Label *TenthsPixmap;
    Rosegarden::Label *ThousandthsPixmap;
    Rosegarden::Label *TempoDisplay;
    QPushButton *TimeDisplayButton;
    QPushButton *PanelOpenButton;
    QPushButton *RewindButton;
    QPushButton *RewindEndButton;
    QPushButton *PlayButton;
    QPushButton *StopButton;
    QPushButton *FfwdButton;
    QPushButton *FfwdEndButton;
    QPushButton *ToEndButton;

    void setupUi(QFrame *RosegardenTransport)
    {
        if (RosegardenTransport->objectName().isEmpty())
            RosegardenTransport->setObjectName(QString::fromUtf8("RosegardenTransport"));
        RosegardenTransport->resize(416, 134);
        RosegardenTransport->setMinimumSize(QSize(416, 87));
        RosegardenTransport->setFrameShape(QFrame::NoFrame);
        RosegardenTransport->setFrameShadow(QFrame::Plain);
        RecordingFrame = new QFrame(RosegardenTransport);
        RecordingFrame->setObjectName(QString::fromUtf8("RecordingFrame"));
        RecordingFrame->setGeometry(QRect(0, 87, 416, 47));
        RecordingFrame->setLineWidth(0);
        PanelCloseButton = new QPushButton(RecordingFrame);
        PanelCloseButton->setObjectName(QString::fromUtf8("PanelCloseButton"));
        PanelCloseButton->setGeometry(QRect(3, 4, 25, 23));
        QFont font;
        font.setFamily(QString::fromUtf8("lucida"));
        font.setBold(true);
        font.setWeight(75);
        PanelCloseButton->setFont(font);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/pixmaps/transport/button-flip-closed.png"), QSize(), QIcon::Normal, QIcon::Off);
        icon.addFile(QString::fromUtf8(":/pixmaps/transport/button-flip-open.png"), QSize(), QIcon::Normal, QIcon::On);
        PanelCloseButton->setIcon(icon);
        PanelCloseButton->setFlat(true);
        PanicButton = new QPushButton(RecordingFrame);
        PanicButton->setObjectName(QString::fromUtf8("PanicButton"));
        PanicButton->setGeometry(QRect(174, 4, 42, 38));
        QFont font1;
        font1.setFamily(QString::fromUtf8("lucidatypewriter"));
        font1.setPointSize(8);
        PanicButton->setFont(font1);
        PanicButton->setCursor(QCursor(Qt::ArrowCursor));
        PanicButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/pixmaps/transport/button-panic.png"), QSize(), QIcon::Normal, QIcon::Off);
        PanicButton->setIcon(icon1);
        PanicButton->setCheckable(false);
        MetronomeButton = new QPushButton(RecordingFrame);
        MetronomeButton->setObjectName(QString::fromUtf8("MetronomeButton"));
        MetronomeButton->setGeometry(QRect(217, 4, 42, 38));
        MetronomeButton->setFont(font1);
        MetronomeButton->setCursor(QCursor(Qt::ArrowCursor));
        MetronomeButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/pixmaps/transport/button-metronome.png"), QSize(), QIcon::Normal, QIcon::Off);
        MetronomeButton->setIcon(icon2);
        MetronomeButton->setCheckable(true);
        MidiEventFrame = new QFrame(RecordingFrame);
        MidiEventFrame->setObjectName(QString::fromUtf8("MidiEventFrame"));
        MidiEventFrame->setGeometry(QRect(31, 7, 137, 34));
        QPalette palette;
        QBrush brush(QColor(0, 0, 0, 255));
        brush.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::WindowText, brush);
        QBrush brush1(QColor(192, 192, 192, 255));
        brush1.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Button, brush1);
        QBrush brush2(QColor(255, 255, 255, 255));
        brush2.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Light, brush2);
        QBrush brush3(QColor(223, 223, 223, 255));
        brush3.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        QBrush brush4(QColor(96, 96, 96, 255));
        brush4.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Dark, brush4);
        QBrush brush5(QColor(128, 128, 128, 255));
        brush5.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette.setBrush(QPalette::Active, QPalette::Text, brush);
        palette.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette.setBrush(QPalette::Active, QPalette::Window, brush);
        palette.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette.setBrush(QPalette::Active, QPalette::Link, brush);
        palette.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette.setBrush(QPalette::Inactive, QPalette::WindowText, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        QBrush brush6(QColor(220, 220, 220, 255));
        brush6.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        MidiEventFrame->setPalette(palette);
        MidiEventFrame->setAutoFillBackground(true);
        InLabel = new QLabel(MidiEventFrame);
        InLabel->setObjectName(QString::fromUtf8("InLabel"));
        InLabel->setGeometry(QRect(18, 1, 25, 16));
        QPalette palette1;
        QBrush brush7(QColor(192, 216, 255, 255));
        brush7.setStyle(Qt::SolidPattern);
        palette1.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette1.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette1.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette1.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette1.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette1.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette1.setBrush(QPalette::Active, QPalette::Text, brush);
        palette1.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette1.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette1.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette1.setBrush(QPalette::Active, QPalette::Window, brush);
        palette1.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette1.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette1.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette1.setBrush(QPalette::Active, QPalette::Link, brush);
        palette1.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette1.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette1.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette1.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette1.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette1.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette1.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette1.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette1.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette1.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette1.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette1.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette1.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette1.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette1.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette1.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette1.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        InLabel->setPalette(palette1);
        QFont font2;
        font2.setFamily(QString::fromUtf8("lucida"));
        font2.setItalic(true);
        InLabel->setFont(font2);
        InLabel->setWordWrap(false);
        OutLabel = new QLabel(MidiEventFrame);
        OutLabel->setObjectName(QString::fromUtf8("OutLabel"));
        OutLabel->setGeometry(QRect(8, 15, 25, 16));
        QPalette palette2;
        palette2.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette2.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette2.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette2.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette2.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette2.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette2.setBrush(QPalette::Active, QPalette::Text, brush);
        palette2.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette2.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette2.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette2.setBrush(QPalette::Active, QPalette::Window, brush);
        palette2.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette2.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette2.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette2.setBrush(QPalette::Active, QPalette::Link, brush);
        palette2.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette2.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette2.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette2.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette2.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette2.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette2.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette2.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette2.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette2.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette2.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette2.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette2.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette2.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette2.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette2.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette2.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette2.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette2.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette2.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette2.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        OutLabel->setPalette(palette2);
        OutLabel->setFont(font2);
        OutLabel->setWordWrap(false);
        InDisplay = new QLabel(MidiEventFrame);
        InDisplay->setObjectName(QString::fromUtf8("InDisplay"));
        InDisplay->setGeometry(QRect(40, 1, 82, 16));
        QPalette palette3;
        palette3.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette3.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette3.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette3.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette3.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette3.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette3.setBrush(QPalette::Active, QPalette::Text, brush);
        palette3.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette3.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette3.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette3.setBrush(QPalette::Active, QPalette::Window, brush);
        palette3.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette3.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette3.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette3.setBrush(QPalette::Active, QPalette::Link, brush);
        palette3.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette3.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette3.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette3.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette3.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette3.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette3.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette3.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette3.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette3.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette3.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette3.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette3.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette3.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette3.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette3.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette3.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette3.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette3.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette3.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette3.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette3.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        InDisplay->setPalette(palette3);
        InDisplay->setFont(font);
        InDisplay->setWordWrap(false);
        OutDisplay = new QLabel(MidiEventFrame);
        OutDisplay->setObjectName(QString::fromUtf8("OutDisplay"));
        OutDisplay->setGeometry(QRect(40, 15, 82, 16));
        QPalette palette4;
        palette4.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette4.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette4.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette4.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette4.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette4.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette4.setBrush(QPalette::Active, QPalette::Text, brush);
        palette4.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette4.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette4.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette4.setBrush(QPalette::Active, QPalette::Window, brush);
        palette4.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette4.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette4.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette4.setBrush(QPalette::Active, QPalette::Link, brush);
        palette4.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette4.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette4.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette4.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette4.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette4.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette4.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette4.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette4.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette4.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette4.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette4.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette4.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette4.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette4.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette4.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette4.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette4.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette4.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette4.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette4.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette4.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        OutDisplay->setPalette(palette4);
        OutDisplay->setFont(font);
        OutDisplay->setWordWrap(false);
        RecordButton = new QPushButton(RecordingFrame);
        RecordButton->setObjectName(QString::fromUtf8("RecordButton"));
        RecordButton->setGeometry(QRect(370, 4, 42, 38));
        RecordButton->setCursor(QCursor(Qt::ArrowCursor));
        RecordButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/pixmaps/transport/button-record.png"), QSize(), QIcon::Normal, QIcon::Off);
        RecordButton->setIcon(icon3);
        RecordButton->setIconSize(QSize(36, 22));
        RecordButton->setCheckable(true);
        LoopButton = new QPushButton(RecordingFrame);
        LoopButton->setObjectName(QString::fromUtf8("LoopButton"));
        LoopButton->setGeometry(QRect(320, 4, 32, 38));
        LoopButton->setFont(font1);
        LoopButton->setCursor(QCursor(Qt::ArrowCursor));
        LoopButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/pixmaps/transport/button-loop-3.png"), QSize(), QIcon::Normal, QIcon::Off);
        LoopButton->setIcon(icon4);
        LoopButton->setIconSize(QSize(30, 20));
        LoopButton->setCheckable(true);
        SoloButton = new QPushButton(RecordingFrame);
        SoloButton->setObjectName(QString::fromUtf8("SoloButton"));
        SoloButton->setGeometry(QRect(260, 4, 42, 38));
        SoloButton->setFont(font1);
        SoloButton->setCursor(QCursor(Qt::ArrowCursor));
        SoloButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/pixmaps/transport/button-solo.png"), QSize(), QIcon::Normal, QIcon::Off);
        SoloButton->setIcon(icon5);
        SoloButton->setCheckable(false);
        SetStartLPButton = new QPushButton(RecordingFrame);
        SetStartLPButton->setObjectName(QString::fromUtf8("SetStartLPButton"));
        SetStartLPButton->setGeometry(QRect(303, 4, 16, 38));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/pixmaps/transport/button-loopstart.png"), QSize(), QIcon::Normal, QIcon::Off);
        SetStartLPButton->setIcon(icon6);
        SetStopLPButton = new QPushButton(RecordingFrame);
        SetStopLPButton->setObjectName(QString::fromUtf8("SetStopLPButton"));
        SetStopLPButton->setGeometry(QRect(353, 4, 16, 38));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/pixmaps/transport/button-loopend.png"), QSize(), QIcon::Normal, QIcon::Off);
        SetStopLPButton->setIcon(icon7);
        MainFrame = new QFrame(RosegardenTransport);
        MainFrame->setObjectName(QString::fromUtf8("MainFrame"));
        MainFrame->setGeometry(QRect(0, 0, 416, 87));
        MainFrame->setLineWidth(0);
        LCDBoxFrame = new QFrame(MainFrame);
        LCDBoxFrame->setObjectName(QString::fromUtf8("LCDBoxFrame"));
        LCDBoxFrame->setEnabled(true);
        LCDBoxFrame->setGeometry(QRect(31, 7, 225, 71));
        QPalette palette5;
        palette5.setBrush(QPalette::Active, QPalette::WindowText, brush2);
        QBrush brush8(QColor(221, 221, 221, 255));
        brush8.setStyle(Qt::SolidPattern);
        palette5.setBrush(QPalette::Active, QPalette::Button, brush8);
        QBrush brush9(QColor(206, 206, 206, 255));
        brush9.setStyle(Qt::SolidPattern);
        palette5.setBrush(QPalette::Active, QPalette::Light, brush9);
        QBrush brush10(QColor(127, 127, 127, 255));
        brush10.setStyle(Qt::SolidPattern);
        palette5.setBrush(QPalette::Active, QPalette::Midlight, brush10);
        palette5.setBrush(QPalette::Active, QPalette::Dark, brush);
        palette5.setBrush(QPalette::Active, QPalette::Mid, brush2);
        palette5.setBrush(QPalette::Active, QPalette::Text, brush2);
        palette5.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette5.setBrush(QPalette::Active, QPalette::ButtonText, brush2);
        palette5.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette5.setBrush(QPalette::Active, QPalette::Window, brush);
        palette5.setBrush(QPalette::Active, QPalette::Shadow, brush2);
        palette5.setBrush(QPalette::Active, QPalette::AlternateBase, brush);
        palette5.setBrush(QPalette::Inactive, QPalette::WindowText, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::Button, brush8);
        palette5.setBrush(QPalette::Inactive, QPalette::Light, brush9);
        palette5.setBrush(QPalette::Inactive, QPalette::Midlight, brush10);
        palette5.setBrush(QPalette::Inactive, QPalette::Dark, brush);
        palette5.setBrush(QPalette::Inactive, QPalette::Mid, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::Text, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::ButtonText, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette5.setBrush(QPalette::Inactive, QPalette::Shadow, brush2);
        palette5.setBrush(QPalette::Inactive, QPalette::AlternateBase, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::WindowText, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::Button, brush8);
        palette5.setBrush(QPalette::Disabled, QPalette::Light, brush9);
        palette5.setBrush(QPalette::Disabled, QPalette::Midlight, brush10);
        palette5.setBrush(QPalette::Disabled, QPalette::Dark, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::Mid, brush2);
        palette5.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette5.setBrush(QPalette::Disabled, QPalette::ButtonText, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::Base, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette5.setBrush(QPalette::Disabled, QPalette::Shadow, brush2);
        palette5.setBrush(QPalette::Disabled, QPalette::AlternateBase, brush);
        LCDBoxFrame->setPalette(palette5);
        LCDBoxFrame->setAutoFillBackground(true);
        LCDBoxFrame->setLineWidth(0);
        UnitHoursPixmap = new Rosegarden::Label(LCDBoxFrame);
        UnitHoursPixmap->setObjectName(QString::fromUtf8("UnitHoursPixmap"));
        UnitHoursPixmap->setGeometry(QRect(39, 6, 20, 34));
        UnitHoursPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        UnitHoursPixmap->setProperty("scaledContents", QVariant(true));
        UnitHoursPixmap->setProperty("buddy", QVariant(QByteArray("")));
        HourColonPixmap = new Rosegarden::Label(LCDBoxFrame);
        HourColonPixmap->setObjectName(QString::fromUtf8("HourColonPixmap"));
        HourColonPixmap->setGeometry(QRect(59, 6, 5, 34));
        HourColonPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-colon.png"))));
        HourColonPixmap->setProperty("scaledContents", QVariant(true));
        HourColonPixmap->setProperty("buddy", QVariant(QByteArray("")));
        TenMinutesPixmap = new Rosegarden::Label(LCDBoxFrame);
        TenMinutesPixmap->setObjectName(QString::fromUtf8("TenMinutesPixmap"));
        TenMinutesPixmap->setGeometry(QRect(64, 6, 20, 34));
        TenMinutesPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        TenMinutesPixmap->setProperty("scaledContents", QVariant(true));
        TenMinutesPixmap->setProperty("buddy", QVariant(QByteArray("")));
        UnitMinutesPixmap = new Rosegarden::Label(LCDBoxFrame);
        UnitMinutesPixmap->setObjectName(QString::fromUtf8("UnitMinutesPixmap"));
        UnitMinutesPixmap->setGeometry(QRect(84, 6, 20, 34));
        UnitMinutesPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        UnitMinutesPixmap->setProperty("scaledContents", QVariant(true));
        UnitMinutesPixmap->setProperty("buddy", QVariant(QByteArray("")));
        TenSecondsPixmap = new Rosegarden::Label(LCDBoxFrame);
        TenSecondsPixmap->setObjectName(QString::fromUtf8("TenSecondsPixmap"));
        TenSecondsPixmap->setGeometry(QRect(109, 6, 20, 34));
        TenSecondsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        TenSecondsPixmap->setProperty("scaledContents", QVariant(true));
        TenSecondsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        UnitSecondsPixmap = new Rosegarden::Label(LCDBoxFrame);
        UnitSecondsPixmap->setObjectName(QString::fromUtf8("UnitSecondsPixmap"));
        UnitSecondsPixmap->setGeometry(QRect(129, 6, 20, 34));
        UnitSecondsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        UnitSecondsPixmap->setProperty("scaledContents", QVariant(true));
        UnitSecondsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        HundredthsPixmap = new Rosegarden::Label(LCDBoxFrame);
        HundredthsPixmap->setObjectName(QString::fromUtf8("HundredthsPixmap"));
        HundredthsPixmap->setGeometry(QRect(169, 16, 14, 24));
        HundredthsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        HundredthsPixmap->setProperty("scaledContents", QVariant(true));
        HundredthsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        NegativePixmap = new Rosegarden::Label(LCDBoxFrame);
        NegativePixmap->setObjectName(QString::fromUtf8("NegativePixmap"));
        NegativePixmap->setGeometry(QRect(2, 6, 20, 34));
        NegativePixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led--.png"))));
        NegativePixmap->setProperty("scaledContents", QVariant(true));
        NegativePixmap->setProperty("buddy", QVariant(QByteArray("")));
        TenHoursPixmap = new Rosegarden::Label(LCDBoxFrame);
        TenHoursPixmap->setObjectName(QString::fromUtf8("TenHoursPixmap"));
        TenHoursPixmap->setGeometry(QRect(19, 6, 20, 34));
        TenHoursPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        TenHoursPixmap->setProperty("scaledContents", QVariant(true));
        TenHoursPixmap->setProperty("buddy", QVariant(QByteArray("")));
        TimeSigLabel = new QLabel(LCDBoxFrame);
        TimeSigLabel->setObjectName(QString::fromUtf8("TimeSigLabel"));
        TimeSigLabel->setGeometry(QRect(6, 51, 28, 16));
        QPalette palette6;
        palette6.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette6.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette6.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette6.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette6.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette6.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette6.setBrush(QPalette::Active, QPalette::Text, brush);
        palette6.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette6.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette6.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette6.setBrush(QPalette::Active, QPalette::Window, brush);
        palette6.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette6.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette6.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette6.setBrush(QPalette::Active, QPalette::Link, brush);
        palette6.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette6.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette6.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette6.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette6.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette6.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette6.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette6.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette6.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette6.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette6.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette6.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette6.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette6.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette6.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette6.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette6.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette6.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette6.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette6.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette6.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette6.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        TimeSigLabel->setPalette(palette6);
        TimeSigLabel->setFont(font2);
        TimeSigLabel->setWordWrap(false);
        DivisionLabel = new QLabel(LCDBoxFrame);
        DivisionLabel->setObjectName(QString::fromUtf8("DivisionLabel"));
        DivisionLabel->setGeometry(QRect(66, 51, 24, 16));
        QPalette palette7;
        palette7.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette7.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette7.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette7.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette7.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette7.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette7.setBrush(QPalette::Active, QPalette::Text, brush);
        palette7.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette7.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette7.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette7.setBrush(QPalette::Active, QPalette::Window, brush);
        palette7.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette7.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette7.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette7.setBrush(QPalette::Active, QPalette::Link, brush);
        palette7.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette7.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette7.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette7.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette7.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette7.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette7.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette7.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette7.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette7.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette7.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette7.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette7.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette7.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette7.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette7.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette7.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette7.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette7.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette7.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette7.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette7.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        DivisionLabel->setPalette(palette7);
        DivisionLabel->setFont(font2);
        DivisionLabel->setWordWrap(false);
        DivisionDisplay = new QLabel(LCDBoxFrame);
        DivisionDisplay->setObjectName(QString::fromUtf8("DivisionDisplay"));
        DivisionDisplay->setGeometry(QRect(86, 51, 27, 16));
        QPalette palette8;
        palette8.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette8.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette8.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette8.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette8.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette8.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette8.setBrush(QPalette::Active, QPalette::Text, brush);
        palette8.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette8.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette8.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette8.setBrush(QPalette::Active, QPalette::Window, brush);
        palette8.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette8.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette8.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette8.setBrush(QPalette::Active, QPalette::Link, brush);
        palette8.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette8.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette8.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette8.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette8.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette8.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette8.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette8.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette8.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette8.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette8.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette8.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette8.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette8.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette8.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette8.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette8.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette8.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette8.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette8.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette8.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette8.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        DivisionDisplay->setPalette(palette8);
        DivisionDisplay->setFont(font);
        DivisionDisplay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        DivisionDisplay->setWordWrap(false);
        TempoLabel = new QLabel(LCDBoxFrame);
        TempoLabel->setObjectName(QString::fromUtf8("TempoLabel"));
        TempoLabel->setGeometry(QRect(124, 51, 43, 16));
        QPalette palette9;
        palette9.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette9.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette9.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette9.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette9.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette9.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette9.setBrush(QPalette::Active, QPalette::Text, brush);
        palette9.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette9.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette9.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette9.setBrush(QPalette::Active, QPalette::Window, brush);
        palette9.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette9.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette9.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette9.setBrush(QPalette::Active, QPalette::Link, brush);
        palette9.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette9.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette9.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette9.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette9.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette9.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette9.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette9.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette9.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette9.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette9.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette9.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette9.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette9.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette9.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette9.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette9.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette9.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette9.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette9.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette9.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette9.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        TempoLabel->setPalette(palette9);
        TempoLabel->setFont(font2);
        TempoLabel->setWordWrap(false);
        TimeSigDisplay = new Rosegarden::Label(LCDBoxFrame);
        TimeSigDisplay->setObjectName(QString::fromUtf8("TimeSigDisplay"));
        TimeSigDisplay->setGeometry(QRect(23, 51, 33, 16));
        ToEndLabel = new QLabel(LCDBoxFrame);
        ToEndLabel->setObjectName(QString::fromUtf8("ToEndLabel"));
        ToEndLabel->setGeometry(QRect(192, 1, 28, 16));
        QPalette palette10;
        palette10.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette10.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette10.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette10.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette10.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette10.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette10.setBrush(QPalette::Active, QPalette::Text, brush);
        palette10.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette10.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette10.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette10.setBrush(QPalette::Active, QPalette::Window, brush);
        palette10.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette10.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette10.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette10.setBrush(QPalette::Active, QPalette::Link, brush);
        palette10.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette10.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette10.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette10.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette10.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette10.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette10.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette10.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette10.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette10.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette10.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette10.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette10.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette10.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette10.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette10.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette10.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette10.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette10.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette10.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette10.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette10.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        ToEndLabel->setPalette(palette10);
        ToEndLabel->setFont(font2);
        ToEndLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ToEndLabel->setWordWrap(false);
        TimeDisplayLabel = new QLabel(LCDBoxFrame);
        TimeDisplayLabel->setObjectName(QString::fromUtf8("TimeDisplayLabel"));
        TimeDisplayLabel->setGeometry(QRect(157, 1, 35, 16));
        QPalette palette11;
        palette11.setBrush(QPalette::Active, QPalette::WindowText, brush7);
        palette11.setBrush(QPalette::Active, QPalette::Button, brush1);
        palette11.setBrush(QPalette::Active, QPalette::Light, brush2);
        palette11.setBrush(QPalette::Active, QPalette::Midlight, brush3);
        palette11.setBrush(QPalette::Active, QPalette::Dark, brush4);
        palette11.setBrush(QPalette::Active, QPalette::Mid, brush5);
        palette11.setBrush(QPalette::Active, QPalette::Text, brush);
        palette11.setBrush(QPalette::Active, QPalette::BrightText, brush2);
        palette11.setBrush(QPalette::Active, QPalette::ButtonText, brush);
        palette11.setBrush(QPalette::Active, QPalette::Base, brush2);
        palette11.setBrush(QPalette::Active, QPalette::Window, brush);
        palette11.setBrush(QPalette::Active, QPalette::Shadow, brush);
        palette11.setBrush(QPalette::Active, QPalette::Highlight, brush);
        palette11.setBrush(QPalette::Active, QPalette::HighlightedText, brush2);
        palette11.setBrush(QPalette::Active, QPalette::Link, brush);
        palette11.setBrush(QPalette::Active, QPalette::LinkVisited, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::WindowText, brush7);
        palette11.setBrush(QPalette::Inactive, QPalette::Button, brush1);
        palette11.setBrush(QPalette::Inactive, QPalette::Light, brush2);
        palette11.setBrush(QPalette::Inactive, QPalette::Midlight, brush6);
        palette11.setBrush(QPalette::Inactive, QPalette::Dark, brush4);
        palette11.setBrush(QPalette::Inactive, QPalette::Mid, brush5);
        palette11.setBrush(QPalette::Inactive, QPalette::Text, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::BrightText, brush2);
        palette11.setBrush(QPalette::Inactive, QPalette::ButtonText, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::Base, brush2);
        palette11.setBrush(QPalette::Inactive, QPalette::Window, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::Shadow, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::Highlight, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::HighlightedText, brush2);
        palette11.setBrush(QPalette::Inactive, QPalette::Link, brush);
        palette11.setBrush(QPalette::Inactive, QPalette::LinkVisited, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::WindowText, brush5);
        palette11.setBrush(QPalette::Disabled, QPalette::Button, brush1);
        palette11.setBrush(QPalette::Disabled, QPalette::Light, brush2);
        palette11.setBrush(QPalette::Disabled, QPalette::Midlight, brush6);
        palette11.setBrush(QPalette::Disabled, QPalette::Dark, brush4);
        palette11.setBrush(QPalette::Disabled, QPalette::Mid, brush5);
        palette11.setBrush(QPalette::Disabled, QPalette::Text, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::BrightText, brush2);
        palette11.setBrush(QPalette::Disabled, QPalette::ButtonText, brush5);
        palette11.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette11.setBrush(QPalette::Disabled, QPalette::Window, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::Shadow, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::Highlight, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::HighlightedText, brush2);
        palette11.setBrush(QPalette::Disabled, QPalette::Link, brush);
        palette11.setBrush(QPalette::Disabled, QPalette::LinkVisited, brush);
        TimeDisplayLabel->setPalette(palette11);
        TimeDisplayLabel->setFont(font2);
        TimeDisplayLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TimeDisplayLabel->setWordWrap(false);
        TenThousandthsPixmap = new Rosegarden::Label(LCDBoxFrame);
        TenThousandthsPixmap->setObjectName(QString::fromUtf8("TenThousandthsPixmap"));
        TenThousandthsPixmap->setGeometry(QRect(201, 16, 14, 24));
        TenThousandthsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        TenThousandthsPixmap->setProperty("scaledContents", QVariant(true));
        TenThousandthsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        MinuteColonPixmap = new Rosegarden::Label(LCDBoxFrame);
        MinuteColonPixmap->setObjectName(QString::fromUtf8("MinuteColonPixmap"));
        MinuteColonPixmap->setGeometry(QRect(104, 6, 5, 34));
        MinuteColonPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-colon.png"))));
        MinuteColonPixmap->setProperty("scaledContents", QVariant(true));
        MinuteColonPixmap->setProperty("buddy", QVariant(QByteArray("")));
        SecondColonPixmap = new Rosegarden::Label(LCDBoxFrame);
        SecondColonPixmap->setObjectName(QString::fromUtf8("SecondColonPixmap"));
        SecondColonPixmap->setGeometry(QRect(183, 14, 4, 27));
        SecondColonPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8("image9"))));
        SecondColonPixmap->setProperty("scaledContents", QVariant(true));
        SecondColonPixmap->setProperty("buddy", QVariant(QByteArray("")));
        HundredthColonPixmap = new Rosegarden::Label(LCDBoxFrame);
        HundredthColonPixmap->setObjectName(QString::fromUtf8("HundredthColonPixmap"));
        HundredthColonPixmap->setGeometry(QRect(151, 14, 4, 27));
        HundredthColonPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8("image9"))));
        HundredthColonPixmap->setProperty("scaledContents", QVariant(true));
        HundredthColonPixmap->setProperty("buddy", QVariant(QByteArray("")));
        TenthsPixmap = new Rosegarden::Label(LCDBoxFrame);
        TenthsPixmap->setObjectName(QString::fromUtf8("TenthsPixmap"));
        TenthsPixmap->setGeometry(QRect(155, 16, 14, 24));
        TenthsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        TenthsPixmap->setProperty("scaledContents", QVariant(true));
        TenthsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        ThousandthsPixmap = new Rosegarden::Label(LCDBoxFrame);
        ThousandthsPixmap->setObjectName(QString::fromUtf8("ThousandthsPixmap"));
        ThousandthsPixmap->setGeometry(QRect(187, 16, 14, 24));
        ThousandthsPixmap->setProperty("pixmap", QVariant(QPixmap(QString::fromUtf8(":/pixmaps/transport/led-0.png"))));
        ThousandthsPixmap->setProperty("scaledContents", QVariant(true));
        ThousandthsPixmap->setProperty("buddy", QVariant(QByteArray("")));
        TempoDisplay = new Rosegarden::Label(LCDBoxFrame);
        TempoDisplay->setObjectName(QString::fromUtf8("TempoDisplay"));
        TempoDisplay->setGeometry(QRect(159, 51, 55, 16));
        TimeDisplayButton = new QPushButton(MainFrame);
        TimeDisplayButton->setObjectName(QString::fromUtf8("TimeDisplayButton"));
        TimeDisplayButton->setGeometry(QRect(3, 4, 25, 25));
        TimeDisplayButton->setFont(font);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/pixmaps/transport/button-bar-time.png"), QSize(), QIcon::Normal, QIcon::Off);
        TimeDisplayButton->setIcon(icon8);
        TimeDisplayButton->setCheckable(false);
        TimeDisplayButton->setFlat(true);
        PanelOpenButton = new QPushButton(MainFrame);
        PanelOpenButton->setObjectName(QString::fromUtf8("PanelOpenButton"));
        PanelOpenButton->setGeometry(QRect(3, 56, 25, 25));
        PanelOpenButton->setFont(font);
        PanelOpenButton->setIcon(icon);
        PanelOpenButton->setCheckable(true);
        PanelOpenButton->setChecked(true);
        PanelOpenButton->setFlat(true);
        RewindButton = new QPushButton(MainFrame);
        RewindButton->setObjectName(QString::fromUtf8("RewindButton"));
        RewindButton->setGeometry(QRect(260, 4, 42, 38));
        RewindButton->setCursor(QCursor(Qt::ArrowCursor));
        RewindButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/pixmaps/transport/button-rewind.png"), QSize(), QIcon::Normal, QIcon::Off);
        RewindButton->setIcon(icon9);
        RewindButton->setAutoRepeat(true);
        RewindEndButton = new QPushButton(MainFrame);
        RewindEndButton->setObjectName(QString::fromUtf8("RewindEndButton"));
        RewindEndButton->setGeometry(QRect(260, 43, 42, 38));
        RewindEndButton->setCursor(QCursor(Qt::ArrowCursor));
        RewindEndButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/pixmaps/transport/button-rewind-end.png"), QSize(), QIcon::Normal, QIcon::Off);
        RewindEndButton->setIcon(icon10);
        PlayButton = new QPushButton(MainFrame);
        PlayButton->setObjectName(QString::fromUtf8("PlayButton"));
        PlayButton->setGeometry(QRect(303, 4, 66, 38));
        PlayButton->setCursor(QCursor(Qt::ArrowCursor));
        PlayButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/pixmaps/transport/button-playpause.png"), QSize(), QIcon::Normal, QIcon::Off);
        PlayButton->setIcon(icon11);
        PlayButton->setIconSize(QSize(36, 20));
        PlayButton->setCheckable(true);
        StopButton = new QPushButton(MainFrame);
        StopButton->setObjectName(QString::fromUtf8("StopButton"));
        StopButton->setGeometry(QRect(303, 43, 66, 38));
        StopButton->setCursor(QCursor(Qt::ArrowCursor));
        StopButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/pixmaps/transport/button-stop.png"), QSize(), QIcon::Normal, QIcon::Off);
        StopButton->setIcon(icon12);
        StopButton->setIconSize(QSize(36, 20));
        FfwdButton = new QPushButton(MainFrame);
        FfwdButton->setObjectName(QString::fromUtf8("FfwdButton"));
        FfwdButton->setGeometry(QRect(370, 4, 42, 38));
        FfwdButton->setCursor(QCursor(Qt::ArrowCursor));
        FfwdButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/pixmaps/transport/button-ffwd.png"), QSize(), QIcon::Normal, QIcon::Off);
        FfwdButton->setIcon(icon13);
        FfwdButton->setAutoRepeat(true);
        FfwdEndButton = new QPushButton(MainFrame);
        FfwdEndButton->setObjectName(QString::fromUtf8("FfwdEndButton"));
        FfwdEndButton->setGeometry(QRect(370, 43, 42, 38));
        FfwdEndButton->setCursor(QCursor(Qt::ArrowCursor));
        FfwdEndButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/pixmaps/transport/button-ffwd-end.png"), QSize(), QIcon::Normal, QIcon::Off);
        FfwdEndButton->setIcon(icon14);
        ToEndButton = new QPushButton(MainFrame);
        ToEndButton->setObjectName(QString::fromUtf8("ToEndButton"));
        ToEndButton->setGeometry(QRect(3, 28, 25, 25));
        ToEndButton->setFont(font);
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/pixmaps/transport/button-from-end.png"), QSize(), QIcon::Normal, QIcon::Off);
        ToEndButton->setIcon(icon15);
        ToEndButton->setCheckable(true);
        ToEndButton->setFlat(true);
        QWidget::setTabOrder(PlayButton, StopButton);
        QWidget::setTabOrder(StopButton, MetronomeButton);
        QWidget::setTabOrder(MetronomeButton, RecordButton);
        QWidget::setTabOrder(RecordButton, RewindEndButton);
        QWidget::setTabOrder(RewindEndButton, RewindButton);
        QWidget::setTabOrder(RewindButton, FfwdButton);
        QWidget::setTabOrder(FfwdButton, FfwdEndButton);

        retranslateUi(RosegardenTransport);

        PlayButton->setDefault(false);


        QMetaObject::connectSlotsByName(RosegardenTransport);
    } // setupUi

    void retranslateUi(QFrame *RosegardenTransport)
    {
        RosegardenTransport->setProperty("caption", QVariant(QCoreApplication::translate("RosegardenTransport", "Rosegarden Transport", nullptr)));
#if QT_CONFIG(tooltip)
        PanelCloseButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Hide additional controls", nullptr));
#endif // QT_CONFIG(tooltip)
        PanelCloseButton->setText(QString());
#if QT_CONFIG(tooltip)
        PanicButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Panic Button", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        PanicButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Resets all MIDI devices if you've got stuck notes", nullptr));
#endif // QT_CONFIG(whatsthis)
        PanicButton->setText(QString());
#if QT_CONFIG(tooltip)
        MetronomeButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Metronome", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        MetronomeButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Provides a metronome click for you to play along with", nullptr));
#endif // QT_CONFIG(whatsthis)
        MetronomeButton->setText(QString());
#if QT_CONFIG(whatsthis)
        MidiEventFrame->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Shows MIDI activity in and out of Rosegarden", nullptr));
#endif // QT_CONFIG(whatsthis)
        InLabel->setText(QCoreApplication::translate("RosegardenTransport", "IN", nullptr));
        OutLabel->setText(QCoreApplication::translate("RosegardenTransport", "OUT", nullptr));
        InDisplay->setText(QCoreApplication::translate("RosegardenTransport", "NO EVENTS", nullptr));
        OutDisplay->setText(QCoreApplication::translate("RosegardenTransport", "NO EVENTS", nullptr));
#if QT_CONFIG(tooltip)
        RecordButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Record", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        RecordButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Record either MIDI or audio", nullptr));
#endif // QT_CONFIG(whatsthis)
        RecordButton->setText(QString());
#if QT_CONFIG(tooltip)
        LoopButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Loop", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        LoopButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Turn on and off the loop markers (if set)", nullptr));
#endif // QT_CONFIG(whatsthis)
        LoopButton->setText(QString());
#if QT_CONFIG(tooltip)
        SoloButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Solo", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        SoloButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Mutes all but the currently selected track", nullptr));
#endif // QT_CONFIG(whatsthis)
        SoloButton->setText(QString());
#if QT_CONFIG(tooltip)
        SetStartLPButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Start loop or range here", nullptr));
#endif // QT_CONFIG(tooltip)
        SetStartLPButton->setText(QString());
#if QT_CONFIG(tooltip)
        SetStopLPButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "End loop or range here", nullptr));
#endif // QT_CONFIG(tooltip)
        SetStopLPButton->setText(QString());
        TimeSigLabel->setText(QCoreApplication::translate("RosegardenTransport", "SIG", nullptr));
        DivisionLabel->setText(QCoreApplication::translate("RosegardenTransport", "DIV", nullptr));
        DivisionDisplay->setText(QCoreApplication::translate("RosegardenTransport", "/16", nullptr));
        TempoLabel->setText(QCoreApplication::translate("RosegardenTransport", "TEMPO", nullptr));
        ToEndLabel->setText(QCoreApplication::translate("RosegardenTransport", "END", nullptr));
        TimeDisplayLabel->setText(QCoreApplication::translate("RosegardenTransport", "BAR", nullptr));
#if QT_CONFIG(tooltip)
        TimeDisplayButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Switch between real time, musical time, and frame count", nullptr));
#endif // QT_CONFIG(tooltip)
        TimeDisplayButton->setText(QString());
#if QT_CONFIG(tooltip)
        PanelOpenButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Show additional controls", nullptr));
#endif // QT_CONFIG(tooltip)
        PanelOpenButton->setText(QString());
#if QT_CONFIG(tooltip)
        RewindButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Rewind", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        RewindButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Moves the current pointer position back one bar.", nullptr));
#endif // QT_CONFIG(whatsthis)
        RewindButton->setText(QString());
#if QT_CONFIG(tooltip)
        RewindEndButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Rewind to beginning", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        RewindEndButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Moves the pointer position to the start of the composition. (This may mean going forwards if the pointer is currently before the start.)", nullptr));
#endif // QT_CONFIG(whatsthis)
        RewindEndButton->setText(QString());
#if QT_CONFIG(tooltip)
        PlayButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Play/Pause", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        PlayButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Plays from the current pointer position, or pauses playback if already in progress.", nullptr));
#endif // QT_CONFIG(whatsthis)
        PlayButton->setText(QString());
#if QT_CONFIG(tooltip)
        StopButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Stop", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        StopButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Stops playback or recording.", nullptr));
#endif // QT_CONFIG(whatsthis)
        StopButton->setText(QString());
#if QT_CONFIG(tooltip)
        FfwdButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Fast forward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        FfwdButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Moves the current pointer position forwards one bar.", nullptr));
#endif // QT_CONFIG(whatsthis)
        FfwdButton->setText(QString());
#if QT_CONFIG(tooltip)
        FfwdEndButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Fast forward to end", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        FfwdEndButton->setWhatsThis(QCoreApplication::translate("RosegardenTransport", "Moves the pointer position to the end of the composition.  (This may mean going backwards if the pointer is already beyond the end.)", nullptr));
#endif // QT_CONFIG(whatsthis)
        FfwdEndButton->setText(QString());
#if QT_CONFIG(tooltip)
        ToEndButton->setToolTip(QCoreApplication::translate("RosegardenTransport", "Display time to end", nullptr));
#endif // QT_CONFIG(tooltip)
        ToEndButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class RosegardenTransport: public Ui_RosegardenTransport {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ROSEGARDENTRANSPORTUI_H
