/****************************************************************************
** Meta object code from reading C++ file 'AudioManagerDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/AudioManagerDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AudioManagerDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__AudioManagerDialog_t {
    QByteArrayData data[36];
    char stringdata0[534];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__AudioManagerDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__AudioManagerDialog_t qt_meta_stringdata_Rosegarden__AudioManagerDialog = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Rosegarden::AudioManagerDialog"
QT_MOC_LITERAL(1, 31, 12), // "addAudioFile"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 11), // "AudioFileId"
QT_MOC_LITERAL(4, 57, 15), // "deleteAudioFile"
QT_MOC_LITERAL(5, 73, 13), // "playAudioFile"
QT_MOC_LITERAL(6, 87, 8), // "RealTime"
QT_MOC_LITERAL(7, 96, 22), // "cancelPlayingAudioFile"
QT_MOC_LITERAL(8, 119, 19), // "deleteAllAudioFiles"
QT_MOC_LITERAL(9, 139, 16), // "segmentsSelected"
QT_MOC_LITERAL(10, 156, 16), // "SegmentSelection"
QT_MOC_LITERAL(11, 173, 14), // "deleteSegments"
QT_MOC_LITERAL(12, 188, 18), // "insertAudioSegment"
QT_MOC_LITERAL(13, 207, 7), // "closing"
QT_MOC_LITERAL(14, 215, 7), // "slotAdd"
QT_MOC_LITERAL(15, 223, 15), // "slotPlayPreview"
QT_MOC_LITERAL(16, 239, 10), // "slotInsert"
QT_MOC_LITERAL(17, 250, 10), // "slotRemove"
QT_MOC_LITERAL(18, 261, 13), // "slotRemoveAll"
QT_MOC_LITERAL(19, 275, 19), // "slotRemoveAllUnused"
QT_MOC_LITERAL(20, 295, 16), // "slotDeleteUnused"
QT_MOC_LITERAL(21, 312, 15), // "slotExportAudio"
QT_MOC_LITERAL(22, 328, 17), // "slotHelpRequested"
QT_MOC_LITERAL(23, 346, 13), // "slotHelpAbout"
QT_MOC_LITERAL(24, 360, 20), // "slotSelectionChanged"
QT_MOC_LITERAL(25, 381, 20), // "slotPopulateFileList"
QT_MOC_LITERAL(26, 402, 19), // "slotCommandExecuted"
QT_MOC_LITERAL(27, 422, 20), // "slotSegmentSelection"
QT_MOC_LITERAL(28, 443, 9), // "slotClose"
QT_MOC_LITERAL(29, 453, 11), // "slotDropped"
QT_MOC_LITERAL(30, 465, 11), // "QDropEvent*"
QT_MOC_LITERAL(31, 477, 5), // "event"
QT_MOC_LITERAL(32, 483, 12), // "QTreeWidget*"
QT_MOC_LITERAL(33, 496, 11), // "QList<QUrl>"
QT_MOC_LITERAL(34, 508, 2), // "sl"
QT_MOC_LITERAL(35, 511, 22) // "slotCancelPlayingAudio"

    },
    "Rosegarden::AudioManagerDialog\0"
    "addAudioFile\0\0AudioFileId\0deleteAudioFile\0"
    "playAudioFile\0RealTime\0cancelPlayingAudioFile\0"
    "deleteAllAudioFiles\0segmentsSelected\0"
    "SegmentSelection\0deleteSegments\0"
    "insertAudioSegment\0closing\0slotAdd\0"
    "slotPlayPreview\0slotInsert\0slotRemove\0"
    "slotRemoveAll\0slotRemoveAllUnused\0"
    "slotDeleteUnused\0slotExportAudio\0"
    "slotHelpRequested\0slotHelpAbout\0"
    "slotSelectionChanged\0slotPopulateFileList\0"
    "slotCommandExecuted\0slotSegmentSelection\0"
    "slotClose\0slotDropped\0QDropEvent*\0"
    "event\0QTreeWidget*\0QList<QUrl>\0sl\0"
    "slotCancelPlayingAudio"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__AudioManagerDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x06 /* Public */,
       4,    1,  147,    2, 0x06 /* Public */,
       5,    3,  150,    2, 0x06 /* Public */,
       7,    1,  157,    2, 0x06 /* Public */,
       8,    0,  160,    2, 0x06 /* Public */,
       9,    1,  161,    2, 0x06 /* Public */,
      11,    1,  164,    2, 0x06 /* Public */,
      12,    3,  167,    2, 0x06 /* Public */,
      13,    0,  174,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    0,  175,    2, 0x0a /* Public */,
      15,    0,  176,    2, 0x0a /* Public */,
      16,    0,  177,    2, 0x0a /* Public */,
      17,    0,  178,    2, 0x0a /* Public */,
      18,    0,  179,    2, 0x0a /* Public */,
      19,    0,  180,    2, 0x0a /* Public */,
      20,    0,  181,    2, 0x0a /* Public */,
      21,    0,  182,    2, 0x0a /* Public */,
      22,    0,  183,    2, 0x0a /* Public */,
      23,    0,  184,    2, 0x0a /* Public */,
      24,    0,  185,    2, 0x0a /* Public */,
      25,    0,  186,    2, 0x0a /* Public */,
      26,    0,  187,    2, 0x0a /* Public */,
      27,    1,  188,    2, 0x0a /* Public */,
      28,    0,  191,    2, 0x0a /* Public */,
      29,    3,  192,    2, 0x09 /* Protected */,
      35,    0,  199,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6, 0x80000000 | 6,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6, 0x80000000 | 6,    2,    2,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 30, 0x80000000 | 32, 0x80000000 | 33,   31,    2,   34,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::AudioManagerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioManagerDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->addAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 1: _t->deleteAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 2: _t->playAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1])),(*reinterpret_cast< const RealTime(*)>(_a[2])),(*reinterpret_cast< const RealTime(*)>(_a[3]))); break;
        case 3: _t->cancelPlayingAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 4: _t->deleteAllAudioFiles(); break;
        case 5: _t->segmentsSelected((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 6: _t->deleteSegments((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 7: _t->insertAudioSegment((*reinterpret_cast< AudioFileId(*)>(_a[1])),(*reinterpret_cast< const RealTime(*)>(_a[2])),(*reinterpret_cast< const RealTime(*)>(_a[3]))); break;
        case 8: _t->closing(); break;
        case 9: _t->slotAdd(); break;
        case 10: _t->slotPlayPreview(); break;
        case 11: _t->slotInsert(); break;
        case 12: _t->slotRemove(); break;
        case 13: _t->slotRemoveAll(); break;
        case 14: _t->slotRemoveAllUnused(); break;
        case 15: _t->slotDeleteUnused(); break;
        case 16: _t->slotExportAudio(); break;
        case 17: _t->slotHelpRequested(); break;
        case 18: _t->slotHelpAbout(); break;
        case 19: _t->slotSelectionChanged(); break;
        case 20: _t->slotPopulateFileList(); break;
        case 21: _t->slotCommandExecuted(); break;
        case 22: _t->slotSegmentSelection((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 23: _t->slotClose(); break;
        case 24: _t->slotDropped((*reinterpret_cast< QDropEvent*(*)>(_a[1])),(*reinterpret_cast< QTreeWidget*(*)>(_a[2])),(*reinterpret_cast< const QList<QUrl>(*)>(_a[3]))); break;
        case 25: _t->slotCancelPlayingAudio(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AudioManagerDialog::*)(AudioFileId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::addAudioFile)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(AudioFileId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::deleteAudioFile)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(AudioFileId , const RealTime & , const RealTime & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::playAudioFile)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(AudioFileId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::cancelPlayingAudioFile)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::deleteAllAudioFiles)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(const SegmentSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::segmentsSelected)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(const SegmentSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::deleteSegments)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)(AudioFileId , const RealTime & , const RealTime & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::insertAudioSegment)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (AudioManagerDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioManagerDialog::closing)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::AudioManagerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__AudioManagerDialog.data,
    qt_meta_data_Rosegarden__AudioManagerDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::AudioManagerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::AudioManagerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__AudioManagerDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::AudioManagerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::AudioManagerDialog::addAudioFile(AudioFileId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::AudioManagerDialog::deleteAudioFile(AudioFileId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::AudioManagerDialog::playAudioFile(AudioFileId _t1, const RealTime & _t2, const RealTime & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::AudioManagerDialog::cancelPlayingAudioFile(AudioFileId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::AudioManagerDialog::deleteAllAudioFiles()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::AudioManagerDialog::segmentsSelected(const SegmentSelection & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Rosegarden::AudioManagerDialog::deleteSegments(const SegmentSelection & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::AudioManagerDialog::insertAudioSegment(AudioFileId _t1, const RealTime & _t2, const RealTime & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::AudioManagerDialog::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
