/****************************************************************************
** Meta object code from reading C++ file 'ControlRulerWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/rulers/ControlRulerWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ControlRulerWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__ControlRulerWidget_t {
    QByteArrayData data[24];
    char stringdata0[370];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__ControlRulerWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__ControlRulerWidget_t qt_meta_stringdata_Rosegarden__ControlRulerWidget = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Rosegarden::ControlRulerWidget"
QT_MOC_LITERAL(1, 31, 26), // "childRulerSelectionChanged"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 20), // "rulerSelectionUpdate"
QT_MOC_LITERAL(4, 80, 15), // "showContextHelp"
QT_MOC_LITERAL(5, 96, 15), // "slotRemoveRuler"
QT_MOC_LITERAL(6, 112, 17), // "slotSetPannedRect"
QT_MOC_LITERAL(7, 130, 10), // "pannedRect"
QT_MOC_LITERAL(8, 141, 25), // "slotSetCurrentViewSegment"
QT_MOC_LITERAL(9, 167, 12), // "ViewSegment*"
QT_MOC_LITERAL(10, 180, 20), // "slotSelectionChanged"
QT_MOC_LITERAL(11, 201, 15), // "EventSelection*"
QT_MOC_LITERAL(12, 217, 26), // "slotHoveredOverNoteChanged"
QT_MOC_LITERAL(13, 244, 7), // "evPitch"
QT_MOC_LITERAL(14, 252, 9), // "haveEvent"
QT_MOC_LITERAL(15, 262, 5), // "timeT"
QT_MOC_LITERAL(16, 268, 6), // "evTime"
QT_MOC_LITERAL(17, 275, 16), // "slotUpdateRulers"
QT_MOC_LITERAL(18, 292, 9), // "startTime"
QT_MOC_LITERAL(19, 302, 7), // "endTime"
QT_MOC_LITERAL(20, 310, 11), // "slotSetTool"
QT_MOC_LITERAL(21, 322, 10), // "tabChanged"
QT_MOC_LITERAL(22, 333, 5), // "index"
QT_MOC_LITERAL(23, 339, 30) // "slotChildRulerSelectionChanged"

    },
    "Rosegarden::ControlRulerWidget\0"
    "childRulerSelectionChanged\0\0"
    "rulerSelectionUpdate\0showContextHelp\0"
    "slotRemoveRuler\0slotSetPannedRect\0"
    "pannedRect\0slotSetCurrentViewSegment\0"
    "ViewSegment*\0slotSelectionChanged\0"
    "EventSelection*\0slotHoveredOverNoteChanged\0"
    "evPitch\0haveEvent\0timeT\0evTime\0"
    "slotUpdateRulers\0startTime\0endTime\0"
    "slotSetTool\0tabChanged\0index\0"
    "slotChildRulerSelectionChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__ControlRulerWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    1,   76,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   79,    2, 0x0a /* Public */,
       6,    1,   82,    2, 0x0a /* Public */,
       8,    1,   85,    2, 0x0a /* Public */,
      10,    1,   88,    2, 0x0a /* Public */,
      12,    3,   91,    2, 0x0a /* Public */,
      17,    2,   98,    2, 0x0a /* Public */,
      20,    1,  103,    2, 0x0a /* Public */,
      21,    1,  106,    2, 0x08 /* Private */,
      23,    1,  109,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QRectF,    7,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool, 0x80000000 | 15,   13,   14,   16,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,   18,   19,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, 0x80000000 | 11,    2,

       0        // eod
};

void Rosegarden::ControlRulerWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ControlRulerWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->childRulerSelectionChanged(); break;
        case 1: _t->rulerSelectionUpdate(); break;
        case 2: _t->showContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->slotRemoveRuler((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotSetPannedRect((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 5: _t->slotSetCurrentViewSegment((*reinterpret_cast< ViewSegment*(*)>(_a[1]))); break;
        case 6: _t->slotSelectionChanged((*reinterpret_cast< EventSelection*(*)>(_a[1]))); break;
        case 7: _t->slotHoveredOverNoteChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< timeT(*)>(_a[3]))); break;
        case 8: _t->slotUpdateRulers((*reinterpret_cast< timeT(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 9: _t->slotSetTool((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->tabChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->slotChildRulerSelectionChanged((*reinterpret_cast< EventSelection*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ControlRulerWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ControlRulerWidget::childRulerSelectionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ControlRulerWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ControlRulerWidget::rulerSelectionUpdate)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ControlRulerWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ControlRulerWidget::showContextHelp)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::ControlRulerWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__ControlRulerWidget.data,
    qt_meta_data_Rosegarden__ControlRulerWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::ControlRulerWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::ControlRulerWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__ControlRulerWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Rosegarden::ControlRulerWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::ControlRulerWidget::childRulerSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::ControlRulerWidget::rulerSelectionUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::ControlRulerWidget::showContextHelp(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
