/****************************************************************************
** Meta object code from reading C++ file 'NotationView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/notation/NotationView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NotationView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__NotationView_t {
    QByteArrayData data[196];
    char stringdata0[3818];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__NotationView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__NotationView_t qt_meta_stringdata_Rosegarden__NotationView = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::NotationView"
QT_MOC_LITERAL(1, 25, 4), // "play"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 4), // "stop"
QT_MOC_LITERAL(4, 36, 12), // "stepBackward"
QT_MOC_LITERAL(5, 49, 11), // "stepForward"
QT_MOC_LITERAL(6, 61, 14), // "rewindPlayback"
QT_MOC_LITERAL(7, 76, 19), // "fastForwardPlayback"
QT_MOC_LITERAL(8, 96, 25), // "rewindPlaybackToBeginning"
QT_MOC_LITERAL(9, 122, 24), // "fastForwardPlaybackToEnd"
QT_MOC_LITERAL(10, 147, 5), // "panic"
QT_MOC_LITERAL(11, 153, 18), // "editTriggerSegment"
QT_MOC_LITERAL(12, 172, 25), // "stepByStepTargetRequested"
QT_MOC_LITERAL(13, 198, 28), // "slotInsertableNoteOnReceived"
QT_MOC_LITERAL(14, 227, 5), // "pitch"
QT_MOC_LITERAL(15, 233, 8), // "velocity"
QT_MOC_LITERAL(16, 242, 29), // "slotInsertableNoteOffReceived"
QT_MOC_LITERAL(17, 272, 29), // "slotStepByStepTargetRequested"
QT_MOC_LITERAL(18, 302, 19), // "slotRegenerateScene"
QT_MOC_LITERAL(19, 322, 21), // "slotUpdateWindowTitle"
QT_MOC_LITERAL(20, 344, 1), // "m"
QT_MOC_LITERAL(21, 346, 17), // "slotPrintLilyPond"
QT_MOC_LITERAL(22, 364, 19), // "slotPreviewLilyPond"
QT_MOC_LITERAL(23, 384, 11), // "slotEditCut"
QT_MOC_LITERAL(24, 396, 12), // "slotEditCopy"
QT_MOC_LITERAL(25, 409, 13), // "slotEditPaste"
QT_MOC_LITERAL(26, 423, 14), // "slotEditDelete"
QT_MOC_LITERAL(27, 438, 19), // "slotEditCutAndClose"
QT_MOC_LITERAL(28, 458, 20), // "slotEditGeneralPaste"
QT_MOC_LITERAL(29, 479, 15), // "slotEditAddClef"
QT_MOC_LITERAL(30, 495, 23), // "slotEditAddClefLinkOnly"
QT_MOC_LITERAL(31, 519, 23), // "slotEditAddKeySignature"
QT_MOC_LITERAL(32, 543, 22), // "slotEditAddSustainDown"
QT_MOC_LITERAL(33, 566, 20), // "slotEditAddSustainUp"
QT_MOC_LITERAL(34, 587, 18), // "slotEditAddSustain"
QT_MOC_LITERAL(35, 606, 4), // "down"
QT_MOC_LITERAL(36, 611, 17), // "slotEditTranspose"
QT_MOC_LITERAL(37, 629, 20), // "slotEditSwitchPreset"
QT_MOC_LITERAL(38, 650, 20), // "slotPreviewSelection"
QT_MOC_LITERAL(39, 671, 13), // "slotClearLoop"
QT_MOC_LITERAL(40, 685, 18), // "slotClearSelection"
QT_MOC_LITERAL(41, 704, 17), // "slotEscapePressed"
QT_MOC_LITERAL(42, 722, 23), // "slotEditSelectFromStart"
QT_MOC_LITERAL(43, 746, 19), // "slotEditSelectToEnd"
QT_MOC_LITERAL(44, 766, 24), // "slotEditSelectWholeStaff"
QT_MOC_LITERAL(45, 791, 16), // "slotSearchSelect"
QT_MOC_LITERAL(46, 808, 19), // "slotFilterSelection"
QT_MOC_LITERAL(47, 828, 27), // "slotSelectEvenlySpacedNotes"
QT_MOC_LITERAL(48, 856, 14), // "slotVelocityUp"
QT_MOC_LITERAL(49, 871, 16), // "slotVelocityDown"
QT_MOC_LITERAL(50, 888, 17), // "slotSetVelocities"
QT_MOC_LITERAL(51, 906, 18), // "slotSetControllers"
QT_MOC_LITERAL(52, 925, 20), // "slotPlaceControllers"
QT_MOC_LITERAL(53, 946, 17), // "slotSetSelectTool"
QT_MOC_LITERAL(54, 964, 23), // "slotSetSelectNoTiesTool"
QT_MOC_LITERAL(55, 988, 16), // "slotSetEraseTool"
QT_MOC_LITERAL(56, 1005, 23), // "slotSetNoteRestInserter"
QT_MOC_LITERAL(57, 1029, 24), // "slotInsertNoteFromAction"
QT_MOC_LITERAL(58, 1054, 24), // "slotInsertRestFromAction"
QT_MOC_LITERAL(59, 1079, 17), // "slotSwitchToNotes"
QT_MOC_LITERAL(60, 1097, 17), // "slotSwitchToRests"
QT_MOC_LITERAL(61, 1115, 13), // "slotToggleDot"
QT_MOC_LITERAL(62, 1129, 10), // "slotAddDot"
QT_MOC_LITERAL(63, 1140, 22), // "slotAddDotNotationOnly"
QT_MOC_LITERAL(64, 1163, 15), // "slotSetNoteType"
QT_MOC_LITERAL(65, 1179, 27), // "slotSetNoteTypeNotationOnly"
QT_MOC_LITERAL(66, 1207, 14), // "slotNoteAction"
QT_MOC_LITERAL(67, 1222, 10), // "slotDummy1"
QT_MOC_LITERAL(68, 1233, 16), // "slotNoAccidental"
QT_MOC_LITERAL(69, 1250, 20), // "slotFollowAccidental"
QT_MOC_LITERAL(70, 1271, 9), // "slotSharp"
QT_MOC_LITERAL(71, 1281, 8), // "slotFlat"
QT_MOC_LITERAL(72, 1290, 11), // "slotNatural"
QT_MOC_LITERAL(73, 1302, 15), // "slotDoubleSharp"
QT_MOC_LITERAL(74, 1318, 14), // "slotDoubleFlat"
QT_MOC_LITERAL(75, 1333, 14), // "slotClefAction"
QT_MOC_LITERAL(76, 1348, 8), // "slotText"
QT_MOC_LITERAL(77, 1357, 15), // "slotGuitarChord"
QT_MOC_LITERAL(78, 1373, 14), // "slotLinearMode"
QT_MOC_LITERAL(79, 1388, 22), // "slotContinuousPageMode"
QT_MOC_LITERAL(80, 1411, 17), // "slotMultiPageMode"
QT_MOC_LITERAL(81, 1429, 13), // "slotHighlight"
QT_MOC_LITERAL(82, 1443, 20), // "slotShowHeadersGroup"
QT_MOC_LITERAL(83, 1464, 24), // "slotChangeFontFromAction"
QT_MOC_LITERAL(84, 1489, 28), // "slotChangeFontSizeFromAction"
QT_MOC_LITERAL(85, 1518, 27), // "slotChangeSpacingFromAction"
QT_MOC_LITERAL(86, 1546, 20), // "slotUpdateMenuStates"
QT_MOC_LITERAL(87, 1567, 28), // "slotTransformsNormalizeRests"
QT_MOC_LITERAL(88, 1596, 27), // "slotTransformsCollapseNotes"
QT_MOC_LITERAL(89, 1624, 22), // "slotTransformsQuantize"
QT_MOC_LITERAL(90, 1647, 23), // "slotTransformsInterpret"
QT_MOC_LITERAL(91, 1671, 16), // "slotMakeOrnament"
QT_MOC_LITERAL(92, 1688, 15), // "slotUseOrnament"
QT_MOC_LITERAL(93, 1704, 18), // "slotRemoveOrnament"
QT_MOC_LITERAL(94, 1723, 22), // "slotEditOrnamentInline"
QT_MOC_LITERAL(95, 1746, 25), // "slotShowOrnamentExpansion"
QT_MOC_LITERAL(96, 1772, 16), // "slotMaskOrnament"
QT_MOC_LITERAL(97, 1789, 18), // "slotUnmaskOrnament"
QT_MOC_LITERAL(98, 1808, 18), // "slotUnadoptSegment"
QT_MOC_LITERAL(99, 1827, 21), // "slotGroupSimpleTuplet"
QT_MOC_LITERAL(100, 1849, 22), // "slotGroupGeneralTuplet"
QT_MOC_LITERAL(101, 1872, 15), // "slotGroupTuplet"
QT_MOC_LITERAL(102, 1888, 6), // "simple"
QT_MOC_LITERAL(103, 1895, 26), // "slotUpdateInsertModeStatus"
QT_MOC_LITERAL(104, 1922, 33), // "slotUpdateInsertModeStatusTri..."
QT_MOC_LITERAL(105, 1956, 32), // "slotUpdateInsertModeStatusTuplet"
QT_MOC_LITERAL(106, 1989, 15), // "slotRescaleHalf"
QT_MOC_LITERAL(107, 2005, 17), // "slotRescaleDouble"
QT_MOC_LITERAL(108, 2023, 11), // "slotRescale"
QT_MOC_LITERAL(109, 2035, 15), // "slotTransposeUp"
QT_MOC_LITERAL(110, 2051, 17), // "slotTransposeDown"
QT_MOC_LITERAL(111, 2069, 21), // "slotTransposeUpOctave"
QT_MOC_LITERAL(112, 2091, 23), // "slotTransposeDownOctave"
QT_MOC_LITERAL(113, 2115, 13), // "slotTranspose"
QT_MOC_LITERAL(114, 2129, 21), // "slotDiatonicTranspose"
QT_MOC_LITERAL(115, 2151, 10), // "slotInvert"
QT_MOC_LITERAL(116, 2162, 14), // "slotRetrograde"
QT_MOC_LITERAL(117, 2177, 20), // "slotRetrogradeInvert"
QT_MOC_LITERAL(118, 2198, 11), // "slotJogLeft"
QT_MOC_LITERAL(119, 2210, 12), // "slotJogRight"
QT_MOC_LITERAL(120, 2223, 14), // "slotEditLyrics"
QT_MOC_LITERAL(121, 2238, 16), // "slotStepBackward"
QT_MOC_LITERAL(122, 2255, 15), // "slotStepForward"
QT_MOC_LITERAL(123, 2271, 18), // "slotCurrentStaffUp"
QT_MOC_LITERAL(124, 2290, 20), // "slotCurrentStaffDown"
QT_MOC_LITERAL(125, 2311, 23), // "slotCurrentSegmentPrior"
QT_MOC_LITERAL(126, 2335, 22), // "slotCurrentSegmentNext"
QT_MOC_LITERAL(127, 2358, 21), // "slotToggleChordsRuler"
QT_MOC_LITERAL(128, 2380, 22), // "slotToggleRawNoteRuler"
QT_MOC_LITERAL(129, 2403, 20), // "slotToggleTempoRuler"
QT_MOC_LITERAL(130, 2424, 23), // "slotToggleVelocityRuler"
QT_MOC_LITERAL(131, 2448, 24), // "slotTogglePitchbendRuler"
QT_MOC_LITERAL(132, 2473, 26), // "slotToggleKeyPressureRuler"
QT_MOC_LITERAL(133, 2500, 30), // "slotToggleChannelPressureRuler"
QT_MOC_LITERAL(134, 2531, 19), // "slotAddControlRuler"
QT_MOC_LITERAL(135, 2551, 8), // "QAction*"
QT_MOC_LITERAL(136, 2560, 12), // "slotAddTempo"
QT_MOC_LITERAL(137, 2573, 20), // "slotAddTimeSignature"
QT_MOC_LITERAL(138, 2594, 21), // "slotCheckForParallels"
QT_MOC_LITERAL(139, 2616, 24), // "slotToggleGeneralToolBar"
QT_MOC_LITERAL(140, 2641, 22), // "slotToggleToolsToolBar"
QT_MOC_LITERAL(141, 2664, 25), // "slotToggleDurationToolBar"
QT_MOC_LITERAL(142, 2690, 26), // "slotToggleInterpretToolBar"
QT_MOC_LITERAL(143, 2717, 28), // "slotToggleAccidentalsToolBar"
QT_MOC_LITERAL(144, 2746, 22), // "slotToggleClefsToolBar"
QT_MOC_LITERAL(145, 2769, 22), // "slotToggleMarksToolBar"
QT_MOC_LITERAL(146, 2792, 22), // "slotToggleGroupToolBar"
QT_MOC_LITERAL(147, 2815, 24), // "slotToggleSymbolsToolBar"
QT_MOC_LITERAL(148, 2840, 23), // "slotToggleLayoutToolBar"
QT_MOC_LITERAL(149, 2864, 22), // "slotToggleLayerToolBar"
QT_MOC_LITERAL(150, 2887, 23), // "slotToggleRulersToolBar"
QT_MOC_LITERAL(151, 2911, 26), // "slotToggleTransportToolBar"
QT_MOC_LITERAL(152, 2938, 18), // "slotScrollToFollow"
QT_MOC_LITERAL(153, 2957, 8), // "slotLoop"
QT_MOC_LITERAL(154, 2966, 15), // "slotLoopChanged"
QT_MOC_LITERAL(155, 2982, 31), // "slotInsertableNoteEventReceived"
QT_MOC_LITERAL(156, 3014, 6), // "noteOn"
QT_MOC_LITERAL(157, 3021, 20), // "slotToggleStepByStep"
QT_MOC_LITERAL(158, 3042, 13), // "slotDebugDump"
QT_MOC_LITERAL(159, 3056, 15), // "slotBarDataDump"
QT_MOC_LITERAL(160, 3072, 16), // "slotSymbolAction"
QT_MOC_LITERAL(161, 3089, 32), // "slotMoveEventsUpStaffInteractive"
QT_MOC_LITERAL(162, 3122, 34), // "slotMoveEventsDownStaffIntera..."
QT_MOC_LITERAL(163, 3157, 21), // "slotMoveEventsUpStaff"
QT_MOC_LITERAL(164, 3179, 23), // "slotMoveEventsDownStaff"
QT_MOC_LITERAL(165, 3203, 26), // "slotHoveredOverNoteChanged"
QT_MOC_LITERAL(166, 3230, 34), // "slotHoveredOverAbsoluteTimeCh..."
QT_MOC_LITERAL(167, 3265, 19), // "slotShowContextHelp"
QT_MOC_LITERAL(168, 3285, 20), // "slotFontComboChanged"
QT_MOC_LITERAL(169, 3306, 20), // "slotSizeComboChanged"
QT_MOC_LITERAL(170, 3327, 23), // "slotSpacingComboChanged"
QT_MOC_LITERAL(171, 3351, 15), // "slotEditElement"
QT_MOC_LITERAL(172, 3367, 14), // "NotationStaff*"
QT_MOC_LITERAL(173, 3382, 16), // "NotationElement*"
QT_MOC_LITERAL(174, 3399, 27), // "slotExtendSelectionBackward"
QT_MOC_LITERAL(175, 3427, 26), // "slotExtendSelectionForward"
QT_MOC_LITERAL(176, 3454, 30), // "slotExtendSelectionBackwardBar"
QT_MOC_LITERAL(177, 3485, 29), // "slotExtendSelectionForwardBar"
QT_MOC_LITERAL(178, 3515, 3), // "bar"
QT_MOC_LITERAL(179, 3519, 8), // "slotHelp"
QT_MOC_LITERAL(180, 3528, 12), // "slotTutorial"
QT_MOC_LITERAL(181, 3541, 17), // "slotBugGuidelines"
QT_MOC_LITERAL(182, 3559, 13), // "slotHelpAbout"
QT_MOC_LITERAL(183, 3573, 15), // "slotHelpAboutQt"
QT_MOC_LITERAL(184, 3589, 10), // "slotDonate"
QT_MOC_LITERAL(185, 3600, 16), // "slotCycleSlashes"
QT_MOC_LITERAL(186, 3617, 12), // "slotAddLayer"
QT_MOC_LITERAL(187, 3630, 25), // "slotNewLayerFromSelection"
QT_MOC_LITERAL(188, 3656, 13), // "slotConfigure"
QT_MOC_LITERAL(189, 3670, 22), // "slotExpressionSequence"
QT_MOC_LITERAL(190, 3693, 21), // "slotPitchBendSequence"
QT_MOC_LITERAL(191, 3715, 22), // "slotControllerSequence"
QT_MOC_LITERAL(192, 3738, 24), // "slotCheckShowHeadersMenu"
QT_MOC_LITERAL(193, 3763, 7), // "checked"
QT_MOC_LITERAL(194, 3771, 21), // "slotInterpretActivate"
QT_MOC_LITERAL(195, 3793, 24) // "slotRulerSelectionUpdate"

    },
    "Rosegarden::NotationView\0play\0\0stop\0"
    "stepBackward\0stepForward\0rewindPlayback\0"
    "fastForwardPlayback\0rewindPlaybackToBeginning\0"
    "fastForwardPlaybackToEnd\0panic\0"
    "editTriggerSegment\0stepByStepTargetRequested\0"
    "slotInsertableNoteOnReceived\0pitch\0"
    "velocity\0slotInsertableNoteOffReceived\0"
    "slotStepByStepTargetRequested\0"
    "slotRegenerateScene\0slotUpdateWindowTitle\0"
    "m\0slotPrintLilyPond\0slotPreviewLilyPond\0"
    "slotEditCut\0slotEditCopy\0slotEditPaste\0"
    "slotEditDelete\0slotEditCutAndClose\0"
    "slotEditGeneralPaste\0slotEditAddClef\0"
    "slotEditAddClefLinkOnly\0slotEditAddKeySignature\0"
    "slotEditAddSustainDown\0slotEditAddSustainUp\0"
    "slotEditAddSustain\0down\0slotEditTranspose\0"
    "slotEditSwitchPreset\0slotPreviewSelection\0"
    "slotClearLoop\0slotClearSelection\0"
    "slotEscapePressed\0slotEditSelectFromStart\0"
    "slotEditSelectToEnd\0slotEditSelectWholeStaff\0"
    "slotSearchSelect\0slotFilterSelection\0"
    "slotSelectEvenlySpacedNotes\0slotVelocityUp\0"
    "slotVelocityDown\0slotSetVelocities\0"
    "slotSetControllers\0slotPlaceControllers\0"
    "slotSetSelectTool\0slotSetSelectNoTiesTool\0"
    "slotSetEraseTool\0slotSetNoteRestInserter\0"
    "slotInsertNoteFromAction\0"
    "slotInsertRestFromAction\0slotSwitchToNotes\0"
    "slotSwitchToRests\0slotToggleDot\0"
    "slotAddDot\0slotAddDotNotationOnly\0"
    "slotSetNoteType\0slotSetNoteTypeNotationOnly\0"
    "slotNoteAction\0slotDummy1\0slotNoAccidental\0"
    "slotFollowAccidental\0slotSharp\0slotFlat\0"
    "slotNatural\0slotDoubleSharp\0slotDoubleFlat\0"
    "slotClefAction\0slotText\0slotGuitarChord\0"
    "slotLinearMode\0slotContinuousPageMode\0"
    "slotMultiPageMode\0slotHighlight\0"
    "slotShowHeadersGroup\0slotChangeFontFromAction\0"
    "slotChangeFontSizeFromAction\0"
    "slotChangeSpacingFromAction\0"
    "slotUpdateMenuStates\0slotTransformsNormalizeRests\0"
    "slotTransformsCollapseNotes\0"
    "slotTransformsQuantize\0slotTransformsInterpret\0"
    "slotMakeOrnament\0slotUseOrnament\0"
    "slotRemoveOrnament\0slotEditOrnamentInline\0"
    "slotShowOrnamentExpansion\0slotMaskOrnament\0"
    "slotUnmaskOrnament\0slotUnadoptSegment\0"
    "slotGroupSimpleTuplet\0slotGroupGeneralTuplet\0"
    "slotGroupTuplet\0simple\0"
    "slotUpdateInsertModeStatus\0"
    "slotUpdateInsertModeStatusTriplet\0"
    "slotUpdateInsertModeStatusTuplet\0"
    "slotRescaleHalf\0slotRescaleDouble\0"
    "slotRescale\0slotTransposeUp\0"
    "slotTransposeDown\0slotTransposeUpOctave\0"
    "slotTransposeDownOctave\0slotTranspose\0"
    "slotDiatonicTranspose\0slotInvert\0"
    "slotRetrograde\0slotRetrogradeInvert\0"
    "slotJogLeft\0slotJogRight\0slotEditLyrics\0"
    "slotStepBackward\0slotStepForward\0"
    "slotCurrentStaffUp\0slotCurrentStaffDown\0"
    "slotCurrentSegmentPrior\0slotCurrentSegmentNext\0"
    "slotToggleChordsRuler\0slotToggleRawNoteRuler\0"
    "slotToggleTempoRuler\0slotToggleVelocityRuler\0"
    "slotTogglePitchbendRuler\0"
    "slotToggleKeyPressureRuler\0"
    "slotToggleChannelPressureRuler\0"
    "slotAddControlRuler\0QAction*\0slotAddTempo\0"
    "slotAddTimeSignature\0slotCheckForParallels\0"
    "slotToggleGeneralToolBar\0"
    "slotToggleToolsToolBar\0slotToggleDurationToolBar\0"
    "slotToggleInterpretToolBar\0"
    "slotToggleAccidentalsToolBar\0"
    "slotToggleClefsToolBar\0slotToggleMarksToolBar\0"
    "slotToggleGroupToolBar\0slotToggleSymbolsToolBar\0"
    "slotToggleLayoutToolBar\0slotToggleLayerToolBar\0"
    "slotToggleRulersToolBar\0"
    "slotToggleTransportToolBar\0"
    "slotScrollToFollow\0slotLoop\0slotLoopChanged\0"
    "slotInsertableNoteEventReceived\0noteOn\0"
    "slotToggleStepByStep\0slotDebugDump\0"
    "slotBarDataDump\0slotSymbolAction\0"
    "slotMoveEventsUpStaffInteractive\0"
    "slotMoveEventsDownStaffInteractive\0"
    "slotMoveEventsUpStaff\0slotMoveEventsDownStaff\0"
    "slotHoveredOverNoteChanged\0"
    "slotHoveredOverAbsoluteTimeChanged\0"
    "slotShowContextHelp\0slotFontComboChanged\0"
    "slotSizeComboChanged\0slotSpacingComboChanged\0"
    "slotEditElement\0NotationStaff*\0"
    "NotationElement*\0slotExtendSelectionBackward\0"
    "slotExtendSelectionForward\0"
    "slotExtendSelectionBackwardBar\0"
    "slotExtendSelectionForwardBar\0bar\0"
    "slotHelp\0slotTutorial\0slotBugGuidelines\0"
    "slotHelpAbout\0slotHelpAboutQt\0slotDonate\0"
    "slotCycleSlashes\0slotAddLayer\0"
    "slotNewLayerFromSelection\0slotConfigure\0"
    "slotExpressionSequence\0slotPitchBendSequence\0"
    "slotControllerSequence\0slotCheckShowHeadersMenu\0"
    "checked\0slotInterpretActivate\0"
    "slotRulerSelectionUpdate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__NotationView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     186,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  944,    2, 0x06 /* Public */,
       3,    0,  945,    2, 0x06 /* Public */,
       4,    0,  946,    2, 0x06 /* Public */,
       5,    0,  947,    2, 0x06 /* Public */,
       6,    0,  948,    2, 0x06 /* Public */,
       7,    0,  949,    2, 0x06 /* Public */,
       8,    0,  950,    2, 0x06 /* Public */,
       9,    0,  951,    2, 0x06 /* Public */,
      10,    0,  952,    2, 0x06 /* Public */,
      11,    1,  953,    2, 0x06 /* Public */,
      12,    1,  956,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    2,  959,    2, 0x0a /* Public */,
      16,    2,  964,    2, 0x0a /* Public */,
      17,    1,  969,    2, 0x0a /* Public */,
      18,    0,  972,    2, 0x09 /* Protected */,
      19,    1,  973,    2, 0x09 /* Protected */,
      19,    0,  976,    2, 0x29 /* Protected | MethodCloned */,
      21,    0,  977,    2, 0x09 /* Protected */,
      22,    0,  978,    2, 0x09 /* Protected */,
      23,    0,  979,    2, 0x09 /* Protected */,
      24,    0,  980,    2, 0x09 /* Protected */,
      25,    0,  981,    2, 0x09 /* Protected */,
      26,    0,  982,    2, 0x09 /* Protected */,
      27,    0,  983,    2, 0x09 /* Protected */,
      28,    0,  984,    2, 0x09 /* Protected */,
      29,    0,  985,    2, 0x09 /* Protected */,
      30,    0,  986,    2, 0x09 /* Protected */,
      31,    0,  987,    2, 0x09 /* Protected */,
      32,    0,  988,    2, 0x09 /* Protected */,
      33,    0,  989,    2, 0x09 /* Protected */,
      34,    1,  990,    2, 0x09 /* Protected */,
      36,    0,  993,    2, 0x09 /* Protected */,
      37,    0,  994,    2, 0x09 /* Protected */,
      38,    0,  995,    2, 0x09 /* Protected */,
      39,    0,  996,    2, 0x09 /* Protected */,
      40,    0,  997,    2, 0x09 /* Protected */,
      41,    0,  998,    2, 0x09 /* Protected */,
      42,    0,  999,    2, 0x09 /* Protected */,
      43,    0, 1000,    2, 0x09 /* Protected */,
      44,    0, 1001,    2, 0x09 /* Protected */,
      45,    0, 1002,    2, 0x09 /* Protected */,
      46,    0, 1003,    2, 0x09 /* Protected */,
      47,    0, 1004,    2, 0x09 /* Protected */,
      48,    0, 1005,    2, 0x09 /* Protected */,
      49,    0, 1006,    2, 0x09 /* Protected */,
      50,    0, 1007,    2, 0x09 /* Protected */,
      51,    0, 1008,    2, 0x09 /* Protected */,
      52,    0, 1009,    2, 0x09 /* Protected */,
      53,    0, 1010,    2, 0x09 /* Protected */,
      54,    0, 1011,    2, 0x09 /* Protected */,
      55,    0, 1012,    2, 0x09 /* Protected */,
      56,    0, 1013,    2, 0x09 /* Protected */,
      57,    0, 1014,    2, 0x09 /* Protected */,
      58,    0, 1015,    2, 0x09 /* Protected */,
      59,    0, 1016,    2, 0x09 /* Protected */,
      60,    0, 1017,    2, 0x09 /* Protected */,
      61,    0, 1018,    2, 0x09 /* Protected */,
      62,    0, 1019,    2, 0x09 /* Protected */,
      63,    0, 1020,    2, 0x09 /* Protected */,
      64,    0, 1021,    2, 0x09 /* Protected */,
      65,    0, 1022,    2, 0x09 /* Protected */,
      66,    0, 1023,    2, 0x09 /* Protected */,
      67,    0, 1024,    2, 0x09 /* Protected */,
      68,    0, 1025,    2, 0x09 /* Protected */,
      69,    0, 1026,    2, 0x09 /* Protected */,
      70,    0, 1027,    2, 0x09 /* Protected */,
      71,    0, 1028,    2, 0x09 /* Protected */,
      72,    0, 1029,    2, 0x09 /* Protected */,
      73,    0, 1030,    2, 0x09 /* Protected */,
      74,    0, 1031,    2, 0x09 /* Protected */,
      75,    0, 1032,    2, 0x09 /* Protected */,
      76,    0, 1033,    2, 0x09 /* Protected */,
      77,    0, 1034,    2, 0x09 /* Protected */,
      78,    0, 1035,    2, 0x09 /* Protected */,
      79,    0, 1036,    2, 0x09 /* Protected */,
      80,    0, 1037,    2, 0x09 /* Protected */,
      81,    0, 1038,    2, 0x09 /* Protected */,
      82,    0, 1039,    2, 0x09 /* Protected */,
      83,    0, 1040,    2, 0x09 /* Protected */,
      84,    0, 1041,    2, 0x09 /* Protected */,
      85,    0, 1042,    2, 0x09 /* Protected */,
      86,    0, 1043,    2, 0x09 /* Protected */,
      87,    0, 1044,    2, 0x09 /* Protected */,
      88,    0, 1045,    2, 0x09 /* Protected */,
      89,    0, 1046,    2, 0x09 /* Protected */,
      90,    0, 1047,    2, 0x09 /* Protected */,
      91,    0, 1048,    2, 0x09 /* Protected */,
      92,    0, 1049,    2, 0x09 /* Protected */,
      93,    0, 1050,    2, 0x09 /* Protected */,
      94,    0, 1051,    2, 0x09 /* Protected */,
      95,    0, 1052,    2, 0x09 /* Protected */,
      96,    0, 1053,    2, 0x09 /* Protected */,
      97,    0, 1054,    2, 0x09 /* Protected */,
      98,    0, 1055,    2, 0x09 /* Protected */,
      99,    0, 1056,    2, 0x09 /* Protected */,
     100,    0, 1057,    2, 0x09 /* Protected */,
     101,    1, 1058,    2, 0x09 /* Protected */,
     103,    0, 1061,    2, 0x09 /* Protected */,
     104,    0, 1062,    2, 0x09 /* Protected */,
     105,    0, 1063,    2, 0x09 /* Protected */,
     106,    0, 1064,    2, 0x09 /* Protected */,
     107,    0, 1065,    2, 0x09 /* Protected */,
     108,    0, 1066,    2, 0x09 /* Protected */,
     109,    0, 1067,    2, 0x09 /* Protected */,
     110,    0, 1068,    2, 0x09 /* Protected */,
     111,    0, 1069,    2, 0x09 /* Protected */,
     112,    0, 1070,    2, 0x09 /* Protected */,
     113,    0, 1071,    2, 0x09 /* Protected */,
     114,    0, 1072,    2, 0x09 /* Protected */,
     115,    0, 1073,    2, 0x09 /* Protected */,
     116,    0, 1074,    2, 0x09 /* Protected */,
     117,    0, 1075,    2, 0x09 /* Protected */,
     118,    0, 1076,    2, 0x09 /* Protected */,
     119,    0, 1077,    2, 0x09 /* Protected */,
     120,    0, 1078,    2, 0x09 /* Protected */,
     121,    0, 1079,    2, 0x09 /* Protected */,
     122,    0, 1080,    2, 0x09 /* Protected */,
     123,    0, 1081,    2, 0x09 /* Protected */,
     124,    0, 1082,    2, 0x09 /* Protected */,
     125,    0, 1083,    2, 0x09 /* Protected */,
     126,    0, 1084,    2, 0x09 /* Protected */,
     127,    0, 1085,    2, 0x09 /* Protected */,
     128,    0, 1086,    2, 0x09 /* Protected */,
     129,    0, 1087,    2, 0x09 /* Protected */,
     130,    0, 1088,    2, 0x09 /* Protected */,
     131,    0, 1089,    2, 0x09 /* Protected */,
     132,    0, 1090,    2, 0x09 /* Protected */,
     133,    0, 1091,    2, 0x09 /* Protected */,
     134,    1, 1092,    2, 0x09 /* Protected */,
     136,    0, 1095,    2, 0x09 /* Protected */,
     137,    0, 1096,    2, 0x09 /* Protected */,
     138,    0, 1097,    2, 0x09 /* Protected */,
     139,    0, 1098,    2, 0x09 /* Protected */,
     140,    0, 1099,    2, 0x09 /* Protected */,
     141,    0, 1100,    2, 0x09 /* Protected */,
     142,    0, 1101,    2, 0x09 /* Protected */,
     143,    0, 1102,    2, 0x09 /* Protected */,
     144,    0, 1103,    2, 0x09 /* Protected */,
     145,    0, 1104,    2, 0x09 /* Protected */,
     146,    0, 1105,    2, 0x09 /* Protected */,
     147,    0, 1106,    2, 0x09 /* Protected */,
     148,    0, 1107,    2, 0x09 /* Protected */,
     149,    0, 1108,    2, 0x09 /* Protected */,
     150,    0, 1109,    2, 0x09 /* Protected */,
     151,    0, 1110,    2, 0x09 /* Protected */,
     152,    0, 1111,    2, 0x09 /* Protected */,
     153,    0, 1112,    2, 0x09 /* Protected */,
     154,    0, 1113,    2, 0x09 /* Protected */,
     155,    3, 1114,    2, 0x09 /* Protected */,
     157,    0, 1121,    2, 0x09 /* Protected */,
     158,    0, 1122,    2, 0x09 /* Protected */,
     159,    0, 1123,    2, 0x09 /* Protected */,
     160,    0, 1124,    2, 0x09 /* Protected */,
     161,    0, 1125,    2, 0x09 /* Protected */,
     162,    0, 1126,    2, 0x09 /* Protected */,
     163,    0, 1127,    2, 0x09 /* Protected */,
     164,    0, 1128,    2, 0x09 /* Protected */,
     165,    1, 1129,    2, 0x09 /* Protected */,
     166,    1, 1132,    2, 0x09 /* Protected */,
     167,    1, 1135,    2, 0x09 /* Protected */,
     168,    1, 1138,    2, 0x09 /* Protected */,
     169,    1, 1141,    2, 0x09 /* Protected */,
     170,    1, 1144,    2, 0x09 /* Protected */,
     171,    2, 1147,    2, 0x09 /* Protected */,
     174,    0, 1152,    2, 0x09 /* Protected */,
     175,    0, 1153,    2, 0x09 /* Protected */,
     176,    0, 1154,    2, 0x09 /* Protected */,
     177,    0, 1155,    2, 0x09 /* Protected */,
     174,    1, 1156,    2, 0x09 /* Protected */,
     175,    1, 1159,    2, 0x09 /* Protected */,
     179,    0, 1162,    2, 0x09 /* Protected */,
     180,    0, 1163,    2, 0x09 /* Protected */,
     181,    0, 1164,    2, 0x09 /* Protected */,
     182,    0, 1165,    2, 0x09 /* Protected */,
     183,    0, 1166,    2, 0x09 /* Protected */,
     184,    0, 1167,    2, 0x09 /* Protected */,
     185,    0, 1168,    2, 0x09 /* Protected */,
     186,    0, 1169,    2, 0x09 /* Protected */,
     187,    0, 1170,    2, 0x09 /* Protected */,
     188,    0, 1171,    2, 0x09 /* Protected */,
     189,    0, 1172,    2, 0x09 /* Protected */,
     190,    0, 1173,    2, 0x09 /* Protected */,
     191,    0, 1174,    2, 0x09 /* Protected */,
     192,    1, 1175,    2, 0x09 /* Protected */,
     194,    0, 1178,    2, 0x09 /* Protected */,
     195,    0, 1179,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QObjectStar,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   14,   15,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   14,   15,
    QMetaType::Void, QMetaType::QObjectStar,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  102,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 135,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool,   14,   15,  156,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::UInt,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 172, 0x80000000 | 173,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  178,
    QMetaType::Void, QMetaType::Bool,  178,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  193,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::NotationView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotationView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->play(); break;
        case 1: _t->stop(); break;
        case 2: _t->stepBackward(); break;
        case 3: _t->stepForward(); break;
        case 4: _t->rewindPlayback(); break;
        case 5: _t->fastForwardPlayback(); break;
        case 6: _t->rewindPlaybackToBeginning(); break;
        case 7: _t->fastForwardPlaybackToEnd(); break;
        case 8: _t->panic(); break;
        case 9: _t->editTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->stepByStepTargetRequested((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 11: _t->slotInsertableNoteOnReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 12: _t->slotInsertableNoteOffReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 13: _t->slotStepByStepTargetRequested((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 14: _t->slotRegenerateScene(); break;
        case 15: _t->slotUpdateWindowTitle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->slotUpdateWindowTitle(); break;
        case 17: _t->slotPrintLilyPond(); break;
        case 18: _t->slotPreviewLilyPond(); break;
        case 19: _t->slotEditCut(); break;
        case 20: _t->slotEditCopy(); break;
        case 21: _t->slotEditPaste(); break;
        case 22: _t->slotEditDelete(); break;
        case 23: _t->slotEditCutAndClose(); break;
        case 24: _t->slotEditGeneralPaste(); break;
        case 25: _t->slotEditAddClef(); break;
        case 26: _t->slotEditAddClefLinkOnly(); break;
        case 27: _t->slotEditAddKeySignature(); break;
        case 28: _t->slotEditAddSustainDown(); break;
        case 29: _t->slotEditAddSustainUp(); break;
        case 30: _t->slotEditAddSustain((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->slotEditTranspose(); break;
        case 32: _t->slotEditSwitchPreset(); break;
        case 33: _t->slotPreviewSelection(); break;
        case 34: _t->slotClearLoop(); break;
        case 35: _t->slotClearSelection(); break;
        case 36: _t->slotEscapePressed(); break;
        case 37: _t->slotEditSelectFromStart(); break;
        case 38: _t->slotEditSelectToEnd(); break;
        case 39: _t->slotEditSelectWholeStaff(); break;
        case 40: _t->slotSearchSelect(); break;
        case 41: _t->slotFilterSelection(); break;
        case 42: _t->slotSelectEvenlySpacedNotes(); break;
        case 43: _t->slotVelocityUp(); break;
        case 44: _t->slotVelocityDown(); break;
        case 45: _t->slotSetVelocities(); break;
        case 46: _t->slotSetControllers(); break;
        case 47: _t->slotPlaceControllers(); break;
        case 48: _t->slotSetSelectTool(); break;
        case 49: _t->slotSetSelectNoTiesTool(); break;
        case 50: _t->slotSetEraseTool(); break;
        case 51: _t->slotSetNoteRestInserter(); break;
        case 52: _t->slotInsertNoteFromAction(); break;
        case 53: _t->slotInsertRestFromAction(); break;
        case 54: _t->slotSwitchToNotes(); break;
        case 55: _t->slotSwitchToRests(); break;
        case 56: _t->slotToggleDot(); break;
        case 57: _t->slotAddDot(); break;
        case 58: _t->slotAddDotNotationOnly(); break;
        case 59: _t->slotSetNoteType(); break;
        case 60: _t->slotSetNoteTypeNotationOnly(); break;
        case 61: _t->slotNoteAction(); break;
        case 62: _t->slotDummy1(); break;
        case 63: _t->slotNoAccidental(); break;
        case 64: _t->slotFollowAccidental(); break;
        case 65: _t->slotSharp(); break;
        case 66: _t->slotFlat(); break;
        case 67: _t->slotNatural(); break;
        case 68: _t->slotDoubleSharp(); break;
        case 69: _t->slotDoubleFlat(); break;
        case 70: _t->slotClefAction(); break;
        case 71: _t->slotText(); break;
        case 72: _t->slotGuitarChord(); break;
        case 73: _t->slotLinearMode(); break;
        case 74: _t->slotContinuousPageMode(); break;
        case 75: _t->slotMultiPageMode(); break;
        case 76: _t->slotHighlight(); break;
        case 77: _t->slotShowHeadersGroup(); break;
        case 78: _t->slotChangeFontFromAction(); break;
        case 79: _t->slotChangeFontSizeFromAction(); break;
        case 80: _t->slotChangeSpacingFromAction(); break;
        case 81: _t->slotUpdateMenuStates(); break;
        case 82: _t->slotTransformsNormalizeRests(); break;
        case 83: _t->slotTransformsCollapseNotes(); break;
        case 84: _t->slotTransformsQuantize(); break;
        case 85: _t->slotTransformsInterpret(); break;
        case 86: _t->slotMakeOrnament(); break;
        case 87: _t->slotUseOrnament(); break;
        case 88: _t->slotRemoveOrnament(); break;
        case 89: _t->slotEditOrnamentInline(); break;
        case 90: _t->slotShowOrnamentExpansion(); break;
        case 91: _t->slotMaskOrnament(); break;
        case 92: _t->slotUnmaskOrnament(); break;
        case 93: _t->slotUnadoptSegment(); break;
        case 94: _t->slotGroupSimpleTuplet(); break;
        case 95: _t->slotGroupGeneralTuplet(); break;
        case 96: _t->slotGroupTuplet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 97: _t->slotUpdateInsertModeStatus(); break;
        case 98: _t->slotUpdateInsertModeStatusTriplet(); break;
        case 99: _t->slotUpdateInsertModeStatusTuplet(); break;
        case 100: _t->slotRescaleHalf(); break;
        case 101: _t->slotRescaleDouble(); break;
        case 102: _t->slotRescale(); break;
        case 103: _t->slotTransposeUp(); break;
        case 104: _t->slotTransposeDown(); break;
        case 105: _t->slotTransposeUpOctave(); break;
        case 106: _t->slotTransposeDownOctave(); break;
        case 107: _t->slotTranspose(); break;
        case 108: _t->slotDiatonicTranspose(); break;
        case 109: _t->slotInvert(); break;
        case 110: _t->slotRetrograde(); break;
        case 111: _t->slotRetrogradeInvert(); break;
        case 112: _t->slotJogLeft(); break;
        case 113: _t->slotJogRight(); break;
        case 114: _t->slotEditLyrics(); break;
        case 115: _t->slotStepBackward(); break;
        case 116: _t->slotStepForward(); break;
        case 117: _t->slotCurrentStaffUp(); break;
        case 118: _t->slotCurrentStaffDown(); break;
        case 119: _t->slotCurrentSegmentPrior(); break;
        case 120: _t->slotCurrentSegmentNext(); break;
        case 121: _t->slotToggleChordsRuler(); break;
        case 122: _t->slotToggleRawNoteRuler(); break;
        case 123: _t->slotToggleTempoRuler(); break;
        case 124: _t->slotToggleVelocityRuler(); break;
        case 125: _t->slotTogglePitchbendRuler(); break;
        case 126: _t->slotToggleKeyPressureRuler(); break;
        case 127: _t->slotToggleChannelPressureRuler(); break;
        case 128: _t->slotAddControlRuler((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 129: _t->slotAddTempo(); break;
        case 130: _t->slotAddTimeSignature(); break;
        case 131: _t->slotCheckForParallels(); break;
        case 132: _t->slotToggleGeneralToolBar(); break;
        case 133: _t->slotToggleToolsToolBar(); break;
        case 134: _t->slotToggleDurationToolBar(); break;
        case 135: _t->slotToggleInterpretToolBar(); break;
        case 136: _t->slotToggleAccidentalsToolBar(); break;
        case 137: _t->slotToggleClefsToolBar(); break;
        case 138: _t->slotToggleMarksToolBar(); break;
        case 139: _t->slotToggleGroupToolBar(); break;
        case 140: _t->slotToggleSymbolsToolBar(); break;
        case 141: _t->slotToggleLayoutToolBar(); break;
        case 142: _t->slotToggleLayerToolBar(); break;
        case 143: _t->slotToggleRulersToolBar(); break;
        case 144: _t->slotToggleTransportToolBar(); break;
        case 145: _t->slotScrollToFollow(); break;
        case 146: _t->slotLoop(); break;
        case 147: _t->slotLoopChanged(); break;
        case 148: _t->slotInsertableNoteEventReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 149: _t->slotToggleStepByStep(); break;
        case 150: _t->slotDebugDump(); break;
        case 151: _t->slotBarDataDump(); break;
        case 152: _t->slotSymbolAction(); break;
        case 153: _t->slotMoveEventsUpStaffInteractive(); break;
        case 154: _t->slotMoveEventsDownStaffInteractive(); break;
        case 155: _t->slotMoveEventsUpStaff(); break;
        case 156: _t->slotMoveEventsDownStaff(); break;
        case 157: _t->slotHoveredOverNoteChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 158: _t->slotHoveredOverAbsoluteTimeChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 159: _t->slotShowContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 160: _t->slotFontComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 161: _t->slotSizeComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 162: _t->slotSpacingComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 163: _t->slotEditElement((*reinterpret_cast< NotationStaff*(*)>(_a[1])),(*reinterpret_cast< NotationElement*(*)>(_a[2]))); break;
        case 164: _t->slotExtendSelectionBackward(); break;
        case 165: _t->slotExtendSelectionForward(); break;
        case 166: _t->slotExtendSelectionBackwardBar(); break;
        case 167: _t->slotExtendSelectionForwardBar(); break;
        case 168: _t->slotExtendSelectionBackward((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 169: _t->slotExtendSelectionForward((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 170: _t->slotHelp(); break;
        case 171: _t->slotTutorial(); break;
        case 172: _t->slotBugGuidelines(); break;
        case 173: _t->slotHelpAbout(); break;
        case 174: _t->slotHelpAboutQt(); break;
        case 175: _t->slotDonate(); break;
        case 176: _t->slotCycleSlashes(); break;
        case 177: _t->slotAddLayer(); break;
        case 178: _t->slotNewLayerFromSelection(); break;
        case 179: _t->slotConfigure(); break;
        case 180: _t->slotExpressionSequence(); break;
        case 181: _t->slotPitchBendSequence(); break;
        case 182: _t->slotControllerSequence(); break;
        case 183: _t->slotCheckShowHeadersMenu((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 184: _t->slotInterpretActivate(); break;
        case 185: _t->slotRulerSelectionUpdate(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 128:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::play)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::stop)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::stepBackward)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::stepForward)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::rewindPlayback)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::fastForwardPlayback)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::rewindPlaybackToBeginning)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::fastForwardPlaybackToEnd)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (NotationView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::panic)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (NotationView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::editTriggerSegment)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (NotationView::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationView::stepByStepTargetRequested)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::NotationView::staticMetaObject = { {
    QMetaObject::SuperData::link<EditViewBase::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__NotationView.data,
    qt_meta_data_Rosegarden__NotationView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::NotationView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::NotationView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__NotationView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return EditViewBase::qt_metacast(_clname);
}

int Rosegarden::NotationView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EditViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 186)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 186;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 186)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 186;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::NotationView::play()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::NotationView::stop()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::NotationView::stepBackward()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::NotationView::stepForward()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::NotationView::rewindPlayback()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::NotationView::fastForwardPlayback()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::NotationView::rewindPlaybackToBeginning()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::NotationView::fastForwardPlaybackToEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Rosegarden::NotationView::panic()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Rosegarden::NotationView::editTriggerSegment(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void Rosegarden::NotationView::stepByStepTargetRequested(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
