/****************************************************************************
** Meta object code from reading C++ file 'TrackParameterBox.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/parameters/TrackParameterBox.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TrackParameterBox.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TrackParameterBox_t {
    QByteArrayData data[27];
    char stringdata0[500];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TrackParameterBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TrackParameterBox_t qt_meta_stringdata_Rosegarden__TrackParameterBox = {
    {
QT_MOC_LITERAL(0, 0, 29), // "Rosegarden::TrackParameterBox"
QT_MOC_LITERAL(1, 30, 18), // "instrumentSelected"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 7), // "TrackId"
QT_MOC_LITERAL(4, 58, 21), // "slotDocColoursChanged"
QT_MOC_LITERAL(5, 80, 14), // "devicesChanged"
QT_MOC_LITERAL(6, 95, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(7, 114, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(8, 134, 20), // "slotDocumentModified"
QT_MOC_LITERAL(9, 155, 25), // "slotPlaybackDeviceChanged"
QT_MOC_LITERAL(10, 181, 5), // "index"
QT_MOC_LITERAL(11, 187, 21), // "slotInstrumentChanged"
QT_MOC_LITERAL(12, 209, 18), // "slotArchiveChanged"
QT_MOC_LITERAL(13, 228, 7), // "checked"
QT_MOC_LITERAL(14, 236, 26), // "slotRecordingDeviceChanged"
QT_MOC_LITERAL(15, 263, 27), // "slotRecordingChannelChanged"
QT_MOC_LITERAL(16, 291, 22), // "slotThruRoutingChanged"
QT_MOC_LITERAL(17, 314, 23), // "slotNotationSizeChanged"
QT_MOC_LITERAL(18, 338, 22), // "slotBracketTypeChanged"
QT_MOC_LITERAL(19, 361, 15), // "slotLoadPressed"
QT_MOC_LITERAL(20, 377, 15), // "slotClefChanged"
QT_MOC_LITERAL(21, 393, 4), // "clef"
QT_MOC_LITERAL(22, 398, 20), // "slotTransposeChanged"
QT_MOC_LITERAL(23, 419, 17), // "slotLowestPressed"
QT_MOC_LITERAL(24, 437, 18), // "slotHighestPressed"
QT_MOC_LITERAL(25, 456, 16), // "slotColorChanged"
QT_MOC_LITERAL(26, 473, 26) // "slotResetToDefaultsPressed"

    },
    "Rosegarden::TrackParameterBox\0"
    "instrumentSelected\0\0TrackId\0"
    "slotDocColoursChanged\0devicesChanged\0"
    "slotDocumentLoaded\0RosegardenDocument*\0"
    "slotDocumentModified\0slotPlaybackDeviceChanged\0"
    "index\0slotInstrumentChanged\0"
    "slotArchiveChanged\0checked\0"
    "slotRecordingDeviceChanged\0"
    "slotRecordingChannelChanged\0"
    "slotThruRoutingChanged\0slotNotationSizeChanged\0"
    "slotBracketTypeChanged\0slotLoadPressed\0"
    "slotClefChanged\0clef\0slotTransposeChanged\0"
    "slotLowestPressed\0slotHighestPressed\0"
    "slotColorChanged\0slotResetToDefaultsPressed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TrackParameterBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  114,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  119,    2, 0x0a /* Public */,
       5,    0,  120,    2, 0x0a /* Public */,
       6,    1,  121,    2, 0x08 /* Private */,
       8,    1,  124,    2, 0x08 /* Private */,
       9,    1,  127,    2, 0x08 /* Private */,
      11,    1,  130,    2, 0x08 /* Private */,
      12,    1,  133,    2, 0x08 /* Private */,
      14,    1,  136,    2, 0x08 /* Private */,
      15,    1,  139,    2, 0x08 /* Private */,
      16,    1,  142,    2, 0x08 /* Private */,
      17,    1,  145,    2, 0x08 /* Private */,
      18,    1,  148,    2, 0x08 /* Private */,
      19,    0,  151,    2, 0x08 /* Private */,
      20,    1,  152,    2, 0x08 /* Private */,
      22,    1,  155,    2, 0x08 /* Private */,
      23,    0,  158,    2, 0x08 /* Private */,
      24,    0,  159,    2, 0x08 /* Private */,
      25,    1,  160,    2, 0x08 /* Private */,
      26,    0,  163,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TrackParameterBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TrackParameterBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->instrumentSelected((*reinterpret_cast< TrackId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->slotDocColoursChanged(); break;
        case 2: _t->devicesChanged(); break;
        case 3: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 4: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotPlaybackDeviceChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotInstrumentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotArchiveChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotRecordingDeviceChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotRecordingChannelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->slotThruRoutingChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->slotNotationSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slotBracketTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slotLoadPressed(); break;
        case 14: _t->slotClefChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->slotTransposeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slotLowestPressed(); break;
        case 17: _t->slotHighestPressed(); break;
        case 18: _t->slotColorChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->slotResetToDefaultsPressed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TrackParameterBox::*)(TrackId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TrackParameterBox::instrumentSelected)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TrackParameterBox::staticMetaObject = { {
    QMetaObject::SuperData::link<RosegardenParameterBox::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TrackParameterBox.data,
    qt_meta_data_Rosegarden__TrackParameterBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TrackParameterBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TrackParameterBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TrackParameterBox.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return RosegardenParameterBox::qt_metacast(_clname);
}

int Rosegarden::TrackParameterBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RosegardenParameterBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TrackParameterBox::instrumentSelected(TrackId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
