/****************************************************************************
** Meta object code from reading C++ file 'TransportDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/TransportDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TransportDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TransportDialog_t {
    QByteArrayData data[38];
    char stringdata0[654];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TransportDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TransportDialog_t qt_meta_stringdata_Rosegarden__TransportDialog = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::TransportDialog"
QT_MOC_LITERAL(1, 28, 6), // "closed"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 17), // "editTimeSignature"
QT_MOC_LITERAL(4, 54, 8), // "QWidget*"
QT_MOC_LITERAL(5, 63, 17), // "editTransportTime"
QT_MOC_LITERAL(6, 81, 5), // "panic"
QT_MOC_LITERAL(7, 87, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(8, 106, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(9, 126, 3), // "doc"
QT_MOC_LITERAL(10, 130, 20), // "slotClearMidiInLabel"
QT_MOC_LITERAL(11, 151, 21), // "slotClearMidiOutLabel"
QT_MOC_LITERAL(12, 173, 21), // "slotChangeTimeDisplay"
QT_MOC_LITERAL(13, 195, 15), // "slotChangeToEnd"
QT_MOC_LITERAL(14, 211, 26), // "slotPanelOpenButtonClicked"
QT_MOC_LITERAL(15, 238, 27), // "slotPanelCloseButtonClicked"
QT_MOC_LITERAL(16, 266, 13), // "slotEditTempo"
QT_MOC_LITERAL(17, 280, 21), // "slotEditTimeSignature"
QT_MOC_LITERAL(18, 302, 12), // "slotEditTime"
QT_MOC_LITERAL(19, 315, 16), // "slotTempoChanged"
QT_MOC_LITERAL(20, 332, 6), // "tempoT"
QT_MOC_LITERAL(21, 339, 15), // "slotMidiInLabel"
QT_MOC_LITERAL(22, 355, 18), // "const MappedEvent*"
QT_MOC_LITERAL(23, 374, 2), // "mE"
QT_MOC_LITERAL(24, 377, 16), // "slotMidiOutLabel"
QT_MOC_LITERAL(25, 394, 11), // "slotPlaying"
QT_MOC_LITERAL(26, 406, 7), // "checked"
QT_MOC_LITERAL(27, 414, 13), // "slotRecording"
QT_MOC_LITERAL(28, 428, 22), // "slotMetronomeActivated"
QT_MOC_LITERAL(29, 451, 17), // "computeSampleRate"
QT_MOC_LITERAL(30, 469, 17), // "cycleThroughModes"
QT_MOC_LITERAL(31, 487, 11), // "displayTime"
QT_MOC_LITERAL(32, 499, 21), // "slotLoopButtonClicked"
QT_MOC_LITERAL(33, 521, 35), // "slotSetStartLoopingPointAtMar..."
QT_MOC_LITERAL(34, 557, 34), // "slotSetStopLoopingPointAtMark..."
QT_MOC_LITERAL(35, 592, 15), // "slotLoopChanged"
QT_MOC_LITERAL(36, 608, 18), // "slotMetronomeTimer"
QT_MOC_LITERAL(37, 627, 26) // "slotMetronomeButtonClicked"

    },
    "Rosegarden::TransportDialog\0closed\0\0"
    "editTimeSignature\0QWidget*\0editTransportTime\0"
    "panic\0slotDocumentLoaded\0RosegardenDocument*\0"
    "doc\0slotClearMidiInLabel\0slotClearMidiOutLabel\0"
    "slotChangeTimeDisplay\0slotChangeToEnd\0"
    "slotPanelOpenButtonClicked\0"
    "slotPanelCloseButtonClicked\0slotEditTempo\0"
    "slotEditTimeSignature\0slotEditTime\0"
    "slotTempoChanged\0tempoT\0slotMidiInLabel\0"
    "const MappedEvent*\0mE\0slotMidiOutLabel\0"
    "slotPlaying\0checked\0slotRecording\0"
    "slotMetronomeActivated\0computeSampleRate\0"
    "cycleThroughModes\0displayTime\0"
    "slotLoopButtonClicked\0"
    "slotSetStartLoopingPointAtMarkerPos\0"
    "slotSetStopLoopingPointAtMarkerPos\0"
    "slotLoopChanged\0slotMetronomeTimer\0"
    "slotMetronomeButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TransportDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      29,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  159,    2, 0x06 /* Public */,
       3,    1,  160,    2, 0x06 /* Public */,
       5,    1,  163,    2, 0x06 /* Public */,
       6,    0,  166,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  167,    2, 0x0a /* Public */,
      10,    0,  170,    2, 0x0a /* Public */,
      11,    0,  171,    2, 0x0a /* Public */,
      12,    0,  172,    2, 0x0a /* Public */,
      13,    0,  173,    2, 0x0a /* Public */,
      14,    0,  174,    2, 0x0a /* Public */,
      15,    0,  175,    2, 0x0a /* Public */,
      16,    0,  176,    2, 0x0a /* Public */,
      17,    0,  177,    2, 0x0a /* Public */,
      18,    0,  178,    2, 0x0a /* Public */,
      19,    1,  179,    2, 0x0a /* Public */,
      21,    1,  182,    2, 0x0a /* Public */,
      24,    1,  185,    2, 0x0a /* Public */,
      25,    1,  188,    2, 0x0a /* Public */,
      27,    1,  191,    2, 0x0a /* Public */,
      28,    1,  194,    2, 0x0a /* Public */,
      29,    0,  197,    2, 0x08 /* Private */,
      30,    0,  198,    2, 0x08 /* Private */,
      31,    0,  199,    2, 0x08 /* Private */,
      32,    0,  200,    2, 0x08 /* Private */,
      33,    0,  201,    2, 0x08 /* Private */,
      34,    0,  202,    2, 0x08 /* Private */,
      35,    0,  203,    2, 0x08 /* Private */,
      36,    0,  204,    2, 0x08 /* Private */,
      37,    0,  205,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 20,    2,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TransportDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TransportDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closed(); break;
        case 1: _t->editTimeSignature((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 2: _t->editTransportTime((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 3: _t->panic(); break;
        case 4: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 5: _t->slotClearMidiInLabel(); break;
        case 6: _t->slotClearMidiOutLabel(); break;
        case 7: _t->slotChangeTimeDisplay(); break;
        case 8: _t->slotChangeToEnd(); break;
        case 9: _t->slotPanelOpenButtonClicked(); break;
        case 10: _t->slotPanelCloseButtonClicked(); break;
        case 11: _t->slotEditTempo(); break;
        case 12: _t->slotEditTimeSignature(); break;
        case 13: _t->slotEditTime(); break;
        case 14: _t->slotTempoChanged((*reinterpret_cast< tempoT(*)>(_a[1]))); break;
        case 15: _t->slotMidiInLabel((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 16: _t->slotMidiOutLabel((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 17: _t->slotPlaying((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->slotRecording((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->slotMetronomeActivated((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->computeSampleRate(); break;
        case 21: _t->cycleThroughModes(); break;
        case 22: _t->displayTime(); break;
        case 23: _t->slotLoopButtonClicked(); break;
        case 24: _t->slotSetStartLoopingPointAtMarkerPos(); break;
        case 25: _t->slotSetStopLoopingPointAtMarkerPos(); break;
        case 26: _t->slotLoopChanged(); break;
        case 27: _t->slotMetronomeTimer(); break;
        case 28: _t->slotMetronomeButtonClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TransportDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransportDialog::closed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TransportDialog::*)(QWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransportDialog::editTimeSignature)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TransportDialog::*)(QWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransportDialog::editTransportTime)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TransportDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransportDialog::panic)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TransportDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TransportDialog.data,
    qt_meta_data_Rosegarden__TransportDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TransportDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TransportDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TransportDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::TransportDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TransportDialog::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::TransportDialog::editTimeSignature(QWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::TransportDialog::editTransportTime(QWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::TransportDialog::panic()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
