/****************************************************************************
** Meta object code from reading C++ file 'PitchTrackerView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/pitchtracker/PitchTrackerView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PitchTrackerView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__PitchTrackerView_t {
    QByteArrayData data[11];
    char stringdata0[170];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__PitchTrackerView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__PitchTrackerView_t qt_meta_stringdata_Rosegarden__PitchTrackerView = {
    {
QT_MOC_LITERAL(0, 0, 28), // "Rosegarden::PitchTrackerView"
QT_MOC_LITERAL(1, 29, 16), // "slotUpdateValues"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 5), // "timeT"
QT_MOC_LITERAL(4, 53, 4), // "time"
QT_MOC_LITERAL(5, 58, 16), // "slotStartTracker"
QT_MOC_LITERAL(6, 75, 15), // "slotStopTracker"
QT_MOC_LITERAL(7, 91, 16), // "slotPlaybackJump"
QT_MOC_LITERAL(8, 108, 23), // "slotNewTuningFromAction"
QT_MOC_LITERAL(9, 132, 8), // "QAction*"
QT_MOC_LITERAL(10, 141, 28) // "slotNewPitchEstimationMethod"

    },
    "Rosegarden::PitchTrackerView\0"
    "slotUpdateValues\0\0timeT\0time\0"
    "slotStartTracker\0slotStopTracker\0"
    "slotPlaybackJump\0slotNewTuningFromAction\0"
    "QAction*\0slotNewPitchEstimationMethod"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__PitchTrackerView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x09 /* Protected */,
       5,    0,   47,    2, 0x09 /* Protected */,
       6,    0,   48,    2, 0x09 /* Protected */,
       7,    0,   49,    2, 0x09 /* Protected */,
       8,    1,   50,    2, 0x09 /* Protected */,
      10,    1,   53,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 9,    2,

       0        // eod
};

void Rosegarden::PitchTrackerView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PitchTrackerView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotUpdateValues((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 1: _t->slotStartTracker(); break;
        case 2: _t->slotStopTracker(); break;
        case 3: _t->slotPlaybackJump(); break;
        case 4: _t->slotNewTuningFromAction((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 5: _t->slotNewPitchEstimationMethod((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::PitchTrackerView::staticMetaObject = { {
    QMetaObject::SuperData::link<NotationView::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__PitchTrackerView.data,
    qt_meta_data_Rosegarden__PitchTrackerView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::PitchTrackerView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::PitchTrackerView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__PitchTrackerView.stringdata0))
        return static_cast<void*>(this);
    return NotationView::qt_metacast(_clname);
}

int Rosegarden::PitchTrackerView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NotationView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
