/****************************************************************************
** Meta object code from reading C++ file 'DeviceManagerDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/studio/DeviceManagerDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DeviceManagerDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__DeviceManagerDialog_t {
    QByteArrayData data[29];
    char stringdata0[534];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__DeviceManagerDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__DeviceManagerDialog_t qt_meta_stringdata_Rosegarden__DeviceManagerDialog = {
    {
QT_MOC_LITERAL(0, 0, 31), // "Rosegarden::DeviceManagerDialog"
QT_MOC_LITERAL(1, 32, 9), // "editBanks"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 8), // "DeviceId"
QT_MOC_LITERAL(4, 52, 15), // "editControllers"
QT_MOC_LITERAL(5, 68, 17), // "deviceNameChanged"
QT_MOC_LITERAL(6, 86, 18), // "deviceNamesChanged"
QT_MOC_LITERAL(7, 105, 21), // "slotOutputPortClicked"
QT_MOC_LITERAL(8, 127, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(9, 144, 6), // "twItem"
QT_MOC_LITERAL(10, 151, 6), // "column"
QT_MOC_LITERAL(11, 158, 26), // "slotPlaybackDeviceSelected"
QT_MOC_LITERAL(12, 185, 20), // "slotInputPortClicked"
QT_MOC_LITERAL(13, 206, 24), // "slotRecordDeviceSelected"
QT_MOC_LITERAL(14, 231, 21), // "slotDeviceItemChanged"
QT_MOC_LITERAL(15, 253, 8), // "slotEdit"
QT_MOC_LITERAL(16, 262, 4), // "item"
QT_MOC_LITERAL(17, 267, 22), // "slotRefreshOutputPorts"
QT_MOC_LITERAL(18, 290, 21), // "slotRefreshInputPorts"
QT_MOC_LITERAL(19, 312, 21), // "slotAddPlaybackDevice"
QT_MOC_LITERAL(20, 334, 19), // "slotAddRecordDevice"
QT_MOC_LITERAL(21, 354, 24), // "slotDeletePlaybackDevice"
QT_MOC_LITERAL(22, 379, 22), // "slotDeleteRecordDevice"
QT_MOC_LITERAL(23, 402, 31), // "slotManageBanksOfPlaybackDevice"
QT_MOC_LITERAL(24, 434, 29), // "slotEditControllerDefinitions"
QT_MOC_LITERAL(25, 464, 4), // "show"
QT_MOC_LITERAL(26, 469, 17), // "slotHelpRequested"
QT_MOC_LITERAL(27, 487, 25), // "slotResyncDevicesReceived"
QT_MOC_LITERAL(28, 513, 20) // "slotCloseButtonPress"

    },
    "Rosegarden::DeviceManagerDialog\0"
    "editBanks\0\0DeviceId\0editControllers\0"
    "deviceNameChanged\0deviceNamesChanged\0"
    "slotOutputPortClicked\0QTreeWidgetItem*\0"
    "twItem\0column\0slotPlaybackDeviceSelected\0"
    "slotInputPortClicked\0slotRecordDeviceSelected\0"
    "slotDeviceItemChanged\0slotEdit\0item\0"
    "slotRefreshOutputPorts\0slotRefreshInputPorts\0"
    "slotAddPlaybackDevice\0slotAddRecordDevice\0"
    "slotDeletePlaybackDevice\0"
    "slotDeleteRecordDevice\0"
    "slotManageBanksOfPlaybackDevice\0"
    "slotEditControllerDefinitions\0show\0"
    "slotHelpRequested\0slotResyncDevicesReceived\0"
    "slotCloseButtonPress"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__DeviceManagerDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  124,    2, 0x06 /* Public */,
       4,    1,  127,    2, 0x06 /* Public */,
       5,    1,  130,    2, 0x06 /* Public */,
       6,    0,  133,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,  134,    2, 0x0a /* Public */,
      11,    0,  139,    2, 0x0a /* Public */,
      12,    2,  140,    2, 0x0a /* Public */,
      13,    0,  145,    2, 0x0a /* Public */,
      14,    2,  146,    2, 0x0a /* Public */,
      15,    2,  151,    2, 0x0a /* Public */,
      17,    0,  156,    2, 0x0a /* Public */,
      18,    0,  157,    2, 0x0a /* Public */,
      19,    0,  158,    2, 0x0a /* Public */,
      20,    0,  159,    2, 0x0a /* Public */,
      21,    0,  160,    2, 0x0a /* Public */,
      22,    0,  161,    2, 0x0a /* Public */,
      23,    0,  162,    2, 0x0a /* Public */,
      24,    0,  163,    2, 0x0a /* Public */,
      25,    0,  164,    2, 0x0a /* Public */,
      26,    0,  165,    2, 0x0a /* Public */,
      27,    0,  166,    2, 0x0a /* Public */,
      28,    0,  167,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int,    9,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int,    9,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int,    9,   10,
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int,   16,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::DeviceManagerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DeviceManagerDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editBanks((*reinterpret_cast< DeviceId(*)>(_a[1]))); break;
        case 1: _t->editControllers((*reinterpret_cast< DeviceId(*)>(_a[1]))); break;
        case 2: _t->deviceNameChanged((*reinterpret_cast< DeviceId(*)>(_a[1]))); break;
        case 3: _t->deviceNamesChanged(); break;
        case 4: _t->slotOutputPortClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->slotPlaybackDeviceSelected(); break;
        case 6: _t->slotInputPortClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->slotRecordDeviceSelected(); break;
        case 8: _t->slotDeviceItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->slotEdit((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->slotRefreshOutputPorts(); break;
        case 11: _t->slotRefreshInputPorts(); break;
        case 12: _t->slotAddPlaybackDevice(); break;
        case 13: _t->slotAddRecordDevice(); break;
        case 14: _t->slotDeletePlaybackDevice(); break;
        case 15: _t->slotDeleteRecordDevice(); break;
        case 16: _t->slotManageBanksOfPlaybackDevice(); break;
        case 17: _t->slotEditControllerDefinitions(); break;
        case 18: _t->show(); break;
        case 19: _t->slotHelpRequested(); break;
        case 20: _t->slotResyncDevicesReceived(); break;
        case 21: _t->slotCloseButtonPress(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DeviceManagerDialog::*)(DeviceId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDialog::editBanks)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDialog::*)(DeviceId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDialog::editControllers)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDialog::*)(DeviceId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDialog::deviceNameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DeviceManagerDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DeviceManagerDialog::deviceNamesChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::DeviceManagerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__DeviceManagerDialog.data,
    qt_meta_data_Rosegarden__DeviceManagerDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::DeviceManagerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::DeviceManagerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__DeviceManagerDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DeviceManagerDialogUi"))
        return static_cast< Ui::DeviceManagerDialogUi*>(this);
    if (!strcmp(_clname, "StudioObserver"))
        return static_cast< StudioObserver*>(this);
    if (!strcmp(_clname, "DeviceObserver"))
        return static_cast< DeviceObserver*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::DeviceManagerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::DeviceManagerDialog::editBanks(DeviceId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::DeviceManagerDialog::editControllers(DeviceId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::DeviceManagerDialog::deviceNameChanged(DeviceId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::DeviceManagerDialog::deviceNamesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
