/****************************************************************************
** Meta object code from reading C++ file 'RosegardenDocument.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/document/RosegardenDocument.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosegardenDocument.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__RosegardenDocument_t {
    QByteArrayData data[23];
    char stringdata0[407];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__RosegardenDocument_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__RosegardenDocument_t qt_meta_stringdata_Rosegarden__RosegardenDocument = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Rosegarden::RosegardenDocument"
QT_MOC_LITERAL(1, 31, 16), // "documentModified"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 24), // "recordMIDISegmentUpdated"
QT_MOC_LITERAL(4, 74, 8), // "Segment*"
QT_MOC_LITERAL(5, 83, 13), // "recordSegment"
QT_MOC_LITERAL(6, 97, 5), // "timeT"
QT_MOC_LITERAL(7, 103, 11), // "updatedFrom"
QT_MOC_LITERAL(8, 115, 23), // "newMIDIRecordingSegment"
QT_MOC_LITERAL(9, 139, 24), // "newAudioRecordingSegment"
QT_MOC_LITERAL(10, 164, 21), // "stoppedAudioRecording"
QT_MOC_LITERAL(11, 186, 20), // "stoppedMIDIRecording"
QT_MOC_LITERAL(12, 207, 18), // "audioFileFinalized"
QT_MOC_LITERAL(13, 226, 19), // "playPositionChanged"
QT_MOC_LITERAL(14, 246, 11), // "loopChanged"
QT_MOC_LITERAL(15, 258, 17), // "docColoursChanged"
QT_MOC_LITERAL(16, 276, 14), // "devicesResyncd"
QT_MOC_LITERAL(17, 291, 18), // "slotUpdateAllViews"
QT_MOC_LITERAL(18, 310, 25), // "RosegardenMainViewWidget*"
QT_MOC_LITERAL(19, 336, 6), // "sender"
QT_MOC_LITERAL(20, 343, 20), // "slotDocumentModified"
QT_MOC_LITERAL(21, 364, 20), // "slotDocumentRestored"
QT_MOC_LITERAL(22, 385, 21) // "slotDocColoursChanged"

    },
    "Rosegarden::RosegardenDocument\0"
    "documentModified\0\0recordMIDISegmentUpdated\0"
    "Segment*\0recordSegment\0timeT\0updatedFrom\0"
    "newMIDIRecordingSegment\0"
    "newAudioRecordingSegment\0stoppedAudioRecording\0"
    "stoppedMIDIRecording\0audioFileFinalized\0"
    "playPositionChanged\0loopChanged\0"
    "docColoursChanged\0devicesResyncd\0"
    "slotUpdateAllViews\0RosegardenMainViewWidget*\0"
    "sender\0slotDocumentModified\0"
    "slotDocumentRestored\0slotDocColoursChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__RosegardenDocument[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       3,    2,   92,    2, 0x06 /* Public */,
       8,    1,   97,    2, 0x06 /* Public */,
       9,    1,  100,    2, 0x06 /* Public */,
      10,    0,  103,    2, 0x06 /* Public */,
      11,    0,  104,    2, 0x06 /* Public */,
      12,    1,  105,    2, 0x06 /* Public */,
      13,    1,  108,    2, 0x06 /* Public */,
      14,    0,  111,    2, 0x06 /* Public */,
      15,    0,  112,    2, 0x06 /* Public */,
      16,    0,  113,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,  114,    2, 0x0a /* Public */,
      20,    0,  117,    2, 0x0a /* Public */,
      21,    0,  118,    2, 0x0a /* Public */,
      22,    0,  119,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 6,    5,    7,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::RosegardenDocument::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RosegardenDocument *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->documentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->recordMIDISegmentUpdated((*reinterpret_cast< Segment*(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 2: _t->newMIDIRecordingSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 3: _t->newAudioRecordingSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 4: _t->stoppedAudioRecording(); break;
        case 5: _t->stoppedMIDIRecording(); break;
        case 6: _t->audioFileFinalized((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 7: _t->playPositionChanged((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 8: _t->loopChanged(); break;
        case 9: _t->docColoursChanged(); break;
        case 10: _t->devicesResyncd(); break;
        case 11: _t->slotUpdateAllViews((*reinterpret_cast< RosegardenMainViewWidget*(*)>(_a[1]))); break;
        case 12: _t->slotDocumentModified(); break;
        case 13: _t->slotDocumentRestored(); break;
        case 14: _t->slotDocColoursChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RosegardenDocument::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::documentModified)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)(Segment * , timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::recordMIDISegmentUpdated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::newMIDIRecordingSegment)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::newAudioRecordingSegment)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::stoppedAudioRecording)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::stoppedMIDIRecording)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::audioFileFinalized)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)(timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::playPositionChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::loopChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::docColoursChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (RosegardenDocument::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenDocument::devicesResyncd)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::RosegardenDocument::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__RosegardenDocument.data,
    qt_meta_data_Rosegarden__RosegardenDocument,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::RosegardenDocument::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::RosegardenDocument::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__RosegardenDocument.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::RosegardenDocument::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::RosegardenDocument::documentModified(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::RosegardenDocument::recordMIDISegmentUpdated(Segment * _t1, timeT _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::RosegardenDocument::newMIDIRecordingSegment(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::RosegardenDocument::newAudioRecordingSegment(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::RosegardenDocument::stoppedAudioRecording()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::RosegardenDocument::stoppedMIDIRecording()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::RosegardenDocument::audioFileFinalized(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::RosegardenDocument::playPositionChanged(timeT _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::RosegardenDocument::loopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Rosegarden::RosegardenDocument::docColoursChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Rosegarden::RosegardenDocument::devicesResyncd()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
