/****************************************************************************
** Meta object code from reading C++ file 'CompositionView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/compositionview/CompositionView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CompositionView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__CompositionView_t {
    QByteArrayData data[27];
    char stringdata0[438];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__CompositionView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__CompositionView_t qt_meta_stringdata_Rosegarden__CompositionView = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::CompositionView"
QT_MOC_LITERAL(1, 28, 11), // "editSegment"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 8), // "Segment*"
QT_MOC_LITERAL(4, 50, 10), // "editRepeat"
QT_MOC_LITERAL(5, 61, 5), // "timeT"
QT_MOC_LITERAL(6, 67, 15), // "showContextHelp"
QT_MOC_LITERAL(7, 83, 22), // "slotExternalWheelEvent"
QT_MOC_LITERAL(8, 106, 12), // "QWheelEvent*"
QT_MOC_LITERAL(9, 119, 25), // "slotExternalKeyPressEvent"
QT_MOC_LITERAL(10, 145, 10), // "QKeyEvent*"
QT_MOC_LITERAL(11, 156, 27), // "slotExternalKeyReleaseEvent"
QT_MOC_LITERAL(12, 184, 13), // "slotUpdateAll"
QT_MOC_LITERAL(13, 198, 18), // "slotAllNeedRefresh"
QT_MOC_LITERAL(14, 217, 4), // "rect"
QT_MOC_LITERAL(15, 222, 14), // "slotUpdateSize"
QT_MOC_LITERAL(16, 237, 19), // "slotUpdateArtifacts"
QT_MOC_LITERAL(17, 257, 15), // "slotUpdateTimer"
QT_MOC_LITERAL(18, 273, 22), // "slotRefreshColourCache"
QT_MOC_LITERAL(19, 296, 19), // "slotToolHelpChanged"
QT_MOC_LITERAL(20, 316, 27), // "slotNewMIDIRecordingSegment"
QT_MOC_LITERAL(21, 344, 28), // "slotNewAudioRecordingSegment"
QT_MOC_LITERAL(22, 373, 20), // "slotStoppedRecording"
QT_MOC_LITERAL(23, 394, 17), // "slotControlChange"
QT_MOC_LITERAL(24, 412, 11), // "Instrument*"
QT_MOC_LITERAL(25, 424, 10), // "instrument"
QT_MOC_LITERAL(26, 435, 2) // "cc"

    },
    "Rosegarden::CompositionView\0editSegment\0"
    "\0Segment*\0editRepeat\0timeT\0showContextHelp\0"
    "slotExternalWheelEvent\0QWheelEvent*\0"
    "slotExternalKeyPressEvent\0QKeyEvent*\0"
    "slotExternalKeyReleaseEvent\0slotUpdateAll\0"
    "slotAllNeedRefresh\0rect\0slotUpdateSize\0"
    "slotUpdateArtifacts\0slotUpdateTimer\0"
    "slotRefreshColourCache\0slotToolHelpChanged\0"
    "slotNewMIDIRecordingSegment\0"
    "slotNewAudioRecordingSegment\0"
    "slotStoppedRecording\0slotControlChange\0"
    "Instrument*\0instrument\0cc"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__CompositionView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x06 /* Public */,
       4,    2,  102,    2, 0x06 /* Public */,
       6,    1,  107,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  110,    2, 0x0a /* Public */,
       9,    1,  113,    2, 0x0a /* Public */,
      11,    1,  116,    2, 0x0a /* Public */,
      12,    0,  119,    2, 0x0a /* Public */,
      13,    1,  120,    2, 0x0a /* Public */,
      15,    0,  123,    2, 0x0a /* Public */,
      16,    0,  124,    2, 0x08 /* Private */,
      17,    0,  125,    2, 0x08 /* Private */,
      18,    0,  126,    2, 0x08 /* Private */,
      19,    1,  127,    2, 0x08 /* Private */,
      20,    1,  130,    2, 0x08 /* Private */,
      21,    1,  133,    2, 0x08 /* Private */,
      22,    0,  136,    2, 0x08 /* Private */,
      23,    2,  137,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRect,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24, QMetaType::Int,   25,   26,

       0        // eod
};

void Rosegarden::CompositionView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CompositionView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 1: _t->editRepeat((*reinterpret_cast< Segment*(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 2: _t->showContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->slotExternalWheelEvent((*reinterpret_cast< QWheelEvent*(*)>(_a[1]))); break;
        case 4: _t->slotExternalKeyPressEvent((*reinterpret_cast< QKeyEvent*(*)>(_a[1]))); break;
        case 5: _t->slotExternalKeyReleaseEvent((*reinterpret_cast< QKeyEvent*(*)>(_a[1]))); break;
        case 6: _t->slotUpdateAll(); break;
        case 7: _t->slotAllNeedRefresh((*reinterpret_cast< const QRect(*)>(_a[1]))); break;
        case 8: _t->slotUpdateSize(); break;
        case 9: _t->slotUpdateArtifacts(); break;
        case 10: _t->slotUpdateTimer(); break;
        case 11: _t->slotRefreshColourCache(); break;
        case 12: _t->slotToolHelpChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->slotNewMIDIRecordingSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 14: _t->slotNewAudioRecordingSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 15: _t->slotStoppedRecording(); break;
        case 16: _t->slotControlChange((*reinterpret_cast< Instrument*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Instrument* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CompositionView::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionView::editSegment)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CompositionView::*)(Segment * , timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionView::editRepeat)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CompositionView::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CompositionView::showContextHelp)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::CompositionView::staticMetaObject = { {
    QMetaObject::SuperData::link<RosegardenScrollView::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__CompositionView.data,
    qt_meta_data_Rosegarden__CompositionView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::CompositionView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::CompositionView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__CompositionView.stringdata0))
        return static_cast<void*>(this);
    return RosegardenScrollView::qt_metacast(_clname);
}

int Rosegarden::CompositionView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RosegardenScrollView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::CompositionView::editSegment(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::CompositionView::editRepeat(Segment * _t1, timeT _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::CompositionView::showContextHelp(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
