/****************************************************************************
** Meta object code from reading C++ file 'TriggerSegmentManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/segment/TriggerSegmentManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TriggerSegmentManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TriggerSegmentManager_t {
    QByteArrayData data[18];
    char stringdata0[246];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TriggerSegmentManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TriggerSegmentManager_t qt_meta_stringdata_Rosegarden__TriggerSegmentManager = {
    {
QT_MOC_LITERAL(0, 0, 33), // "Rosegarden::TriggerSegmentMan..."
QT_MOC_LITERAL(1, 34, 18), // "editTriggerSegment"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 7), // "closing"
QT_MOC_LITERAL(4, 62, 10), // "slotUpdate"
QT_MOC_LITERAL(5, 73, 7), // "slotAdd"
QT_MOC_LITERAL(6, 81, 10), // "slotDelete"
QT_MOC_LITERAL(7, 92, 13), // "slotDeleteAll"
QT_MOC_LITERAL(8, 106, 9), // "slotClose"
QT_MOC_LITERAL(9, 116, 8), // "slotEdit"
QT_MOC_LITERAL(10, 125, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(11, 142, 15), // "slotItemClicked"
QT_MOC_LITERAL(12, 158, 14), // "slotPasteAsNew"
QT_MOC_LITERAL(13, 173, 15), // "slotMusicalTime"
QT_MOC_LITERAL(14, 189, 12), // "slotRealTime"
QT_MOC_LITERAL(15, 202, 11), // "slotRawTime"
QT_MOC_LITERAL(16, 214, 17), // "slotHelpRequested"
QT_MOC_LITERAL(17, 232, 13) // "slotHelpAbout"

    },
    "Rosegarden::TriggerSegmentManager\0"
    "editTriggerSegment\0\0closing\0slotUpdate\0"
    "slotAdd\0slotDelete\0slotDeleteAll\0"
    "slotClose\0slotEdit\0QTreeWidgetItem*\0"
    "slotItemClicked\0slotPasteAsNew\0"
    "slotMusicalTime\0slotRealTime\0slotRawTime\0"
    "slotHelpRequested\0slotHelpAbout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TriggerSegmentManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       3,    0,   92,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   93,    2, 0x08 /* Private */,
       5,    0,   94,    2, 0x08 /* Private */,
       6,    0,   95,    2, 0x08 /* Private */,
       7,    0,   96,    2, 0x08 /* Private */,
       8,    0,   97,    2, 0x08 /* Private */,
       9,    1,   98,    2, 0x08 /* Private */,
      11,    1,  101,    2, 0x08 /* Private */,
      12,    0,  104,    2, 0x08 /* Private */,
      13,    0,  105,    2, 0x08 /* Private */,
      14,    0,  106,    2, 0x08 /* Private */,
      15,    0,  107,    2, 0x08 /* Private */,
      16,    0,  108,    2, 0x08 /* Private */,
      17,    0,  109,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TriggerSegmentManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TriggerSegmentManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->closing(); break;
        case 2: _t->slotUpdate(); break;
        case 3: _t->slotAdd(); break;
        case 4: _t->slotDelete(); break;
        case 5: _t->slotDeleteAll(); break;
        case 6: _t->slotClose(); break;
        case 7: _t->slotEdit((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 8: _t->slotItemClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 9: _t->slotPasteAsNew(); break;
        case 10: _t->slotMusicalTime(); break;
        case 11: _t->slotRealTime(); break;
        case 12: _t->slotRawTime(); break;
        case 13: _t->slotHelpRequested(); break;
        case 14: _t->slotHelpAbout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TriggerSegmentManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TriggerSegmentManager::editTriggerSegment)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TriggerSegmentManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TriggerSegmentManager::closing)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TriggerSegmentManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TriggerSegmentManager.data,
    qt_meta_data_Rosegarden__TriggerSegmentManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TriggerSegmentManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TriggerSegmentManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TriggerSegmentManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::TriggerSegmentManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TriggerSegmentManager::editTriggerSegment(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::TriggerSegmentManager::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
