/****************************************************************************
** Meta object code from reading C++ file 'RosegardenMainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/application/RosegardenMainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosegardenMainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__RosegardenMainWindow_t {
    QByteArrayData data[276];
    char stringdata0[4920];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__RosegardenMainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__RosegardenMainWindow_t qt_meta_stringdata_Rosegarden__RosegardenMainWindow = {
    {
QT_MOC_LITERAL(0, 0, 32), // "Rosegarden::RosegardenMainWindow"
QT_MOC_LITERAL(1, 33, 20), // "startupStatusMessage"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 7), // "message"
QT_MOC_LITERAL(4, 63, 21), // "documentAboutToChange"
QT_MOC_LITERAL(5, 85, 14), // "documentLoaded"
QT_MOC_LITERAL(6, 100, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(7, 120, 16), // "segmentsSelected"
QT_MOC_LITERAL(8, 137, 16), // "SegmentSelection"
QT_MOC_LITERAL(9, 154, 14), // "pluginSelected"
QT_MOC_LITERAL(10, 169, 12), // "InstrumentId"
QT_MOC_LITERAL(11, 182, 14), // "pluginBypassed"
QT_MOC_LITERAL(12, 197, 15), // "slotUpdateTitle"
QT_MOC_LITERAL(13, 213, 8), // "modified"
QT_MOC_LITERAL(14, 222, 18), // "slotOpenDroppedURL"
QT_MOC_LITERAL(15, 241, 3), // "url"
QT_MOC_LITERAL(16, 245, 12), // "slotFileSave"
QT_MOC_LITERAL(17, 258, 17), // "slotExportProject"
QT_MOC_LITERAL(18, 276, 14), // "slotExportMIDI"
QT_MOC_LITERAL(19, 291, 16), // "slotExportCsound"
QT_MOC_LITERAL(20, 308, 13), // "slotExportMup"
QT_MOC_LITERAL(21, 322, 18), // "slotExportLilyPond"
QT_MOC_LITERAL(22, 341, 17), // "slotPrintLilyPond"
QT_MOC_LITERAL(23, 359, 19), // "slotPreviewLilyPond"
QT_MOC_LITERAL(24, 379, 18), // "slotExportMusicXml"
QT_MOC_LITERAL(25, 398, 13), // "slotExportWAV"
QT_MOC_LITERAL(26, 412, 8), // "slotQuit"
QT_MOC_LITERAL(27, 421, 11), // "slotEditCut"
QT_MOC_LITERAL(28, 433, 12), // "slotEditCopy"
QT_MOC_LITERAL(29, 446, 13), // "slotEditPaste"
QT_MOC_LITERAL(30, 460, 12), // "slotCutRange"
QT_MOC_LITERAL(31, 473, 13), // "slotCopyRange"
QT_MOC_LITERAL(32, 487, 14), // "slotPasteRange"
QT_MOC_LITERAL(33, 502, 15), // "slotDeleteRange"
QT_MOC_LITERAL(34, 518, 15), // "slotInsertRange"
QT_MOC_LITERAL(35, 534, 22), // "slotPasteConductorData"
QT_MOC_LITERAL(36, 557, 20), // "slotEraseRangeTempos"
QT_MOC_LITERAL(37, 578, 13), // "slotSelectAll"
QT_MOC_LITERAL(38, 592, 25), // "slotSelectRelatedSegments"
QT_MOC_LITERAL(39, 618, 26), // "slotDeleteSelectedSegments"
QT_MOC_LITERAL(40, 645, 21), // "slotQuantizeSelection"
QT_MOC_LITERAL(41, 667, 27), // "slotRepeatQuantizeSelection"
QT_MOC_LITERAL(42, 695, 18), // "slotGrooveQuantize"
QT_MOC_LITERAL(43, 714, 14), // "slotFitToBeats"
QT_MOC_LITERAL(44, 729, 20), // "slotRescaleSelection"
QT_MOC_LITERAL(45, 750, 22), // "slotAutoSplitSelection"
QT_MOC_LITERAL(46, 773, 12), // "slotJogRight"
QT_MOC_LITERAL(47, 786, 11), // "slotJogLeft"
QT_MOC_LITERAL(48, 798, 25), // "slotSplitSelectionByPitch"
QT_MOC_LITERAL(49, 824, 31), // "slotSplitSelectionByRecordedSrc"
QT_MOC_LITERAL(50, 856, 24), // "slotSplitSelectionAtTime"
QT_MOC_LITERAL(51, 881, 24), // "slotSplitSelectionByDrum"
QT_MOC_LITERAL(52, 906, 22), // "slotHarmonizeSelection"
QT_MOC_LITERAL(53, 929, 19), // "slotCreateAnacrusis"
QT_MOC_LITERAL(54, 949, 24), // "slotSetSegmentStartTimes"
QT_MOC_LITERAL(55, 974, 23), // "slotSetSegmentDurations"
QT_MOC_LITERAL(56, 998, 16), // "slotJoinSegments"
QT_MOC_LITERAL(57, 1015, 16), // "slotToggleRepeat"
QT_MOC_LITERAL(58, 1032, 20), // "slotExpandFiguration"
QT_MOC_LITERAL(59, 1053, 21), // "slotUpdateFigurations"
QT_MOC_LITERAL(60, 1075, 24), // "slotTempoToSegmentLength"
QT_MOC_LITERAL(61, 1100, 8), // "QWidget*"
QT_MOC_LITERAL(62, 1109, 6), // "parent"
QT_MOC_LITERAL(63, 1116, 23), // "slotToggleSegmentLabels"
QT_MOC_LITERAL(64, 1140, 8), // "slotEdit"
QT_MOC_LITERAL(65, 1149, 19), // "slotEditInEventList"
QT_MOC_LITERAL(66, 1169, 16), // "slotEditInMatrix"
QT_MOC_LITERAL(67, 1186, 26), // "slotEditInPercussionMatrix"
QT_MOC_LITERAL(68, 1213, 22), // "slotEditInPitchTracker"
QT_MOC_LITERAL(69, 1236, 18), // "slotEditAsNotation"
QT_MOC_LITERAL(70, 1255, 14), // "slotEditTempos"
QT_MOC_LITERAL(71, 1270, 5), // "timeT"
QT_MOC_LITERAL(72, 1276, 10), // "openAtTime"
QT_MOC_LITERAL(73, 1287, 13), // "slotEditTempo"
QT_MOC_LITERAL(74, 1301, 6), // "atTime"
QT_MOC_LITERAL(75, 1308, 21), // "slotEditTimeSignature"
QT_MOC_LITERAL(76, 1330, 21), // "slotEditTransportTime"
QT_MOC_LITERAL(77, 1352, 27), // "slotChangeCompositionLength"
QT_MOC_LITERAL(78, 1380, 26), // "slotEditDocumentProperties"
QT_MOC_LITERAL(79, 1407, 18), // "slotResetConfigDlg"
QT_MOC_LITERAL(80, 1426, 21), // "slotResetDocConfigDlg"
QT_MOC_LITERAL(81, 1448, 21), // "slotManageMIDIDevices"
QT_MOC_LITERAL(82, 1470, 16), // "slotManageSynths"
QT_MOC_LITERAL(83, 1487, 18), // "slotOpenAudioMixer"
QT_MOC_LITERAL(84, 1506, 17), // "slotOpenMidiMixer"
QT_MOC_LITERAL(85, 1524, 13), // "slotEditBanks"
QT_MOC_LITERAL(86, 1538, 8), // "DeviceId"
QT_MOC_LITERAL(87, 1547, 25), // "slotEditControlParameters"
QT_MOC_LITERAL(88, 1573, 15), // "slotEditMarkers"
QT_MOC_LITERAL(89, 1589, 32), // "slotPopulateTrackInstrumentPopup"
QT_MOC_LITERAL(90, 1622, 20), // "slotRemapInstruments"
QT_MOC_LITERAL(91, 1643, 21), // "slotModifyMIDIFilters"
QT_MOC_LITERAL(92, 1665, 19), // "slotManageMetronome"
QT_MOC_LITERAL(93, 1685, 21), // "slotSaveDefaultStudio"
QT_MOC_LITERAL(94, 1707, 16), // "slotImportStudio"
QT_MOC_LITERAL(95, 1724, 23), // "slotImportDefaultStudio"
QT_MOC_LITERAL(96, 1748, 24), // "slotImportStudioFromFile"
QT_MOC_LITERAL(97, 1773, 8), // "filename"
QT_MOC_LITERAL(98, 1782, 20), // "slotResetMidiNetwork"
QT_MOC_LITERAL(99, 1803, 17), // "slotToggleToolBar"
QT_MOC_LITERAL(100, 1821, 29), // "slotUpdateTransportVisibility"
QT_MOC_LITERAL(101, 1851, 29), // "slotToggleTransportVisibility"
QT_MOC_LITERAL(102, 1881, 22), // "slotToggleToolsToolBar"
QT_MOC_LITERAL(103, 1904, 23), // "slotToggleTracksToolBar"
QT_MOC_LITERAL(104, 1928, 24), // "slotToggleEditorsToolBar"
QT_MOC_LITERAL(105, 1953, 26), // "slotToggleTransportToolBar"
QT_MOC_LITERAL(106, 1980, 21), // "slotToggleZoomToolBar"
QT_MOC_LITERAL(107, 2002, 19), // "slotToggleStatusBar"
QT_MOC_LITERAL(108, 2022, 14), // "slotFullScreen"
QT_MOC_LITERAL(109, 2037, 13), // "slotStatusMsg"
QT_MOC_LITERAL(110, 2051, 4), // "text"
QT_MOC_LITERAL(111, 2056, 17), // "slotStatusHelpMsg"
QT_MOC_LITERAL(112, 2074, 19), // "slotEnableTransport"
QT_MOC_LITERAL(113, 2094, 19), // "slotPointerSelected"
QT_MOC_LITERAL(114, 2114, 17), // "slotEraseSelected"
QT_MOC_LITERAL(115, 2132, 16), // "slotDrawSelected"
QT_MOC_LITERAL(116, 2149, 16), // "slotMoveSelected"
QT_MOC_LITERAL(117, 2166, 18), // "slotResizeSelected"
QT_MOC_LITERAL(118, 2185, 16), // "slotJoinSelected"
QT_MOC_LITERAL(119, 2202, 17), // "slotSplitSelected"
QT_MOC_LITERAL(120, 2220, 12), // "slotAddTrack"
QT_MOC_LITERAL(121, 2233, 13), // "slotAddTracks"
QT_MOC_LITERAL(122, 2247, 15), // "slotDeleteTrack"
QT_MOC_LITERAL(123, 2263, 15), // "slotMoveTrackUp"
QT_MOC_LITERAL(124, 2279, 17), // "slotMoveTrackDown"
QT_MOC_LITERAL(125, 2297, 26), // "slotUpdateForPointerChange"
QT_MOC_LITERAL(126, 2324, 19), // "slotSetPlayPosition"
QT_MOC_LITERAL(127, 2344, 4), // "time"
QT_MOC_LITERAL(128, 2349, 15), // "slotLoopChanged"
QT_MOC_LITERAL(129, 2365, 8), // "slotPlay"
QT_MOC_LITERAL(130, 2374, 8), // "slotStop"
QT_MOC_LITERAL(131, 2383, 10), // "slotRewind"
QT_MOC_LITERAL(132, 2394, 15), // "slotFastforward"
QT_MOC_LITERAL(133, 2410, 10), // "slotRecord"
QT_MOC_LITERAL(134, 2421, 16), // "slotToggleRecord"
QT_MOC_LITERAL(135, 2438, 21), // "slotRewindToBeginning"
QT_MOC_LITERAL(136, 2460, 20), // "slotFastForwardToEnd"
QT_MOC_LITERAL(137, 2481, 14), // "slotJumpToTime"
QT_MOC_LITERAL(138, 2496, 8), // "RealTime"
QT_MOC_LITERAL(139, 2505, 15), // "slotStartAtTime"
QT_MOC_LITERAL(140, 2521, 22), // "slotRefreshTimeDisplay"
QT_MOC_LITERAL(141, 2544, 8), // "slotLoop"
QT_MOC_LITERAL(142, 2553, 18), // "slotScrollToFollow"
QT_MOC_LITERAL(143, 2572, 19), // "slotSequencerExited"
QT_MOC_LITERAL(144, 2592, 18), // "slotCloseTransport"
QT_MOC_LITERAL(145, 2611, 19), // "slotDeleteTransport"
QT_MOC_LITERAL(146, 2631, 16), // "slotActivateTool"
QT_MOC_LITERAL(147, 2648, 8), // "toolName"
QT_MOC_LITERAL(148, 2657, 19), // "slotToggleMetronome"
QT_MOC_LITERAL(149, 2677, 14), // "slotToggleSolo"
QT_MOC_LITERAL(150, 2692, 26), // "slotToggleSoloCurrentTrack"
QT_MOC_LITERAL(151, 2719, 21), // "slotToggleTrackLabels"
QT_MOC_LITERAL(152, 2741, 16), // "slotToggleRulers"
QT_MOC_LITERAL(153, 2758, 20), // "slotToggleTempoRuler"
QT_MOC_LITERAL(154, 2779, 24), // "slotToggleChordNameRuler"
QT_MOC_LITERAL(155, 2804, 18), // "slotTogglePreviews"
QT_MOC_LITERAL(156, 2823, 25), // "slotHideShowParameterArea"
QT_MOC_LITERAL(157, 2849, 23), // "slotSelectPreviousTrack"
QT_MOC_LITERAL(158, 2873, 19), // "slotSelectNextTrack"
QT_MOC_LITERAL(159, 2893, 28), // "slotToggleRecordCurrentTrack"
QT_MOC_LITERAL(160, 2922, 22), // "slotConfigureShortcuts"
QT_MOC_LITERAL(161, 2945, 13), // "slotConfigure"
QT_MOC_LITERAL(162, 2959, 14), // "slotChangeZoom"
QT_MOC_LITERAL(163, 2974, 5), // "index"
QT_MOC_LITERAL(164, 2980, 10), // "slotZoomIn"
QT_MOC_LITERAL(165, 2991, 11), // "slotZoomOut"
QT_MOC_LITERAL(166, 3003, 20), // "slotDocumentModified"
QT_MOC_LITERAL(167, 3024, 16), // "slotStateChanged"
QT_MOC_LITERAL(168, 3041, 1), // "s"
QT_MOC_LITERAL(169, 3043, 9), // "noReverse"
QT_MOC_LITERAL(170, 3053, 17), // "slotTestClipboard"
QT_MOC_LITERAL(171, 3071, 12), // "slotPlayList"
QT_MOC_LITERAL(172, 3084, 16), // "slotPlayListPlay"
QT_MOC_LITERAL(173, 3101, 8), // "slotHelp"
QT_MOC_LITERAL(174, 3110, 12), // "slotTutorial"
QT_MOC_LITERAL(175, 3123, 17), // "slotBugGuidelines"
QT_MOC_LITERAL(176, 3141, 13), // "slotHelpAbout"
QT_MOC_LITERAL(177, 3155, 15), // "slotHelpAboutQt"
QT_MOC_LITERAL(178, 3171, 10), // "slotDonate"
QT_MOC_LITERAL(179, 3182, 25), // "slotManageTriggerSegments"
QT_MOC_LITERAL(180, 3208, 16), // "slotAudioManager"
QT_MOC_LITERAL(181, 3225, 16), // "slotAddAudioFile"
QT_MOC_LITERAL(182, 3242, 11), // "AudioFileId"
QT_MOC_LITERAL(183, 3254, 19), // "slotDeleteAudioFile"
QT_MOC_LITERAL(184, 3274, 17), // "slotPlayAudioFile"
QT_MOC_LITERAL(185, 3292, 26), // "slotCancelAudioPlayingFile"
QT_MOC_LITERAL(186, 3319, 23), // "slotDeleteAllAudioFiles"
QT_MOC_LITERAL(187, 3343, 18), // "slotDeleteSegments"
QT_MOC_LITERAL(188, 3362, 21), // "slotRepeatingSegments"
QT_MOC_LITERAL(189, 3384, 17), // "slotLinksToCopies"
QT_MOC_LITERAL(190, 3402, 19), // "slotRelabelSegments"
QT_MOC_LITERAL(191, 3422, 21), // "slotTransposeSegments"
QT_MOC_LITERAL(192, 3444, 22), // "slotTransposeSemitones"
QT_MOC_LITERAL(193, 3467, 16), // "slotSwitchPreset"
QT_MOC_LITERAL(194, 3484, 13), // "slotInterpret"
QT_MOC_LITERAL(195, 3498, 9), // "slotPanic"
QT_MOC_LITERAL(196, 3508, 12), // "slotAutoSave"
QT_MOC_LITERAL(197, 3521, 26), // "slotUpdateAutoSaveInterval"
QT_MOC_LITERAL(198, 3548, 8), // "interval"
QT_MOC_LITERAL(199, 3557, 18), // "slotPlayListClosed"
QT_MOC_LITERAL(200, 3576, 20), // "slotBankEditorClosed"
QT_MOC_LITERAL(201, 3597, 28), // "slotSynthPluginManagerClosed"
QT_MOC_LITERAL(202, 3626, 19), // "slotMidiMixerClosed"
QT_MOC_LITERAL(203, 3646, 23), // "slotControlEditorClosed"
QT_MOC_LITERAL(204, 3670, 22), // "slotMarkerEditorClosed"
QT_MOC_LITERAL(205, 3693, 19), // "slotTempoViewClosed"
QT_MOC_LITERAL(206, 3713, 24), // "slotTriggerManagerClosed"
QT_MOC_LITERAL(207, 3738, 22), // "slotAudioManagerClosed"
QT_MOC_LITERAL(208, 3761, 20), // "slotUpdateMonitoring"
QT_MOC_LITERAL(209, 3782, 20), // "slotShowPluginDialog"
QT_MOC_LITERAL(210, 3803, 12), // "instrumentId"
QT_MOC_LITERAL(211, 3816, 18), // "slotPluginSelected"
QT_MOC_LITERAL(212, 3835, 6), // "plugin"
QT_MOC_LITERAL(213, 3842, 20), // "slotChangePluginPort"
QT_MOC_LITERAL(214, 3863, 11), // "pluginIndex"
QT_MOC_LITERAL(215, 3875, 9), // "portIndex"
QT_MOC_LITERAL(216, 3885, 5), // "value"
QT_MOC_LITERAL(217, 3891, 21), // "slotPluginPortChanged"
QT_MOC_LITERAL(218, 3913, 23), // "slotChangePluginProgram"
QT_MOC_LITERAL(219, 3937, 7), // "program"
QT_MOC_LITERAL(220, 3945, 24), // "slotPluginProgramChanged"
QT_MOC_LITERAL(221, 3970, 29), // "slotChangePluginConfiguration"
QT_MOC_LITERAL(222, 4000, 6), // "global"
QT_MOC_LITERAL(223, 4007, 9), // "configKey"
QT_MOC_LITERAL(224, 4017, 11), // "configValue"
QT_MOC_LITERAL(225, 4029, 25), // "slotPluginDialogDestroyed"
QT_MOC_LITERAL(226, 4055, 18), // "slotPluginBypassed"
QT_MOC_LITERAL(227, 4074, 8), // "bypassed"
QT_MOC_LITERAL(228, 4083, 17), // "slotShowPluginGUI"
QT_MOC_LITERAL(229, 4101, 17), // "slotStopPluginGUI"
QT_MOC_LITERAL(230, 4119, 19), // "slotPluginGUIExited"
QT_MOC_LITERAL(231, 4139, 26), // "slotDocumentDevicesResyncd"
QT_MOC_LITERAL(232, 4166, 21), // "slotTestStartupTester"
QT_MOC_LITERAL(233, 4188, 13), // "slotDebugDump"
QT_MOC_LITERAL(234, 4202, 16), // "slotShowToolHelp"
QT_MOC_LITERAL(235, 4219, 25), // "slotNewerVersionAvailable"
QT_MOC_LITERAL(236, 4245, 14), // "slotAddMarker2"
QT_MOC_LITERAL(237, 4260, 18), // "slotPreviousMarker"
QT_MOC_LITERAL(238, 4279, 14), // "slotNextMarker"
QT_MOC_LITERAL(239, 4294, 18), // "slotSetQuickMarker"
QT_MOC_LITERAL(240, 4313, 21), // "slotJumpToQuickMarker"
QT_MOC_LITERAL(241, 4335, 18), // "slotDisplayWarning"
QT_MOC_LITERAL(242, 4354, 4), // "type"
QT_MOC_LITERAL(243, 4359, 15), // "informativeText"
QT_MOC_LITERAL(244, 4375, 25), // "slotAboutToExecuteCommand"
QT_MOC_LITERAL(245, 4401, 17), // "slotCommandUndone"
QT_MOC_LITERAL(246, 4419, 17), // "slotCommandRedone"
QT_MOC_LITERAL(247, 4437, 18), // "slotUpdatePosition"
QT_MOC_LITERAL(248, 4456, 14), // "slotToggleMute"
QT_MOC_LITERAL(249, 4471, 20), // "setupRecentFilesMenu"
QT_MOC_LITERAL(250, 4492, 12), // "signalAction"
QT_MOC_LITERAL(251, 4505, 16), // "slotHandleInputs"
QT_MOC_LITERAL(252, 4522, 12), // "slotUpdateUI"
QT_MOC_LITERAL(253, 4535, 18), // "slotUpdateCPUMeter"
QT_MOC_LITERAL(254, 4554, 17), // "slotMuteAllTracks"
QT_MOC_LITERAL(255, 4572, 19), // "slotUnmuteAllTracks"
QT_MOC_LITERAL(256, 4592, 19), // "slotCommandExecuted"
QT_MOC_LITERAL(257, 4612, 11), // "slotFileNew"
QT_MOC_LITERAL(258, 4624, 12), // "slotFileOpen"
QT_MOC_LITERAL(259, 4637, 19), // "slotFileOpenExample"
QT_MOC_LITERAL(260, 4657, 20), // "slotFileOpenTemplate"
QT_MOC_LITERAL(261, 4678, 18), // "slotFileOpenRecent"
QT_MOC_LITERAL(262, 4697, 14), // "slotFileSaveAs"
QT_MOC_LITERAL(263, 4712, 22), // "slotFileSaveAsTemplate"
QT_MOC_LITERAL(264, 4735, 13), // "slotFileClose"
QT_MOC_LITERAL(265, 4749, 17), // "slotImportProject"
QT_MOC_LITERAL(266, 4767, 14), // "slotImportMIDI"
QT_MOC_LITERAL(267, 4782, 17), // "slotRevertToSaved"
QT_MOC_LITERAL(268, 4800, 14), // "slotImportRG21"
QT_MOC_LITERAL(269, 4815, 18), // "slotImportMusicXML"
QT_MOC_LITERAL(270, 4834, 9), // "slotMerge"
QT_MOC_LITERAL(271, 4844, 13), // "slotMergeMIDI"
QT_MOC_LITERAL(272, 4858, 13), // "slotMergeRG21"
QT_MOC_LITERAL(273, 4872, 17), // "slotMergeMusicXML"
QT_MOC_LITERAL(274, 4890, 22), // "slotMetronomeActivated"
QT_MOC_LITERAL(275, 4913, 6) // "active"

    },
    "Rosegarden::RosegardenMainWindow\0"
    "startupStatusMessage\0\0message\0"
    "documentAboutToChange\0documentLoaded\0"
    "RosegardenDocument*\0segmentsSelected\0"
    "SegmentSelection\0pluginSelected\0"
    "InstrumentId\0pluginBypassed\0slotUpdateTitle\0"
    "modified\0slotOpenDroppedURL\0url\0"
    "slotFileSave\0slotExportProject\0"
    "slotExportMIDI\0slotExportCsound\0"
    "slotExportMup\0slotExportLilyPond\0"
    "slotPrintLilyPond\0slotPreviewLilyPond\0"
    "slotExportMusicXml\0slotExportWAV\0"
    "slotQuit\0slotEditCut\0slotEditCopy\0"
    "slotEditPaste\0slotCutRange\0slotCopyRange\0"
    "slotPasteRange\0slotDeleteRange\0"
    "slotInsertRange\0slotPasteConductorData\0"
    "slotEraseRangeTempos\0slotSelectAll\0"
    "slotSelectRelatedSegments\0"
    "slotDeleteSelectedSegments\0"
    "slotQuantizeSelection\0slotRepeatQuantizeSelection\0"
    "slotGrooveQuantize\0slotFitToBeats\0"
    "slotRescaleSelection\0slotAutoSplitSelection\0"
    "slotJogRight\0slotJogLeft\0"
    "slotSplitSelectionByPitch\0"
    "slotSplitSelectionByRecordedSrc\0"
    "slotSplitSelectionAtTime\0"
    "slotSplitSelectionByDrum\0"
    "slotHarmonizeSelection\0slotCreateAnacrusis\0"
    "slotSetSegmentStartTimes\0"
    "slotSetSegmentDurations\0slotJoinSegments\0"
    "slotToggleRepeat\0slotExpandFiguration\0"
    "slotUpdateFigurations\0slotTempoToSegmentLength\0"
    "QWidget*\0parent\0slotToggleSegmentLabels\0"
    "slotEdit\0slotEditInEventList\0"
    "slotEditInMatrix\0slotEditInPercussionMatrix\0"
    "slotEditInPitchTracker\0slotEditAsNotation\0"
    "slotEditTempos\0timeT\0openAtTime\0"
    "slotEditTempo\0atTime\0slotEditTimeSignature\0"
    "slotEditTransportTime\0slotChangeCompositionLength\0"
    "slotEditDocumentProperties\0"
    "slotResetConfigDlg\0slotResetDocConfigDlg\0"
    "slotManageMIDIDevices\0slotManageSynths\0"
    "slotOpenAudioMixer\0slotOpenMidiMixer\0"
    "slotEditBanks\0DeviceId\0slotEditControlParameters\0"
    "slotEditMarkers\0slotPopulateTrackInstrumentPopup\0"
    "slotRemapInstruments\0slotModifyMIDIFilters\0"
    "slotManageMetronome\0slotSaveDefaultStudio\0"
    "slotImportStudio\0slotImportDefaultStudio\0"
    "slotImportStudioFromFile\0filename\0"
    "slotResetMidiNetwork\0slotToggleToolBar\0"
    "slotUpdateTransportVisibility\0"
    "slotToggleTransportVisibility\0"
    "slotToggleToolsToolBar\0slotToggleTracksToolBar\0"
    "slotToggleEditorsToolBar\0"
    "slotToggleTransportToolBar\0"
    "slotToggleZoomToolBar\0slotToggleStatusBar\0"
    "slotFullScreen\0slotStatusMsg\0text\0"
    "slotStatusHelpMsg\0slotEnableTransport\0"
    "slotPointerSelected\0slotEraseSelected\0"
    "slotDrawSelected\0slotMoveSelected\0"
    "slotResizeSelected\0slotJoinSelected\0"
    "slotSplitSelected\0slotAddTrack\0"
    "slotAddTracks\0slotDeleteTrack\0"
    "slotMoveTrackUp\0slotMoveTrackDown\0"
    "slotUpdateForPointerChange\0"
    "slotSetPlayPosition\0time\0slotLoopChanged\0"
    "slotPlay\0slotStop\0slotRewind\0"
    "slotFastforward\0slotRecord\0slotToggleRecord\0"
    "slotRewindToBeginning\0slotFastForwardToEnd\0"
    "slotJumpToTime\0RealTime\0slotStartAtTime\0"
    "slotRefreshTimeDisplay\0slotLoop\0"
    "slotScrollToFollow\0slotSequencerExited\0"
    "slotCloseTransport\0slotDeleteTransport\0"
    "slotActivateTool\0toolName\0slotToggleMetronome\0"
    "slotToggleSolo\0slotToggleSoloCurrentTrack\0"
    "slotToggleTrackLabels\0slotToggleRulers\0"
    "slotToggleTempoRuler\0slotToggleChordNameRuler\0"
    "slotTogglePreviews\0slotHideShowParameterArea\0"
    "slotSelectPreviousTrack\0slotSelectNextTrack\0"
    "slotToggleRecordCurrentTrack\0"
    "slotConfigureShortcuts\0slotConfigure\0"
    "slotChangeZoom\0index\0slotZoomIn\0"
    "slotZoomOut\0slotDocumentModified\0"
    "slotStateChanged\0s\0noReverse\0"
    "slotTestClipboard\0slotPlayList\0"
    "slotPlayListPlay\0slotHelp\0slotTutorial\0"
    "slotBugGuidelines\0slotHelpAbout\0"
    "slotHelpAboutQt\0slotDonate\0"
    "slotManageTriggerSegments\0slotAudioManager\0"
    "slotAddAudioFile\0AudioFileId\0"
    "slotDeleteAudioFile\0slotPlayAudioFile\0"
    "slotCancelAudioPlayingFile\0"
    "slotDeleteAllAudioFiles\0slotDeleteSegments\0"
    "slotRepeatingSegments\0slotLinksToCopies\0"
    "slotRelabelSegments\0slotTransposeSegments\0"
    "slotTransposeSemitones\0slotSwitchPreset\0"
    "slotInterpret\0slotPanic\0slotAutoSave\0"
    "slotUpdateAutoSaveInterval\0interval\0"
    "slotPlayListClosed\0slotBankEditorClosed\0"
    "slotSynthPluginManagerClosed\0"
    "slotMidiMixerClosed\0slotControlEditorClosed\0"
    "slotMarkerEditorClosed\0slotTempoViewClosed\0"
    "slotTriggerManagerClosed\0"
    "slotAudioManagerClosed\0slotUpdateMonitoring\0"
    "slotShowPluginDialog\0instrumentId\0"
    "slotPluginSelected\0plugin\0"
    "slotChangePluginPort\0pluginIndex\0"
    "portIndex\0value\0slotPluginPortChanged\0"
    "slotChangePluginProgram\0program\0"
    "slotPluginProgramChanged\0"
    "slotChangePluginConfiguration\0global\0"
    "configKey\0configValue\0slotPluginDialogDestroyed\0"
    "slotPluginBypassed\0bypassed\0"
    "slotShowPluginGUI\0slotStopPluginGUI\0"
    "slotPluginGUIExited\0slotDocumentDevicesResyncd\0"
    "slotTestStartupTester\0slotDebugDump\0"
    "slotShowToolHelp\0slotNewerVersionAvailable\0"
    "slotAddMarker2\0slotPreviousMarker\0"
    "slotNextMarker\0slotSetQuickMarker\0"
    "slotJumpToQuickMarker\0slotDisplayWarning\0"
    "type\0informativeText\0slotAboutToExecuteCommand\0"
    "slotCommandUndone\0slotCommandRedone\0"
    "slotUpdatePosition\0slotToggleMute\0"
    "setupRecentFilesMenu\0signalAction\0"
    "slotHandleInputs\0slotUpdateUI\0"
    "slotUpdateCPUMeter\0slotMuteAllTracks\0"
    "slotUnmuteAllTracks\0slotCommandExecuted\0"
    "slotFileNew\0slotFileOpen\0slotFileOpenExample\0"
    "slotFileOpenTemplate\0slotFileOpenRecent\0"
    "slotFileSaveAs\0slotFileSaveAsTemplate\0"
    "slotFileClose\0slotImportProject\0"
    "slotImportMIDI\0slotRevertToSaved\0"
    "slotImportRG21\0slotImportMusicXML\0"
    "slotMerge\0slotMergeMIDI\0slotMergeRG21\0"
    "slotMergeMusicXML\0slotMetronomeActivated\0"
    "active"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__RosegardenMainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     251,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1, 1269,    2, 0x06 /* Public */,
       4,    0, 1272,    2, 0x06 /* Public */,
       5,    1, 1273,    2, 0x06 /* Public */,
       7,    1, 1276,    2, 0x06 /* Public */,
       9,    3, 1279,    2, 0x06 /* Public */,
      11,    3, 1286,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1, 1293,    2, 0x0a /* Public */,
      12,    0, 1296,    2, 0x2a /* Public | MethodCloned */,
      14,    1, 1297,    2, 0x0a /* Public */,
      16,    0, 1300,    2, 0x0a /* Public */,
      17,    0, 1301,    2, 0x0a /* Public */,
      18,    0, 1302,    2, 0x0a /* Public */,
      19,    0, 1303,    2, 0x0a /* Public */,
      20,    0, 1304,    2, 0x0a /* Public */,
      21,    0, 1305,    2, 0x0a /* Public */,
      22,    0, 1306,    2, 0x0a /* Public */,
      23,    0, 1307,    2, 0x0a /* Public */,
      24,    0, 1308,    2, 0x0a /* Public */,
      25,    0, 1309,    2, 0x0a /* Public */,
      26,    0, 1310,    2, 0x0a /* Public */,
      27,    0, 1311,    2, 0x0a /* Public */,
      28,    0, 1312,    2, 0x0a /* Public */,
      29,    0, 1313,    2, 0x0a /* Public */,
      30,    0, 1314,    2, 0x0a /* Public */,
      31,    0, 1315,    2, 0x0a /* Public */,
      32,    0, 1316,    2, 0x0a /* Public */,
      33,    0, 1317,    2, 0x0a /* Public */,
      34,    0, 1318,    2, 0x0a /* Public */,
      35,    0, 1319,    2, 0x0a /* Public */,
      36,    0, 1320,    2, 0x0a /* Public */,
      37,    0, 1321,    2, 0x0a /* Public */,
      38,    0, 1322,    2, 0x0a /* Public */,
      39,    0, 1323,    2, 0x0a /* Public */,
      40,    0, 1324,    2, 0x0a /* Public */,
      41,    0, 1325,    2, 0x0a /* Public */,
      42,    0, 1326,    2, 0x0a /* Public */,
      43,    0, 1327,    2, 0x0a /* Public */,
      44,    0, 1328,    2, 0x0a /* Public */,
      45,    0, 1329,    2, 0x0a /* Public */,
      46,    0, 1330,    2, 0x0a /* Public */,
      47,    0, 1331,    2, 0x0a /* Public */,
      48,    0, 1332,    2, 0x0a /* Public */,
      49,    0, 1333,    2, 0x0a /* Public */,
      50,    0, 1334,    2, 0x0a /* Public */,
      51,    0, 1335,    2, 0x0a /* Public */,
      52,    0, 1336,    2, 0x0a /* Public */,
      53,    0, 1337,    2, 0x0a /* Public */,
      54,    0, 1338,    2, 0x0a /* Public */,
      55,    0, 1339,    2, 0x0a /* Public */,
      56,    0, 1340,    2, 0x0a /* Public */,
      57,    0, 1341,    2, 0x0a /* Public */,
      58,    0, 1342,    2, 0x0a /* Public */,
      59,    0, 1343,    2, 0x0a /* Public */,
      60,    0, 1344,    2, 0x0a /* Public */,
      60,    1, 1345,    2, 0x0a /* Public */,
      63,    0, 1348,    2, 0x0a /* Public */,
      64,    0, 1349,    2, 0x0a /* Public */,
      65,    0, 1350,    2, 0x0a /* Public */,
      66,    0, 1351,    2, 0x0a /* Public */,
      67,    0, 1352,    2, 0x0a /* Public */,
      68,    0, 1353,    2, 0x0a /* Public */,
      69,    0, 1354,    2, 0x0a /* Public */,
      70,    0, 1355,    2, 0x0a /* Public */,
      70,    1, 1356,    2, 0x0a /* Public */,
      73,    0, 1359,    2, 0x0a /* Public */,
      73,    1, 1360,    2, 0x0a /* Public */,
      73,    1, 1363,    2, 0x0a /* Public */,
      73,    2, 1366,    2, 0x0a /* Public */,
      75,    0, 1371,    2, 0x0a /* Public */,
      75,    1, 1372,    2, 0x0a /* Public */,
      75,    1, 1375,    2, 0x0a /* Public */,
      75,    2, 1378,    2, 0x0a /* Public */,
      76,    0, 1383,    2, 0x0a /* Public */,
      76,    1, 1384,    2, 0x0a /* Public */,
      77,    0, 1387,    2, 0x0a /* Public */,
      78,    0, 1388,    2, 0x0a /* Public */,
      79,    0, 1389,    2, 0x0a /* Public */,
      80,    0, 1390,    2, 0x0a /* Public */,
      81,    0, 1391,    2, 0x0a /* Public */,
      82,    0, 1392,    2, 0x0a /* Public */,
      83,    0, 1393,    2, 0x0a /* Public */,
      84,    0, 1394,    2, 0x0a /* Public */,
      85,    0, 1395,    2, 0x0a /* Public */,
      85,    1, 1396,    2, 0x0a /* Public */,
      87,    1, 1399,    2, 0x0a /* Public */,
      88,    0, 1402,    2, 0x0a /* Public */,
      89,    0, 1403,    2, 0x0a /* Public */,
      90,    0, 1404,    2, 0x0a /* Public */,
      91,    0, 1405,    2, 0x0a /* Public */,
      92,    0, 1406,    2, 0x0a /* Public */,
      93,    0, 1407,    2, 0x0a /* Public */,
      94,    0, 1408,    2, 0x0a /* Public */,
      95,    0, 1409,    2, 0x0a /* Public */,
      96,    1, 1410,    2, 0x0a /* Public */,
      98,    0, 1413,    2, 0x0a /* Public */,
      99,    0, 1414,    2, 0x0a /* Public */,
     100,    0, 1415,    2, 0x0a /* Public */,
     101,    0, 1416,    2, 0x0a /* Public */,
     102,    0, 1417,    2, 0x0a /* Public */,
     103,    0, 1418,    2, 0x0a /* Public */,
     104,    0, 1419,    2, 0x0a /* Public */,
     105,    0, 1420,    2, 0x0a /* Public */,
     106,    0, 1421,    2, 0x0a /* Public */,
     107,    0, 1422,    2, 0x0a /* Public */,
     108,    0, 1423,    2, 0x0a /* Public */,
     109,    1, 1424,    2, 0x0a /* Public */,
     111,    1, 1427,    2, 0x0a /* Public */,
     112,    1, 1430,    2, 0x0a /* Public */,
     113,    0, 1433,    2, 0x0a /* Public */,
     114,    0, 1434,    2, 0x0a /* Public */,
     115,    0, 1435,    2, 0x0a /* Public */,
     116,    0, 1436,    2, 0x0a /* Public */,
     117,    0, 1437,    2, 0x0a /* Public */,
     118,    0, 1438,    2, 0x0a /* Public */,
     119,    0, 1439,    2, 0x0a /* Public */,
     120,    0, 1440,    2, 0x0a /* Public */,
     121,    0, 1441,    2, 0x0a /* Public */,
     122,    0, 1442,    2, 0x0a /* Public */,
     123,    0, 1443,    2, 0x0a /* Public */,
     124,    0, 1444,    2, 0x0a /* Public */,
     125,    0, 1445,    2, 0x0a /* Public */,
     126,    1, 1446,    2, 0x0a /* Public */,
     128,    0, 1449,    2, 0x0a /* Public */,
     129,    0, 1450,    2, 0x0a /* Public */,
     130,    0, 1451,    2, 0x0a /* Public */,
     131,    0, 1452,    2, 0x0a /* Public */,
     132,    0, 1453,    2, 0x0a /* Public */,
     133,    0, 1454,    2, 0x0a /* Public */,
     134,    0, 1455,    2, 0x0a /* Public */,
     135,    0, 1456,    2, 0x0a /* Public */,
     136,    0, 1457,    2, 0x0a /* Public */,
     137,    1, 1458,    2, 0x0a /* Public */,
     139,    1, 1461,    2, 0x0a /* Public */,
     140,    0, 1464,    2, 0x0a /* Public */,
     141,    0, 1465,    2, 0x0a /* Public */,
     142,    0, 1466,    2, 0x0a /* Public */,
     143,    0, 1467,    2, 0x0a /* Public */,
     144,    0, 1468,    2, 0x0a /* Public */,
     145,    0, 1469,    2, 0x0a /* Public */,
     146,    1, 1470,    2, 0x0a /* Public */,
     148,    0, 1473,    2, 0x0a /* Public */,
     149,    1, 1474,    2, 0x0a /* Public */,
     150,    0, 1477,    2, 0x0a /* Public */,
     151,    0, 1478,    2, 0x0a /* Public */,
     152,    0, 1479,    2, 0x0a /* Public */,
     153,    0, 1480,    2, 0x0a /* Public */,
     154,    0, 1481,    2, 0x0a /* Public */,
     155,    0, 1482,    2, 0x0a /* Public */,
     156,    0, 1483,    2, 0x0a /* Public */,
     157,    0, 1484,    2, 0x0a /* Public */,
     158,    0, 1485,    2, 0x0a /* Public */,
     159,    0, 1486,    2, 0x0a /* Public */,
     160,    0, 1487,    2, 0x0a /* Public */,
     161,    0, 1488,    2, 0x0a /* Public */,
     162,    1, 1489,    2, 0x0a /* Public */,
     164,    0, 1492,    2, 0x0a /* Public */,
     165,    0, 1493,    2, 0x0a /* Public */,
     166,    1, 1494,    2, 0x0a /* Public */,
     166,    0, 1497,    2, 0x2a /* Public | MethodCloned */,
     167,    2, 1498,    2, 0x0a /* Public */,
     170,    0, 1503,    2, 0x0a /* Public */,
     171,    0, 1504,    2, 0x0a /* Public */,
     172,    1, 1505,    2, 0x0a /* Public */,
     173,    0, 1508,    2, 0x0a /* Public */,
     174,    0, 1509,    2, 0x0a /* Public */,
     175,    0, 1510,    2, 0x0a /* Public */,
     176,    0, 1511,    2, 0x0a /* Public */,
     177,    0, 1512,    2, 0x0a /* Public */,
     178,    0, 1513,    2, 0x0a /* Public */,
     179,    0, 1514,    2, 0x0a /* Public */,
     180,    0, 1515,    2, 0x0a /* Public */,
     181,    1, 1516,    2, 0x0a /* Public */,
     183,    1, 1519,    2, 0x0a /* Public */,
     184,    3, 1522,    2, 0x0a /* Public */,
     185,    1, 1529,    2, 0x0a /* Public */,
     186,    0, 1532,    2, 0x0a /* Public */,
     187,    1, 1533,    2, 0x0a /* Public */,
     188,    0, 1536,    2, 0x0a /* Public */,
     189,    0, 1537,    2, 0x0a /* Public */,
     190,    0, 1538,    2, 0x0a /* Public */,
     191,    0, 1539,    2, 0x0a /* Public */,
     192,    0, 1540,    2, 0x0a /* Public */,
     193,    0, 1541,    2, 0x0a /* Public */,
     194,    0, 1542,    2, 0x0a /* Public */,
     195,    0, 1543,    2, 0x0a /* Public */,
     196,    0, 1544,    2, 0x0a /* Public */,
     197,    1, 1545,    2, 0x0a /* Public */,
     199,    0, 1548,    2, 0x0a /* Public */,
     200,    0, 1549,    2, 0x0a /* Public */,
     201,    0, 1550,    2, 0x0a /* Public */,
     202,    0, 1551,    2, 0x0a /* Public */,
     203,    0, 1552,    2, 0x0a /* Public */,
     204,    0, 1553,    2, 0x0a /* Public */,
     205,    0, 1554,    2, 0x0a /* Public */,
     206,    0, 1555,    2, 0x0a /* Public */,
     207,    0, 1556,    2, 0x0a /* Public */,
     208,    0, 1557,    2, 0x0a /* Public */,
     209,    3, 1558,    2, 0x0a /* Public */,
     211,    3, 1565,    2, 0x0a /* Public */,
     213,    4, 1572,    2, 0x0a /* Public */,
     217,    3, 1581,    2, 0x0a /* Public */,
     218,    3, 1588,    2, 0x0a /* Public */,
     220,    2, 1595,    2, 0x0a /* Public */,
     221,    5, 1600,    2, 0x0a /* Public */,
     225,    2, 1611,    2, 0x0a /* Public */,
     226,    3, 1616,    2, 0x0a /* Public */,
     228,    2, 1623,    2, 0x0a /* Public */,
     229,    2, 1628,    2, 0x0a /* Public */,
     230,    2, 1633,    2, 0x0a /* Public */,
     231,    0, 1638,    2, 0x0a /* Public */,
     232,    0, 1639,    2, 0x0a /* Public */,
     233,    0, 1640,    2, 0x0a /* Public */,
     234,    1, 1641,    2, 0x0a /* Public */,
     235,    1, 1644,    2, 0x0a /* Public */,
     236,    0, 1647,    2, 0x0a /* Public */,
     237,    0, 1648,    2, 0x0a /* Public */,
     238,    0, 1649,    2, 0x0a /* Public */,
     239,    0, 1650,    2, 0x0a /* Public */,
     240,    0, 1651,    2, 0x0a /* Public */,
     241,    3, 1652,    2, 0x0a /* Public */,
     244,    0, 1659,    2, 0x0a /* Public */,
     245,    0, 1660,    2, 0x0a /* Public */,
     246,    0, 1661,    2, 0x0a /* Public */,
     247,    0, 1662,    2, 0x0a /* Public */,
     248,    0, 1663,    2, 0x0a /* Public */,
     249,    0, 1664,    2, 0x08 /* Private */,
     250,    1, 1665,    2, 0x08 /* Private */,
     251,    0, 1668,    2, 0x08 /* Private */,
     252,    0, 1669,    2, 0x08 /* Private */,
     253,    0, 1670,    2, 0x08 /* Private */,
     254,    0, 1671,    2, 0x08 /* Private */,
     255,    0, 1672,    2, 0x08 /* Private */,
     256,    0, 1673,    2, 0x08 /* Private */,
     257,    0, 1674,    2, 0x08 /* Private */,
     258,    0, 1675,    2, 0x08 /* Private */,
     259,    0, 1676,    2, 0x08 /* Private */,
     260,    0, 1677,    2, 0x08 /* Private */,
     261,    0, 1678,    2, 0x08 /* Private */,
     262,    0, 1679,    2, 0x08 /* Private */,
     263,    0, 1680,    2, 0x08 /* Private */,
     264,    0, 1681,    2, 0x08 /* Private */,
     265,    0, 1682,    2, 0x08 /* Private */,
     266,    0, 1683,    2, 0x08 /* Private */,
     267,    0, 1684,    2, 0x08 /* Private */,
     268,    0, 1685,    2, 0x08 /* Private */,
     269,    0, 1686,    2, 0x08 /* Private */,
     270,    0, 1687,    2, 0x08 /* Private */,
     271,    0, 1688,    2, 0x08 /* Private */,
     272,    0, 1689,    2, 0x08 /* Private */,
     273,    0, 1690,    2, 0x08 /* Private */,
     274,    1, 1691,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Bool,    2,    2,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 61,   62,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 71,   72,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 71,   74,
    QMetaType::Void, 0x80000000 | 61,   62,
    QMetaType::Void, 0x80000000 | 61, 0x80000000 | 71,   62,   74,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 71,   74,
    QMetaType::Void, 0x80000000 | 61,   62,
    QMetaType::Void, 0x80000000 | 61, 0x80000000 | 71,   62,   74,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 61,   62,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 86,    2,
    QMetaType::Void, 0x80000000 | 86,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   97,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,  110,
    QMetaType::Void, QMetaType::QString,  110,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 71,  127,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 138,    2,
    QMetaType::Void, 0x80000000 | 138,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,  147,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  163,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,  168,  169,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 182,    2,
    QMetaType::Void, 0x80000000 | 182,    2,
    QMetaType::Void, 0x80000000 | 182, 0x80000000 | 138, 0x80000000 | 138,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 182,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt,  198,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 61, 0x80000000 | 10, QMetaType::Int,   62,  210,  163,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Int,  210,  163,  212,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Int, QMetaType::Float,  210,  214,  215,  216,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Int,  210,  214,  215,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::QString,  210,  214,  219,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,  210,  214,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Bool, QMetaType::QString, QMetaType::QString,  210,  163,  222,  223,  224,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,  210,  163,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int, QMetaType::Bool,    2,  214,  227,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,    2,  163,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,    2,  163,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Int,    2,  163,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::QString,  242,  110,  243,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  275,

       0        // eod
};

void Rosegarden::RosegardenMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RosegardenMainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startupStatusMessage((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->documentAboutToChange(); break;
        case 2: _t->documentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 3: _t->segmentsSelected((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 4: _t->pluginSelected((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->pluginBypassed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 6: _t->slotUpdateTitle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->slotUpdateTitle(); break;
        case 8: _t->slotOpenDroppedURL((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->slotFileSave(); break;
        case 10: _t->slotExportProject(); break;
        case 11: _t->slotExportMIDI(); break;
        case 12: _t->slotExportCsound(); break;
        case 13: _t->slotExportMup(); break;
        case 14: _t->slotExportLilyPond(); break;
        case 15: _t->slotPrintLilyPond(); break;
        case 16: _t->slotPreviewLilyPond(); break;
        case 17: _t->slotExportMusicXml(); break;
        case 18: _t->slotExportWAV(); break;
        case 19: _t->slotQuit(); break;
        case 20: _t->slotEditCut(); break;
        case 21: _t->slotEditCopy(); break;
        case 22: _t->slotEditPaste(); break;
        case 23: _t->slotCutRange(); break;
        case 24: _t->slotCopyRange(); break;
        case 25: _t->slotPasteRange(); break;
        case 26: _t->slotDeleteRange(); break;
        case 27: _t->slotInsertRange(); break;
        case 28: _t->slotPasteConductorData(); break;
        case 29: _t->slotEraseRangeTempos(); break;
        case 30: _t->slotSelectAll(); break;
        case 31: _t->slotSelectRelatedSegments(); break;
        case 32: _t->slotDeleteSelectedSegments(); break;
        case 33: _t->slotQuantizeSelection(); break;
        case 34: _t->slotRepeatQuantizeSelection(); break;
        case 35: _t->slotGrooveQuantize(); break;
        case 36: _t->slotFitToBeats(); break;
        case 37: _t->slotRescaleSelection(); break;
        case 38: _t->slotAutoSplitSelection(); break;
        case 39: _t->slotJogRight(); break;
        case 40: _t->slotJogLeft(); break;
        case 41: _t->slotSplitSelectionByPitch(); break;
        case 42: _t->slotSplitSelectionByRecordedSrc(); break;
        case 43: _t->slotSplitSelectionAtTime(); break;
        case 44: _t->slotSplitSelectionByDrum(); break;
        case 45: _t->slotHarmonizeSelection(); break;
        case 46: _t->slotCreateAnacrusis(); break;
        case 47: _t->slotSetSegmentStartTimes(); break;
        case 48: _t->slotSetSegmentDurations(); break;
        case 49: _t->slotJoinSegments(); break;
        case 50: _t->slotToggleRepeat(); break;
        case 51: _t->slotExpandFiguration(); break;
        case 52: _t->slotUpdateFigurations(); break;
        case 53: _t->slotTempoToSegmentLength(); break;
        case 54: _t->slotTempoToSegmentLength((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 55: _t->slotToggleSegmentLabels(); break;
        case 56: _t->slotEdit(); break;
        case 57: _t->slotEditInEventList(); break;
        case 58: _t->slotEditInMatrix(); break;
        case 59: _t->slotEditInPercussionMatrix(); break;
        case 60: _t->slotEditInPitchTracker(); break;
        case 61: _t->slotEditAsNotation(); break;
        case 62: _t->slotEditTempos(); break;
        case 63: _t->slotEditTempos((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 64: _t->slotEditTempo(); break;
        case 65: _t->slotEditTempo((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 66: _t->slotEditTempo((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 67: _t->slotEditTempo((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 68: _t->slotEditTimeSignature(); break;
        case 69: _t->slotEditTimeSignature((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 70: _t->slotEditTimeSignature((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 71: _t->slotEditTimeSignature((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 72: _t->slotEditTransportTime(); break;
        case 73: _t->slotEditTransportTime((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 74: _t->slotChangeCompositionLength(); break;
        case 75: _t->slotEditDocumentProperties(); break;
        case 76: _t->slotResetConfigDlg(); break;
        case 77: _t->slotResetDocConfigDlg(); break;
        case 78: _t->slotManageMIDIDevices(); break;
        case 79: _t->slotManageSynths(); break;
        case 80: _t->slotOpenAudioMixer(); break;
        case 81: _t->slotOpenMidiMixer(); break;
        case 82: _t->slotEditBanks(); break;
        case 83: _t->slotEditBanks((*reinterpret_cast< DeviceId(*)>(_a[1]))); break;
        case 84: _t->slotEditControlParameters((*reinterpret_cast< DeviceId(*)>(_a[1]))); break;
        case 85: _t->slotEditMarkers(); break;
        case 86: _t->slotPopulateTrackInstrumentPopup(); break;
        case 87: _t->slotRemapInstruments(); break;
        case 88: _t->slotModifyMIDIFilters(); break;
        case 89: _t->slotManageMetronome(); break;
        case 90: _t->slotSaveDefaultStudio(); break;
        case 91: _t->slotImportStudio(); break;
        case 92: _t->slotImportDefaultStudio(); break;
        case 93: _t->slotImportStudioFromFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 94: _t->slotResetMidiNetwork(); break;
        case 95: _t->slotToggleToolBar(); break;
        case 96: _t->slotUpdateTransportVisibility(); break;
        case 97: _t->slotToggleTransportVisibility(); break;
        case 98: _t->slotToggleToolsToolBar(); break;
        case 99: _t->slotToggleTracksToolBar(); break;
        case 100: _t->slotToggleEditorsToolBar(); break;
        case 101: _t->slotToggleTransportToolBar(); break;
        case 102: _t->slotToggleZoomToolBar(); break;
        case 103: _t->slotToggleStatusBar(); break;
        case 104: _t->slotFullScreen(); break;
        case 105: _t->slotStatusMsg((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 106: _t->slotStatusHelpMsg((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 107: _t->slotEnableTransport((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 108: _t->slotPointerSelected(); break;
        case 109: _t->slotEraseSelected(); break;
        case 110: _t->slotDrawSelected(); break;
        case 111: _t->slotMoveSelected(); break;
        case 112: _t->slotResizeSelected(); break;
        case 113: _t->slotJoinSelected(); break;
        case 114: _t->slotSplitSelected(); break;
        case 115: _t->slotAddTrack(); break;
        case 116: _t->slotAddTracks(); break;
        case 117: _t->slotDeleteTrack(); break;
        case 118: _t->slotMoveTrackUp(); break;
        case 119: _t->slotMoveTrackDown(); break;
        case 120: _t->slotUpdateForPointerChange(); break;
        case 121: _t->slotSetPlayPosition((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 122: _t->slotLoopChanged(); break;
        case 123: _t->slotPlay(); break;
        case 124: _t->slotStop(); break;
        case 125: _t->slotRewind(); break;
        case 126: _t->slotFastforward(); break;
        case 127: _t->slotRecord(); break;
        case 128: _t->slotToggleRecord(); break;
        case 129: _t->slotRewindToBeginning(); break;
        case 130: _t->slotFastForwardToEnd(); break;
        case 131: _t->slotJumpToTime((*reinterpret_cast< RealTime(*)>(_a[1]))); break;
        case 132: _t->slotStartAtTime((*reinterpret_cast< const RealTime(*)>(_a[1]))); break;
        case 133: _t->slotRefreshTimeDisplay(); break;
        case 134: _t->slotLoop(); break;
        case 135: _t->slotScrollToFollow(); break;
        case 136: _t->slotSequencerExited(); break;
        case 137: _t->slotCloseTransport(); break;
        case 138: _t->slotDeleteTransport(); break;
        case 139: _t->slotActivateTool((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 140: _t->slotToggleMetronome(); break;
        case 141: _t->slotToggleSolo((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 142: _t->slotToggleSoloCurrentTrack(); break;
        case 143: _t->slotToggleTrackLabels(); break;
        case 144: _t->slotToggleRulers(); break;
        case 145: _t->slotToggleTempoRuler(); break;
        case 146: _t->slotToggleChordNameRuler(); break;
        case 147: _t->slotTogglePreviews(); break;
        case 148: _t->slotHideShowParameterArea(); break;
        case 149: _t->slotSelectPreviousTrack(); break;
        case 150: _t->slotSelectNextTrack(); break;
        case 151: _t->slotToggleRecordCurrentTrack(); break;
        case 152: _t->slotConfigureShortcuts(); break;
        case 153: _t->slotConfigure(); break;
        case 154: _t->slotChangeZoom((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 155: _t->slotZoomIn(); break;
        case 156: _t->slotZoomOut(); break;
        case 157: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 158: _t->slotDocumentModified(); break;
        case 159: _t->slotStateChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 160: _t->slotTestClipboard(); break;
        case 161: _t->slotPlayList(); break;
        case 162: _t->slotPlayListPlay((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 163: _t->slotHelp(); break;
        case 164: _t->slotTutorial(); break;
        case 165: _t->slotBugGuidelines(); break;
        case 166: _t->slotHelpAbout(); break;
        case 167: _t->slotHelpAboutQt(); break;
        case 168: _t->slotDonate(); break;
        case 169: _t->slotManageTriggerSegments(); break;
        case 170: _t->slotAudioManager(); break;
        case 171: _t->slotAddAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 172: _t->slotDeleteAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 173: _t->slotPlayAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1])),(*reinterpret_cast< const RealTime(*)>(_a[2])),(*reinterpret_cast< const RealTime(*)>(_a[3]))); break;
        case 174: _t->slotCancelAudioPlayingFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 175: _t->slotDeleteAllAudioFiles(); break;
        case 176: _t->slotDeleteSegments((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 177: _t->slotRepeatingSegments(); break;
        case 178: _t->slotLinksToCopies(); break;
        case 179: _t->slotRelabelSegments(); break;
        case 180: _t->slotTransposeSegments(); break;
        case 181: _t->slotTransposeSemitones(); break;
        case 182: _t->slotSwitchPreset(); break;
        case 183: _t->slotInterpret(); break;
        case 184: _t->slotPanic(); break;
        case 185: _t->slotAutoSave(); break;
        case 186: _t->slotUpdateAutoSaveInterval((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 187: _t->slotPlayListClosed(); break;
        case 188: _t->slotBankEditorClosed(); break;
        case 189: _t->slotSynthPluginManagerClosed(); break;
        case 190: _t->slotMidiMixerClosed(); break;
        case 191: _t->slotControlEditorClosed(); break;
        case 192: _t->slotMarkerEditorClosed(); break;
        case 193: _t->slotTempoViewClosed(); break;
        case 194: _t->slotTriggerManagerClosed(); break;
        case 195: _t->slotAudioManagerClosed(); break;
        case 196: _t->slotUpdateMonitoring(); break;
        case 197: _t->slotShowPluginDialog((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< InstrumentId(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 198: _t->slotPluginSelected((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 199: _t->slotChangePluginPort((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< float(*)>(_a[4]))); break;
        case 200: _t->slotPluginPortChanged((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 201: _t->slotChangePluginProgram((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 202: _t->slotPluginProgramChanged((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 203: _t->slotChangePluginConfiguration((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 204: _t->slotPluginDialogDestroyed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 205: _t->slotPluginBypassed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 206: _t->slotShowPluginGUI((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 207: _t->slotStopPluginGUI((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 208: _t->slotPluginGUIExited((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 209: _t->slotDocumentDevicesResyncd(); break;
        case 210: _t->slotTestStartupTester(); break;
        case 211: _t->slotDebugDump(); break;
        case 212: _t->slotShowToolHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 213: _t->slotNewerVersionAvailable((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 214: _t->slotAddMarker2(); break;
        case 215: _t->slotPreviousMarker(); break;
        case 216: _t->slotNextMarker(); break;
        case 217: _t->slotSetQuickMarker(); break;
        case 218: _t->slotJumpToQuickMarker(); break;
        case 219: _t->slotDisplayWarning((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 220: _t->slotAboutToExecuteCommand(); break;
        case 221: _t->slotCommandUndone(); break;
        case 222: _t->slotCommandRedone(); break;
        case 223: _t->slotUpdatePosition(); break;
        case 224: _t->slotToggleMute(); break;
        case 225: _t->setupRecentFilesMenu(); break;
        case 226: _t->signalAction((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 227: _t->slotHandleInputs(); break;
        case 228: _t->slotUpdateUI(); break;
        case 229: _t->slotUpdateCPUMeter(); break;
        case 230: _t->slotMuteAllTracks(); break;
        case 231: _t->slotUnmuteAllTracks(); break;
        case 232: _t->slotCommandExecuted(); break;
        case 233: _t->slotFileNew(); break;
        case 234: _t->slotFileOpen(); break;
        case 235: _t->slotFileOpenExample(); break;
        case 236: _t->slotFileOpenTemplate(); break;
        case 237: _t->slotFileOpenRecent(); break;
        case 238: _t->slotFileSaveAs(); break;
        case 239: _t->slotFileSaveAsTemplate(); break;
        case 240: _t->slotFileClose(); break;
        case 241: _t->slotImportProject(); break;
        case 242: _t->slotImportMIDI(); break;
        case 243: _t->slotRevertToSaved(); break;
        case 244: _t->slotImportRG21(); break;
        case 245: _t->slotImportMusicXML(); break;
        case 246: _t->slotMerge(); break;
        case 247: _t->slotMergeMIDI(); break;
        case 248: _t->slotMergeRG21(); break;
        case 249: _t->slotMergeMusicXML(); break;
        case 250: _t->slotMetronomeActivated((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 54:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 66:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 67:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 70:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 71:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 73:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 197:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RosegardenMainWindow::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::startupStatusMessage)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RosegardenMainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::documentAboutToChange)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RosegardenMainWindow::*)(RosegardenDocument * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::documentLoaded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RosegardenMainWindow::*)(const SegmentSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::segmentsSelected)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RosegardenMainWindow::*)(InstrumentId , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::pluginSelected)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RosegardenMainWindow::*)(InstrumentId , int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainWindow::pluginBypassed)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::RosegardenMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__RosegardenMainWindow.data,
    qt_meta_data_Rosegarden__RosegardenMainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::RosegardenMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::RosegardenMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__RosegardenMainWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::RosegardenMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 251)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 251;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 251)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 251;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::RosegardenMainWindow::startupStatusMessage(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::RosegardenMainWindow::documentAboutToChange()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::RosegardenMainWindow::documentLoaded(RosegardenDocument * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::RosegardenMainWindow::segmentsSelected(const SegmentSelection & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::RosegardenMainWindow::pluginSelected(InstrumentId _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::RosegardenMainWindow::pluginBypassed(InstrumentId _t1, int _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
