/****************************************************************************
** Meta object code from reading C++ file 'MatrixWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/matrix/MatrixWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MatrixWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MatrixWidget_t {
    QByteArrayData data[43];
    char stringdata0[752];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MatrixWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MatrixWidget_t qt_meta_stringdata_Rosegarden__MatrixWidget = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::MatrixWidget"
QT_MOC_LITERAL(1, 25, 11), // "toolChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 18), // "editTriggerSegment"
QT_MOC_LITERAL(4, 57, 14), // "segmentDeleted"
QT_MOC_LITERAL(5, 72, 8), // "Segment*"
QT_MOC_LITERAL(6, 81, 12), // "sceneDeleted"
QT_MOC_LITERAL(7, 94, 16), // "selectionChanged"
QT_MOC_LITERAL(8, 111, 21), // "rulerSelectionChanged"
QT_MOC_LITERAL(9, 133, 20), // "rulerSelectionUpdate"
QT_MOC_LITERAL(10, 154, 15), // "showContextHelp"
QT_MOC_LITERAL(11, 170, 19), // "slotPlayPreviewNote"
QT_MOC_LITERAL(12, 190, 7), // "segment"
QT_MOC_LITERAL(13, 198, 5), // "pitch"
QT_MOC_LITERAL(14, 204, 20), // "slotDocumentModified"
QT_MOC_LITERAL(15, 225, 10), // "slotZoomIn"
QT_MOC_LITERAL(16, 236, 11), // "slotZoomOut"
QT_MOC_LITERAL(17, 248, 16), // "slotScrollRulers"
QT_MOC_LITERAL(18, 265, 22), // "slotDispatchMousePress"
QT_MOC_LITERAL(19, 288, 23), // "const MatrixMouseEvent*"
QT_MOC_LITERAL(20, 312, 21), // "slotDispatchMouseMove"
QT_MOC_LITERAL(21, 334, 24), // "slotDispatchMouseRelease"
QT_MOC_LITERAL(22, 359, 28), // "slotDispatchMouseDoubleClick"
QT_MOC_LITERAL(23, 388, 26), // "slotPointerPositionChanged"
QT_MOC_LITERAL(24, 415, 5), // "timeT"
QT_MOC_LITERAL(25, 421, 1), // "t"
QT_MOC_LITERAL(26, 423, 21), // "slotStandardRulerDrag"
QT_MOC_LITERAL(27, 445, 26), // "slotHScrollBarRangeChanged"
QT_MOC_LITERAL(28, 472, 3), // "min"
QT_MOC_LITERAL(29, 476, 3), // "max"
QT_MOC_LITERAL(30, 480, 25), // "slotHoveredOverKeyChanged"
QT_MOC_LITERAL(31, 506, 14), // "slotKeyPressed"
QT_MOC_LITERAL(32, 521, 15), // "slotKeySelected"
QT_MOC_LITERAL(33, 537, 15), // "slotKeyReleased"
QT_MOC_LITERAL(34, 553, 29), // "slotHorizontalThumbwheelMoved"
QT_MOC_LITERAL(35, 583, 27), // "slotVerticalThumbwheelMoved"
QT_MOC_LITERAL(36, 611, 21), // "slotHVThumbwheelMoved"
QT_MOC_LITERAL(37, 633, 20), // "slotResetZoomClicked"
QT_MOC_LITERAL(38, 654, 16), // "slotPannerZoomIn"
QT_MOC_LITERAL(39, 671, 17), // "slotPannerZoomOut"
QT_MOC_LITERAL(40, 689, 23), // "slotSegmentChangerMoved"
QT_MOC_LITERAL(41, 713, 19), // "slotMouseLeavesView"
QT_MOC_LITERAL(42, 733, 18) // "slotInstrumentGone"

    },
    "Rosegarden::MatrixWidget\0toolChanged\0"
    "\0editTriggerSegment\0segmentDeleted\0"
    "Segment*\0sceneDeleted\0selectionChanged\0"
    "rulerSelectionChanged\0rulerSelectionUpdate\0"
    "showContextHelp\0slotPlayPreviewNote\0"
    "segment\0pitch\0slotDocumentModified\0"
    "slotZoomIn\0slotZoomOut\0slotScrollRulers\0"
    "slotDispatchMousePress\0const MatrixMouseEvent*\0"
    "slotDispatchMouseMove\0slotDispatchMouseRelease\0"
    "slotDispatchMouseDoubleClick\0"
    "slotPointerPositionChanged\0timeT\0t\0"
    "slotStandardRulerDrag\0slotHScrollBarRangeChanged\0"
    "min\0max\0slotHoveredOverKeyChanged\0"
    "slotKeyPressed\0slotKeySelected\0"
    "slotKeyReleased\0slotHorizontalThumbwheelMoved\0"
    "slotVerticalThumbwheelMoved\0"
    "slotHVThumbwheelMoved\0slotResetZoomClicked\0"
    "slotPannerZoomIn\0slotPannerZoomOut\0"
    "slotSegmentChangerMoved\0slotMouseLeavesView\0"
    "slotInstrumentGone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MatrixWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      33,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  179,    2, 0x06 /* Public */,
       3,    1,  182,    2, 0x06 /* Public */,
       4,    1,  185,    2, 0x06 /* Public */,
       6,    0,  188,    2, 0x06 /* Public */,
       7,    0,  189,    2, 0x06 /* Public */,
       8,    0,  190,    2, 0x06 /* Public */,
       9,    0,  191,    2, 0x06 /* Public */,
      10,    1,  192,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    2,  195,    2, 0x0a /* Public */,
      14,    1,  200,    2, 0x08 /* Private */,
      15,    0,  203,    2, 0x08 /* Private */,
      16,    0,  204,    2, 0x08 /* Private */,
      17,    0,  205,    2, 0x08 /* Private */,
      18,    1,  206,    2, 0x08 /* Private */,
      20,    1,  209,    2, 0x08 /* Private */,
      21,    1,  212,    2, 0x08 /* Private */,
      22,    1,  215,    2, 0x08 /* Private */,
      23,    1,  218,    2, 0x08 /* Private */,
      26,    1,  221,    2, 0x08 /* Private */,
      27,    2,  224,    2, 0x08 /* Private */,
      30,    1,  229,    2, 0x08 /* Private */,
      31,    2,  232,    2, 0x08 /* Private */,
      32,    2,  237,    2, 0x08 /* Private */,
      33,    2,  242,    2, 0x08 /* Private */,
      34,    1,  247,    2, 0x08 /* Private */,
      35,    1,  250,    2, 0x08 /* Private */,
      36,    1,  253,    2, 0x08 /* Private */,
      37,    0,  256,    2, 0x08 /* Private */,
      38,    0,  257,    2, 0x08 /* Private */,
      39,    0,  258,    2, 0x08 /* Private */,
      40,    1,  259,    2, 0x08 /* Private */,
      41,    0,  262,    2, 0x08 /* Private */,
      42,    0,  263,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5, QMetaType::Int,   12,   13,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   28,   29,
    QMetaType::Void, QMetaType::UInt,    2,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::MatrixWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MatrixWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->toolChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->editTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->segmentDeleted((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 3: _t->sceneDeleted(); break;
        case 4: _t->selectionChanged(); break;
        case 5: _t->rulerSelectionChanged(); break;
        case 6: _t->rulerSelectionUpdate(); break;
        case 7: _t->showContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->slotPlayPreviewNote((*reinterpret_cast< Segment*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotZoomIn(); break;
        case 11: _t->slotZoomOut(); break;
        case 12: _t->slotScrollRulers(); break;
        case 13: _t->slotDispatchMousePress((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 14: _t->slotDispatchMouseMove((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 15: _t->slotDispatchMouseRelease((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 16: _t->slotDispatchMouseDoubleClick((*reinterpret_cast< const MatrixMouseEvent*(*)>(_a[1]))); break;
        case 17: _t->slotPointerPositionChanged((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 18: _t->slotStandardRulerDrag((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 19: _t->slotHScrollBarRangeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 20: _t->slotHoveredOverKeyChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 21: _t->slotKeyPressed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 22: _t->slotKeySelected((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 23: _t->slotKeyReleased((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 24: _t->slotHorizontalThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->slotVerticalThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->slotHVThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->slotResetZoomClicked(); break;
        case 28: _t->slotPannerZoomIn(); break;
        case 29: _t->slotPannerZoomOut(); break;
        case 30: _t->slotSegmentChangerMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->slotMouseLeavesView(); break;
        case 32: _t->slotInstrumentGone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MatrixWidget::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::toolChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::editTriggerSegment)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)(Segment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::segmentDeleted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::sceneDeleted)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::selectionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::rulerSelectionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::rulerSelectionUpdate)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (MatrixWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixWidget::showContextHelp)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MatrixWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MatrixWidget.data,
    qt_meta_data_Rosegarden__MatrixWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MatrixWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MatrixWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MatrixWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return QWidget::qt_metacast(_clname);
}

int Rosegarden::MatrixWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::MatrixWidget::toolChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::MatrixWidget::editTriggerSegment(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::MatrixWidget::segmentDeleted(Segment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::MatrixWidget::sceneDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::MatrixWidget::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::MatrixWidget::rulerSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::MatrixWidget::rulerSelectionUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::MatrixWidget::showContextHelp(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
