/****************************************************************************
** Meta object code from reading C++ file 'CommandHistory.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/document/CommandHistory.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CommandHistory.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__CommandHistory_t {
    QByteArrayData data[17];
    char stringdata0[245];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__CommandHistory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__CommandHistory_t qt_meta_stringdata_Rosegarden__CommandHistory = {
    {
QT_MOC_LITERAL(0, 0, 26), // "Rosegarden::CommandHistory"
QT_MOC_LITERAL(1, 27, 20), // "updateLinkedSegments"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 8), // "Command*"
QT_MOC_LITERAL(4, 58, 15), // "commandExecuted"
QT_MOC_LITERAL(5, 74, 17), // "commandUnexecuted"
QT_MOC_LITERAL(6, 92, 16), // "documentRestored"
QT_MOC_LITERAL(7, 109, 21), // "aboutToExecuteCommand"
QT_MOC_LITERAL(8, 131, 13), // "commandUndone"
QT_MOC_LITERAL(9, 145, 13), // "commandRedone"
QT_MOC_LITERAL(10, 159, 24), // "commandExecutedInitially"
QT_MOC_LITERAL(11, 184, 13), // "documentSaved"
QT_MOC_LITERAL(12, 198, 4), // "undo"
QT_MOC_LITERAL(13, 203, 4), // "redo"
QT_MOC_LITERAL(14, 208, 13), // "undoActivated"
QT_MOC_LITERAL(15, 222, 8), // "QAction*"
QT_MOC_LITERAL(16, 231, 13) // "redoActivated"

    },
    "Rosegarden::CommandHistory\0"
    "updateLinkedSegments\0\0Command*\0"
    "commandExecuted\0commandUnexecuted\0"
    "documentRestored\0aboutToExecuteCommand\0"
    "commandUndone\0commandRedone\0"
    "commandExecutedInitially\0documentSaved\0"
    "undo\0redo\0undoActivated\0QAction*\0"
    "redoActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__CommandHistory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    0,   82,    2, 0x06 /* Public */,
       5,    1,   83,    2, 0x06 /* Public */,
       6,    0,   86,    2, 0x06 /* Public */,
       7,    0,   87,    2, 0x06 /* Public */,
       8,    0,   88,    2, 0x06 /* Public */,
       9,    0,   89,    2, 0x06 /* Public */,
      10,    0,   90,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   91,    2, 0x0a /* Public */,
      12,    0,   92,    2, 0x0a /* Public */,
      13,    0,   93,    2, 0x0a /* Public */,
      14,    1,   94,    2, 0x09 /* Protected */,
      16,    1,   97,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 15,    2,

       0        // eod
};

void Rosegarden::CommandHistory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CommandHistory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateLinkedSegments((*reinterpret_cast< Command*(*)>(_a[1]))); break;
        case 1: _t->commandExecuted(); break;
        case 2: _t->commandUnexecuted((*reinterpret_cast< Command*(*)>(_a[1]))); break;
        case 3: _t->documentRestored(); break;
        case 4: _t->aboutToExecuteCommand(); break;
        case 5: _t->commandUndone(); break;
        case 6: _t->commandRedone(); break;
        case 7: _t->commandExecutedInitially(); break;
        case 8: _t->documentSaved(); break;
        case 9: _t->undo(); break;
        case 10: _t->redo(); break;
        case 11: _t->undoActivated((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 12: _t->redoActivated((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CommandHistory::*)(Command * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::updateLinkedSegments)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::commandExecuted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)(Command * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::commandUnexecuted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::documentRestored)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::aboutToExecuteCommand)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::commandUndone)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::commandRedone)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (CommandHistory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandHistory::commandExecutedInitially)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::CommandHistory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__CommandHistory.data,
    qt_meta_data_Rosegarden__CommandHistory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::CommandHistory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::CommandHistory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__CommandHistory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::CommandHistory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::CommandHistory::updateLinkedSegments(Command * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::CommandHistory::commandExecuted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::CommandHistory::commandUnexecuted(Command * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::CommandHistory::documentRestored()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::CommandHistory::aboutToExecuteCommand()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::CommandHistory::commandUndone()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::CommandHistory::commandRedone()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::CommandHistory::commandExecutedInitially()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
