/****************************************************************************
** Meta object code from reading C++ file 'ShortcutDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/ShortcutDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ShortcutDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__ShortcutDialog_t {
    QByteArrayData data[18];
    char stringdata0[272];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__ShortcutDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__ShortcutDialog_t qt_meta_stringdata_Rosegarden__ShortcutDialog = {
    {
QT_MOC_LITERAL(0, 0, 26), // "Rosegarden::ShortcutDialog"
QT_MOC_LITERAL(1, 27, 17), // "slotFilterChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 4), // "text"
QT_MOC_LITERAL(4, 51, 20), // "slotSelectionChanged"
QT_MOC_LITERAL(5, 72, 14), // "QItemSelection"
QT_MOC_LITERAL(6, 87, 8), // "selected"
QT_MOC_LITERAL(7, 96, 10), // "deselected"
QT_MOC_LITERAL(8, 107, 24), // "slotResetSelectedClicked"
QT_MOC_LITERAL(9, 132, 7), // "checked"
QT_MOC_LITERAL(10, 140, 26), // "slotRemoveShortcutsClicked"
QT_MOC_LITERAL(11, 167, 19), // "slotResetAllClicked"
QT_MOC_LITERAL(12, 187, 19), // "slotKeyboardChanged"
QT_MOC_LITERAL(13, 207, 5), // "index"
QT_MOC_LITERAL(14, 213, 23), // "slotWarningsWhenChanged"
QT_MOC_LITERAL(15, 237, 15), // "slotDataChanged"
QT_MOC_LITERAL(16, 253, 11), // "QModelIndex"
QT_MOC_LITERAL(17, 265, 6) // "reject"

    },
    "Rosegarden::ShortcutDialog\0slotFilterChanged\0"
    "\0text\0slotSelectionChanged\0QItemSelection\0"
    "selected\0deselected\0slotResetSelectedClicked\0"
    "checked\0slotRemoveShortcutsClicked\0"
    "slotResetAllClicked\0slotKeyboardChanged\0"
    "index\0slotWarningsWhenChanged\0"
    "slotDataChanged\0QModelIndex\0reject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__ShortcutDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x08 /* Private */,
       4,    2,   62,    2, 0x08 /* Private */,
       8,    1,   67,    2, 0x08 /* Private */,
      10,    1,   70,    2, 0x08 /* Private */,
      11,    1,   73,    2, 0x08 /* Private */,
      12,    1,   76,    2, 0x08 /* Private */,
      14,    1,   79,    2, 0x08 /* Private */,
      15,    2,   82,    2, 0x08 /* Private */,
      17,    0,   87,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 5,    6,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 16,    2,    2,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::ShortcutDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ShortcutDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotFilterChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->slotSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 2: _t->slotResetSelectedClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotRemoveShortcutsClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->slotResetAllClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotKeyboardChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotWarningsWhenChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 8: _t->reject(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::ShortcutDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__ShortcutDialog.data,
    qt_meta_data_Rosegarden__ShortcutDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::ShortcutDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::ShortcutDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__ShortcutDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::ShortcutDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
