/****************************************************************************
** Meta object code from reading C++ file 'TempoDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/TempoDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TempoDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TempoDialog_t {
    QByteArrayData data[20];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TempoDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TempoDialog_t qt_meta_stringdata_Rosegarden__TempoDialog = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Rosegarden::TempoDialog"
QT_MOC_LITERAL(1, 24, 11), // "changeTempo"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 5), // "timeT"
QT_MOC_LITERAL(4, 43, 4), // "time"
QT_MOC_LITERAL(5, 48, 6), // "tempoT"
QT_MOC_LITERAL(6, 55, 5), // "tempo"
QT_MOC_LITERAL(7, 61, 6), // "target"
QT_MOC_LITERAL(8, 68, 30), // "TempoDialog::TempoDialogAction"
QT_MOC_LITERAL(9, 99, 6), // "action"
QT_MOC_LITERAL(10, 106, 6), // "accept"
QT_MOC_LITERAL(11, 113, 17), // "slotActionChanged"
QT_MOC_LITERAL(12, 131, 16), // "slotTempoChanged"
QT_MOC_LITERAL(13, 148, 24), // "slotTempoConstantClicked"
QT_MOC_LITERAL(14, 173, 26), // "slotTempoRampToNextClicked"
QT_MOC_LITERAL(15, 200, 28), // "slotTempoRampToTargetClicked"
QT_MOC_LITERAL(16, 229, 14), // "slotTapClicked"
QT_MOC_LITERAL(17, 244, 17), // "slotHelpRequested"
QT_MOC_LITERAL(18, 262, 11), // "slotIsValid"
QT_MOC_LITERAL(19, 274, 5) // "valid"

    },
    "Rosegarden::TempoDialog\0changeTempo\0"
    "\0timeT\0time\0tempoT\0tempo\0target\0"
    "TempoDialog::TempoDialogAction\0action\0"
    "accept\0slotActionChanged\0slotTempoChanged\0"
    "slotTempoConstantClicked\0"
    "slotTempoRampToNextClicked\0"
    "slotTempoRampToTargetClicked\0"
    "slotTapClicked\0slotHelpRequested\0"
    "slotIsValid\0valid"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TempoDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   73,    2, 0x08 /* Private */,
      11,    0,   74,    2, 0x08 /* Private */,
      12,    1,   75,    2, 0x08 /* Private */,
      13,    0,   78,    2, 0x08 /* Private */,
      14,    0,   79,    2, 0x08 /* Private */,
      15,    0,   80,    2, 0x08 /* Private */,
      16,    0,   81,    2, 0x08 /* Private */,
      17,    0,   82,    2, 0x08 /* Private */,
      18,    1,   83,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5, 0x80000000 | 5, 0x80000000 | 8,    4,    6,    7,    9,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   19,

       0        // eod
};

void Rosegarden::TempoDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TempoDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changeTempo((*reinterpret_cast< timeT(*)>(_a[1])),(*reinterpret_cast< tempoT(*)>(_a[2])),(*reinterpret_cast< tempoT(*)>(_a[3])),(*reinterpret_cast< TempoDialog::TempoDialogAction(*)>(_a[4]))); break;
        case 1: _t->accept(); break;
        case 2: _t->slotActionChanged(); break;
        case 3: _t->slotTempoChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->slotTempoConstantClicked(); break;
        case 5: _t->slotTempoRampToNextClicked(); break;
        case 6: _t->slotTempoRampToTargetClicked(); break;
        case 7: _t->slotTapClicked(); break;
        case 8: _t->slotHelpRequested(); break;
        case 9: _t->slotIsValid((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TempoDialog::*)(timeT , tempoT , tempoT , TempoDialog::TempoDialogAction );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TempoDialog::changeTempo)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TempoDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TempoDialog.data,
    qt_meta_data_Rosegarden__TempoDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TempoDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TempoDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TempoDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::TempoDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TempoDialog::changeTempo(timeT _t1, tempoT _t2, tempoT _t3, TempoDialog::TempoDialogAction _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
