/****************************************************************************
** Meta object code from reading C++ file 'EventListEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/eventlist/EventListEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EventListEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__EventListEditor_t {
    QByteArrayData data[33];
    char stringdata0[518];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__EventListEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__EventListEditor_t qt_meta_stringdata_Rosegarden__EventListEditor = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::EventListEditor"
QT_MOC_LITERAL(1, 28, 18), // "editTriggerSegment"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 2), // "id"
QT_MOC_LITERAL(4, 51, 14), // "slotEditInsert"
QT_MOC_LITERAL(5, 66, 14), // "slotEditDelete"
QT_MOC_LITERAL(6, 81, 13), // "slotEditEvent"
QT_MOC_LITERAL(7, 95, 11), // "slotEditCut"
QT_MOC_LITERAL(8, 107, 12), // "slotEditCopy"
QT_MOC_LITERAL(9, 120, 13), // "slotEditPaste"
QT_MOC_LITERAL(10, 134, 15), // "slotEditPasteAt"
QT_MOC_LITERAL(11, 150, 13), // "slotSelectAll"
QT_MOC_LITERAL(12, 164, 18), // "slotClearSelection"
QT_MOC_LITERAL(13, 183, 15), // "slotMusicalTime"
QT_MOC_LITERAL(14, 199, 12), // "slotRealTime"
QT_MOC_LITERAL(15, 212, 11), // "slotRawTime"
QT_MOC_LITERAL(16, 224, 17), // "slotHelpRequested"
QT_MOC_LITERAL(17, 242, 13), // "slotHelpAbout"
QT_MOC_LITERAL(18, 256, 17), // "slotFilterClicked"
QT_MOC_LITERAL(19, 274, 19), // "slotAllOnOffClicked"
QT_MOC_LITERAL(20, 294, 21), // "slotCellDoubleClicked"
QT_MOC_LITERAL(21, 316, 3), // "row"
QT_MOC_LITERAL(22, 320, 6), // "column"
QT_MOC_LITERAL(23, 327, 24), // "slotItemSelectionChanged"
QT_MOC_LITERAL(24, 352, 15), // "slotContextMenu"
QT_MOC_LITERAL(25, 368, 21), // "slotOpenInEventEditor"
QT_MOC_LITERAL(26, 390, 7), // "checked"
QT_MOC_LITERAL(27, 398, 24), // "slotEditTriggeredSegment"
QT_MOC_LITERAL(28, 423, 19), // "slotEditTriggerName"
QT_MOC_LITERAL(29, 443, 20), // "slotEditTriggerPitch"
QT_MOC_LITERAL(30, 464, 23), // "slotEditTriggerVelocity"
QT_MOC_LITERAL(31, 488, 20), // "slotDocumentModified"
QT_MOC_LITERAL(32, 509, 8) // "modified"

    },
    "Rosegarden::EventListEditor\0"
    "editTriggerSegment\0\0id\0slotEditInsert\0"
    "slotEditDelete\0slotEditEvent\0slotEditCut\0"
    "slotEditCopy\0slotEditPaste\0slotEditPasteAt\0"
    "slotSelectAll\0slotClearSelection\0"
    "slotMusicalTime\0slotRealTime\0slotRawTime\0"
    "slotHelpRequested\0slotHelpAbout\0"
    "slotFilterClicked\0slotAllOnOffClicked\0"
    "slotCellDoubleClicked\0row\0column\0"
    "slotItemSelectionChanged\0slotContextMenu\0"
    "slotOpenInEventEditor\0checked\0"
    "slotEditTriggeredSegment\0slotEditTriggerName\0"
    "slotEditTriggerPitch\0slotEditTriggerVelocity\0"
    "slotDocumentModified\0modified"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__EventListEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  147,    2, 0x08 /* Private */,
       5,    0,  148,    2, 0x08 /* Private */,
       6,    0,  149,    2, 0x08 /* Private */,
       7,    0,  150,    2, 0x08 /* Private */,
       8,    0,  151,    2, 0x08 /* Private */,
       9,    0,  152,    2, 0x08 /* Private */,
      10,    0,  153,    2, 0x08 /* Private */,
      11,    0,  154,    2, 0x08 /* Private */,
      12,    0,  155,    2, 0x08 /* Private */,
      13,    0,  156,    2, 0x08 /* Private */,
      14,    0,  157,    2, 0x08 /* Private */,
      15,    0,  158,    2, 0x08 /* Private */,
      16,    0,  159,    2, 0x08 /* Private */,
      17,    0,  160,    2, 0x08 /* Private */,
      18,    1,  161,    2, 0x08 /* Private */,
      19,    1,  164,    2, 0x08 /* Private */,
      20,    2,  167,    2, 0x08 /* Private */,
      23,    0,  172,    2, 0x08 /* Private */,
      24,    1,  173,    2, 0x08 /* Private */,
      25,    1,  176,    2, 0x08 /* Private */,
      27,    1,  179,    2, 0x08 /* Private */,
      28,    0,  182,    2, 0x08 /* Private */,
      29,    0,  183,    2, 0x08 /* Private */,
      30,    0,  184,    2, 0x08 /* Private */,
      31,    1,  185,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   21,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   32,

       0        // eod
};

void Rosegarden::EventListEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EventListEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->slotEditInsert(); break;
        case 2: _t->slotEditDelete(); break;
        case 3: _t->slotEditEvent(); break;
        case 4: _t->slotEditCut(); break;
        case 5: _t->slotEditCopy(); break;
        case 6: _t->slotEditPaste(); break;
        case 7: _t->slotEditPasteAt(); break;
        case 8: _t->slotSelectAll(); break;
        case 9: _t->slotClearSelection(); break;
        case 10: _t->slotMusicalTime(); break;
        case 11: _t->slotRealTime(); break;
        case 12: _t->slotRawTime(); break;
        case 13: _t->slotHelpRequested(); break;
        case 14: _t->slotHelpAbout(); break;
        case 15: _t->slotFilterClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->slotAllOnOffClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->slotCellDoubleClicked((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 18: _t->slotItemSelectionChanged(); break;
        case 19: _t->slotContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 20: _t->slotOpenInEventEditor((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->slotEditTriggeredSegment((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->slotEditTriggerName(); break;
        case 23: _t->slotEditTriggerPitch(); break;
        case 24: _t->slotEditTriggerVelocity(); break;
        case 25: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EventListEditor::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EventListEditor::editTriggerSegment)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::EventListEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<EditViewBase::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__EventListEditor.data,
    qt_meta_data_Rosegarden__EventListEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::EventListEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::EventListEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__EventListEditor.stringdata0))
        return static_cast<void*>(this);
    return EditViewBase::qt_metacast(_clname);
}

int Rosegarden::EventListEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EditViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::EventListEditor::editTriggerSegment(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
