/****************************************************************************
** Meta object code from reading C++ file 'TempoAndTimeSignatureEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/tempo/TempoAndTimeSignatureEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TempoAndTimeSignatureEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor_t {
    QByteArrayData data[21];
    char stringdata0[341];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor_t qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor = {
    {
QT_MOC_LITERAL(0, 0, 39), // "Rosegarden::TempoAndTimeSigna..."
QT_MOC_LITERAL(1, 40, 7), // "closing"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 20), // "slotDocumentModified"
QT_MOC_LITERAL(4, 70, 8), // "modified"
QT_MOC_LITERAL(5, 79, 14), // "slotEditDelete"
QT_MOC_LITERAL(6, 94, 18), // "slotAddTempoChange"
QT_MOC_LITERAL(7, 113, 26), // "slotAddTimeSignatureChange"
QT_MOC_LITERAL(8, 140, 12), // "slotEditItem"
QT_MOC_LITERAL(9, 153, 15), // "slotPopupEditor"
QT_MOC_LITERAL(10, 169, 3), // "row"
QT_MOC_LITERAL(11, 173, 3), // "col"
QT_MOC_LITERAL(12, 177, 13), // "slotSelectAll"
QT_MOC_LITERAL(13, 191, 18), // "slotClearSelection"
QT_MOC_LITERAL(14, 210, 20), // "slotViewMusicalTimes"
QT_MOC_LITERAL(15, 231, 17), // "slotViewRealTimes"
QT_MOC_LITERAL(16, 249, 16), // "slotViewRawTimes"
QT_MOC_LITERAL(17, 266, 17), // "slotHelpRequested"
QT_MOC_LITERAL(18, 284, 13), // "slotHelpAbout"
QT_MOC_LITERAL(19, 298, 17), // "slotFilterClicked"
QT_MOC_LITERAL(20, 316, 24) // "slotItemSelectionChanged"

    },
    "Rosegarden::TempoAndTimeSignatureEditor\0"
    "closing\0\0slotDocumentModified\0modified\0"
    "slotEditDelete\0slotAddTempoChange\0"
    "slotAddTimeSignatureChange\0slotEditItem\0"
    "slotPopupEditor\0row\0col\0slotSelectAll\0"
    "slotClearSelection\0slotViewMusicalTimes\0"
    "slotViewRealTimes\0slotViewRawTimes\0"
    "slotHelpRequested\0slotHelpAbout\0"
    "slotFilterClicked\0slotItemSelectionChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TempoAndTimeSignatureEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   94,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   95,    2, 0x08 /* Private */,
       5,    0,   98,    2, 0x08 /* Private */,
       6,    0,   99,    2, 0x08 /* Private */,
       7,    0,  100,    2, 0x08 /* Private */,
       8,    0,  101,    2, 0x08 /* Private */,
       9,    2,  102,    2, 0x08 /* Private */,
      12,    0,  107,    2, 0x08 /* Private */,
      13,    0,  108,    2, 0x08 /* Private */,
      14,    0,  109,    2, 0x08 /* Private */,
      15,    0,  110,    2, 0x08 /* Private */,
      16,    0,  111,    2, 0x08 /* Private */,
      17,    0,  112,    2, 0x08 /* Private */,
      18,    0,  113,    2, 0x08 /* Private */,
      19,    1,  114,    2, 0x08 /* Private */,
      20,    0,  117,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::TempoAndTimeSignatureEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TempoAndTimeSignatureEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closing(); break;
        case 1: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->slotEditDelete(); break;
        case 3: _t->slotAddTempoChange(); break;
        case 4: _t->slotAddTimeSignatureChange(); break;
        case 5: _t->slotEditItem(); break;
        case 6: _t->slotPopupEditor((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->slotSelectAll(); break;
        case 8: _t->slotClearSelection(); break;
        case 9: _t->slotViewMusicalTimes(); break;
        case 10: _t->slotViewRealTimes(); break;
        case 11: _t->slotViewRawTimes(); break;
        case 12: _t->slotHelpRequested(); break;
        case 13: _t->slotHelpAbout(); break;
        case 14: _t->slotFilterClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->slotItemSelectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TempoAndTimeSignatureEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TempoAndTimeSignatureEditor::closing)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TempoAndTimeSignatureEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<EditViewBase::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor.data,
    qt_meta_data_Rosegarden__TempoAndTimeSignatureEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TempoAndTimeSignatureEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TempoAndTimeSignatureEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TempoAndTimeSignatureEditor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return EditViewBase::qt_metacast(_clname);
}

int Rosegarden::TempoAndTimeSignatureEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EditViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::TempoAndTimeSignatureEditor::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
