/****************************************************************************
** Meta object code from reading C++ file 'AudioInstrumentParameterPanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/parameters/AudioInstrumentParameterPanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AudioInstrumentParameterPanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel_t {
    QByteArrayData data[28];
    char stringdata0[415];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel_t qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel = {
    {
QT_MOC_LITERAL(0, 0, 41), // "Rosegarden::AudioInstrumentPa..."
QT_MOC_LITERAL(1, 42, 20), // "slotSelectAudioLevel"
QT_MOC_LITERAL(2, 63, 0), // ""
QT_MOC_LITERAL(3, 64, 2), // "dB"
QT_MOC_LITERAL(4, 67, 26), // "slotSelectAudioRecordLevel"
QT_MOC_LITERAL(5, 94, 17), // "slotAudioChannels"
QT_MOC_LITERAL(6, 112, 8), // "channels"
QT_MOC_LITERAL(7, 121, 16), // "slotSelectPlugin"
QT_MOC_LITERAL(8, 138, 5), // "index"
QT_MOC_LITERAL(9, 144, 10), // "slotSetPan"
QT_MOC_LITERAL(10, 155, 3), // "pan"
QT_MOC_LITERAL(11, 159, 18), // "slotPluginSelected"
QT_MOC_LITERAL(12, 178, 12), // "InstrumentId"
QT_MOC_LITERAL(13, 191, 12), // "instrumentId"
QT_MOC_LITERAL(14, 204, 6), // "plugin"
QT_MOC_LITERAL(15, 211, 18), // "slotPluginBypassed"
QT_MOC_LITERAL(16, 230, 11), // "pluginIndex"
QT_MOC_LITERAL(17, 242, 2), // "bp"
QT_MOC_LITERAL(18, 245, 22), // "slotSynthButtonClicked"
QT_MOC_LITERAL(19, 268, 25), // "slotSynthGUIButtonClicked"
QT_MOC_LITERAL(20, 294, 16), // "slotLabelClicked"
QT_MOC_LITERAL(21, 311, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(22, 330, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(23, 350, 20), // "slotDocumentModified"
QT_MOC_LITERAL(24, 371, 17), // "slotControlChange"
QT_MOC_LITERAL(25, 389, 11), // "Instrument*"
QT_MOC_LITERAL(26, 401, 10), // "instrument"
QT_MOC_LITERAL(27, 412, 2) // "cc"

    },
    "Rosegarden::AudioInstrumentParameterPanel\0"
    "slotSelectAudioLevel\0\0dB\0"
    "slotSelectAudioRecordLevel\0slotAudioChannels\0"
    "channels\0slotSelectPlugin\0index\0"
    "slotSetPan\0pan\0slotPluginSelected\0"
    "InstrumentId\0instrumentId\0plugin\0"
    "slotPluginBypassed\0pluginIndex\0bp\0"
    "slotSynthButtonClicked\0slotSynthGUIButtonClicked\0"
    "slotLabelClicked\0slotDocumentLoaded\0"
    "RosegardenDocument*\0slotDocumentModified\0"
    "slotControlChange\0Instrument*\0instrument\0"
    "cc"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__AudioInstrumentParameterPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x0a /* Public */,
       4,    1,   82,    2, 0x0a /* Public */,
       5,    1,   85,    2, 0x0a /* Public */,
       7,    1,   88,    2, 0x0a /* Public */,
       9,    1,   91,    2, 0x0a /* Public */,
      11,    3,   94,    2, 0x0a /* Public */,
      15,    3,  101,    2, 0x0a /* Public */,
      18,    0,  108,    2, 0x0a /* Public */,
      19,    0,  109,    2, 0x0a /* Public */,
      20,    0,  110,    2, 0x08 /* Private */,
      21,    1,  111,    2, 0x08 /* Private */,
      23,    1,  114,    2, 0x08 /* Private */,
      24,    2,  117,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Float,    3,
    QMetaType::Void, QMetaType::Float,    3,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Float,   10,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int, QMetaType::Int,   13,    8,   14,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int, QMetaType::Bool,   13,   16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 25, QMetaType::Int,   26,   27,

       0        // eod
};

void Rosegarden::AudioInstrumentParameterPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioInstrumentParameterPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotSelectAudioLevel((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 1: _t->slotSelectAudioRecordLevel((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 2: _t->slotAudioChannels((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->slotSelectPlugin((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotSetPan((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 5: _t->slotPluginSelected((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->slotPluginBypassed((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 7: _t->slotSynthButtonClicked(); break;
        case 8: _t->slotSynthGUIButtonClicked(); break;
        case 9: _t->slotLabelClicked(); break;
        case 10: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 11: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotControlChange((*reinterpret_cast< Instrument*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Instrument* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::AudioInstrumentParameterPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<InstrumentParameterPanel::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel.data,
    qt_meta_data_Rosegarden__AudioInstrumentParameterPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::AudioInstrumentParameterPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::AudioInstrumentParameterPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__AudioInstrumentParameterPanel.stringdata0))
        return static_cast<void*>(this);
    return InstrumentParameterPanel::qt_metacast(_clname);
}

int Rosegarden::AudioInstrumentParameterPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = InstrumentParameterPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
