/****************************************************************************
** Meta object code from reading C++ file 'Panner.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/widgets/Panner.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Panner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__Panner_t {
    QByteArrayData data[13];
    char stringdata0[167];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__Panner_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__Panner_t qt_meta_stringdata_Rosegarden__Panner = {
    {
QT_MOC_LITERAL(0, 0, 18), // "Rosegarden::Panner"
QT_MOC_LITERAL(1, 19, 17), // "pannedRectChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 13), // "pannerChanged"
QT_MOC_LITERAL(4, 52, 6), // "zoomIn"
QT_MOC_LITERAL(5, 59, 7), // "zoomOut"
QT_MOC_LITERAL(6, 67, 17), // "slotSetPannedRect"
QT_MOC_LITERAL(7, 85, 23), // "slotShowPositionPointer"
QT_MOC_LITERAL(8, 109, 1), // "x"
QT_MOC_LITERAL(9, 111, 3), // "top"
QT_MOC_LITERAL(10, 115, 6), // "height"
QT_MOC_LITERAL(11, 122, 23), // "slotHidePositionPointer"
QT_MOC_LITERAL(12, 146, 20) // "slotSceneRectChanged"

    },
    "Rosegarden::Panner\0pannedRectChanged\0"
    "\0pannerChanged\0zoomIn\0zoomOut\0"
    "slotSetPannedRect\0slotShowPositionPointer\0"
    "x\0top\0height\0slotHidePositionPointer\0"
    "slotSceneRectChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__Panner[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       3,    1,   62,    2, 0x06 /* Public */,
       4,    0,   65,    2, 0x06 /* Public */,
       5,    0,   66,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   67,    2, 0x0a /* Public */,
       7,    1,   70,    2, 0x0a /* Public */,
       7,    2,   73,    2, 0x0a /* Public */,
      11,    0,   78,    2, 0x0a /* Public */,
      12,    1,   79,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QRectF,    2,
    QMetaType::Void, QMetaType::QRectF,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QRectF,    2,
    QMetaType::Void, QMetaType::Float,    8,
    QMetaType::Void, QMetaType::QPointF, QMetaType::Float,    9,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRectF,    2,

       0        // eod
};

void Rosegarden::Panner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Panner *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pannedRectChanged((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 1: _t->pannerChanged((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 2: _t->zoomIn(); break;
        case 3: _t->zoomOut(); break;
        case 4: _t->slotSetPannedRect((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 5: _t->slotShowPositionPointer((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 6: _t->slotShowPositionPointer((*reinterpret_cast< QPointF(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 7: _t->slotHidePositionPointer(); break;
        case 8: _t->slotSceneRectChanged((*reinterpret_cast< const QRectF(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Panner::*)(QRectF );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panner::pannedRectChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Panner::*)(QRectF );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panner::pannerChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Panner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panner::zoomIn)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Panner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panner::zoomOut)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::Panner::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__Panner.data,
    qt_meta_data_Rosegarden__Panner,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::Panner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::Panner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__Panner.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int Rosegarden::Panner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::Panner::pannedRectChanged(QRectF _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::Panner::pannerChanged(QRectF _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::Panner::zoomIn()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::Panner::zoomOut()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
