/****************************************************************************
** Meta object code from reading C++ file 'SegmentParameterBox.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/parameters/SegmentParameterBox.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SegmentParameterBox.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__SegmentParameterBox_t {
    QByteArrayData data[21];
    char stringdata0[377];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__SegmentParameterBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__SegmentParameterBox_t qt_meta_stringdata_Rosegarden__SegmentParameterBox = {
    {
QT_MOC_LITERAL(0, 0, 31), // "Rosegarden::SegmentParameterBox"
QT_MOC_LITERAL(1, 32, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(4, 72, 3), // "doc"
QT_MOC_LITERAL(5, 76, 20), // "slotDocumentModified"
QT_MOC_LITERAL(6, 97, 20), // "slotSelectionChanged"
QT_MOC_LITERAL(7, 118, 16), // "SegmentSelection"
QT_MOC_LITERAL(8, 135, 8), // "segments"
QT_MOC_LITERAL(9, 144, 20), // "slotEditSegmentLabel"
QT_MOC_LITERAL(10, 165, 17), // "slotRepeatClicked"
QT_MOC_LITERAL(11, 183, 7), // "checked"
QT_MOC_LITERAL(12, 191, 21), // "slotTransposeSelected"
QT_MOC_LITERAL(13, 213, 20), // "slotQuantizeSelected"
QT_MOC_LITERAL(14, 234, 6), // "qIndex"
QT_MOC_LITERAL(15, 241, 17), // "slotDelaySelected"
QT_MOC_LITERAL(16, 259, 17), // "slotColourChanged"
QT_MOC_LITERAL(17, 277, 21), // "slotDocColoursChanged"
QT_MOC_LITERAL(18, 299, 30), // "slotExcludeFromPrintingClicked"
QT_MOC_LITERAL(19, 330, 23), // "slotChangeLinkTranspose"
QT_MOC_LITERAL(20, 354, 22) // "slotResetLinkTranspose"

    },
    "Rosegarden::SegmentParameterBox\0"
    "slotDocumentLoaded\0\0RosegardenDocument*\0"
    "doc\0slotDocumentModified\0slotSelectionChanged\0"
    "SegmentSelection\0segments\0"
    "slotEditSegmentLabel\0slotRepeatClicked\0"
    "checked\0slotTransposeSelected\0"
    "slotQuantizeSelected\0qIndex\0"
    "slotDelaySelected\0slotColourChanged\0"
    "slotDocColoursChanged\0"
    "slotExcludeFromPrintingClicked\0"
    "slotChangeLinkTranspose\0slotResetLinkTranspose"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__SegmentParameterBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x08 /* Private */,
       5,    1,   82,    2, 0x08 /* Private */,
       6,    1,   85,    2, 0x08 /* Private */,
       9,    0,   88,    2, 0x08 /* Private */,
      10,    1,   89,    2, 0x08 /* Private */,
      12,    1,   92,    2, 0x08 /* Private */,
      13,    1,   95,    2, 0x08 /* Private */,
      15,    1,   98,    2, 0x08 /* Private */,
      16,    1,  101,    2, 0x08 /* Private */,
      17,    0,  104,    2, 0x08 /* Private */,
      18,    1,  105,    2, 0x08 /* Private */,
      19,    0,  108,    2, 0x08 /* Private */,
      20,    0,  109,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::SegmentParameterBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SegmentParameterBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 1: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->slotSelectionChanged((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 3: _t->slotEditSegmentLabel(); break;
        case 4: _t->slotRepeatClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotTransposeSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotQuantizeSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotDelaySelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotColourChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotDocColoursChanged(); break;
        case 10: _t->slotExcludeFromPrintingClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->slotChangeLinkTranspose(); break;
        case 12: _t->slotResetLinkTranspose(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::SegmentParameterBox::staticMetaObject = { {
    QMetaObject::SuperData::link<RosegardenParameterBox::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__SegmentParameterBox.data,
    qt_meta_data_Rosegarden__SegmentParameterBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::SegmentParameterBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::SegmentParameterBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__SegmentParameterBox.stringdata0))
        return static_cast<void*>(this);
    return RosegardenParameterBox::qt_metacast(_clname);
}

int Rosegarden::SegmentParameterBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = RosegardenParameterBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
