/****************************************************************************
** Meta object code from reading C++ file 'TextEventDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/TextEventDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TextEventDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__TextEventDialog_t {
    QByteArrayData data[13];
    char stringdata0[275];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__TextEventDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__TextEventDialog_t qt_meta_stringdata_Rosegarden__TextEventDialog = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::TextEventDialog"
QT_MOC_LITERAL(1, 28, 15), // "slotTextChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 15), // "slotTypeChanged"
QT_MOC_LITERAL(4, 61, 6), // "slotOK"
QT_MOC_LITERAL(5, 68, 17), // "slotHelpRequested"
QT_MOC_LITERAL(6, 86, 26), // "slotDynamicShortcutChanged"
QT_MOC_LITERAL(7, 113, 28), // "slotDirectionShortcutChanged"
QT_MOC_LITERAL(8, 142, 33), // "slotLocalDirectionShortcutCha..."
QT_MOC_LITERAL(9, 176, 24), // "slotTempoShortcutChanged"
QT_MOC_LITERAL(10, 201, 29), // "slotLocalTempoShortcutChanged"
QT_MOC_LITERAL(11, 231, 28), // "slotLilyPondDirectiveChanged"
QT_MOC_LITERAL(12, 260, 14) // "slotUpdateSize"

    },
    "Rosegarden::TextEventDialog\0slotTextChanged\0"
    "\0slotTypeChanged\0slotOK\0slotHelpRequested\0"
    "slotDynamicShortcutChanged\0"
    "slotDirectionShortcutChanged\0"
    "slotLocalDirectionShortcutChanged\0"
    "slotTempoShortcutChanged\0"
    "slotLocalTempoShortcutChanged\0"
    "slotLilyPondDirectiveChanged\0"
    "slotUpdateSize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__TextEventDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x08 /* Private */,
       3,    1,   72,    2, 0x08 /* Private */,
       4,    0,   75,    2, 0x08 /* Private */,
       5,    0,   76,    2, 0x08 /* Private */,
       6,    1,   77,    2, 0x08 /* Private */,
       7,    1,   80,    2, 0x08 /* Private */,
       8,    1,   83,    2, 0x08 /* Private */,
       9,    1,   86,    2, 0x08 /* Private */,
      10,    1,   89,    2, 0x08 /* Private */,
      11,    1,   92,    2, 0x08 /* Private */,
      12,    1,   95,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void Rosegarden::TextEventDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextEventDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->slotTypeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->slotOK(); break;
        case 3: _t->slotHelpRequested(); break;
        case 4: _t->slotDynamicShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotDirectionShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->slotLocalDirectionShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->slotTempoShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->slotLocalTempoShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->slotLilyPondDirectiveChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->slotUpdateSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::TextEventDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__TextEventDialog.data,
    qt_meta_data_Rosegarden__TextEventDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::TextEventDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::TextEventDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__TextEventDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::TextEventDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
