/****************************************************************************
** Meta object code from reading C++ file 'NotationScene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/notation/NotationScene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NotationScene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__NotationScene_t {
    QByteArrayData data[19];
    char stringdata0[308];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__NotationScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__NotationScene_t qt_meta_stringdata_Rosegarden__NotationScene = {
    {
QT_MOC_LITERAL(0, 0, 25), // "Rosegarden::NotationScene"
QT_MOC_LITERAL(1, 26, 20), // "sceneNeedsRebuilding"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 12), // "eventRemoved"
QT_MOC_LITERAL(4, 61, 6), // "Event*"
QT_MOC_LITERAL(5, 68, 18), // "selectionChangedES"
QT_MOC_LITERAL(6, 87, 15), // "EventSelection*"
QT_MOC_LITERAL(7, 103, 13), // "layoutUpdated"
QT_MOC_LITERAL(8, 117, 5), // "timeT"
QT_MOC_LITERAL(9, 123, 17), // "staffsPositionned"
QT_MOC_LITERAL(10, 141, 19), // "currentStaffChanged"
QT_MOC_LITERAL(11, 161, 25), // "currentViewSegmentChanged"
QT_MOC_LITERAL(12, 187, 12), // "ViewSegment*"
QT_MOC_LITERAL(13, 200, 22), // "hoveredOverNoteChanged"
QT_MOC_LITERAL(14, 223, 8), // "noteName"
QT_MOC_LITERAL(15, 232, 30), // "hoveredOverAbsoluteTimeChanged"
QT_MOC_LITERAL(16, 263, 4), // "time"
QT_MOC_LITERAL(17, 268, 19), // "slotMouseLeavesView"
QT_MOC_LITERAL(18, 288, 19) // "slotCommandExecuted"

    },
    "Rosegarden::NotationScene\0"
    "sceneNeedsRebuilding\0\0eventRemoved\0"
    "Event*\0selectionChangedES\0EventSelection*\0"
    "layoutUpdated\0timeT\0staffsPositionned\0"
    "currentStaffChanged\0currentViewSegmentChanged\0"
    "ViewSegment*\0hoveredOverNoteChanged\0"
    "noteName\0hoveredOverAbsoluteTimeChanged\0"
    "time\0slotMouseLeavesView\0slotCommandExecuted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__NotationScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    1,   70,    2, 0x06 /* Public */,
       5,    1,   73,    2, 0x06 /* Public */,
       7,    2,   76,    2, 0x06 /* Public */,
       9,    0,   81,    2, 0x06 /* Public */,
      10,    0,   82,    2, 0x06 /* Public */,
      11,    1,   83,    2, 0x06 /* Public */,
      13,    1,   86,    2, 0x06 /* Public */,
      15,    1,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    0,   92,    2, 0x0a /* Public */,
      18,    0,   93,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 8,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::UInt,   16,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::NotationScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotationScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneNeedsRebuilding(); break;
        case 1: _t->eventRemoved((*reinterpret_cast< Event*(*)>(_a[1]))); break;
        case 2: _t->selectionChangedES((*reinterpret_cast< EventSelection*(*)>(_a[1]))); break;
        case 3: _t->layoutUpdated((*reinterpret_cast< timeT(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 4: _t->staffsPositionned(); break;
        case 5: _t->currentStaffChanged(); break;
        case 6: _t->currentViewSegmentChanged((*reinterpret_cast< ViewSegment*(*)>(_a[1]))); break;
        case 7: _t->hoveredOverNoteChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->hoveredOverAbsoluteTimeChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 9: _t->slotMouseLeavesView(); break;
        case 10: _t->slotCommandExecuted(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NotationScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::sceneNeedsRebuilding)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(Event * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::eventRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(EventSelection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::selectionChangedES)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(timeT , timeT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::layoutUpdated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::staffsPositionned)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::currentStaffChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(ViewSegment * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::currentViewSegmentChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::hoveredOverNoteChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (NotationScene::*)(unsigned int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationScene::hoveredOverAbsoluteTimeChanged)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::NotationScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__NotationScene.data,
    qt_meta_data_Rosegarden__NotationScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::NotationScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::NotationScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__NotationScene.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int Rosegarden::NotationScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::NotationScene::sceneNeedsRebuilding()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::NotationScene::eventRemoved(Event * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::NotationScene::selectionChangedES(EventSelection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::NotationScene::layoutUpdated(timeT _t1, timeT _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::NotationScene::staffsPositionned()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::NotationScene::currentStaffChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::NotationScene::currentViewSegmentChanged(ViewSegment * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::NotationScene::hoveredOverNoteChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::NotationScene::hoveredOverAbsoluteTimeChanged(unsigned int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
