// This file is autogenerated. Changes will be overwritten.
#include "CZ4SVKUTXB/moc_AllocateChannels.cpp"
#include "CZ4SVKUTXB/moc_Instrument.cpp"
#include "CZ4SVKUTXB/moc_InstrumentStaticSignals.cpp"
#include "CZ4SVKUTXB/moc_Segment.cpp"
#include "CZ4SVKUTXB/moc_SegmentLinker.cpp"
#include "CZ4SVKUTXB/moc_Typematic.cpp"
#include "JAWNUOOXPG/moc_EventQuantizeCommand.cpp"
#include "VCBS5DYREG/moc_AdoptSegmentCommand.cpp"
#include "DV7UALDUNI/moc_CommandHistory.cpp"
#include "DV7UALDUNI/moc_CommandRegistry.cpp"
#include "DV7UALDUNI/moc_RoseXmlHandler.cpp"
#include "DV7UALDUNI/moc_RosegardenDocument.cpp"
#include "IPVUKHDIVI/moc_CompositionPosition.cpp"
#include "IPVUKHDIVI/moc_LircClient.cpp"
#include "IPVUKHDIVI/moc_LircCommander.cpp"
#include "IPVUKHDIVI/moc_RosegardenApplication.cpp"
#include "IPVUKHDIVI/moc_RosegardenMainViewWidget.cpp"
#include "IPVUKHDIVI/moc_RosegardenMainWindow.cpp"
#include "IPVUKHDIVI/moc_StartupTester.cpp"
#include "IPVUKHDIVI/moc_TranzportClient.cpp"
#include "WGF4VMHOWF/moc_AudioConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_AudioPropertiesPage.cpp"
#include "WGF4VMHOWF/moc_CommentsConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_DocumentMetaConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_GeneralConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_HeadersConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_MIDIConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_MatrixConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_NotationConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_PitchTrackerConfigurationPage.cpp"
#include "WGF4VMHOWF/moc_TabbedConfigurationPage.cpp"
#include "SX4JZQ6BRK/moc_AboutDialog.cpp"
#include "SX4JZQ6BRK/moc_AddTracksDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioFileLocationDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioManagerDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioPlayingDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioPluginConnectionDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioPluginDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioPluginParameterDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioPluginPresetDialog.cpp"
#include "SX4JZQ6BRK/moc_AudioSplitDialog.cpp"
#include "SX4JZQ6BRK/moc_BeatsBarsDialog.cpp"
#include "SX4JZQ6BRK/moc_CheckForParallelsDialog.cpp"
#include "SX4JZQ6BRK/moc_ClefDialog.cpp"
#include "SX4JZQ6BRK/moc_CommentsPopupDialog.cpp"
#include "SX4JZQ6BRK/moc_CompositionLengthDialog.cpp"
#include "SX4JZQ6BRK/moc_ConfigureDialog.cpp"
#include "SX4JZQ6BRK/moc_ConfigureDialogBase.cpp"
#include "SX4JZQ6BRK/moc_CountdownBar.cpp"
#include "SX4JZQ6BRK/moc_CountdownDialog.cpp"
#include "SX4JZQ6BRK/moc_DocumentConfigureDialog.cpp"
#include "SX4JZQ6BRK/moc_EventFilterDialog.cpp"
#include "SX4JZQ6BRK/moc_EventParameterDialog.cpp"
#include "SX4JZQ6BRK/moc_ExportDeviceDialog.cpp"
#include "SX4JZQ6BRK/moc_FileLocateDialog.cpp"
#include "SX4JZQ6BRK/moc_FileMergeDialog.cpp"
#include "SX4JZQ6BRK/moc_FloatEdit.cpp"
#include "SX4JZQ6BRK/moc_GeneratedRegionDialog.cpp"
#include "SX4JZQ6BRK/moc_IdentifyTextCodecDialog.cpp"
#include "SX4JZQ6BRK/moc_ImportDeviceDialog.cpp"
#include "SX4JZQ6BRK/moc_InsertTupletDialog.cpp"
#include "SX4JZQ6BRK/moc_InterpretDialog.cpp"
#include "SX4JZQ6BRK/moc_IntervalDialog.cpp"
#include "SX4JZQ6BRK/moc_KeySignatureDialog.cpp"
#include "SX4JZQ6BRK/moc_LibrarianDialog.cpp"
#include "SX4JZQ6BRK/moc_LilyPondOptionsDialog.cpp"
#include "SX4JZQ6BRK/moc_LyricEditDialog.cpp"
#include "SX4JZQ6BRK/moc_MakeOrnamentDialog.cpp"
#include "SX4JZQ6BRK/moc_ManageMetronomeDialog.cpp"
#include "SX4JZQ6BRK/moc_MarkerModifyDialog.cpp"
#include "SX4JZQ6BRK/moc_MusicXMLOptionsDialog.cpp"
#include "SX4JZQ6BRK/moc_OutOfProcessorPower.cpp"
#include "SX4JZQ6BRK/moc_PasteNotationDialog.cpp"
#include "SX4JZQ6BRK/moc_PitchBendSequenceDialog.cpp"
#include "SX4JZQ6BRK/moc_PitchDialog.cpp"
#include "SX4JZQ6BRK/moc_PitchPickerDialog.cpp"
#include "SX4JZQ6BRK/moc_QuantizeDialog.cpp"
#include "SX4JZQ6BRK/moc_RescaleDialog.cpp"
#include "SX4JZQ6BRK/moc_SelectBankDialog.cpp"
#include "SX4JZQ6BRK/moc_SelectDialog.cpp"
#include "SX4JZQ6BRK/moc_ShortcutDelegate.cpp"
#include "SX4JZQ6BRK/moc_ShortcutDialog.cpp"
#include "SX4JZQ6BRK/moc_ShortcutWarnDialog.cpp"
#include "SX4JZQ6BRK/moc_ShowSequencerStatusDialog.cpp"
#include "SX4JZQ6BRK/moc_SplitByPitchDialog.cpp"
#include "SX4JZQ6BRK/moc_SplitByRecordingSrcDialog.cpp"
#include "SX4JZQ6BRK/moc_TempoDialog.cpp"
#include "SX4JZQ6BRK/moc_TextEventDialog.cpp"
#include "SX4JZQ6BRK/moc_TimeDialog.cpp"
#include "SX4JZQ6BRK/moc_TimeSignatureDialog.cpp"
#include "SX4JZQ6BRK/moc_TrackLabelDialog.cpp"
#include "SX4JZQ6BRK/moc_TransportDialog.cpp"
#include "SX4JZQ6BRK/moc_TriggerSegmentDialog.cpp"
#include "SX4JZQ6BRK/moc_TupletDialog.cpp"
#include "SX4JZQ6BRK/moc_UnusedAudioSelectionDialog.cpp"
#include "SX4JZQ6BRK/moc_UseOrnamentDialog.cpp"
#include "3LAKNDBSXT/moc_EditEvent.cpp"
#include "3LAKNDBSXT/moc_EventTypeDialog.cpp"
#include "3LAKNDBSXT/moc_EventWidget.cpp"
#include "HRM3UXEAKL/moc_EventListEditor.cpp"
#include "WFQOKPZUST/moc_GuitarChordEditorDialog.cpp"
#include "WFQOKPZUST/moc_GuitarChordSelectorDialog.cpp"
#include "IQKBEBYK4N/moc_MatrixEraser.cpp"
#include "IQKBEBYK4N/moc_MatrixMover.cpp"
#include "IQKBEBYK4N/moc_MatrixPainter.cpp"
#include "IQKBEBYK4N/moc_MatrixResizer.cpp"
#include "IQKBEBYK4N/moc_MatrixScene.cpp"
#include "IQKBEBYK4N/moc_MatrixSelector.cpp"
#include "IQKBEBYK4N/moc_MatrixTool.cpp"
#include "IQKBEBYK4N/moc_MatrixToolBox.cpp"
#include "IQKBEBYK4N/moc_MatrixVelocity.cpp"
#include "IQKBEBYK4N/moc_MatrixView.cpp"
#include "IQKBEBYK4N/moc_MatrixWidget.cpp"
#include "IQKBEBYK4N/moc_PianoKeyboard.cpp"
#include "XVAGPESQ7F/moc_ClefInserter.cpp"
#include "XVAGPESQ7F/moc_FontViewFrame.cpp"
#include "XVAGPESQ7F/moc_GuitarChordInserter.cpp"
#include "XVAGPESQ7F/moc_HeadersGroup.cpp"
#include "XVAGPESQ7F/moc_NotationEraser.cpp"
#include "XVAGPESQ7F/moc_NotationScene.cpp"
#include "XVAGPESQ7F/moc_NotationSelector.cpp"
#include "XVAGPESQ7F/moc_NotationStaff.cpp"
#include "XVAGPESQ7F/moc_NotationTool.cpp"
#include "XVAGPESQ7F/moc_NotationToolBox.cpp"
#include "XVAGPESQ7F/moc_NotationVLayout.cpp"
#include "XVAGPESQ7F/moc_NotationView.cpp"
#include "XVAGPESQ7F/moc_NotationWidget.cpp"
#include "XVAGPESQ7F/moc_NoteFontViewer.cpp"
#include "XVAGPESQ7F/moc_NoteRestInserter.cpp"
#include "XVAGPESQ7F/moc_StaffHeader.cpp"
#include "XVAGPESQ7F/moc_SymbolInserter.cpp"
#include "XVAGPESQ7F/moc_TextInserter.cpp"
#include "S6RWFA53GF/moc_AudioInstrumentParameterPanel.cpp"
#include "S6RWFA53GF/moc_InstrumentParameterBox.cpp"
#include "S6RWFA53GF/moc_InstrumentParameterPanel.cpp"
#include "S6RWFA53GF/moc_MIDIInstrumentParameterPanel.cpp"
#include "S6RWFA53GF/moc_RosegardenParameterArea.cpp"
#include "S6RWFA53GF/moc_RosegardenParameterBox.cpp"
#include "S6RWFA53GF/moc_SegmentParameterBox.cpp"
#include "S6RWFA53GF/moc_TrackParameterBox.cpp"
#include "BBG343WLUY/moc_PitchGraphWidget.cpp"
#include "BBG343WLUY/moc_PitchTrackerView.cpp"
#include "J35TVOVX5J/moc_MarkerEditor.cpp"
#include "J35TVOVX5J/moc_PlayList.cpp"
#include "J35TVOVX5J/moc_PlayListDialog.cpp"
#include "J35TVOVX5J/moc_PlayListView.cpp"
#include "J35TVOVX5J/moc_TrackButtons.cpp"
#include "J35TVOVX5J/moc_TrackEditor.cpp"
#include "J35TVOVX5J/moc_TrackLabel.cpp"
#include "J35TVOVX5J/moc_TriggerSegmentManager.cpp"
#include "JGISQ4BW6I/moc_AudioPeaksGenerator.cpp"
#include "JGISQ4BW6I/moc_CompositionModelImpl.cpp"
#include "JGISQ4BW6I/moc_CompositionView.cpp"
#include "JGISQ4BW6I/moc_SegmentEraser.cpp"
#include "JGISQ4BW6I/moc_SegmentJoiner.cpp"
#include "JGISQ4BW6I/moc_SegmentMover.cpp"
#include "JGISQ4BW6I/moc_SegmentPencil.cpp"
#include "JGISQ4BW6I/moc_SegmentResizer.cpp"
#include "JGISQ4BW6I/moc_SegmentSelector.cpp"
#include "JGISQ4BW6I/moc_SegmentSplitter.cpp"
#include "JGISQ4BW6I/moc_SegmentTool.cpp"
#include "JGISQ4BW6I/moc_SegmentToolBox.cpp"
#include "KKQKDM5WND/moc_TempoAndTimeSignatureEditor.cpp"
#include "Y5U73OBC5B/moc_ActionFileParser.cpp"
#include "Y5U73OBC5B/moc_AutoScroller.cpp"
#include "Y5U73OBC5B/moc_BaseTool.cpp"
#include "Y5U73OBC5B/moc_BaseToolBox.cpp"
#include "Y5U73OBC5B/moc_EditTempoController.cpp"
#include "Y5U73OBC5B/moc_EditViewBase.cpp"
#include "Y5U73OBC5B/moc_FileSource.cpp"
#include "Y5U73OBC5B/moc_LilyPondProcessor.cpp"
#include "Y5U73OBC5B/moc_PresetHandlerDialog.cpp"
#include "Y5U73OBC5B/moc_ProjectPackager.cpp"
#include "Y5U73OBC5B/moc_RecentFiles.cpp"
#include "Y5U73OBC5B/moc_RosegardenScrollView.cpp"
#include "Y5U73OBC5B/moc_ThornStyle.cpp"
#include "QEKSZUDHWI/moc_ChordNameRuler.cpp"
#include "QEKSZUDHWI/moc_ControlEraser.cpp"
#include "QEKSZUDHWI/moc_ControlMover.cpp"
#include "QEKSZUDHWI/moc_ControlPainter.cpp"
#include "QEKSZUDHWI/moc_ControlRuler.cpp"
#include "QEKSZUDHWI/moc_ControlRulerTabBar.cpp"
#include "QEKSZUDHWI/moc_ControlRulerWidget.cpp"
#include "QEKSZUDHWI/moc_ControlSelector.cpp"
#include "QEKSZUDHWI/moc_ControlTool.cpp"
#include "QEKSZUDHWI/moc_ControlToolBox.cpp"
#include "QEKSZUDHWI/moc_ControllerEventsRuler.cpp"
#include "QEKSZUDHWI/moc_LoopRuler.cpp"
#include "QEKSZUDHWI/moc_MarkerRuler.cpp"
#include "QEKSZUDHWI/moc_PercussionPitchRuler.cpp"
#include "QEKSZUDHWI/moc_PitchRuler.cpp"
#include "QEKSZUDHWI/moc_PropertyAdjuster.cpp"
#include "QEKSZUDHWI/moc_PropertyBox.cpp"
#include "QEKSZUDHWI/moc_RawNoteRuler.cpp"
#include "QEKSZUDHWI/moc_StandardRuler.cpp"
#include "QEKSZUDHWI/moc_TempoRuler.cpp"
#include "QEKSZUDHWI/moc_TextRuler.cpp"
#include "5FGW2KV7EB/moc_ChannelManager.cpp"
#include "5FGW2KV7EB/moc_MidiFilterDialog.cpp"
#include "5FGW2KV7EB/moc_SequenceManager.cpp"
#include "CJKKGESP6Q/moc_AudioMixerWindow2.cpp"
#include "CJKKGESP6Q/moc_AudioPluginLV2GUI.cpp"
#include "CJKKGESP6Q/moc_AudioPluginLV2GUIManager.cpp"
#include "CJKKGESP6Q/moc_AudioPluginLV2GUIWindow.cpp"
#include "CJKKGESP6Q/moc_AudioStrip.cpp"
#include "CJKKGESP6Q/moc_BankEditorDialog.cpp"
#include "CJKKGESP6Q/moc_ControlEditorDialog.cpp"
#include "CJKKGESP6Q/moc_ControlParameterEditDialog.cpp"
#include "CJKKGESP6Q/moc_DeviceManagerDialog.cpp"
#include "CJKKGESP6Q/moc_LV2Gtk2.cpp"
#include "CJKKGESP6Q/moc_MidiKeyMappingEditor.cpp"
#include "CJKKGESP6Q/moc_MidiMixerWindow.cpp"
#include "CJKKGESP6Q/moc_MidiProgramsEditor.cpp"
#include "CJKKGESP6Q/moc_MidiStrip.cpp"
#include "CJKKGESP6Q/moc_MixerWindow.cpp"
#include "CJKKGESP6Q/moc_NameSetEditor.cpp"
#include "CJKKGESP6Q/moc_RemapInstrumentDialog.cpp"
#include "CJKKGESP6Q/moc_SynthPluginManagerDialog.cpp"
#include "CJKKGESP6Q/moc_TimerCallbackAssistant.cpp"
#include "WJQGMJHSF5/moc_AudioFaderBox.cpp"
#include "WJQGMJHSF5/moc_AudioListView.cpp"
#include "WJQGMJHSF5/moc_AudioRouteMenu.cpp"
#include "WJQGMJHSF5/moc_BaseTextFloat.cpp"
#include "WJQGMJHSF5/moc_CheckButton.cpp"
#include "WJQGMJHSF5/moc_CollapsingFrame.cpp"
#include "WJQGMJHSF5/moc_ColorCombo.cpp"
#include "WJQGMJHSF5/moc_DeferScrollArea.cpp"
#include "WJQGMJHSF5/moc_DiatonicPitchChooser.cpp"
#include "WJQGMJHSF5/moc_Fader.cpp"
#include "WJQGMJHSF5/moc_FileDialog.cpp"
#include "WJQGMJHSF5/moc_FontRequester.cpp"
#include "WJQGMJHSF5/moc_IconButton.cpp"
#include "WJQGMJHSF5/moc_IconStackedWidget.cpp"
#include "WJQGMJHSF5/moc_InputDialog.cpp"
#include "WJQGMJHSF5/moc_Label.cpp"
#include "WJQGMJHSF5/moc_Led.cpp"
#include "WJQGMJHSF5/moc_LedButton.cpp"
#include "WJQGMJHSF5/moc_LilyVersionAwareCheckBox.cpp"
#include "WJQGMJHSF5/moc_LineEdit.cpp"
#include "WJQGMJHSF5/moc_Panned.cpp"
#include "WJQGMJHSF5/moc_Panner.cpp"
#include "WJQGMJHSF5/moc_PitchChooser.cpp"
#include "WJQGMJHSF5/moc_PitchDragLabel.cpp"
#include "WJQGMJHSF5/moc_PluginControl.cpp"
#include "WJQGMJHSF5/moc_PluginPushButton.cpp"
#include "WJQGMJHSF5/moc_ProgressBar.cpp"
#include "WJQGMJHSF5/moc_QuantizeParameters.cpp"
#include "WJQGMJHSF5/moc_Rotary.cpp"
#include "WJQGMJHSF5/moc_ScrollBox.cpp"
#include "WJQGMJHSF5/moc_ScrollBoxDialog.cpp"
#include "WJQGMJHSF5/moc_StartupLogo.cpp"
#include "WJQGMJHSF5/moc_Thumbwheel.cpp"
#include "WJQGMJHSF5/moc_TimeWidget2.cpp"
#include "WJQGMJHSF5/moc_TristateCheckBox.cpp"
#include "WJQGMJHSF5/moc_VUMeter.cpp"
#include "WJQGMJHSF5/moc_WarningDialog.cpp"
#include "WJQGMJHSF5/moc_WarningGroupBox.cpp"
#include "WJQGMJHSF5/moc_WarningWidget.cpp"
#include "WJQGMJHSF5/moc_WheelyButton.cpp"
#include "RS2VDF5MOJ/moc_RosegardenSequencer.cpp"
#include "QXM35NU5JP/moc_AudioFileManager.cpp"
#include "QXM35NU5JP/moc_AudioFileTimeStretcher.cpp"
#include "QXM35NU5JP/moc_ExternalController.cpp"
#include "QXM35NU5JP/moc_LV2Worker.cpp"
#include "QXM35NU5JP/moc_MidiFile.cpp"
#include "QXM35NU5JP/moc_PeakFile.cpp"
#include "QXM35NU5JP/moc_PeakFileManager.cpp"
