/****************************************************************************
** Meta object code from reading C++ file 'Panned.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/widgets/Panned.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Panned.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__Panned_t {
    QByteArrayData data[13];
    char stringdata0[173];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__Panned_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__Panned_t qt_meta_stringdata_Rosegarden__Panned = {
    {
QT_MOC_LITERAL(0, 0, 18), // "Rosegarden::Panned"
QT_MOC_LITERAL(1, 19, 15), // "viewportChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 13), // "viewportScene"
QT_MOC_LITERAL(4, 50, 18), // "wheelEventReceived"
QT_MOC_LITERAL(5, 69, 12), // "QWheelEvent*"
QT_MOC_LITERAL(6, 82, 22), // "pannedContentsScrolled"
QT_MOC_LITERAL(7, 105, 11), // "mouseLeaves"
QT_MOC_LITERAL(8, 117, 6), // "zoomIn"
QT_MOC_LITERAL(9, 124, 7), // "zoomOut"
QT_MOC_LITERAL(10, 132, 15), // "slotSetViewport"
QT_MOC_LITERAL(11, 148, 21), // "slotEmulateWheelEvent"
QT_MOC_LITERAL(12, 170, 2) // "ev"

    },
    "Rosegarden::Panned\0viewportChanged\0\0"
    "viewportScene\0wheelEventReceived\0"
    "QWheelEvent*\0pannedContentsScrolled\0"
    "mouseLeaves\0zoomIn\0zoomOut\0slotSetViewport\0"
    "slotEmulateWheelEvent\0ev"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__Panned[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       6,    0,   60,    2, 0x06 /* Public */,
       7,    0,   61,    2, 0x06 /* Public */,
       8,    0,   62,    2, 0x06 /* Public */,
       9,    0,   63,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   64,    2, 0x0a /* Public */,
      11,    1,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QRectF,    3,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QRectF,    3,
    QMetaType::Void, 0x80000000 | 5,   12,

       0        // eod
};

void Rosegarden::Panned::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Panned *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->viewportChanged((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 1: _t->wheelEventReceived((*reinterpret_cast< QWheelEvent*(*)>(_a[1]))); break;
        case 2: _t->pannedContentsScrolled(); break;
        case 3: _t->mouseLeaves(); break;
        case 4: _t->zoomIn(); break;
        case 5: _t->zoomOut(); break;
        case 6: _t->slotSetViewport((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 7: _t->slotEmulateWheelEvent((*reinterpret_cast< QWheelEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Panned::*)(QRectF );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::viewportChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Panned::*)(QWheelEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::wheelEventReceived)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Panned::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::pannedContentsScrolled)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Panned::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::mouseLeaves)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Panned::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::zoomIn)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Panned::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Panned::zoomOut)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::Panned::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__Panned.data,
    qt_meta_data_Rosegarden__Panned,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::Panned::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::Panned::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__Panned.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int Rosegarden::Panned::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::Panned::viewportChanged(QRectF _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::Panned::wheelEventReceived(QWheelEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::Panned::pannedContentsScrolled()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::Panned::mouseLeaves()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::Panned::zoomIn()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::Panned::zoomOut()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
