/****************************************************************************
** Meta object code from reading C++ file 'RosegardenMainViewWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/application/RosegardenMainViewWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosegardenMainViewWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget_t {
    QByteArrayData data[64];
    char stringdata0[1149];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget_t qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget = {
    {
QT_MOC_LITERAL(0, 0, 36), // "Rosegarden::RosegardenMainVie..."
QT_MOC_LITERAL(1, 37, 12), // "activateTool"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 8), // "toolName"
QT_MOC_LITERAL(4, 60, 11), // "stateChange"
QT_MOC_LITERAL(5, 72, 16), // "segmentsSelected"
QT_MOC_LITERAL(6, 89, 16), // "SegmentSelection"
QT_MOC_LITERAL(7, 106, 10), // "toggleSolo"
QT_MOC_LITERAL(8, 117, 25), // "stepByStepTargetRequested"
QT_MOC_LITERAL(9, 143, 12), // "addAudioFile"
QT_MOC_LITERAL(10, 156, 11), // "AudioFileId"
QT_MOC_LITERAL(11, 168, 23), // "instrumentLevelsChanged"
QT_MOC_LITERAL(12, 192, 12), // "InstrumentId"
QT_MOC_LITERAL(13, 205, 9), // "LevelInfo"
QT_MOC_LITERAL(14, 215, 15), // "slotEditSegment"
QT_MOC_LITERAL(15, 231, 8), // "Segment*"
QT_MOC_LITERAL(16, 240, 23), // "slotEditSegmentNotation"
QT_MOC_LITERAL(17, 264, 24), // "slotEditSegmentsNotation"
QT_MOC_LITERAL(18, 289, 21), // "std::vector<Segment*>"
QT_MOC_LITERAL(19, 311, 21), // "slotEditSegmentMatrix"
QT_MOC_LITERAL(20, 333, 22), // "slotEditSegmentsMatrix"
QT_MOC_LITERAL(21, 356, 31), // "slotEditSegmentPercussionMatrix"
QT_MOC_LITERAL(22, 388, 32), // "slotEditSegmentsPercussionMatrix"
QT_MOC_LITERAL(23, 421, 24), // "slotEditSegmentEventList"
QT_MOC_LITERAL(24, 446, 25), // "slotEditSegmentsEventList"
QT_MOC_LITERAL(25, 472, 27), // "slotEditSegmentPitchTracker"
QT_MOC_LITERAL(26, 500, 28), // "slotEditSegmentsPitchTracker"
QT_MOC_LITERAL(27, 529, 22), // "slotEditTriggerSegment"
QT_MOC_LITERAL(28, 552, 20), // "slotEditSegmentAudio"
QT_MOC_LITERAL(29, 573, 14), // "slotEditRepeat"
QT_MOC_LITERAL(30, 588, 5), // "timeT"
QT_MOC_LITERAL(31, 594, 23), // "slotSelectTrackSegments"
QT_MOC_LITERAL(32, 618, 21), // "slotSelectAllSegments"
QT_MOC_LITERAL(33, 640, 25), // "slotSelectRelatedSegments"
QT_MOC_LITERAL(34, 666, 20), // "slotSelectedSegments"
QT_MOC_LITERAL(35, 687, 8), // "segments"
QT_MOC_LITERAL(36, 696, 29), // "slotPropagateSegmentSelection"
QT_MOC_LITERAL(37, 726, 14), // "slotShowRulers"
QT_MOC_LITERAL(38, 741, 18), // "slotShowTempoRuler"
QT_MOC_LITERAL(39, 760, 22), // "slotShowChordNameRuler"
QT_MOC_LITERAL(40, 783, 16), // "slotShowPreviews"
QT_MOC_LITERAL(41, 800, 21), // "slotShowSegmentLabels"
QT_MOC_LITERAL(42, 822, 16), // "slotDeleteTracks"
QT_MOC_LITERAL(43, 839, 20), // "std::vector<TrackId>"
QT_MOC_LITERAL(44, 860, 6), // "tracks"
QT_MOC_LITERAL(45, 867, 34), // "slotAddAudioSegmentDefaultPos..."
QT_MOC_LITERAL(46, 902, 8), // "RealTime"
QT_MOC_LITERAL(47, 911, 9), // "startTime"
QT_MOC_LITERAL(48, 921, 7), // "endTime"
QT_MOC_LITERAL(49, 929, 19), // "slotAddAudioSegment"
QT_MOC_LITERAL(50, 949, 7), // "audioId"
QT_MOC_LITERAL(51, 957, 7), // "TrackId"
QT_MOC_LITERAL(52, 965, 7), // "trackId"
QT_MOC_LITERAL(53, 973, 8), // "position"
QT_MOC_LITERAL(54, 982, 16), // "slotDroppedAudio"
QT_MOC_LITERAL(55, 999, 9), // "audioDesc"
QT_MOC_LITERAL(56, 1009, 19), // "slotDroppedNewAudio"
QT_MOC_LITERAL(57, 1029, 23), // "slotAddCommandToHistory"
QT_MOC_LITERAL(58, 1053, 8), // "Command*"
QT_MOC_LITERAL(59, 1062, 7), // "command"
QT_MOC_LITERAL(60, 1070, 30), // "slotSynchroniseWithComposition"
QT_MOC_LITERAL(61, 1101, 22), // "slotExternalController"
QT_MOC_LITERAL(62, 1124, 18), // "const MappedEvent*"
QT_MOC_LITERAL(63, 1143, 5) // "event"

    },
    "Rosegarden::RosegardenMainViewWidget\0"
    "activateTool\0\0toolName\0stateChange\0"
    "segmentsSelected\0SegmentSelection\0"
    "toggleSolo\0stepByStepTargetRequested\0"
    "addAudioFile\0AudioFileId\0"
    "instrumentLevelsChanged\0InstrumentId\0"
    "LevelInfo\0slotEditSegment\0Segment*\0"
    "slotEditSegmentNotation\0"
    "slotEditSegmentsNotation\0std::vector<Segment*>\0"
    "slotEditSegmentMatrix\0slotEditSegmentsMatrix\0"
    "slotEditSegmentPercussionMatrix\0"
    "slotEditSegmentsPercussionMatrix\0"
    "slotEditSegmentEventList\0"
    "slotEditSegmentsEventList\0"
    "slotEditSegmentPitchTracker\0"
    "slotEditSegmentsPitchTracker\0"
    "slotEditTriggerSegment\0slotEditSegmentAudio\0"
    "slotEditRepeat\0timeT\0slotSelectTrackSegments\0"
    "slotSelectAllSegments\0slotSelectRelatedSegments\0"
    "slotSelectedSegments\0segments\0"
    "slotPropagateSegmentSelection\0"
    "slotShowRulers\0slotShowTempoRuler\0"
    "slotShowChordNameRuler\0slotShowPreviews\0"
    "slotShowSegmentLabels\0slotDeleteTracks\0"
    "std::vector<TrackId>\0tracks\0"
    "slotAddAudioSegmentDefaultPosition\0"
    "RealTime\0startTime\0endTime\0"
    "slotAddAudioSegment\0audioId\0TrackId\0"
    "trackId\0position\0slotDroppedAudio\0"
    "audioDesc\0slotDroppedNewAudio\0"
    "slotAddCommandToHistory\0Command*\0"
    "command\0slotSynchroniseWithComposition\0"
    "slotExternalController\0const MappedEvent*\0"
    "event"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__RosegardenMainViewWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      39,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  209,    2, 0x06 /* Public */,
       4,    2,  212,    2, 0x06 /* Public */,
       5,    1,  217,    2, 0x06 /* Public */,
       7,    1,  220,    2, 0x06 /* Public */,
       8,    1,  223,    2, 0x06 /* Public */,
       9,    1,  226,    2, 0x06 /* Public */,
      11,    2,  229,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    1,  234,    2, 0x0a /* Public */,
      16,    1,  237,    2, 0x0a /* Public */,
      17,    1,  240,    2, 0x0a /* Public */,
      19,    1,  243,    2, 0x0a /* Public */,
      20,    1,  246,    2, 0x0a /* Public */,
      21,    1,  249,    2, 0x0a /* Public */,
      22,    1,  252,    2, 0x0a /* Public */,
      23,    1,  255,    2, 0x0a /* Public */,
      24,    1,  258,    2, 0x0a /* Public */,
      25,    1,  261,    2, 0x0a /* Public */,
      26,    1,  264,    2, 0x0a /* Public */,
      27,    1,  267,    2, 0x0a /* Public */,
      28,    1,  270,    2, 0x0a /* Public */,
      29,    2,  273,    2, 0x0a /* Public */,
      31,    1,  278,    2, 0x0a /* Public */,
      32,    0,  281,    2, 0x0a /* Public */,
      33,    0,  282,    2, 0x0a /* Public */,
      34,    1,  283,    2, 0x0a /* Public */,
      36,    1,  286,    2, 0x0a /* Public */,
      37,    1,  289,    2, 0x0a /* Public */,
      38,    1,  292,    2, 0x0a /* Public */,
      39,    1,  295,    2, 0x0a /* Public */,
      40,    1,  298,    2, 0x0a /* Public */,
      41,    1,  301,    2, 0x0a /* Public */,
      42,    1,  304,    2, 0x0a /* Public */,
      45,    3,  307,    2, 0x0a /* Public */,
      49,    5,  314,    2, 0x0a /* Public */,
      54,    1,  325,    2, 0x0a /* Public */,
      56,    1,  328,    2, 0x0a /* Public */,
      57,    1,  331,    2, 0x0a /* Public */,
      60,    0,  334,    2, 0x0a /* Public */,
      61,    1,  335,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    2,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QObjectStar,    2,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 13,    2,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 18,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 30,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,   35,
    QMetaType::Void, 0x80000000 | 6,   35,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 46, 0x80000000 | 46,    2,   47,   48,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 51, 0x80000000 | 30, 0x80000000 | 46, 0x80000000 | 46,   50,   52,   53,   47,   48,
    QMetaType::Void, QMetaType::QString,   55,
    QMetaType::Void, QMetaType::QString,   55,
    QMetaType::Void, 0x80000000 | 58,   59,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 62,   63,

       0        // eod
};

void Rosegarden::RosegardenMainViewWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RosegardenMainViewWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activateTool((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->stateChange((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->segmentsSelected((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 3: _t->toggleSolo((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->stepByStepTargetRequested((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 5: _t->addAudioFile((*reinterpret_cast< AudioFileId(*)>(_a[1]))); break;
        case 6: _t->instrumentLevelsChanged((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< const LevelInfo(*)>(_a[2]))); break;
        case 7: _t->slotEditSegment((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 8: _t->slotEditSegmentNotation((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 9: _t->slotEditSegmentsNotation((*reinterpret_cast< const std::vector<Segment*>(*)>(_a[1]))); break;
        case 10: _t->slotEditSegmentMatrix((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 11: _t->slotEditSegmentsMatrix((*reinterpret_cast< const std::vector<Segment*>(*)>(_a[1]))); break;
        case 12: _t->slotEditSegmentPercussionMatrix((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 13: _t->slotEditSegmentsPercussionMatrix((*reinterpret_cast< const std::vector<Segment*>(*)>(_a[1]))); break;
        case 14: _t->slotEditSegmentEventList((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 15: _t->slotEditSegmentsEventList((*reinterpret_cast< const std::vector<Segment*>(*)>(_a[1]))); break;
        case 16: _t->slotEditSegmentPitchTracker((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 17: _t->slotEditSegmentsPitchTracker((*reinterpret_cast< const std::vector<Segment*>(*)>(_a[1]))); break;
        case 18: _t->slotEditTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->slotEditSegmentAudio((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 20: _t->slotEditRepeat((*reinterpret_cast< Segment*(*)>(_a[1])),(*reinterpret_cast< timeT(*)>(_a[2]))); break;
        case 21: _t->slotSelectTrackSegments((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->slotSelectAllSegments(); break;
        case 23: _t->slotSelectRelatedSegments(); break;
        case 24: _t->slotSelectedSegments((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 25: _t->slotPropagateSegmentSelection((*reinterpret_cast< const SegmentSelection(*)>(_a[1]))); break;
        case 26: _t->slotShowRulers((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->slotShowTempoRuler((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->slotShowChordNameRuler((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 29: _t->slotShowPreviews((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->slotShowSegmentLabels((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->slotDeleteTracks((*reinterpret_cast< const std::vector<TrackId>(*)>(_a[1]))); break;
        case 32: _t->slotAddAudioSegmentDefaultPosition((*reinterpret_cast< AudioFileId(*)>(_a[1])),(*reinterpret_cast< const RealTime(*)>(_a[2])),(*reinterpret_cast< const RealTime(*)>(_a[3]))); break;
        case 33: _t->slotAddAudioSegment((*reinterpret_cast< AudioFileId(*)>(_a[1])),(*reinterpret_cast< TrackId(*)>(_a[2])),(*reinterpret_cast< timeT(*)>(_a[3])),(*reinterpret_cast< const RealTime(*)>(_a[4])),(*reinterpret_cast< const RealTime(*)>(_a[5]))); break;
        case 34: _t->slotDroppedAudio((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 35: _t->slotDroppedNewAudio((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 36: _t->slotAddCommandToHistory((*reinterpret_cast< Command*(*)>(_a[1]))); break;
        case 37: _t->slotSynchroniseWithComposition(); break;
        case 38: _t->slotExternalController((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Segment* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RosegardenMainViewWidget::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::activateTool)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(QString , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::stateChange)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(const SegmentSelection & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::segmentsSelected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::toggleSolo)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::stepByStepTargetRequested)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(AudioFileId );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::addAudioFile)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (RosegardenMainViewWidget::*)(InstrumentId , const LevelInfo & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RosegardenMainViewWidget::instrumentLevelsChanged)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::RosegardenMainViewWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget.data,
    qt_meta_data_Rosegarden__RosegardenMainViewWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::RosegardenMainViewWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::RosegardenMainViewWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__RosegardenMainViewWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Rosegarden::RosegardenMainViewWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::RosegardenMainViewWidget::activateTool(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::RosegardenMainViewWidget::stateChange(QString _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::RosegardenMainViewWidget::segmentsSelected(const SegmentSelection & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::RosegardenMainViewWidget::toggleSolo(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::RosegardenMainViewWidget::stepByStepTargetRequested(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::RosegardenMainViewWidget::addAudioFile(AudioFileId _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Rosegarden::RosegardenMainViewWidget::instrumentLevelsChanged(InstrumentId _t1, const LevelInfo & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
