/****************************************************************************
** Meta object code from reading C++ file 'NotationWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/notation/NotationWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NotationWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__NotationWidget_t {
    QByteArrayData data[89];
    char stringdata0[1555];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__NotationWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__NotationWidget_t qt_meta_stringdata_Rosegarden__NotationWidget = {
    {
QT_MOC_LITERAL(0, 0, 26), // "Rosegarden::NotationWidget"
QT_MOC_LITERAL(1, 27, 20), // "sceneNeedsRebuilding"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 11), // "toolChanged"
QT_MOC_LITERAL(4, 61, 22), // "hoveredOverNoteChanged"
QT_MOC_LITERAL(5, 84, 24), // "headersVisibilityChanged"
QT_MOC_LITERAL(6, 109, 15), // "showContextHelp"
QT_MOC_LITERAL(7, 125, 21), // "rulerSelectionChanged"
QT_MOC_LITERAL(8, 147, 20), // "rulerSelectionUpdate"
QT_MOC_LITERAL(9, 168, 12), // "adjustNeeded"
QT_MOC_LITERAL(10, 181, 4), // "last"
QT_MOC_LITERAL(11, 186, 11), // "editElement"
QT_MOC_LITERAL(12, 198, 14), // "NotationStaff*"
QT_MOC_LITERAL(13, 213, 16), // "NotationElement*"
QT_MOC_LITERAL(14, 230, 19), // "currentSegmentPrior"
QT_MOC_LITERAL(15, 250, 18), // "currentSegmentNext"
QT_MOC_LITERAL(16, 269, 11), // "slotSetTool"
QT_MOC_LITERAL(17, 281, 4), // "name"
QT_MOC_LITERAL(18, 286, 17), // "slotSetSelectTool"
QT_MOC_LITERAL(19, 304, 23), // "slotSetSelectNoTiesTool"
QT_MOC_LITERAL(20, 328, 16), // "slotSetEraseTool"
QT_MOC_LITERAL(21, 345, 23), // "slotSetNoteRestInserter"
QT_MOC_LITERAL(22, 369, 19), // "slotSetNoteInserter"
QT_MOC_LITERAL(23, 389, 19), // "slotSetRestInserter"
QT_MOC_LITERAL(24, 409, 19), // "slotSetInsertedNote"
QT_MOC_LITERAL(25, 429, 10), // "Note::Type"
QT_MOC_LITERAL(26, 440, 4), // "type"
QT_MOC_LITERAL(27, 445, 4), // "dots"
QT_MOC_LITERAL(28, 450, 17), // "slotSetAccidental"
QT_MOC_LITERAL(29, 468, 10), // "Accidental"
QT_MOC_LITERAL(30, 479, 10), // "accidental"
QT_MOC_LITERAL(31, 490, 6), // "follow"
QT_MOC_LITERAL(32, 497, 19), // "slotSetClefInserter"
QT_MOC_LITERAL(33, 517, 19), // "slotSetInsertedClef"
QT_MOC_LITERAL(34, 537, 4), // "Clef"
QT_MOC_LITERAL(35, 542, 19), // "slotSetTextInserter"
QT_MOC_LITERAL(36, 562, 26), // "slotSetGuitarChordInserter"
QT_MOC_LITERAL(37, 589, 17), // "slotSetLinearMode"
QT_MOC_LITERAL(38, 607, 25), // "slotSetContinuousPageMode"
QT_MOC_LITERAL(39, 633, 20), // "slotSetMultiPageMode"
QT_MOC_LITERAL(40, 654, 15), // "slotSetFontName"
QT_MOC_LITERAL(41, 670, 15), // "slotSetFontSize"
QT_MOC_LITERAL(42, 686, 18), // "slotScrollToFollow"
QT_MOC_LITERAL(43, 705, 21), // "slotSetSymbolInserter"
QT_MOC_LITERAL(44, 727, 21), // "slotSetInsertedSymbol"
QT_MOC_LITERAL(45, 749, 6), // "Symbol"
QT_MOC_LITERAL(46, 756, 23), // "slotToggleVelocityRuler"
QT_MOC_LITERAL(47, 780, 24), // "slotTogglePitchbendRuler"
QT_MOC_LITERAL(48, 805, 26), // "slotToggleKeyPressureRuler"
QT_MOC_LITERAL(49, 832, 30), // "slotToggleChannelPressureRuler"
QT_MOC_LITERAL(50, 863, 19), // "slotAddControlRuler"
QT_MOC_LITERAL(51, 883, 8), // "QAction*"
QT_MOC_LITERAL(52, 892, 21), // "slotRegenerateHeaders"
QT_MOC_LITERAL(53, 914, 21), // "slotShowHeaderToolTip"
QT_MOC_LITERAL(54, 936, 11), // "toolTipText"
QT_MOC_LITERAL(55, 948, 26), // "slotPointerPositionChanged"
QT_MOC_LITERAL(56, 975, 5), // "timeT"
QT_MOC_LITERAL(57, 981, 1), // "t"
QT_MOC_LITERAL(58, 983, 21), // "slotStandardRulerDrag"
QT_MOC_LITERAL(59, 1005, 20), // "slotZoomInFromPanner"
QT_MOC_LITERAL(60, 1026, 21), // "slotZoomOutFromPanner"
QT_MOC_LITERAL(61, 1048, 11), // "slotHScroll"
QT_MOC_LITERAL(62, 1060, 26), // "slotHScrollBarRangeChanged"
QT_MOC_LITERAL(63, 1087, 3), // "min"
QT_MOC_LITERAL(64, 1091, 3), // "max"
QT_MOC_LITERAL(65, 1095, 29), // "slotHorizontalThumbwheelMoved"
QT_MOC_LITERAL(66, 1125, 27), // "slotVerticalThumbwheelMoved"
QT_MOC_LITERAL(67, 1153, 26), // "slotPrimaryThumbwheelMoved"
QT_MOC_LITERAL(68, 1180, 20), // "slotResetZoomClicked"
QT_MOC_LITERAL(69, 1201, 20), // "slotSyncPannerZoomIn"
QT_MOC_LITERAL(70, 1222, 21), // "slotSyncPannerZoomOut"
QT_MOC_LITERAL(71, 1244, 19), // "slotGenerateHeaders"
QT_MOC_LITERAL(72, 1264, 18), // "slotHeadersResized"
QT_MOC_LITERAL(73, 1283, 5), // "width"
QT_MOC_LITERAL(74, 1289, 30), // "slotAdjustHeadersHorizontalPos"
QT_MOC_LITERAL(75, 1320, 28), // "slotAdjustHeadersVerticalPos"
QT_MOC_LITERAL(76, 1349, 1), // "r"
QT_MOC_LITERAL(77, 1351, 16), // "slotCloseHeaders"
QT_MOC_LITERAL(78, 1368, 23), // "slotSegmentChangerMoved"
QT_MOC_LITERAL(79, 1392, 16), // "slotStaffChanged"
QT_MOC_LITERAL(80, 1409, 22), // "slotUpdateRawNoteRuler"
QT_MOC_LITERAL(81, 1432, 12), // "ViewSegment*"
QT_MOC_LITERAL(82, 1445, 34), // "slotUpdateSegmentChangerBackg..."
QT_MOC_LITERAL(83, 1480, 11), // "resizeEvent"
QT_MOC_LITERAL(84, 1492, 13), // "QResizeEvent*"
QT_MOC_LITERAL(85, 1506, 5), // "event"
QT_MOC_LITERAL(86, 1512, 19), // "slotResizeTimerDone"
QT_MOC_LITERAL(87, 1532, 10), // "slotZoomIn"
QT_MOC_LITERAL(88, 1543, 11) // "slotZoomOut"

    },
    "Rosegarden::NotationWidget\0"
    "sceneNeedsRebuilding\0\0toolChanged\0"
    "hoveredOverNoteChanged\0headersVisibilityChanged\0"
    "showContextHelp\0rulerSelectionChanged\0"
    "rulerSelectionUpdate\0adjustNeeded\0"
    "last\0editElement\0NotationStaff*\0"
    "NotationElement*\0currentSegmentPrior\0"
    "currentSegmentNext\0slotSetTool\0name\0"
    "slotSetSelectTool\0slotSetSelectNoTiesTool\0"
    "slotSetEraseTool\0slotSetNoteRestInserter\0"
    "slotSetNoteInserter\0slotSetRestInserter\0"
    "slotSetInsertedNote\0Note::Type\0type\0"
    "dots\0slotSetAccidental\0Accidental\0"
    "accidental\0follow\0slotSetClefInserter\0"
    "slotSetInsertedClef\0Clef\0slotSetTextInserter\0"
    "slotSetGuitarChordInserter\0slotSetLinearMode\0"
    "slotSetContinuousPageMode\0"
    "slotSetMultiPageMode\0slotSetFontName\0"
    "slotSetFontSize\0slotScrollToFollow\0"
    "slotSetSymbolInserter\0slotSetInsertedSymbol\0"
    "Symbol\0slotToggleVelocityRuler\0"
    "slotTogglePitchbendRuler\0"
    "slotToggleKeyPressureRuler\0"
    "slotToggleChannelPressureRuler\0"
    "slotAddControlRuler\0QAction*\0"
    "slotRegenerateHeaders\0slotShowHeaderToolTip\0"
    "toolTipText\0slotPointerPositionChanged\0"
    "timeT\0t\0slotStandardRulerDrag\0"
    "slotZoomInFromPanner\0slotZoomOutFromPanner\0"
    "slotHScroll\0slotHScrollBarRangeChanged\0"
    "min\0max\0slotHorizontalThumbwheelMoved\0"
    "slotVerticalThumbwheelMoved\0"
    "slotPrimaryThumbwheelMoved\0"
    "slotResetZoomClicked\0slotSyncPannerZoomIn\0"
    "slotSyncPannerZoomOut\0slotGenerateHeaders\0"
    "slotHeadersResized\0width\0"
    "slotAdjustHeadersHorizontalPos\0"
    "slotAdjustHeadersVerticalPos\0r\0"
    "slotCloseHeaders\0slotSegmentChangerMoved\0"
    "slotStaffChanged\0slotUpdateRawNoteRuler\0"
    "ViewSegment*\0slotUpdateSegmentChangerBackground\0"
    "resizeEvent\0QResizeEvent*\0event\0"
    "slotResizeTimerDone\0slotZoomIn\0"
    "slotZoomOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__NotationWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      64,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  334,    2, 0x06 /* Public */,
       3,    1,  335,    2, 0x06 /* Public */,
       4,    1,  338,    2, 0x06 /* Public */,
       5,    1,  341,    2, 0x06 /* Public */,
       6,    1,  344,    2, 0x06 /* Public */,
       7,    0,  347,    2, 0x06 /* Public */,
       8,    0,  348,    2, 0x06 /* Public */,
       9,    1,  349,    2, 0x06 /* Public */,
      11,    2,  352,    2, 0x06 /* Public */,
      14,    0,  357,    2, 0x06 /* Public */,
      15,    0,  358,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,  359,    2, 0x0a /* Public */,
      18,    0,  362,    2, 0x0a /* Public */,
      19,    0,  363,    2, 0x0a /* Public */,
      20,    0,  364,    2, 0x0a /* Public */,
      21,    0,  365,    2, 0x0a /* Public */,
      22,    0,  366,    2, 0x0a /* Public */,
      23,    0,  367,    2, 0x0a /* Public */,
      24,    2,  368,    2, 0x0a /* Public */,
      28,    2,  373,    2, 0x0a /* Public */,
      32,    0,  378,    2, 0x0a /* Public */,
      33,    1,  379,    2, 0x0a /* Public */,
      35,    0,  382,    2, 0x0a /* Public */,
      36,    0,  383,    2, 0x0a /* Public */,
      37,    0,  384,    2, 0x0a /* Public */,
      38,    0,  385,    2, 0x0a /* Public */,
      39,    0,  386,    2, 0x0a /* Public */,
      40,    1,  387,    2, 0x0a /* Public */,
      41,    1,  390,    2, 0x0a /* Public */,
      42,    0,  393,    2, 0x0a /* Public */,
      43,    0,  394,    2, 0x0a /* Public */,
      44,    1,  395,    2, 0x0a /* Public */,
      46,    0,  398,    2, 0x0a /* Public */,
      47,    0,  399,    2, 0x0a /* Public */,
      48,    0,  400,    2, 0x0a /* Public */,
      49,    0,  401,    2, 0x0a /* Public */,
      50,    1,  402,    2, 0x0a /* Public */,
      52,    0,  405,    2, 0x0a /* Public */,
      53,    1,  406,    2, 0x0a /* Public */,
      55,    1,  409,    2, 0x08 /* Private */,
      58,    1,  412,    2, 0x08 /* Private */,
      59,    0,  415,    2, 0x08 /* Private */,
      60,    0,  416,    2, 0x08 /* Private */,
      61,    0,  417,    2, 0x08 /* Private */,
      62,    2,  418,    2, 0x08 /* Private */,
      65,    1,  423,    2, 0x08 /* Private */,
      66,    1,  426,    2, 0x08 /* Private */,
      67,    1,  429,    2, 0x08 /* Private */,
      68,    0,  432,    2, 0x08 /* Private */,
      69,    0,  433,    2, 0x08 /* Private */,
      70,    0,  434,    2, 0x08 /* Private */,
      71,    0,  435,    2, 0x08 /* Private */,
      72,    1,  436,    2, 0x08 /* Private */,
      74,    1,  439,    2, 0x08 /* Private */,
      75,    1,  442,    2, 0x08 /* Private */,
      77,    0,  445,    2, 0x08 /* Private */,
      78,    1,  446,    2, 0x08 /* Private */,
      79,    0,  449,    2, 0x08 /* Private */,
      80,    1,  450,    2, 0x08 /* Private */,
      82,    0,  453,    2, 0x08 /* Private */,
      83,    1,  454,    2, 0x08 /* Private */,
      86,    0,  457,    2, 0x08 /* Private */,
      87,    0,  458,    2, 0x08 /* Private */,
      88,    0,  459,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 13,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 25, QMetaType::Int,   26,   27,
    QMetaType::Void, 0x80000000 | 29, QMetaType::Bool,   30,   31,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 34,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 45,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 51,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   54,
    QMetaType::Void, 0x80000000 | 56,   57,
    QMetaType::Void, 0x80000000 | 56,   57,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   63,   64,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   73,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::QRectF,   76,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 81,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 84,   85,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::NotationWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotationWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneNeedsRebuilding(); break;
        case 1: _t->toolChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->hoveredOverNoteChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->headersVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->showContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->rulerSelectionChanged(); break;
        case 6: _t->rulerSelectionUpdate(); break;
        case 7: _t->adjustNeeded((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->editElement((*reinterpret_cast< NotationStaff*(*)>(_a[1])),(*reinterpret_cast< NotationElement*(*)>(_a[2]))); break;
        case 9: _t->currentSegmentPrior(); break;
        case 10: _t->currentSegmentNext(); break;
        case 11: _t->slotSetTool((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 12: _t->slotSetSelectTool(); break;
        case 13: _t->slotSetSelectNoTiesTool(); break;
        case 14: _t->slotSetEraseTool(); break;
        case 15: _t->slotSetNoteRestInserter(); break;
        case 16: _t->slotSetNoteInserter(); break;
        case 17: _t->slotSetRestInserter(); break;
        case 18: _t->slotSetInsertedNote((*reinterpret_cast< Note::Type(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 19: _t->slotSetAccidental((*reinterpret_cast< const Accidental(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 20: _t->slotSetClefInserter(); break;
        case 21: _t->slotSetInsertedClef((*reinterpret_cast< const Clef(*)>(_a[1]))); break;
        case 22: _t->slotSetTextInserter(); break;
        case 23: _t->slotSetGuitarChordInserter(); break;
        case 24: _t->slotSetLinearMode(); break;
        case 25: _t->slotSetContinuousPageMode(); break;
        case 26: _t->slotSetMultiPageMode(); break;
        case 27: _t->slotSetFontName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->slotSetFontSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->slotScrollToFollow(); break;
        case 30: _t->slotSetSymbolInserter(); break;
        case 31: _t->slotSetInsertedSymbol((*reinterpret_cast< Symbol(*)>(_a[1]))); break;
        case 32: _t->slotToggleVelocityRuler(); break;
        case 33: _t->slotTogglePitchbendRuler(); break;
        case 34: _t->slotToggleKeyPressureRuler(); break;
        case 35: _t->slotToggleChannelPressureRuler(); break;
        case 36: _t->slotAddControlRuler((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 37: _t->slotRegenerateHeaders(); break;
        case 38: _t->slotShowHeaderToolTip((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 39: _t->slotPointerPositionChanged((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 40: _t->slotStandardRulerDrag((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 41: _t->slotZoomInFromPanner(); break;
        case 42: _t->slotZoomOutFromPanner(); break;
        case 43: _t->slotHScroll(); break;
        case 44: _t->slotHScrollBarRangeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 45: _t->slotHorizontalThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->slotVerticalThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 47: _t->slotPrimaryThumbwheelMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 48: _t->slotResetZoomClicked(); break;
        case 49: _t->slotSyncPannerZoomIn(); break;
        case 50: _t->slotSyncPannerZoomOut(); break;
        case 51: _t->slotGenerateHeaders(); break;
        case 52: _t->slotHeadersResized((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 53: _t->slotAdjustHeadersHorizontalPos((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 54: _t->slotAdjustHeadersVerticalPos((*reinterpret_cast< QRectF(*)>(_a[1]))); break;
        case 55: _t->slotCloseHeaders(); break;
        case 56: _t->slotSegmentChangerMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 57: _t->slotStaffChanged(); break;
        case 58: _t->slotUpdateRawNoteRuler((*reinterpret_cast< ViewSegment*(*)>(_a[1]))); break;
        case 59: _t->slotUpdateSegmentChangerBackground(); break;
        case 60: _t->resizeEvent((*reinterpret_cast< QResizeEvent*(*)>(_a[1]))); break;
        case 61: _t->slotResizeTimerDone(); break;
        case 62: _t->slotZoomIn(); break;
        case 63: _t->slotZoomOut(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NotationWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::sceneNeedsRebuilding)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::toolChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::hoveredOverNoteChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::headersVisibilityChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::showContextHelp)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::rulerSelectionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::rulerSelectionUpdate)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::adjustNeeded)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)(NotationStaff * , NotationElement * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::editElement)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::currentSegmentPrior)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (NotationWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NotationWidget::currentSegmentNext)) {
                *result = 10;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::NotationWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__NotationWidget.data,
    qt_meta_data_Rosegarden__NotationWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::NotationWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::NotationWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__NotationWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return QWidget::qt_metacast(_clname);
}

int Rosegarden::NotationWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 64)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 64;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 64)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 64;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::NotationWidget::sceneNeedsRebuilding()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::NotationWidget::toolChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::NotationWidget::hoveredOverNoteChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::NotationWidget::headersVisibilityChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::NotationWidget::showContextHelp(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::NotationWidget::rulerSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::NotationWidget::rulerSelectionUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::NotationWidget::adjustNeeded(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::NotationWidget::editElement(NotationStaff * _t1, NotationElement * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Rosegarden::NotationWidget::currentSegmentPrior()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Rosegarden::NotationWidget::currentSegmentNext()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
