/****************************************************************************
** Meta object code from reading C++ file 'SynthPluginManagerDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/studio/SynthPluginManagerDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SynthPluginManagerDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog_t {
    QByteArrayData data[16];
    char stringdata0[232];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog_t qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog = {
    {
QT_MOC_LITERAL(0, 0, 36), // "Rosegarden::SynthPluginManage..."
QT_MOC_LITERAL(1, 37, 7), // "closing"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 14), // "pluginSelected"
QT_MOC_LITERAL(4, 61, 12), // "InstrumentId"
QT_MOC_LITERAL(5, 74, 11), // "pluginIndex"
QT_MOC_LITERAL(6, 86, 6), // "plugin"
QT_MOC_LITERAL(7, 93, 16), // "showPluginDialog"
QT_MOC_LITERAL(8, 110, 8), // "QWidget*"
QT_MOC_LITERAL(9, 119, 13), // "showPluginGUI"
QT_MOC_LITERAL(10, 133, 9), // "slotClose"
QT_MOC_LITERAL(11, 143, 17), // "slotPluginChanged"
QT_MOC_LITERAL(12, 161, 5), // "index"
QT_MOC_LITERAL(13, 167, 25), // "slotControlsButtonClicked"
QT_MOC_LITERAL(14, 193, 20), // "slotGUIButtonClicked"
QT_MOC_LITERAL(15, 214, 17) // "slotHelpRequested"

    },
    "Rosegarden::SynthPluginManagerDialog\0"
    "closing\0\0pluginSelected\0InstrumentId\0"
    "pluginIndex\0plugin\0showPluginDialog\0"
    "QWidget*\0showPluginGUI\0slotClose\0"
    "slotPluginChanged\0index\0"
    "slotControlsButtonClicked\0"
    "slotGUIButtonClicked\0slotHelpRequested"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__SynthPluginManagerDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    3,   60,    2, 0x06 /* Public */,
       7,    3,   67,    2, 0x06 /* Public */,
       9,    2,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   79,    2, 0x09 /* Protected */,
      11,    1,   80,    2, 0x09 /* Protected */,
      13,    0,   83,    2, 0x09 /* Protected */,
      14,    0,   84,    2, 0x09 /* Protected */,
      15,    0,   85,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,    2,    5,    6,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 4, QMetaType::Int,    2,    2,    5,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    2,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::SynthPluginManagerDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SynthPluginManagerDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closing(); break;
        case 1: _t->pluginSelected((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 2: _t->showPluginDialog((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< InstrumentId(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->showPluginGUI((*reinterpret_cast< InstrumentId(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->slotClose(); break;
        case 5: _t->slotPluginChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotControlsButtonClicked(); break;
        case 7: _t->slotGUIButtonClicked(); break;
        case 8: _t->slotHelpRequested(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SynthPluginManagerDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SynthPluginManagerDialog::closing)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SynthPluginManagerDialog::*)(InstrumentId , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SynthPluginManagerDialog::pluginSelected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SynthPluginManagerDialog::*)(QWidget * , InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SynthPluginManagerDialog::showPluginDialog)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SynthPluginManagerDialog::*)(InstrumentId , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SynthPluginManagerDialog::showPluginGUI)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::SynthPluginManagerDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog.data,
    qt_meta_data_Rosegarden__SynthPluginManagerDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::SynthPluginManagerDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::SynthPluginManagerDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__SynthPluginManagerDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::SynthPluginManagerDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::SynthPluginManagerDialog::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::SynthPluginManagerDialog::pluginSelected(InstrumentId _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::SynthPluginManagerDialog::showPluginDialog(QWidget * _t1, InstrumentId _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::SynthPluginManagerDialog::showPluginGUI(InstrumentId _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
