/****************************************************************************
** Meta object code from reading C++ file 'ExternalController.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/sound/ExternalController.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ExternalController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__ExternalController_t {
    QByteArrayData data[18];
    char stringdata0[267];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__ExternalController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__ExternalController_t qt_meta_stringdata_Rosegarden__ExternalController = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Rosegarden::ExternalController"
QT_MOC_LITERAL(1, 31, 22), // "externalControllerRMVW"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 18), // "const MappedEvent*"
QT_MOC_LITERAL(4, 74, 5), // "event"
QT_MOC_LITERAL(5, 80, 21), // "externalControllerMMW"
QT_MOC_LITERAL(6, 102, 22), // "externalControllerAMW2"
QT_MOC_LITERAL(7, 125, 18), // "slotDocumentLoaded"
QT_MOC_LITERAL(8, 144, 19), // "RosegardenDocument*"
QT_MOC_LITERAL(9, 164, 3), // "doc"
QT_MOC_LITERAL(10, 168, 20), // "slotDocumentModified"
QT_MOC_LITERAL(11, 189, 17), // "slotControlChange"
QT_MOC_LITERAL(12, 207, 11), // "Instrument*"
QT_MOC_LITERAL(13, 219, 10), // "instrument"
QT_MOC_LITERAL(14, 230, 2), // "cc"
QT_MOC_LITERAL(15, 233, 11), // "slotPlaying"
QT_MOC_LITERAL(16, 245, 7), // "checked"
QT_MOC_LITERAL(17, 253, 13) // "slotRecording"

    },
    "Rosegarden::ExternalController\0"
    "externalControllerRMVW\0\0const MappedEvent*\0"
    "event\0externalControllerMMW\0"
    "externalControllerAMW2\0slotDocumentLoaded\0"
    "RosegardenDocument*\0doc\0slotDocumentModified\0"
    "slotControlChange\0Instrument*\0instrument\0"
    "cc\0slotPlaying\0checked\0slotRecording"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__ExternalController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       5,    1,   57,    2, 0x06 /* Public */,
       6,    1,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   63,    2, 0x08 /* Private */,
      10,    1,   66,    2, 0x08 /* Private */,
      11,    2,   69,    2, 0x08 /* Private */,
      15,    1,   74,    2, 0x08 /* Private */,
      17,    1,   77,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int,   13,   14,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,   16,

       0        // eod
};

void Rosegarden::ExternalController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExternalController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->externalControllerRMVW((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 1: _t->externalControllerMMW((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 2: _t->externalControllerAMW2((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 3: _t->slotDocumentLoaded((*reinterpret_cast< RosegardenDocument*(*)>(_a[1]))); break;
        case 4: _t->slotDocumentModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotControlChange((*reinterpret_cast< Instrument*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->slotPlaying((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->slotRecording((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Instrument* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ExternalController::*)(const MappedEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExternalController::externalControllerRMVW)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ExternalController::*)(const MappedEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExternalController::externalControllerMMW)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ExternalController::*)(const MappedEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExternalController::externalControllerAMW2)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::ExternalController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__ExternalController.data,
    qt_meta_data_Rosegarden__ExternalController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::ExternalController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::ExternalController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__ExternalController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::ExternalController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::ExternalController::externalControllerRMVW(const MappedEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::ExternalController::externalControllerMMW(const MappedEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::ExternalController::externalControllerAMW2(const MappedEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
