/****************************************************************************
** Meta object code from reading C++ file 'SequenceManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/seqmanager/SequenceManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SequenceManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__SequenceManager_t {
    QByteArrayData data[34];
    char stringdata0[493];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__SequenceManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__SequenceManager_t qt_meta_stringdata_Rosegarden__SequenceManager = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::SequenceManager"
QT_MOC_LITERAL(1, 28, 16), // "sigProgramChange"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 7), // "bankMSB"
QT_MOC_LITERAL(4, 54, 7), // "bankLSB"
QT_MOC_LITERAL(5, 62, 13), // "programChange"
QT_MOC_LITERAL(6, 76, 24), // "insertableNoteOnReceived"
QT_MOC_LITERAL(7, 101, 5), // "pitch"
QT_MOC_LITERAL(8, 107, 8), // "velocity"
QT_MOC_LITERAL(9, 116, 25), // "insertableNoteOffReceived"
QT_MOC_LITERAL(10, 142, 11), // "sendWarning"
QT_MOC_LITERAL(11, 154, 4), // "type"
QT_MOC_LITERAL(12, 159, 4), // "text"
QT_MOC_LITERAL(13, 164, 15), // "informativeText"
QT_MOC_LITERAL(14, 180, 18), // "signalTempoChanged"
QT_MOC_LITERAL(15, 199, 6), // "tempoT"
QT_MOC_LITERAL(16, 206, 5), // "tempo"
QT_MOC_LITERAL(17, 212, 17), // "signalMidiInLabel"
QT_MOC_LITERAL(18, 230, 18), // "const MappedEvent*"
QT_MOC_LITERAL(19, 249, 5), // "event"
QT_MOC_LITERAL(20, 255, 18), // "signalMidiOutLabel"
QT_MOC_LITERAL(21, 274, 13), // "signalPlaying"
QT_MOC_LITERAL(22, 288, 7), // "checked"
QT_MOC_LITERAL(23, 296, 15), // "signalRecording"
QT_MOC_LITERAL(24, 312, 24), // "signalMetronomeActivated"
QT_MOC_LITERAL(25, 337, 6), // "update"
QT_MOC_LITERAL(26, 344, 6), // "rewind"
QT_MOC_LITERAL(27, 351, 11), // "fastforward"
QT_MOC_LITERAL(28, 363, 17), // "rewindToBeginning"
QT_MOC_LITERAL(29, 381, 16), // "fastForwardToEnd"
QT_MOC_LITERAL(30, 398, 15), // "slotLoopChanged"
QT_MOC_LITERAL(31, 414, 16), // "slotExportUpdate"
QT_MOC_LITERAL(32, 431, 25), // "slotCountdownTimerTimeout"
QT_MOC_LITERAL(33, 457, 35) // "slotScheduledCompositionMappe..."

    },
    "Rosegarden::SequenceManager\0"
    "sigProgramChange\0\0bankMSB\0bankLSB\0"
    "programChange\0insertableNoteOnReceived\0"
    "pitch\0velocity\0insertableNoteOffReceived\0"
    "sendWarning\0type\0text\0informativeText\0"
    "signalTempoChanged\0tempoT\0tempo\0"
    "signalMidiInLabel\0const MappedEvent*\0"
    "event\0signalMidiOutLabel\0signalPlaying\0"
    "checked\0signalRecording\0"
    "signalMetronomeActivated\0update\0rewind\0"
    "fastforward\0rewindToBeginning\0"
    "fastForwardToEnd\0slotLoopChanged\0"
    "slotExportUpdate\0slotCountdownTimerTimeout\0"
    "slotScheduledCompositionMapperReset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__SequenceManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,  109,    2, 0x06 /* Public */,
       6,    2,  116,    2, 0x06 /* Public */,
       9,    2,  121,    2, 0x06 /* Public */,
      10,    3,  126,    2, 0x06 /* Public */,
      14,    1,  133,    2, 0x06 /* Public */,
      17,    1,  136,    2, 0x06 /* Public */,
      20,    1,  139,    2, 0x06 /* Public */,
      21,    1,  142,    2, 0x06 /* Public */,
      23,    1,  145,    2, 0x06 /* Public */,
      24,    1,  148,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      25,    0,  151,    2, 0x0a /* Public */,
      26,    0,  152,    2, 0x0a /* Public */,
      27,    0,  153,    2, 0x0a /* Public */,
      28,    0,  154,    2, 0x0a /* Public */,
      29,    0,  155,    2, 0x0a /* Public */,
      30,    0,  156,    2, 0x0a /* Public */,
      31,    0,  157,    2, 0x0a /* Public */,
      32,    0,  158,    2, 0x08 /* Private */,
      33,    0,  159,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    4,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    7,    8,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    7,    8,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::QString,   11,   12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool,   22,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::SequenceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SequenceManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sigProgramChange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->insertableNoteOnReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->insertableNoteOffReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->sendWarning((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 4: _t->signalTempoChanged((*reinterpret_cast< tempoT(*)>(_a[1]))); break;
        case 5: _t->signalMidiInLabel((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 6: _t->signalMidiOutLabel((*reinterpret_cast< const MappedEvent*(*)>(_a[1]))); break;
        case 7: _t->signalPlaying((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->signalRecording((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->signalMetronomeActivated((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->update(); break;
        case 11: _t->rewind(); break;
        case 12: _t->fastforward(); break;
        case 13: _t->rewindToBeginning(); break;
        case 14: _t->fastForwardToEnd(); break;
        case 15: _t->slotLoopChanged(); break;
        case 16: _t->slotExportUpdate(); break;
        case 17: _t->slotCountdownTimerTimeout(); break;
        case 18: _t->slotScheduledCompositionMapperReset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SequenceManager::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::sigProgramChange)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::insertableNoteOnReceived)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::insertableNoteOffReceived)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(int , QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::sendWarning)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(tempoT );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalTempoChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(const MappedEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalMidiInLabel)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(const MappedEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalMidiOutLabel)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalPlaying)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalRecording)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (SequenceManager::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SequenceManager::signalMetronomeActivated)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::SequenceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__SequenceManager.data,
    qt_meta_data_Rosegarden__SequenceManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::SequenceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::SequenceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__SequenceManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CompositionObserver"))
        return static_cast< CompositionObserver*>(this);
    return QObject::qt_metacast(_clname);
}

int Rosegarden::SequenceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::SequenceManager::sigProgramChange(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::SequenceManager::insertableNoteOnReceived(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Rosegarden::SequenceManager::insertableNoteOffReceived(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Rosegarden::SequenceManager::sendWarning(int _t1, QString _t2, QString _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Rosegarden::SequenceManager::signalTempoChanged(tempoT _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Rosegarden::SequenceManager::signalMidiInLabel(const MappedEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Rosegarden::SequenceManager::signalMidiOutLabel(const MappedEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Rosegarden::SequenceManager::signalPlaying(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Rosegarden::SequenceManager::signalRecording(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Rosegarden::SequenceManager::signalMetronomeActivated(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
