/****************************************************************************
** Meta object code from reading C++ file 'ProjectPackager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/general/ProjectPackager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ProjectPackager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__ProjectPackager_t {
    QByteArrayData data[19];
    char stringdata0[221];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__ProjectPackager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__ProjectPackager_t qt_meta_stringdata_Rosegarden__ProjectPackager = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Rosegarden::ProjectPackager"
QT_MOC_LITERAL(1, 28, 4), // "puke"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 5), // "error"
QT_MOC_LITERAL(4, 40, 14), // "rmdirRecursive"
QT_MOC_LITERAL(5, 55, 7), // "dirName"
QT_MOC_LITERAL(6, 63, 6), // "reject"
QT_MOC_LITERAL(7, 70, 13), // "runPackUnpack"
QT_MOC_LITERAL(8, 84, 8), // "exitCode"
QT_MOC_LITERAL(9, 93, 20), // "QProcess::ExitStatus"
QT_MOC_LITERAL(10, 114, 7), // "runPack"
QT_MOC_LITERAL(11, 122, 17), // "startAudioEncoder"
QT_MOC_LITERAL(12, 140, 5), // "files"
QT_MOC_LITERAL(13, 146, 10), // "finishPack"
QT_MOC_LITERAL(14, 157, 9), // "runUnpack"
QT_MOC_LITERAL(15, 167, 17), // "startAudioDecoder"
QT_MOC_LITERAL(16, 185, 9), // "flacFiles"
QT_MOC_LITERAL(17, 195, 12), // "wavpackFiles"
QT_MOC_LITERAL(18, 208, 12) // "finishUnpack"

    },
    "Rosegarden::ProjectPackager\0puke\0\0"
    "error\0rmdirRecursive\0dirName\0reject\0"
    "runPackUnpack\0exitCode\0QProcess::ExitStatus\0"
    "runPack\0startAudioEncoder\0files\0"
    "finishPack\0runUnpack\0startAudioDecoder\0"
    "flacFiles\0wavpackFiles\0finishUnpack"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__ProjectPackager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x09 /* Protected */,
       4,    1,   67,    2, 0x09 /* Protected */,
       6,    0,   70,    2, 0x09 /* Protected */,
       7,    2,   71,    2, 0x09 /* Protected */,
      10,    0,   76,    2, 0x09 /* Protected */,
      11,    1,   77,    2, 0x09 /* Protected */,
      13,    2,   80,    2, 0x09 /* Protected */,
      14,    0,   85,    2, 0x09 /* Protected */,
      15,    2,   86,    2, 0x09 /* Protected */,
      18,    2,   91,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Bool, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 9,    8,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,   12,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 9,    8,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList, QMetaType::QStringList,   16,   17,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 9,    8,    2,

       0        // eod
};

void Rosegarden::ProjectPackager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProjectPackager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->puke((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: { bool _r = _t->rmdirRecursive((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->reject(); break;
        case 3: _t->runPackUnpack((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 4: _t->runPack(); break;
        case 5: _t->startAudioEncoder((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 6: _t->finishPack((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 7: _t->runUnpack(); break;
        case 8: _t->startAudioDecoder((*reinterpret_cast< QStringList(*)>(_a[1])),(*reinterpret_cast< QStringList(*)>(_a[2]))); break;
        case 9: _t->finishUnpack((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::ProjectPackager::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__ProjectPackager.data,
    qt_meta_data_Rosegarden__ProjectPackager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::ProjectPackager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::ProjectPackager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__ProjectPackager.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::ProjectPackager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
