/****************************************************************************
** Meta object code from reading C++ file 'BankEditorDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/studio/BankEditorDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'BankEditorDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__BankEditorDialog_t {
    QByteArrayData data[28];
    char stringdata0[387];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__BankEditorDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__BankEditorDialog_t qt_meta_stringdata_Rosegarden__BankEditorDialog = {
    {
QT_MOC_LITERAL(0, 0, 28), // "Rosegarden::BankEditorDialog"
QT_MOC_LITERAL(1, 29, 7), // "closing"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 18), // "deviceNamesChanged"
QT_MOC_LITERAL(4, 57, 17), // "slotEditLibrarian"
QT_MOC_LITERAL(5, 75, 13), // "slotFileClose"
QT_MOC_LITERAL(6, 89, 8), // "slotEdit"
QT_MOC_LITERAL(7, 98, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(8, 115, 4), // "item"
QT_MOC_LITERAL(9, 120, 6), // "column"
QT_MOC_LITERAL(10, 127, 15), // "slotItemChanged"
QT_MOC_LITERAL(11, 143, 16), // "slotUpdateEditor"
QT_MOC_LITERAL(12, 160, 11), // "currentItem"
QT_MOC_LITERAL(13, 172, 12), // "previousItem"
QT_MOC_LITERAL(14, 185, 11), // "slotAddBank"
QT_MOC_LITERAL(15, 197, 17), // "slotAddKeyMapping"
QT_MOC_LITERAL(16, 215, 10), // "slotDelete"
QT_MOC_LITERAL(17, 226, 13), // "slotDeleteAll"
QT_MOC_LITERAL(18, 240, 10), // "slotImport"
QT_MOC_LITERAL(19, 251, 10), // "slotExport"
QT_MOC_LITERAL(20, 262, 8), // "slotCopy"
QT_MOC_LITERAL(21, 271, 9), // "slotPaste"
QT_MOC_LITERAL(22, 281, 20), // "slotVariationToggled"
QT_MOC_LITERAL(23, 302, 20), // "slotVariationChanged"
QT_MOC_LITERAL(24, 323, 5), // "index"
QT_MOC_LITERAL(25, 329, 25), // "slotBankSelectTypeChanged"
QT_MOC_LITERAL(26, 355, 17), // "slotHelpRequested"
QT_MOC_LITERAL(27, 373, 13) // "slotHelpAbout"

    },
    "Rosegarden::BankEditorDialog\0closing\0"
    "\0deviceNamesChanged\0slotEditLibrarian\0"
    "slotFileClose\0slotEdit\0QTreeWidgetItem*\0"
    "item\0column\0slotItemChanged\0"
    "slotUpdateEditor\0currentItem\0previousItem\0"
    "slotAddBank\0slotAddKeyMapping\0slotDelete\0"
    "slotDeleteAll\0slotImport\0slotExport\0"
    "slotCopy\0slotPaste\0slotVariationToggled\0"
    "slotVariationChanged\0index\0"
    "slotBankSelectTypeChanged\0slotHelpRequested\0"
    "slotHelpAbout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__BankEditorDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x06 /* Public */,
       3,    0,  115,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  116,    2, 0x0a /* Public */,
       5,    0,  117,    2, 0x0a /* Public */,
       6,    2,  118,    2, 0x08 /* Private */,
      10,    2,  123,    2, 0x08 /* Private */,
      11,    2,  128,    2, 0x08 /* Private */,
      14,    0,  133,    2, 0x08 /* Private */,
      15,    0,  134,    2, 0x08 /* Private */,
      16,    0,  135,    2, 0x08 /* Private */,
      17,    0,  136,    2, 0x08 /* Private */,
      18,    0,  137,    2, 0x08 /* Private */,
      19,    0,  138,    2, 0x08 /* Private */,
      20,    0,  139,    2, 0x08 /* Private */,
      21,    0,  140,    2, 0x08 /* Private */,
      22,    0,  141,    2, 0x08 /* Private */,
      23,    1,  142,    2, 0x08 /* Private */,
      25,    1,  145,    2, 0x08 /* Private */,
      26,    0,  148,    2, 0x08 /* Private */,
      27,    0,  149,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int,    8,    9,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int,    8,    9,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,   12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Rosegarden::BankEditorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BankEditorDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closing(); break;
        case 1: _t->deviceNamesChanged(); break;
        case 2: _t->slotEditLibrarian(); break;
        case 3: _t->slotFileClose(); break;
        case 4: _t->slotEdit((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->slotItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->slotUpdateEditor((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 7: _t->slotAddBank(); break;
        case 8: _t->slotAddKeyMapping(); break;
        case 9: _t->slotDelete(); break;
        case 10: _t->slotDeleteAll(); break;
        case 11: _t->slotImport(); break;
        case 12: _t->slotExport(); break;
        case 13: _t->slotCopy(); break;
        case 14: _t->slotPaste(); break;
        case 15: _t->slotVariationToggled(); break;
        case 16: _t->slotVariationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->slotBankSelectTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->slotHelpRequested(); break;
        case 19: _t->slotHelpAbout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BankEditorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BankEditorDialog::closing)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BankEditorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BankEditorDialog::deviceNamesChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::BankEditorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__BankEditorDialog.data,
    qt_meta_data_Rosegarden__BankEditorDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::BankEditorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::BankEditorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__BankEditorDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ActionFileClient"))
        return static_cast< ActionFileClient*>(this);
    if (!strcmp(_clname, "StudioObserver"))
        return static_cast< StudioObserver*>(this);
    if (!strcmp(_clname, "DeviceObserver"))
        return static_cast< DeviceObserver*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Rosegarden::BankEditorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::BankEditorDialog::closing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Rosegarden::BankEditorDialog::deviceNamesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
