/****************************************************************************
** Meta object code from reading C++ file 'SelectDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/dialogs/SelectDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SelectDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__SelectDialog_t {
    QByteArrayData data[29];
    char stringdata0[474];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__SelectDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__SelectDialog_t qt_meta_stringdata_Rosegarden__SelectDialog = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Rosegarden::SelectDialog"
QT_MOC_LITERAL(1, 25, 6), // "accept"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 4), // "help"
QT_MOC_LITERAL(4, 38, 15), // "slotUseAllBreve"
QT_MOC_LITERAL(5, 54, 19), // "slotUseAllSemiBreve"
QT_MOC_LITERAL(6, 74, 15), // "slotUseAllMinim"
QT_MOC_LITERAL(7, 90, 18), // "slotUseAllCrotchet"
QT_MOC_LITERAL(8, 109, 16), // "slotUseAllQuaver"
QT_MOC_LITERAL(9, 126, 20), // "slotUseAllSemiQuaver"
QT_MOC_LITERAL(10, 147, 18), // "slotUseAllDemiSemi"
QT_MOC_LITERAL(11, 166, 22), // "slotUseAllHemiDemiSemi"
QT_MOC_LITERAL(12, 189, 16), // "slotUseAllNormal"
QT_MOC_LITERAL(13, 206, 16), // "slotUseAllDotted"
QT_MOC_LITERAL(14, 223, 22), // "slotUseAllDoubleDotted"
QT_MOC_LITERAL(15, 246, 16), // "slotUseAllTuplet"
QT_MOC_LITERAL(16, 263, 20), // "slotUseAllRestNormal"
QT_MOC_LITERAL(17, 284, 20), // "slotUseAllRestDotted"
QT_MOC_LITERAL(18, 305, 26), // "slotUseAllRestDoubleDotted"
QT_MOC_LITERAL(19, 332, 20), // "slotUseAllRestTuplet"
QT_MOC_LITERAL(20, 353, 18), // "slotUseAllDuration"
QT_MOC_LITERAL(21, 372, 11), // "slotUseRow1"
QT_MOC_LITERAL(22, 384, 11), // "slotUseRow2"
QT_MOC_LITERAL(23, 396, 11), // "slotUseRow3"
QT_MOC_LITERAL(24, 408, 11), // "slotUseRow4"
QT_MOC_LITERAL(25, 420, 11), // "slotUseRow5"
QT_MOC_LITERAL(26, 432, 11), // "slotUseRow6"
QT_MOC_LITERAL(27, 444, 11), // "slotUseRow7"
QT_MOC_LITERAL(28, 456, 17) // "slotUseAllSpecial"

    },
    "Rosegarden::SelectDialog\0accept\0\0help\0"
    "slotUseAllBreve\0slotUseAllSemiBreve\0"
    "slotUseAllMinim\0slotUseAllCrotchet\0"
    "slotUseAllQuaver\0slotUseAllSemiQuaver\0"
    "slotUseAllDemiSemi\0slotUseAllHemiDemiSemi\0"
    "slotUseAllNormal\0slotUseAllDotted\0"
    "slotUseAllDoubleDotted\0slotUseAllTuplet\0"
    "slotUseAllRestNormal\0slotUseAllRestDotted\0"
    "slotUseAllRestDoubleDotted\0"
    "slotUseAllRestTuplet\0slotUseAllDuration\0"
    "slotUseRow1\0slotUseRow2\0slotUseRow3\0"
    "slotUseRow4\0slotUseRow5\0slotUseRow6\0"
    "slotUseRow7\0slotUseAllSpecial"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__SelectDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x0a /* Public */,
       3,    0,  150,    2, 0x0a /* Public */,
       4,    1,  151,    2, 0x09 /* Protected */,
       5,    1,  154,    2, 0x09 /* Protected */,
       6,    1,  157,    2, 0x09 /* Protected */,
       7,    1,  160,    2, 0x09 /* Protected */,
       8,    1,  163,    2, 0x09 /* Protected */,
       9,    1,  166,    2, 0x09 /* Protected */,
      10,    1,  169,    2, 0x09 /* Protected */,
      11,    1,  172,    2, 0x09 /* Protected */,
      12,    1,  175,    2, 0x09 /* Protected */,
      13,    1,  178,    2, 0x09 /* Protected */,
      14,    1,  181,    2, 0x09 /* Protected */,
      15,    1,  184,    2, 0x09 /* Protected */,
      16,    1,  187,    2, 0x09 /* Protected */,
      17,    1,  190,    2, 0x09 /* Protected */,
      18,    1,  193,    2, 0x09 /* Protected */,
      19,    1,  196,    2, 0x09 /* Protected */,
      20,    1,  199,    2, 0x09 /* Protected */,
      21,    1,  202,    2, 0x09 /* Protected */,
      22,    1,  205,    2, 0x09 /* Protected */,
      23,    1,  208,    2, 0x09 /* Protected */,
      24,    1,  211,    2, 0x09 /* Protected */,
      25,    1,  214,    2, 0x09 /* Protected */,
      26,    1,  217,    2, 0x09 /* Protected */,
      27,    1,  220,    2, 0x09 /* Protected */,
      28,    1,  223,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void Rosegarden::SelectDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SelectDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->help(); break;
        case 2: _t->slotUseAllBreve((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotUseAllSemiBreve((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->slotUseAllMinim((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotUseAllCrotchet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->slotUseAllQuaver((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->slotUseAllSemiQuaver((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotUseAllDemiSemi((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->slotUseAllHemiDemiSemi((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotUseAllNormal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->slotUseAllDotted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotUseAllDoubleDotted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->slotUseAllTuplet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->slotUseAllRestNormal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->slotUseAllRestDotted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->slotUseAllRestDoubleDotted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->slotUseAllRestTuplet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->slotUseAllDuration((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->slotUseRow1((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->slotUseRow2((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->slotUseRow3((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->slotUseRow4((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->slotUseRow5((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->slotUseRow6((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->slotUseRow7((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->slotUseAllSpecial((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::SelectDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__SelectDialog.data,
    qt_meta_data_Rosegarden__SelectDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::SelectDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::SelectDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__SelectDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Rosegarden::SelectDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
