/****************************************************************************
** Meta object code from reading C++ file 'MatrixView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../rosegarden/src/gui/editors/matrix/MatrixView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MatrixView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Rosegarden__MatrixView_t {
    QByteArrayData data[120];
    char stringdata0[2205];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Rosegarden__MatrixView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Rosegarden__MatrixView_t qt_meta_stringdata_Rosegarden__MatrixView = {
    {
QT_MOC_LITERAL(0, 0, 22), // "Rosegarden::MatrixView"
QT_MOC_LITERAL(1, 23, 18), // "editTriggerSegment"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 4), // "play"
QT_MOC_LITERAL(4, 48, 4), // "stop"
QT_MOC_LITERAL(5, 53, 14), // "rewindPlayback"
QT_MOC_LITERAL(6, 68, 19), // "fastForwardPlayback"
QT_MOC_LITERAL(7, 88, 25), // "rewindPlaybackToBeginning"
QT_MOC_LITERAL(8, 114, 24), // "fastForwardPlaybackToEnd"
QT_MOC_LITERAL(9, 139, 5), // "panic"
QT_MOC_LITERAL(10, 145, 25), // "stepByStepTargetRequested"
QT_MOC_LITERAL(11, 171, 24), // "noteInsertedFromKeyboard"
QT_MOC_LITERAL(12, 196, 8), // "Segment*"
QT_MOC_LITERAL(13, 205, 7), // "segment"
QT_MOC_LITERAL(14, 213, 5), // "pitch"
QT_MOC_LITERAL(15, 219, 28), // "slotInsertableNoteOnReceived"
QT_MOC_LITERAL(16, 248, 8), // "velocity"
QT_MOC_LITERAL(17, 257, 29), // "slotInsertableNoteOffReceived"
QT_MOC_LITERAL(18, 287, 29), // "slotStepByStepTargetRequested"
QT_MOC_LITERAL(19, 317, 18), // "slotSegmentDeleted"
QT_MOC_LITERAL(20, 336, 16), // "slotSceneDeleted"
QT_MOC_LITERAL(21, 353, 12), // "slotQuantize"
QT_MOC_LITERAL(22, 366, 18), // "slotRepeatQuantize"
QT_MOC_LITERAL(23, 385, 17), // "slotCollapseNotes"
QT_MOC_LITERAL(24, 403, 10), // "slotLegato"
QT_MOC_LITERAL(25, 414, 14), // "slotVelocityUp"
QT_MOC_LITERAL(26, 429, 16), // "slotVelocityDown"
QT_MOC_LITERAL(27, 446, 17), // "slotSetVelocities"
QT_MOC_LITERAL(28, 464, 26), // "slotSetVelocitiesToCurrent"
QT_MOC_LITERAL(29, 491, 18), // "slotSetControllers"
QT_MOC_LITERAL(30, 510, 20), // "slotPlaceControllers"
QT_MOC_LITERAL(31, 531, 18), // "slotTriggerSegment"
QT_MOC_LITERAL(32, 550, 18), // "slotRemoveTriggers"
QT_MOC_LITERAL(33, 569, 13), // "slotSelectAll"
QT_MOC_LITERAL(34, 583, 20), // "slotPreviewSelection"
QT_MOC_LITERAL(35, 604, 13), // "slotClearLoop"
QT_MOC_LITERAL(36, 618, 18), // "slotClearSelection"
QT_MOC_LITERAL(37, 637, 17), // "slotEscapePressed"
QT_MOC_LITERAL(38, 655, 19), // "slotFilterSelection"
QT_MOC_LITERAL(39, 675, 23), // "slotEditAddKeySignature"
QT_MOC_LITERAL(40, 699, 23), // "slotCurrentSegmentPrior"
QT_MOC_LITERAL(41, 723, 22), // "slotCurrentSegmentNext"
QT_MOC_LITERAL(42, 746, 16), // "slotSetPaintTool"
QT_MOC_LITERAL(43, 763, 16), // "slotSetEraseTool"
QT_MOC_LITERAL(44, 780, 17), // "slotSetSelectTool"
QT_MOC_LITERAL(45, 798, 15), // "slotSetMoveTool"
QT_MOC_LITERAL(46, 814, 17), // "slotSetResizeTool"
QT_MOC_LITERAL(47, 832, 19), // "slotSetVelocityTool"
QT_MOC_LITERAL(48, 852, 20), // "slotSetSnapFromIndex"
QT_MOC_LITERAL(49, 873, 21), // "slotSetSnapFromAction"
QT_MOC_LITERAL(50, 895, 11), // "slotSetSnap"
QT_MOC_LITERAL(51, 907, 5), // "timeT"
QT_MOC_LITERAL(52, 913, 21), // "slotQuantizeSelection"
QT_MOC_LITERAL(53, 935, 22), // "slotSetCurrentVelocity"
QT_MOC_LITERAL(54, 958, 35), // "slotSetCurrentVelocityFromSel..."
QT_MOC_LITERAL(55, 994, 18), // "slotScrollToFollow"
QT_MOC_LITERAL(56, 1013, 8), // "slotLoop"
QT_MOC_LITERAL(57, 1022, 15), // "slotLoopChanged"
QT_MOC_LITERAL(58, 1038, 20), // "slotUpdateMenuStates"
QT_MOC_LITERAL(59, 1059, 24), // "slotRulerSelectionUpdate"
QT_MOC_LITERAL(60, 1084, 11), // "slotEditCut"
QT_MOC_LITERAL(61, 1096, 12), // "slotEditCopy"
QT_MOC_LITERAL(62, 1109, 13), // "slotEditPaste"
QT_MOC_LITERAL(63, 1123, 14), // "slotEditDelete"
QT_MOC_LITERAL(64, 1138, 21), // "slotToggleChordsRuler"
QT_MOC_LITERAL(65, 1160, 20), // "slotToggleTempoRuler"
QT_MOC_LITERAL(66, 1181, 23), // "slotToggleVelocityRuler"
QT_MOC_LITERAL(67, 1205, 24), // "slotTogglePitchbendRuler"
QT_MOC_LITERAL(68, 1230, 26), // "slotToggleKeyPressureRuler"
QT_MOC_LITERAL(69, 1257, 30), // "slotToggleChannelPressureRuler"
QT_MOC_LITERAL(70, 1288, 19), // "slotAddControlRuler"
QT_MOC_LITERAL(71, 1308, 8), // "QAction*"
QT_MOC_LITERAL(72, 1317, 8), // "slotHelp"
QT_MOC_LITERAL(73, 1326, 12), // "slotTutorial"
QT_MOC_LITERAL(74, 1339, 17), // "slotBugGuidelines"
QT_MOC_LITERAL(75, 1357, 13), // "slotHelpAbout"
QT_MOC_LITERAL(76, 1371, 15), // "slotHelpAboutQt"
QT_MOC_LITERAL(77, 1387, 10), // "slotDonate"
QT_MOC_LITERAL(78, 1398, 13), // "slotShowNames"
QT_MOC_LITERAL(79, 1412, 13), // "slotHighlight"
QT_MOC_LITERAL(80, 1426, 19), // "slotShowContextHelp"
QT_MOC_LITERAL(81, 1446, 12), // "slotAddTempo"
QT_MOC_LITERAL(82, 1459, 20), // "slotAddTimeSignature"
QT_MOC_LITERAL(83, 1480, 15), // "slotRescaleHalf"
QT_MOC_LITERAL(84, 1496, 17), // "slotRescaleDouble"
QT_MOC_LITERAL(85, 1514, 11), // "slotRescale"
QT_MOC_LITERAL(86, 1526, 15), // "slotTransposeUp"
QT_MOC_LITERAL(87, 1542, 21), // "slotTransposeUpOctave"
QT_MOC_LITERAL(88, 1564, 17), // "slotTransposeDown"
QT_MOC_LITERAL(89, 1582, 23), // "slotTransposeDownOctave"
QT_MOC_LITERAL(90, 1606, 13), // "slotTranspose"
QT_MOC_LITERAL(91, 1620, 21), // "slotDiatonicTranspose"
QT_MOC_LITERAL(92, 1642, 10), // "slotInvert"
QT_MOC_LITERAL(93, 1653, 14), // "slotRetrograde"
QT_MOC_LITERAL(94, 1668, 20), // "slotRetrogradeInvert"
QT_MOC_LITERAL(95, 1689, 11), // "slotJogLeft"
QT_MOC_LITERAL(96, 1701, 12), // "slotJogRight"
QT_MOC_LITERAL(97, 1714, 16), // "slotStepBackward"
QT_MOC_LITERAL(98, 1731, 15), // "slotStepForward"
QT_MOC_LITERAL(99, 1747, 27), // "slotExtendSelectionBackward"
QT_MOC_LITERAL(100, 1775, 26), // "slotExtendSelectionForward"
QT_MOC_LITERAL(101, 1802, 30), // "slotExtendSelectionBackwardBar"
QT_MOC_LITERAL(102, 1833, 29), // "slotExtendSelectionForwardBar"
QT_MOC_LITERAL(103, 1863, 3), // "bar"
QT_MOC_LITERAL(104, 1867, 24), // "slotInsertNoteFromAction"
QT_MOC_LITERAL(105, 1892, 31), // "slotInsertableNoteEventReceived"
QT_MOC_LITERAL(106, 1924, 6), // "noteOn"
QT_MOC_LITERAL(107, 1931, 21), // "slotPitchBendSequence"
QT_MOC_LITERAL(108, 1953, 22), // "slotControllerSequence"
QT_MOC_LITERAL(109, 1976, 19), // "slotConstrainedMove"
QT_MOC_LITERAL(110, 1996, 20), // "slotToggleStepByStep"
QT_MOC_LITERAL(111, 2017, 21), // "slotUpdateWindowTitle"
QT_MOC_LITERAL(112, 2039, 1), // "m"
QT_MOC_LITERAL(113, 2041, 19), // "slotToggleChordMode"
QT_MOC_LITERAL(114, 2061, 24), // "slotToggleGeneralToolBar"
QT_MOC_LITERAL(115, 2086, 22), // "slotToggleToolsToolBar"
QT_MOC_LITERAL(116, 2109, 24), // "slotToggleActionsToolBar"
QT_MOC_LITERAL(117, 2134, 23), // "slotToggleRulersToolBar"
QT_MOC_LITERAL(118, 2158, 26), // "slotToggleTransportToolBar"
QT_MOC_LITERAL(119, 2185, 19) // "slotVelocityChanged"

    },
    "Rosegarden::MatrixView\0editTriggerSegment\0"
    "\0play\0stop\0rewindPlayback\0fastForwardPlayback\0"
    "rewindPlaybackToBeginning\0"
    "fastForwardPlaybackToEnd\0panic\0"
    "stepByStepTargetRequested\0"
    "noteInsertedFromKeyboard\0Segment*\0"
    "segment\0pitch\0slotInsertableNoteOnReceived\0"
    "velocity\0slotInsertableNoteOffReceived\0"
    "slotStepByStepTargetRequested\0"
    "slotSegmentDeleted\0slotSceneDeleted\0"
    "slotQuantize\0slotRepeatQuantize\0"
    "slotCollapseNotes\0slotLegato\0"
    "slotVelocityUp\0slotVelocityDown\0"
    "slotSetVelocities\0slotSetVelocitiesToCurrent\0"
    "slotSetControllers\0slotPlaceControllers\0"
    "slotTriggerSegment\0slotRemoveTriggers\0"
    "slotSelectAll\0slotPreviewSelection\0"
    "slotClearLoop\0slotClearSelection\0"
    "slotEscapePressed\0slotFilterSelection\0"
    "slotEditAddKeySignature\0slotCurrentSegmentPrior\0"
    "slotCurrentSegmentNext\0slotSetPaintTool\0"
    "slotSetEraseTool\0slotSetSelectTool\0"
    "slotSetMoveTool\0slotSetResizeTool\0"
    "slotSetVelocityTool\0slotSetSnapFromIndex\0"
    "slotSetSnapFromAction\0slotSetSnap\0"
    "timeT\0slotQuantizeSelection\0"
    "slotSetCurrentVelocity\0"
    "slotSetCurrentVelocityFromSelection\0"
    "slotScrollToFollow\0slotLoop\0slotLoopChanged\0"
    "slotUpdateMenuStates\0slotRulerSelectionUpdate\0"
    "slotEditCut\0slotEditCopy\0slotEditPaste\0"
    "slotEditDelete\0slotToggleChordsRuler\0"
    "slotToggleTempoRuler\0slotToggleVelocityRuler\0"
    "slotTogglePitchbendRuler\0"
    "slotToggleKeyPressureRuler\0"
    "slotToggleChannelPressureRuler\0"
    "slotAddControlRuler\0QAction*\0slotHelp\0"
    "slotTutorial\0slotBugGuidelines\0"
    "slotHelpAbout\0slotHelpAboutQt\0slotDonate\0"
    "slotShowNames\0slotHighlight\0"
    "slotShowContextHelp\0slotAddTempo\0"
    "slotAddTimeSignature\0slotRescaleHalf\0"
    "slotRescaleDouble\0slotRescale\0"
    "slotTransposeUp\0slotTransposeUpOctave\0"
    "slotTransposeDown\0slotTransposeDownOctave\0"
    "slotTranspose\0slotDiatonicTranspose\0"
    "slotInvert\0slotRetrograde\0"
    "slotRetrogradeInvert\0slotJogLeft\0"
    "slotJogRight\0slotStepBackward\0"
    "slotStepForward\0slotExtendSelectionBackward\0"
    "slotExtendSelectionForward\0"
    "slotExtendSelectionBackwardBar\0"
    "slotExtendSelectionForwardBar\0bar\0"
    "slotInsertNoteFromAction\0"
    "slotInsertableNoteEventReceived\0noteOn\0"
    "slotPitchBendSequence\0slotControllerSequence\0"
    "slotConstrainedMove\0slotToggleStepByStep\0"
    "slotUpdateWindowTitle\0m\0slotToggleChordMode\0"
    "slotToggleGeneralToolBar\0"
    "slotToggleToolsToolBar\0slotToggleActionsToolBar\0"
    "slotToggleRulersToolBar\0"
    "slotToggleTransportToolBar\0"
    "slotVelocityChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Rosegarden__MatrixView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     112,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  574,    2, 0x06 /* Public */,
       3,    0,  577,    2, 0x06 /* Public */,
       4,    0,  578,    2, 0x06 /* Public */,
       5,    0,  579,    2, 0x06 /* Public */,
       6,    0,  580,    2, 0x06 /* Public */,
       7,    0,  581,    2, 0x06 /* Public */,
       8,    0,  582,    2, 0x06 /* Public */,
       9,    0,  583,    2, 0x06 /* Public */,
      10,    1,  584,    2, 0x06 /* Public */,
      11,    2,  587,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    2,  592,    2, 0x0a /* Public */,
      17,    2,  597,    2, 0x0a /* Public */,
      18,    1,  602,    2, 0x0a /* Public */,
      19,    1,  605,    2, 0x09 /* Protected */,
      20,    0,  608,    2, 0x09 /* Protected */,
      21,    0,  609,    2, 0x09 /* Protected */,
      22,    0,  610,    2, 0x09 /* Protected */,
      23,    0,  611,    2, 0x09 /* Protected */,
      24,    0,  612,    2, 0x09 /* Protected */,
      25,    0,  613,    2, 0x09 /* Protected */,
      26,    0,  614,    2, 0x09 /* Protected */,
      27,    0,  615,    2, 0x09 /* Protected */,
      28,    0,  616,    2, 0x09 /* Protected */,
      29,    0,  617,    2, 0x09 /* Protected */,
      30,    0,  618,    2, 0x09 /* Protected */,
      31,    0,  619,    2, 0x09 /* Protected */,
      32,    0,  620,    2, 0x09 /* Protected */,
      33,    0,  621,    2, 0x09 /* Protected */,
      34,    0,  622,    2, 0x09 /* Protected */,
      35,    0,  623,    2, 0x09 /* Protected */,
      36,    0,  624,    2, 0x09 /* Protected */,
      37,    0,  625,    2, 0x09 /* Protected */,
      38,    0,  626,    2, 0x09 /* Protected */,
      39,    0,  627,    2, 0x09 /* Protected */,
      40,    0,  628,    2, 0x09 /* Protected */,
      41,    0,  629,    2, 0x09 /* Protected */,
      42,    0,  630,    2, 0x09 /* Protected */,
      43,    0,  631,    2, 0x09 /* Protected */,
      44,    0,  632,    2, 0x09 /* Protected */,
      45,    0,  633,    2, 0x09 /* Protected */,
      46,    0,  634,    2, 0x09 /* Protected */,
      47,    0,  635,    2, 0x09 /* Protected */,
      48,    1,  636,    2, 0x09 /* Protected */,
      49,    0,  639,    2, 0x09 /* Protected */,
      50,    1,  640,    2, 0x09 /* Protected */,
      52,    1,  643,    2, 0x09 /* Protected */,
      53,    1,  646,    2, 0x09 /* Protected */,
      54,    0,  649,    2, 0x09 /* Protected */,
      55,    0,  650,    2, 0x09 /* Protected */,
      56,    0,  651,    2, 0x09 /* Protected */,
      57,    0,  652,    2, 0x09 /* Protected */,
      58,    0,  653,    2, 0x09 /* Protected */,
      59,    0,  654,    2, 0x09 /* Protected */,
      60,    0,  655,    2, 0x09 /* Protected */,
      61,    0,  656,    2, 0x09 /* Protected */,
      62,    0,  657,    2, 0x09 /* Protected */,
      63,    0,  658,    2, 0x09 /* Protected */,
      64,    0,  659,    2, 0x09 /* Protected */,
      65,    0,  660,    2, 0x09 /* Protected */,
      66,    0,  661,    2, 0x09 /* Protected */,
      67,    0,  662,    2, 0x09 /* Protected */,
      68,    0,  663,    2, 0x09 /* Protected */,
      69,    0,  664,    2, 0x09 /* Protected */,
      70,    1,  665,    2, 0x09 /* Protected */,
      72,    0,  668,    2, 0x09 /* Protected */,
      73,    0,  669,    2, 0x09 /* Protected */,
      74,    0,  670,    2, 0x09 /* Protected */,
      75,    0,  671,    2, 0x09 /* Protected */,
      76,    0,  672,    2, 0x09 /* Protected */,
      77,    0,  673,    2, 0x09 /* Protected */,
      78,    0,  674,    2, 0x09 /* Protected */,
      79,    0,  675,    2, 0x09 /* Protected */,
      80,    1,  676,    2, 0x09 /* Protected */,
      81,    0,  679,    2, 0x09 /* Protected */,
      82,    0,  680,    2, 0x09 /* Protected */,
      83,    0,  681,    2, 0x09 /* Protected */,
      84,    0,  682,    2, 0x09 /* Protected */,
      85,    0,  683,    2, 0x09 /* Protected */,
      86,    0,  684,    2, 0x09 /* Protected */,
      87,    0,  685,    2, 0x09 /* Protected */,
      88,    0,  686,    2, 0x09 /* Protected */,
      89,    0,  687,    2, 0x09 /* Protected */,
      90,    0,  688,    2, 0x09 /* Protected */,
      91,    0,  689,    2, 0x09 /* Protected */,
      92,    0,  690,    2, 0x09 /* Protected */,
      93,    0,  691,    2, 0x09 /* Protected */,
      94,    0,  692,    2, 0x09 /* Protected */,
      95,    0,  693,    2, 0x09 /* Protected */,
      96,    0,  694,    2, 0x09 /* Protected */,
      97,    0,  695,    2, 0x09 /* Protected */,
      98,    0,  696,    2, 0x09 /* Protected */,
      99,    0,  697,    2, 0x09 /* Protected */,
     100,    0,  698,    2, 0x09 /* Protected */,
     101,    0,  699,    2, 0x09 /* Protected */,
     102,    0,  700,    2, 0x09 /* Protected */,
      99,    1,  701,    2, 0x09 /* Protected */,
     100,    1,  704,    2, 0x09 /* Protected */,
     104,    0,  707,    2, 0x09 /* Protected */,
     105,    3,  708,    2, 0x09 /* Protected */,
     107,    0,  715,    2, 0x09 /* Protected */,
     108,    0,  716,    2, 0x09 /* Protected */,
     109,    0,  717,    2, 0x09 /* Protected */,
     110,    0,  718,    2, 0x09 /* Protected */,
     111,    1,  719,    2, 0x09 /* Protected */,
     111,    0,  722,    2, 0x29 /* Protected | MethodCloned */,
     113,    0,  723,    2, 0x09 /* Protected */,
     114,    0,  724,    2, 0x09 /* Protected */,
     115,    0,  725,    2, 0x09 /* Protected */,
     116,    0,  726,    2, 0x09 /* Protected */,
     117,    0,  727,    2, 0x09 /* Protected */,
     118,    0,  728,    2, 0x09 /* Protected */,
     119,    1,  729,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QObjectStar,    2,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int,   13,   14,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   14,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   14,   16,
    QMetaType::Void, QMetaType::QObjectStar,    2,
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 51,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 71,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  103,
    QMetaType::Void, QMetaType::Bool,  103,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool,   14,   16,  106,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  112,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,

       0        // eod
};

void Rosegarden::MatrixView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MatrixView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editTriggerSegment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->play(); break;
        case 2: _t->stop(); break;
        case 3: _t->rewindPlayback(); break;
        case 4: _t->fastForwardPlayback(); break;
        case 5: _t->rewindPlaybackToBeginning(); break;
        case 6: _t->fastForwardPlaybackToEnd(); break;
        case 7: _t->panic(); break;
        case 8: _t->stepByStepTargetRequested((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 9: _t->noteInsertedFromKeyboard((*reinterpret_cast< Segment*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->slotInsertableNoteOnReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: _t->slotInsertableNoteOffReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 12: _t->slotStepByStepTargetRequested((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 13: _t->slotSegmentDeleted((*reinterpret_cast< Segment*(*)>(_a[1]))); break;
        case 14: _t->slotSceneDeleted(); break;
        case 15: _t->slotQuantize(); break;
        case 16: _t->slotRepeatQuantize(); break;
        case 17: _t->slotCollapseNotes(); break;
        case 18: _t->slotLegato(); break;
        case 19: _t->slotVelocityUp(); break;
        case 20: _t->slotVelocityDown(); break;
        case 21: _t->slotSetVelocities(); break;
        case 22: _t->slotSetVelocitiesToCurrent(); break;
        case 23: _t->slotSetControllers(); break;
        case 24: _t->slotPlaceControllers(); break;
        case 25: _t->slotTriggerSegment(); break;
        case 26: _t->slotRemoveTriggers(); break;
        case 27: _t->slotSelectAll(); break;
        case 28: _t->slotPreviewSelection(); break;
        case 29: _t->slotClearLoop(); break;
        case 30: _t->slotClearSelection(); break;
        case 31: _t->slotEscapePressed(); break;
        case 32: _t->slotFilterSelection(); break;
        case 33: _t->slotEditAddKeySignature(); break;
        case 34: _t->slotCurrentSegmentPrior(); break;
        case 35: _t->slotCurrentSegmentNext(); break;
        case 36: _t->slotSetPaintTool(); break;
        case 37: _t->slotSetEraseTool(); break;
        case 38: _t->slotSetSelectTool(); break;
        case 39: _t->slotSetMoveTool(); break;
        case 40: _t->slotSetResizeTool(); break;
        case 41: _t->slotSetVelocityTool(); break;
        case 42: _t->slotSetSnapFromIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 43: _t->slotSetSnapFromAction(); break;
        case 44: _t->slotSetSnap((*reinterpret_cast< timeT(*)>(_a[1]))); break;
        case 45: _t->slotQuantizeSelection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->slotSetCurrentVelocity((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 47: _t->slotSetCurrentVelocityFromSelection(); break;
        case 48: _t->slotScrollToFollow(); break;
        case 49: _t->slotLoop(); break;
        case 50: _t->slotLoopChanged(); break;
        case 51: _t->slotUpdateMenuStates(); break;
        case 52: _t->slotRulerSelectionUpdate(); break;
        case 53: _t->slotEditCut(); break;
        case 54: _t->slotEditCopy(); break;
        case 55: _t->slotEditPaste(); break;
        case 56: _t->slotEditDelete(); break;
        case 57: _t->slotToggleChordsRuler(); break;
        case 58: _t->slotToggleTempoRuler(); break;
        case 59: _t->slotToggleVelocityRuler(); break;
        case 60: _t->slotTogglePitchbendRuler(); break;
        case 61: _t->slotToggleKeyPressureRuler(); break;
        case 62: _t->slotToggleChannelPressureRuler(); break;
        case 63: _t->slotAddControlRuler((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 64: _t->slotHelp(); break;
        case 65: _t->slotTutorial(); break;
        case 66: _t->slotBugGuidelines(); break;
        case 67: _t->slotHelpAbout(); break;
        case 68: _t->slotHelpAboutQt(); break;
        case 69: _t->slotDonate(); break;
        case 70: _t->slotShowNames(); break;
        case 71: _t->slotHighlight(); break;
        case 72: _t->slotShowContextHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 73: _t->slotAddTempo(); break;
        case 74: _t->slotAddTimeSignature(); break;
        case 75: _t->slotRescaleHalf(); break;
        case 76: _t->slotRescaleDouble(); break;
        case 77: _t->slotRescale(); break;
        case 78: _t->slotTransposeUp(); break;
        case 79: _t->slotTransposeUpOctave(); break;
        case 80: _t->slotTransposeDown(); break;
        case 81: _t->slotTransposeDownOctave(); break;
        case 82: _t->slotTranspose(); break;
        case 83: _t->slotDiatonicTranspose(); break;
        case 84: _t->slotInvert(); break;
        case 85: _t->slotRetrograde(); break;
        case 86: _t->slotRetrogradeInvert(); break;
        case 87: _t->slotJogLeft(); break;
        case 88: _t->slotJogRight(); break;
        case 89: _t->slotStepBackward(); break;
        case 90: _t->slotStepForward(); break;
        case 91: _t->slotExtendSelectionBackward(); break;
        case 92: _t->slotExtendSelectionForward(); break;
        case 93: _t->slotExtendSelectionBackwardBar(); break;
        case 94: _t->slotExtendSelectionForwardBar(); break;
        case 95: _t->slotExtendSelectionBackward((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 96: _t->slotExtendSelectionForward((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 97: _t->slotInsertNoteFromAction(); break;
        case 98: _t->slotInsertableNoteEventReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 99: _t->slotPitchBendSequence(); break;
        case 100: _t->slotControllerSequence(); break;
        case 101: _t->slotConstrainedMove(); break;
        case 102: _t->slotToggleStepByStep(); break;
        case 103: _t->slotUpdateWindowTitle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 104: _t->slotUpdateWindowTitle(); break;
        case 105: _t->slotToggleChordMode(); break;
        case 106: _t->slotToggleGeneralToolBar(); break;
        case 107: _t->slotToggleToolsToolBar(); break;
        case 108: _t->slotToggleActionsToolBar(); break;
        case 109: _t->slotToggleRulersToolBar(); break;
        case 110: _t->slotToggleTransportToolBar(); break;
        case 111: _t->slotVelocityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 63:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MatrixView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::editTriggerSegment)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::play)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::stop)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::rewindPlayback)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::fastForwardPlayback)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::rewindPlaybackToBeginning)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::fastForwardPlaybackToEnd)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::panic)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::stepByStepTargetRequested)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (MatrixView::*)(Segment * , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MatrixView::noteInsertedFromKeyboard)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Rosegarden::MatrixView::staticMetaObject = { {
    QMetaObject::SuperData::link<EditViewBase::staticMetaObject>(),
    qt_meta_stringdata_Rosegarden__MatrixView.data,
    qt_meta_data_Rosegarden__MatrixView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Rosegarden::MatrixView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Rosegarden::MatrixView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Rosegarden__MatrixView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SelectionManager"))
        return static_cast< SelectionManager*>(this);
    return EditViewBase::qt_metacast(_clname);
}

int Rosegarden::MatrixView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EditViewBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 112)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 112;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 112)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 112;
    }
    return _id;
}

// SIGNAL 0
void Rosegarden::MatrixView::editTriggerSegment(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Rosegarden::MatrixView::play()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Rosegarden::MatrixView::stop()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Rosegarden::MatrixView::rewindPlayback()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Rosegarden::MatrixView::fastForwardPlayback()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Rosegarden::MatrixView::rewindPlaybackToBeginning()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Rosegarden::MatrixView::fastForwardPlaybackToEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Rosegarden::MatrixView::panic()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Rosegarden::MatrixView::stepByStepTargetRequested(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Rosegarden::MatrixView::noteInsertedFromKeyboard(Segment * _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
